searchNodes=[{"doc":"Numerical Elixir. The Nx library is a collection of functions and data types to work with Numerical Elixir. This module defines the main entry point for building and working with said data-structures. For example, to create an n-dimensional tensor, do: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . shape ( t ) { 2 , 2 } Nx also provides the so-called numerical definitions under the Nx.Defn module. They are a subset of Elixir tailored for numerical computations. For example, it overrides Elixir's default operators so they are tensor-aware: defn softmax ( t ) do Nx . exp ( t ) / Nx . sum ( Nx . exp ( t ) ) end Code inside defn functions can also be given to custom compilers, which can compile said functions just-in-time (JIT) to run on the CPU or on the GPU. References Here is a general outline of the main references in this library: For an introduction, see our Intro to Nx guide This module provides the main API for working with tensors Nx.Defn provides numerical definitions, CPU/GPU compilation, gradients, and more Nx.LinAlg provides functions related to linear algebra Nx.Constants declares many constants commonly used in numerical code Continue reading this documentation for an overview of creating, broadcasting, and accessing/slicing Nx tensors. Creating tensors The main APIs for creating tensors are tensor/2 , from_binary/2 , iota/2 , eye/2 , random_uniform/2 , random_normal/2 , and broadcast/3 . The tensor types can be one of: unsigned integers ( u8 , u16 , u32 , u64 ) signed integers ( s8 , s16 , s32 , s64 ) floats ( f16 , f32 , f64 ) brain floats ( bf16 ) and complex numbers ( c64 , c128 ) The types are tracked as tuples: iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] , type : { :f , 32 } ) # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; But a shortcut atom notation is also available: iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] , type : :f32 ) # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; The tensor dimensions can also be named, via the :names option available to all creation functions: iex&gt; Nx . iota ( { 2 , 3 } , names : [ :x , :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] &gt; Finally, for creating vectors and matrices, a sigil notation is available: iex&gt; import Nx , only : :sigils iex&gt; ~V[1 2 3]f32 # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; iex&gt; import Nx , only : :sigils iex&gt; ~M&#39;&#39;&#39; ...&gt; 1 2 3 ...&gt; 4 5 6 ...&gt; &#39;&#39;&#39;s32 # Nx.Tensor &lt; s32 [ 2 ] [ 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] &gt; All other APIs accept exclusively numbers or tensors, unless explicitly noted otherwise. Broadcasting Broadcasting allows operations on two tensors of different shapes to match. For example, most often operations between tensors have the same shape: iex&gt; a = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; b = Nx . tensor ( [ 10 , 20 , 30 ] ) iex&gt; Nx . add ( a , b ) # Nx.Tensor &lt; s64 [ 3 ] [ 11 , 22 , 33 ] &gt; Now let's imagine you want to multiply a large tensor of dimensions 1000x1000x1000 by 2. If you had to create a similarly large tensor only to perform this operation, it would be inefficient. Therefore, you can simply multiply this large tensor by the scalar 2, and Nx will propagate its dimensions at the time the operation happens, without allocating a large intermediate tensor: iex&gt; Nx . multiply ( Nx . tensor ( [ 1 , 2 , 3 ] ) , 2 ) # Nx.Tensor &lt; s64 [ 3 ] [ 2 , 4 , 6 ] &gt; In practice, broadcasting is not restricted only to scalars; it is a general algorithm that applies to all dimensions of a tensor. When broadcasting, Nx compares the shapes of the two tensors, starting with the trailing ones, such that: If the dimensions have equal size, then they are compatible If one of the dimensions have size of 1, it is &quot;broadcast&quot; to match the dimension of the other In case one tensor has more dimensions than the other, the missing dimensions are considered to be of size one. Here are some examples of how broadcast would work when multiplying two tensors with the following shapes: s64 [ 3 ] * s64 #=&gt; s64[3] s64 [ 255 ] [ 255 ] [ 3 ] * s64 [ 3 ] #=&gt; s64[255][255][3] s64 [ 2 ] [ 1 ] * s [ 1 ] [ 2 ] #=&gt; s64[2][2] s64 [ 5 ] [ 1 ] [ 4 ] [ 1 ] * s64 [ 3 ] [ 4 ] [ 5 ] #=&gt; s64[5][3][4][5] If any of the dimensions do not match or are not 1, an error is raised. Access syntax (slicing) Nx tensors implement Elixir's access syntax. This allows developers to slice tensors up and easily access sub-dimensions and values. Access accepts integers: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t [ 0 ] # Nx.Tensor &lt; s64 [ 2 ] [ 1 , 2 ] &gt; iex&gt; t [ 1 ] # Nx.Tensor &lt; s64 [ 2 ] [ 3 , 4 ] &gt; iex&gt; t [ 1 ] [ 1 ] # Nx.Tensor &lt; s64 4 &gt; If a negative index is given, it accesses the element from the back: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t [ - 1 ] [ - 1 ] # Nx.Tensor &lt; s64 4 &gt; Out of bound access will raise: iex&gt; Nx . tensor ( [ 1 , 2 ] ) [ 2 ] ** (ArgumentError) index 2 is out of bounds for axis 0 in shape {2} iex&gt; Nx . tensor ( [ 1 , 2 ] ) [ - 3 ] ** (ArgumentError) index -3 is out of bounds for axis 0 in shape {2} The index can also be another tensor but in such cases it must be a scalar between 0 and the dimension size. Out of bound dynamic indexes are always clamped to the tensor dimensions: iex&gt; two = Nx . tensor ( 2 ) iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t [ two ] [ two ] # Nx.Tensor &lt; s64 4 &gt; For example, a minus_one dynamic index will be clamped to zero: iex&gt; minus_one = Nx . tensor ( - 1 ) iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t [ minus_one ] [ minus_one ] # Nx.Tensor &lt; s64 1 &gt; Access also accepts ranges. Ranges in Elixir are inclusive: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] ) iex&gt; t [ 0 .. 1 ] # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] ] &gt; Ranges can receive negative positions and they will read from the back. In such cases, the range step must be explicitly given and the right-side of the range must be equal or greater than the left-side: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] ) iex&gt; t [ 1 .. - 2 // 1 ] # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 3 , 4 ] , [ 5 , 6 ] ] &gt; As you can see, accessing with a range does not eliminate the accessed axis. This means that, if you try to cascade ranges, you will always be filtering the highest dimension: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] ) iex&gt; t [ 1 .. - 1 // 1 ] # Drop the first &quot;row&quot; # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] &gt; iex&gt; t [ 1 .. - 1 // 1 ] [ 1 .. - 1 // 1 ] # Drop the first &quot;row&quot; twice # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 5 , 6 ] , [ 7 , 8 ] ] &gt; Therefore, if you want to slice across multiple dimensions, you can wrap the ranges in a list: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] ) iex&gt; t [ [ 1 .. - 1 // 1 , 1 .. - 1 // 1 ] ] # Drop the first &quot;row&quot; and the first &quot;column&quot; # Nx.Tensor &lt; s64 [ 3 ] [ 1 ] [ [ 4 ] , [ 6 ] , [ 8 ] ] &gt; You can also use .. as the full-slice range, which means you want to keep a given dimension as is: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] , [ 7 , 8 ] ] ) iex&gt; t [ [ .. , 1 .. - 1 // 1 ] ] # Drop only the first &quot;column&quot; # Nx.Tensor &lt; s64 [ 4 ] [ 1 ] [ [ 2 ] , [ 4 ] , [ 6 ] , [ 8 ] ] &gt; You can mix both ranges and integers in the list too: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ) iex&gt; t [ [ 1 .. 2 , 2 ] ] # Nx.Tensor &lt; s64 [ 2 ] [ 6 , 9 ] &gt; If the list has less elements than axes, the remaining dimensions are returned in full: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ) iex&gt; t [ [ 1 .. 2 ] ] # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] &gt; The access syntax also pairs nicely with named tensors. By using named tensors, you can pass only the axis you want to slice, leaving the other axes intact: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] , names : [ :x , :y ] ) iex&gt; t [ x : 1 .. 2 ] # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] &gt; iex&gt; t [ x : 1 .. 2 , y : 0 .. 1 ] # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 4 , 5 ] , [ 7 , 8 ] ] &gt; iex&gt; t [ x : 1 , y : 0 .. 1 ] # Nx.Tensor &lt; s64 [ y : 2 ] [ 4 , 5 ] &gt; For a more complex slicing rules, including strides, you can always fallback to Nx.slice/4 . Backends The Nx library has built-in support for multiple backends. A tensor is always handled by a backend, the default backend being Nx.BinaryBackend , which means the tensor is allocated as a binary within the Erlang VM. Most often backends are used to provide a completely different implementation of tensor operations, often accelerated to the GPU. In such cases, you want to guarantee all tensors are allocated in the new backend. This can be done by configuring your runtime: # config/runtime.exs import Config config :nx , default_backend : EXLA.Backend In your notebooks and on Mix.install/2 , you might: Mix . install ( [ { :nx , &quot;&gt;= 0.0.0&quot; } ] , config : [ nx : [ default_backend : EXLA.Backend ] ] ) Or by calling Nx.global_default_backend/1 (less preferrable): Nx . global_default_backend ( EXLA.Backend ) To pass options to the backend, replacing EXLA.Backend by {EXLA.Backend, client: :cuda} or similar. See the documentation for EXLA and Torchx for installation and GPU support. To implement your own backend, check the Nx.Tensor behaviour.","ref":"Nx.html","title":"Nx","type":"module"},{"doc":"Computes the absolute value of each element in the tensor. Examples iex&gt; Nx . abs ( Nx . tensor ( [ - 2 , - 1 , 0 , 1 , 2 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 5 ] [ 2 , 1 , 0 , 1 , 2 ] &gt;","ref":"Nx.html#abs/1","title":"Nx.abs/1","type":"function"},{"doc":"Calculates the inverse cosine of each element in the tensor. It is equivalent to: $$acos(cos(z)) = z$$ Examples iex&gt; Nx . acos ( 0.10000000149011612 ) # Nx.Tensor &lt; f32 1.4706288576126099 &gt; iex&gt; Nx . acos ( Nx . tensor ( [ 0.10000000149011612 , 0.5 , 0.8999999761581421 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.4706288576126099 , 1.0471975803375244 , 0.4510268568992615 ] &gt;","ref":"Nx.html#acos/1","title":"Nx.acos/1","type":"function"},{"doc":"Calculates the inverse hyperbolic cosine of each element in the tensor. It is equivalent to: $$acosh(cosh(z)) = z$$ Examples iex&gt; Nx . acosh ( 1 ) # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; Nx . acosh ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.0 , 1.316957950592041 , 1.7627471685409546 ] &gt;","ref":"Nx.html#acosh/1","title":"Nx.acosh/1","type":"function"},{"doc":"Element-wise addition of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the + operator in place of this function: left + right . Examples Adding scalars iex&gt; Nx . add ( 1 , 2 ) # Nx.Tensor &lt; s64 3 &gt; iex&gt; Nx . add ( 1 , 2.2 ) # Nx.Tensor &lt; f32 3.200000047683716 &gt; Adding a scalar to a tensor iex&gt; Nx . add ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 2 , 3 , 4 ] &gt; iex&gt; Nx . add ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 2 , 3 , 4 ] &gt; Given a float scalar converts the tensor to a float: iex&gt; Nx . add ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1.0 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 2.0 , 3.0 , 4.0 ] &gt; iex&gt; Nx . add ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 2.0 , 3.0 , 4.0 ] &gt; iex&gt; Nx . add ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , type : :f32 , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 2.0 , 3.0 , 4.0 ] &gt; Unsigned tensors become signed and double their size if a negative number is given: iex&gt; Nx . add ( Nx . tensor ( [ 0 , 1 , 2 ] , type : :u8 , names : [ :data ] ) , - 1 ) # Nx.Tensor &lt; s16 [ data : 3 ] [ - 1 , 0 , 1 ] &gt; Adding tensors of the same shape iex&gt; left = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , names : [ nil , :y ] ) iex&gt; Nx . add ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 11 , 22 ] , [ 33 , 44 ] ] &gt; Adding tensors with broadcasting iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ nil , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ :x , nil ] ) iex&gt; Nx . add ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 11 , 21 ] , [ 12 , 22 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ nil , :y ] ) iex&gt; Nx . add ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 11 , 21 ] , [ 12 , 22 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] ) iex&gt; Nx . add ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 2 ] [ [ 11 , 21 ] , [ 32 , 42 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 , 2 ] ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] ) iex&gt; Nx . add ( left , right ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 11 , 22 ] , [ 31 , 42 ] ] &gt;","ref":"Nx.html#add/2","title":"Nx.add/2","type":"function"},{"doc":"Returns a scalar tensor of value 1 if all of the tensor values are not zero. Otherwise the value is 0. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the reduced axes to size 1. Examples iex&gt; Nx . all ( Nx . tensor ( [ 0 , 1 , 2 ] ) ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx . all ( Nx . tensor ( [ [ - 1 , 0 , 1 ] , [ 2 , 3 , 4 ] ] , names : [ :x , :y ] ) , axes : [ :x ] ) # Nx.Tensor &lt; u8 [ y : 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; Nx . all ( Nx . tensor ( [ [ - 1 , 0 , 1 ] , [ 2 , 3 , 4 ] ] , names : [ :x , :y ] ) , axes : [ :y ] ) # Nx.Tensor &lt; u8 [ x : 2 ] [ 0 , 1 ] &gt; Keeping axes iex&gt; Nx . all ( Nx . tensor ( [ [ - 1 , 0 , 1 ] , [ 2 , 3 , 4 ] ] , names : [ :x , :y ] ) , axes : [ :y ] , keep_axes : true ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 1 ] [ [ 0 ] , [ 1 ] ] &gt;","ref":"Nx.html#all/2","title":"Nx.all/2","type":"function"},{"doc":"Returns a scalar tensor of value 1 if all element-wise values are within tolerance of b. Otherwise returns value 0. You may set the absolute tolerance, :atol and relative tolerance :rtol . Given tolerances, this functions returns 1 if absolute ( a - b ) &lt;= ( atol + rtol * absolute ( b ) ) is true for all elements of a and b. Options :rtol - relative tolerance between numbers, as described above. Defaults to 1.0e-5 :atol - absolute tolerance between numbers, as described above. Defaults to 1.0e-8 :equal_nan - if false , NaN will always compare as false. Otherwise NaN will only equal NaN . Defaults to false Examples iex&gt; Nx . all_close ( Nx . tensor ( [ 1.0e10 , 1.0e-7 ] ) , Nx . tensor ( [ 1.00001e10 , 1.0e-8 ] ) ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx . all_close ( Nx . tensor ( [ 1.0e-8 , 1.0e-8 ] ) , Nx . tensor ( [ 1.0e-8 , 1.0e-9 ] ) ) # Nx.Tensor &lt; u8 1 &gt; Although NaN by definition isn't equal to itself, so this implementation also considers all NaN s different from each other by default: iex&gt; Nx . all_close ( Nx . tensor ( :nan ) , Nx . tensor ( :nan ) ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx . all_close ( Nx . tensor ( :nan ) , Nx . tensor ( 0 ) ) # Nx.Tensor &lt; u8 0 &gt; We can change this behavior with the :equal_nan option: iex&gt; t = Nx . tensor ( [ :nan , 1 ] ) iex&gt; Nx . all_close ( t , t , equal_nan : true ) # nan == nan -&gt; true # Nx.Tensor &lt; u8 1 &gt; iex&gt; Nx . all_close ( t , t , equal_nan : false ) # nan == nan -&gt; false, default behavior # Nx.Tensor &lt; u8 0 &gt; Infinities behave as expected, being &quot;close&quot; to themselves but not to other numbers: iex&gt; Nx . all_close ( Nx . tensor ( :infinity ) , Nx . tensor ( :infinity ) ) # Nx.Tensor &lt; u8 1 &gt; iex&gt; Nx . all_close ( Nx . tensor ( :infinity ) , Nx . tensor ( :neg_infinity ) ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx . all_close ( Nx . tensor ( 1.0e30 ) , Nx . tensor ( :infinity ) ) # Nx.Tensor &lt; u8 0 &gt;","ref":"Nx.html#all_close/3","title":"Nx.all_close/3","type":"function"},{"doc":"Returns a scalar tensor of value 1 if any of the tensor values are not zero. Otherwise the value is 0. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the reduced axes to size 1. Examples iex&gt; Nx . any ( Nx . tensor ( [ 0 , 1 , 2 ] ) ) # Nx.Tensor &lt; u8 1 &gt; iex&gt; Nx . any ( Nx . tensor ( [ [ 0 , 1 , 0 ] , [ 0 , 1 , 2 ] ] , names : [ :x , :y ] ) , axes : [ :x ] ) # Nx.Tensor &lt; u8 [ y : 3 ] [ 0 , 1 , 1 ] &gt; iex&gt; Nx . any ( Nx . tensor ( [ [ 0 , 1 , 0 ] , [ 0 , 1 , 2 ] ] , names : [ :x , :y ] ) , axes : [ :y ] ) # Nx.Tensor &lt; u8 [ x : 2 ] [ 1 , 1 ] &gt; Keeping axes iex&gt; Nx . any ( Nx . tensor ( [ [ 0 , 1 , 0 ] , [ 0 , 1 , 2 ] ] , names : [ :x , :y ] ) , axes : [ :y ] , keep_axes : true ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 1 ] [ [ 1 ] , [ 1 ] ] &gt;","ref":"Nx.html#any/2","title":"Nx.any/2","type":"function"},{"doc":"Returns the indices of the maximum values. Options :axis - the axis to aggregate on. If no axis is given, returns the index of the absolute maximum value in the tensor. :keep_axis - whether or not to keep the reduced axis with a size of 1. Defaults to false . :tie_break - how to break ties. one of :high , or :low . default behavior is to always return the lower index. Examples iex&gt; Nx . argmax ( 4 ) # Nx.Tensor &lt; s64 0 &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] ) iex&gt; Nx . argmax ( t ) # Nx.Tensor &lt; s64 10 &gt; If a tensor of floats is given, it still returns integers: iex&gt; Nx . argmax ( Nx . tensor ( [ 2.0 , 4.0 ] ) ) # Nx.Tensor &lt; s64 1 &gt; Aggregating over an axis iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , axis : :x ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 0 , 0 , 0 ] , [ 0 , 1 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 0 , 2 ] , [ 0 , 1 ] ] &gt; Tie breaks iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , tie_break : :low , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 0 , 0 , 0 ] , [ 0 , 1 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , tie_break : :high , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 0 , 0 , 1 ] , [ 0 , 1 , 1 ] ] &gt; Keep axis iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmax ( t , axis : :y , keep_axis : true ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 1 ] [ z : 3 ] [ [ [ 0 , 0 , 0 ] ] , [ [ 0 , 1 , 0 ] ] ] &gt;","ref":"Nx.html#argmax/2","title":"Nx.argmax/2","type":"function"},{"doc":"Returns the indices of the minimum values. Options :axis - the axis to aggregate on. If no axis is given, returns the index of the absolute minimum value in the tensor. :keep_axis - whether or not to keep the reduced axis with a size of 1. Defaults to false . :tie_break - how to break ties. one of :high , or :low . Default behavior is to always return the lower index. Examples iex&gt; Nx . argmin ( 4 ) # Nx.Tensor &lt; s64 0 &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] ) iex&gt; Nx . argmin ( t ) # Nx.Tensor &lt; s64 4 &gt; If a tensor of floats is given, it still returns integers: iex&gt; Nx . argmin ( Nx . tensor ( [ 2.0 , 4.0 ] ) ) # Nx.Tensor &lt; s64 0 &gt; Aggregating over an axis iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , axis : :x ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , axis : 1 ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 1 , 1 , 0 ] , [ 1 , 0 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 1 , 1 ] , [ 1 , 2 ] ] &gt; Tie breaks iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , tie_break : :low , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 1 , 1 , 0 ] , [ 1 , 0 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , tie_break : :high , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 0 , 1 ] ] &gt; Keep axis iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 3 ] , [ 1 , - 5 , 3 ] ] , [ [ 6 , 2 , 3 ] , [ 4 , 8 , 3 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . argmin ( t , axis : :y , keep_axis : true ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 1 ] [ z : 3 ] [ [ [ 1 , 1 , 0 ] ] , [ [ 1 , 0 , 0 ] ] ] &gt;","ref":"Nx.html#argmin/2","title":"Nx.argmin/2","type":"function"},{"doc":"Sorts the tensor along the given axis according to the given direction and returns the corresponding indices of the original tensor in the new sorted positions. If no axis is given, defaults to 0 . Options :axis - The name or number of the corresponding axis on which the sort should be applied :direction - Can be :asc or :desc . Defaults to :asc Examples iex&gt; Nx . argsort ( Nx . tensor ( [ 16 , 23 , 42 , 4 , 8 , 15 ] ) ) # Nx.Tensor &lt; s64 [ 6 ] [ 3 , 4 , 5 , 0 , 1 , 2 ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . argsort ( t , axis : :x ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 0 , 1 ] , [ 0 , 1 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . argsort ( t , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 0 , 2 ] , [ 0 , 2 , 1 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . argsort ( t , axis : :y , direction : :asc ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 0 , 2 ] , [ 0 , 2 , 1 ] ] &gt; Same tensor sorted over different axes: iex&gt; t = Nx . tensor ( ...&gt; [ ...&gt; [ ...&gt; [ 4 , 5 , 2 ] , ...&gt; [ 2 , 5 , 3 ] , ...&gt; [ 5 , 0 , 2 ] ...&gt; ] , ...&gt; [ ...&gt; [ 1 , 9 , 8 ] , ...&gt; [ 2 , 1 , 3 ] , ...&gt; [ 2 , 1 , 4 ] ...&gt; ] ...&gt; ] , ...&gt; names : [ :x , :y , :z ] ...&gt; ) iex&gt; Nx . argsort ( t , axis : :x ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 1 , 0 , 0 ] ] , [ [ 0 , 1 , 1 ] , [ 1 , 0 , 1 ] , [ 0 , 1 , 1 ] ] ] &gt; iex&gt; Nx . argsort ( t , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 1 , 2 , 0 ] , [ 0 , 0 , 2 ] , [ 2 , 1 , 1 ] ] , [ [ 0 , 1 , 1 ] , [ 1 , 2 , 2 ] , [ 2 , 0 , 0 ] ] ] &gt; iex&gt; Nx . argsort ( t , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 2 , 0 , 1 ] , [ 0 , 2 , 1 ] , [ 1 , 2 , 0 ] ] , [ [ 0 , 2 , 1 ] , [ 1 , 0 , 2 ] , [ 1 , 0 , 2 ] ] ] &gt;","ref":"Nx.html#argsort/2","title":"Nx.argsort/2","type":"function"},{"doc":"Changes the type of a tensor. Note conversion between float and integers truncates the result. Consider using round/1 , floor/1 , or ceil/1 before casting from float to integer to guarantee consistent behavior. Casting from a higher precision may lead to an overflow or underflow, which is platform and compiler dependent behaviour. Casting of non-finite types to integer types are handled such as: negative infinity becomes the minimum value for said type positive infinity becomes the maximum value for said type nan becomes zero Examples iex&gt; Nx . as_type ( Nx . tensor ( [ 0 , 1 , 2 ] , names : [ :data ] ) , :f32 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.0 , 1.0 , 2.0 ] &gt; iex&gt; Nx . as_type ( Nx . tensor ( [ 0.0 , 1.0 , 2.0 ] , names : [ :data ] ) , :bf16 ) # Nx.Tensor &lt; bf16 [ data : 3 ] [ 0.0 , 1.0 , 2.0 ] &gt; iex&gt; Nx . as_type ( Nx . tensor ( [ 0.0 , 1.0 , 2.0 ] , names : [ :data ] ) , :s64 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 0 , 1 , 2 ] &gt; Casting numbers as complex will return the corresponding complex with 0 imaginary component: iex&gt; Nx . as_type ( Nx . tensor ( [ 1 , - 2 ] ) , :c64 ) # Nx.Tensor &lt; c64 [ 2 ] [ 1.0 + 0.0 i , - 2.0 + 0.0 i ] &gt; Casting complex numbers will return their real parts as the target type: iex&gt; Nx . as_type ( Nx . tensor ( [ Complex . new ( 1 , 2 ) , Complex . new ( 0 , 3 ) , Complex . new ( 4 , 5 ) ] ) , :f64 ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 0.0 , 4.0 ] &gt; iex&gt; Nx . as_type ( Nx . tensor ( [ Complex . new ( - 1 , 2 ) , Complex . new ( - 2 , 3 ) , Complex . new ( 3 , - 4 ) ] ) , :s64 ) # Nx.Tensor &lt; s64 [ 3 ] [ - 1 , - 2 , 3 ] &gt; Casting of non-finite values to integer types convert to pre-determined integer values: iex&gt; non_finite = Nx . tensor ( [ :infinity , :nan , :neg_infinity ] ) iex&gt; Nx . as_type ( non_finite , :u8 ) # Nx.Tensor &lt; u8 [ 3 ] [ 255 , 0 , 0 ] &gt; iex&gt; Nx . as_type ( non_finite , :s32 ) # Nx.Tensor &lt; s32 [ 3 ] [ 2147483647 , 0 , - 2147483648 ] &gt; Non-finite values between float types are preserved: iex&gt; non_finite = Nx . tensor ( [ :infinity , :nan ] ) iex&gt; Nx . as_type ( non_finite , :f64 ) # Nx.Tensor &lt; f64 [ 2 ] [ Inf , NaN ] &gt; iex&gt; Nx . as_type ( non_finite , :f16 ) # Nx.Tensor &lt; f16 [ 2 ] [ Inf , NaN ] &gt;","ref":"Nx.html#as_type/2","title":"Nx.as_type/2","type":"function"},{"doc":"Calculates the inverse sine of each element in the tensor. It is equivalent to: $$asin(sin(z)) = z$$ Examples iex&gt; Nx . asin ( 0.10000000149011612 ) # Nx.Tensor &lt; f32 0.1001674234867096 &gt; iex&gt; Nx . asin ( Nx . tensor ( [ 0.10000000149011612 , 0.5 , 0.8999999761581421 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.1001674234867096 , 0.5235987901687622 , 1.1197694540023804 ] &gt;","ref":"Nx.html#asin/1","title":"Nx.asin/1","type":"function"},{"doc":"Calculates the inverse hyperbolic sine of each element in the tensor. It is equivalent to: $$asinh(sinh(z)) = z$$ Examples iex&gt; Nx . asinh ( 1 ) # Nx.Tensor &lt; f32 0.8813735842704773 &gt; iex&gt; Nx . asinh ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.8813735842704773 , 1.4436354637145996 , 1.8184465169906616 ] &gt;","ref":"Nx.html#asinh/1","title":"Nx.asinh/1","type":"function"},{"doc":"Element-wise arc tangent of two tensors. If a number is given, it is converted to a tensor. It always returns a float tensor. If any of the input tensors are not float, they are converted to f32. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. Examples Arc tangent between scalars iex&gt; Nx . atan2 ( 1 , 2 ) # Nx.Tensor &lt; f32 0.46364760398864746 &gt; Arc tangent between tensors and scalars iex&gt; Nx . atan2 ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.7853981852531433 , 1.1071487665176392 , 1.249045729637146 ] &gt; iex&gt; Nx . atan2 ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.7853981852531433 , 0.46364760398864746 , 0.32175055146217346 ] &gt; Arc tangent between tensors iex&gt; neg_and_pos_zero_columns = Nx . tensor ( [ [ - 0.0 ] , [ 0.0 ] ] , type : :f64 ) iex&gt; neg_and_pos_zero_rows = Nx . tensor ( [ - 0.0 , 0.0 ] , type : :f64 ) iex&gt; Nx . atan2 ( neg_and_pos_zero_columns , neg_and_pos_zero_rows ) # Nx.Tensor &lt; f64 [ 2 ] [ 2 ] [ [ - 3.141592653589793 , - 0.0 ] , [ 3.141592653589793 , 0.0 ] ] &gt;","ref":"Nx.html#atan2/2","title":"Nx.atan2/2","type":"function"},{"doc":"Calculates the inverse tangent of each element in the tensor. It is equivalent to: $$atan(tan(z)) = z$$ Examples iex&gt; Nx . atan ( 0.10000000149011612 ) # Nx.Tensor &lt; f32 0.09966865181922913 &gt; iex&gt; Nx . atan ( Nx . tensor ( [ 0.10000000149011612 , 0.5 , 0.8999999761581421 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.09966865181922913 , 0.46364760398864746 , 0.7328150868415833 ] &gt;","ref":"Nx.html#atan/1","title":"Nx.atan/1","type":"function"},{"doc":"Calculates the inverse hyperbolic tangent of each element in the tensor. It is equivalent to: $$atanh(tanh(z)) = z$$ Examples iex&gt; Nx . atanh ( 0.10000000149011612 ) # Nx.Tensor &lt; f32 0.10033535212278366 &gt; iex&gt; Nx . atanh ( Nx . tensor ( [ 0.10000000149011612 , 0.5 , 0.8999999761581421 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.10033535212278366 , 0.5493061542510986 , 1.4722193479537964 ] &gt;","ref":"Nx.html#atanh/1","title":"Nx.atanh/1","type":"function"},{"doc":"Returns all of the axes in a tensor. If a shape is given, it returns the axes for the given shape. Examples iex&gt; Nx . axes ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) [ 0 , 1 ] iex&gt; Nx . axes ( 1 ) [ ] iex&gt; Nx . axes ( { 1 , 2 , 3 } ) [ 0 , 1 , 2 ]","ref":"Nx.html#axes/1","title":"Nx.axes/1","type":"function"},{"doc":"Returns the index of the given axis in the tensor. Examples iex&gt; Nx . axis_index ( Nx . iota ( { 100 , 10 , 20 } ) , 0 ) 0 iex&gt; Nx . axis_index ( Nx . iota ( { 100 , 10 , 20 } ) , - 1 ) 2 iex&gt; Nx . axis_index ( Nx . iota ( { 100 , 10 , 20 } , names : [ :batch , :x , :y ] ) , :x ) 1 Error cases iex&gt; Nx . axis_index ( Nx . iota ( { 100 , 10 , 20 } ) , 3 ) ** (ArgumentError) given axis (3) invalid for shape with rank 3 iex&gt; Nx . axis_index ( Nx . iota ( { 100 , 10 , 20 } , names : [ :batch , :x , :y ] ) , :z ) ** (ArgumentError) key :z not found in tensor with names [:batch, :x, :y]","ref":"Nx.html#axis_index/2","title":"Nx.axis_index/2","type":"function"},{"doc":"Returns the size of a given axis of a tensor. It accepts either an atom as the name or an integer as the axis. It raises if the axis/name does not exist. Examples iex&gt; Nx . axis_size ( Nx . iota ( { 100 , 10 , 20 } ) , 0 ) 100 iex&gt; Nx . axis_size ( Nx . iota ( { 100 , 10 , 20 } , names : [ :batch , :x , :y ] ) , :y ) 20","ref":"Nx.html#axis_size/2","title":"Nx.axis_size/2","type":"function"},{"doc":"Copies data to the given backend. If a backend is not given, Nx.Tensor is used, which means the given tensor backend will pick the most appropriate backend to copy the data to. Note this function keeps the data in the original backend. Therefore, use this function with care, as it may duplicate large amounts of data across backends. Generally speaking, you may want to use backend_transfer/2 , unless you explicitly want to copy the data. For convenience, this function accepts tensors and any container (such as maps and tuples as defined by the Nx.Container protocol) and recursively copies all tensors in them. This behaviour exists as it is common to transfer data before and after defn functions. *Note: Nx.default_backend/1 does not affect the behaviour of this function. Examples iex&gt; Nx.backend_copy(Nx.tensor([[1, 2, 3], [4, 5, 6]])) #Nx.Tensor&lt; s64 [ 2 ] [ 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ]","ref":"Nx.html#backend_copy/2","title":"Nx.backend_copy/2","type":"function"},{"doc":"Deallocates data in a device. It returns either :ok or :already_deallocated . For convenience, this function accepts tensors and any container (such as maps and tuples as defined by the Nx.Container protocol) and deallocates all devices in them. This behaviour exists as it is common to deallocate data after defn functions.","ref":"Nx.html#backend_deallocate/1","title":"Nx.backend_deallocate/1","type":"function"},{"doc":"Transfers data to the given backend. This operation can be seen as an equivalent to backend_copy/3 followed by a backend_deallocate/1 on the initial tensor: new_tensor = Nx . backend_copy ( old_tensor , new_backend ) Nx . backend_deallocate ( old_tensor ) If a backend is not given, Nx.Tensor is used, which means the given tensor backend will pick the most appropriate backend to transfer to. For Elixir's builtin tensor, transferring to another backend will call new_backend.from_binary(tensor, binary, opts) . Transferring from a mutable backend, such as GPU memory, implies the data is copied from the GPU to the Erlang VM and then deallocated from the device. For convenience, this function accepts tensors and any container (such as maps and tuples as defined by the Nx.Container protocol) and transfers all tensors in them. This behaviour exists as it is common to transfer data from tuples and maps before and after defn functions. *Note: Nx.default_backend/1 does not affect the behaviour of this function. Examples Transfer a tensor to an EXLA device backend, stored in the GPU: device_tensor = Nx . backend_transfer ( tensor , { EXLA.Backend , client : :cuda } ) Transfer the device tensor back to an Elixir tensor: tensor = Nx . backend_transfer ( device_tensor )","ref":"Nx.html#backend_transfer/2","title":"Nx.backend_transfer/2","type":"function"},{"doc":"Changes the type of a tensor, using a bitcast. The width of input tensor's type must match the width of the output type. bitcast/1 does not change the underlying tensor data, but instead changes how the tensor data is viewed. Machines with different floating-point representations will give different results. For complex numbers, the last axis will change in size depending on whether you are upcasting or downcasting. Examples iex&gt; t = Nx . bitcast ( Nx . tensor ( [ 0 , 0 , 0 ] , names : [ :data ] , type : :s32 ) , :f32 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.0 , 0.0 , 0.0 ] &gt; iex&gt; Nx . bitcast ( t , :s32 ) # Nx.Tensor &lt; s32 [ data : 3 ] [ 0 , 0 , 0 ] &gt; Error cases iex&gt; Nx . bitcast ( Nx . tensor ( [ 0 , 1 , 2 ] , names : [ :data ] , type : :s16 ) , :f32 ) ** (ArgumentError) input type width must match new type width, got input type {:s, 16} and output type {:f, 32} iex&gt; Nx . bitcast ( Nx . tensor ( [ 0 ] , type : :c64 ) , :s64 ) ** (ArgumentError) Nx.bitcast/2 does not support complex inputs iex&gt; Nx . bitcast ( Nx . tensor ( [ 0 ] , type : :s64 ) , :c64 ) ** (ArgumentError) Nx.bitcast/2 does not support complex inputs","ref":"Nx.html#bitcast/2","title":"Nx.bitcast/2","type":"function"},{"doc":"Element-wise bitwise AND of two tensors. Only integer tensors are supported. If a float or complex tensor is given, an error is raised. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the &amp;&amp;&amp; operator in place of this function: left &amp;&amp;&amp; right . Examples bitwise and between scalars iex&gt; Nx . bitwise_and ( 1 , 0 ) # Nx.Tensor &lt; s64 0 &gt; bitwise and between tensors and scalars iex&gt; Nx . bitwise_and ( Nx . tensor ( [ 0 , 1 , 2 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 0 , 1 , 0 ] &gt; iex&gt; Nx . bitwise_and ( Nx . tensor ( [ 0 , - 1 , - 2 ] , names : [ :data ] ) , - 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 0 , - 1 , - 2 ] &gt; bitwise and between tensors iex&gt; Nx . bitwise_and ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] , names : [ :data ] ) , Nx . tensor ( [ 0 , 1 , 0 , 1 ] ) ) # Nx.Tensor &lt; s64 [ data : 4 ] [ 0 , 0 , 0 , 1 ] &gt; Error cases iex&gt; Nx . bitwise_and ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , 1.0 ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_and/2","title":"Nx.bitwise_and/2","type":"function"},{"doc":"Applies bitwise not to each element in the tensor. If you're using Nx.Defn.defn/2 , you can use the ~~~ operator in place of this function: ~~~tensor . Examples iex&gt; Nx . bitwise_not ( 1 ) # Nx.Tensor &lt; s64 - 2 &gt; iex&gt; Nx . bitwise_not ( Nx . tensor ( [ - 1 , 0 , 1 ] , type : :s8 , names : [ :x ] ) ) # Nx.Tensor &lt; s8 [ x : 3 ] [ 0 , - 1 , - 2 ] &gt; iex&gt; Nx . bitwise_not ( Nx . tensor ( [ 0 , 1 , 254 , 255 ] , type : :u8 , names : [ :x ] ) ) # Nx.Tensor &lt; u8 [ x : 4 ] [ 255 , 254 , 1 , 0 ] &gt; Error cases iex&gt; Nx . bitwise_not ( Nx . tensor ( [ 0.0 , 1.0 ] ) ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_not/1","title":"Nx.bitwise_not/1","type":"function"},{"doc":"Element-wise bitwise OR of two tensors. Only integer tensors are supported. If a float or complex tensor is given, an error is raised. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the ||| operator in place of this function: left ||| right . Examples bitwise or between scalars iex&gt; Nx . bitwise_or ( 1 , 0 ) # Nx.Tensor &lt; s64 1 &gt; bitwise or between tensors and scalars iex&gt; Nx . bitwise_or ( Nx . tensor ( [ 0 , 1 , 2 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 1 , 3 ] &gt; iex&gt; Nx . bitwise_or ( Nx . tensor ( [ 0 , - 1 , - 2 ] , names : [ :data ] ) , - 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ - 1 , - 1 , - 1 ] &gt; bitwise or between tensors iex&gt; Nx . bitwise_or ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] , names : [ :data ] ) , Nx . tensor ( [ 0 , 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; s64 [ data : 4 ] [ 0 , 1 , 1 , 1 ] &gt; Error cases iex&gt; Nx . bitwise_or ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , 1.0 ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_or/2","title":"Nx.bitwise_or/2","type":"function"},{"doc":"Element-wise bitwise XOR of two tensors. Only integer tensors are supported. If a float or complex tensor is given, an error is raised. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. Examples Bitwise xor between scalars iex&gt; Nx . bitwise_xor ( 1 , 0 ) # Nx.Tensor &lt; s64 1 &gt; Bitwise xor and between tensors and scalars iex&gt; Nx . bitwise_xor ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 3 , 0 , 1 ] &gt; iex&gt; Nx . bitwise_xor ( Nx . tensor ( [ - 1 , - 2 , - 3 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ - 3 , - 4 , - 1 ] &gt; Bitwise xor between tensors iex&gt; Nx . bitwise_xor ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , Nx . tensor ( [ 0 , 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; s64 [ data : 4 ] [ 0 , 1 , 1 , 0 ] &gt; Error cases iex&gt; Nx . bitwise_xor ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , 1.0 ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#bitwise_xor/2","title":"Nx.bitwise_xor/2","type":"function"},{"doc":"Broadcasts tensor to the given broadcast_shape . The new shape is either a tuple or a tensor which we will retrieve the current shape from. The broadcast shape must be of equal or higher rank than the current shape. An optional :axes can be given to customize how broadcasting happens. axes must be a list with the same length as the tensor shape. Each axis in the list maps to the dimension in the broadcast shape that must match. For example, an axis of [1, 2] says the 0 dimension of the tensor matches to the 1 dimension of the broadcast shape and the 1 dimension of the tensor matches the 2 dimension of the broadcast shape. Each matching dimension must either be 1, for implicit broadcasting, or match the dimension in the broadcast shape. Broadcasting is destructive with respect to names. You can optionally provide new :names for the new tensor. If you pass a tensor with named dimensions, the new tensor will inherit names from that tensor. Examples Without axes Examples iex&gt; Nx . broadcast ( 1 , { 1 , 2 , 3 } ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 1 , 1 , 1 ] , [ 1 , 1 , 1 ] ] ] &gt; iex&gt; Nx . broadcast ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , :y ] ) , Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , names : [ :i , :j ] ) ) # Nx.Tensor &lt; s64 [ i : 2 ] [ j : 2 ] [ [ 1 , 1 ] , [ 2 , 2 ] ] &gt; iex&gt; Nx . broadcast ( Nx . tensor ( [ [ 1 , 2 ] ] , names : [ :x , :y ] ) , Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , names : [ :i , :j ] ) ) # Nx.Tensor &lt; s64 [ i : 2 ] [ j : 2 ] [ [ 1 , 2 ] , [ 1 , 2 ] ] &gt; Note that, even if there is no broadcasting because the shape is the same, names are discarded if none are given: iex&gt; Nx . broadcast ( Nx . iota ( { 2 , 2 } , names : [ :x , :y ] ) , { 2 , 2 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 0 , 1 ] , [ 2 , 3 ] ] &gt; iex&gt; Nx . broadcast ( Nx . iota ( { 2 , 2 } , names : [ :x , :y ] ) , { 2 , 2 } , names : [ :i , :j ] ) # Nx.Tensor &lt; s64 [ i : 2 ] [ j : 2 ] [ [ 0 , 1 ] , [ 2 , 3 ] ] &gt; With axes Using the default broadcast rules, we cannot broadcast a tensor of shape (3) to the shape (3, 2), because the lower dimensions must match. But with Nx.broadcast/3 we can configure how the dimensions match: iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; Nx . broadcast ( t , { 3 , 2 } , axes : [ 0 ] , names : [ :x , :y ] ) # Nx.Tensor &lt; s64 [ x : 3 ] [ y : 2 ] [ [ 1 , 1 ] , [ 2 , 2 ] , [ 3 , 3 ] ] &gt; Or a more complex example: iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; Nx . broadcast ( t , { 2 , 3 , 2 } , axes : [ 1 ] , names : [ :x , :y , :z ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 2 ] [ [ [ 1 , 1 ] , [ 2 , 2 ] , [ 3 , 3 ] ] , [ [ 1 , 1 ] , [ 2 , 2 ] , [ 3 , 3 ] ] ] &gt;","ref":"Nx.html#broadcast/3","title":"Nx.broadcast/3","type":"function"},{"doc":"Returns the byte size of the data in the tensor computed from its shape and type. Examples iex&gt; Nx . byte_size ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) 48 iex&gt; Nx . byte_size ( Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] ) ) 24 iex&gt; Nx . byte_size ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , type : :u8 ) ) 6 iex&gt; Nx . byte_size ( 1 ) 8","ref":"Nx.html#byte_size/1","title":"Nx.byte_size/1","type":"function"},{"doc":"Calculates the cube root of each element in the tensor. It is equivalent to: $$cbrt(z) = z^{\\frac{1}{3}}$$ Examples iex&gt; Nx . cbrt ( 1 ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx . cbrt ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.0 , 1.2599210739135742 , 1.4422495365142822 ] &gt;","ref":"Nx.html#cbrt/1","title":"Nx.cbrt/1","type":"function"},{"doc":"Calculates the ceil of each element in the tensor. If a non-floating tensor is given, it is returned as is. If a floating tensor is given, then we apply the operation, but keep its type. Examples iex&gt; Nx . ceil ( Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ - 1 , 0 , 1 ] &gt; iex&gt; Nx . ceil ( Nx . tensor ( [ - 1.5 , - 0.5 , 0.5 , 1.5 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 4 ] [ - 1.0 , 0.0 , 1.0 , 2.0 ] &gt;","ref":"Nx.html#ceil/1","title":"Nx.ceil/1","type":"function"},{"doc":"Clips the values of the tensor on the closed interval [min, max] . You can pass a tensor to min or max as long as the tensor has a scalar shape. Examples iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) iex&gt; Nx . clip ( t , 2 , 4 ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 2 , 2 , 3 ] , [ 4 , 4 , 4 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) iex&gt; Nx . clip ( t , 2.0 , 3 ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 3 ] [ [ 2.0 , 2.0 , 3.0 ] , [ 3.0 , 3.0 , 3.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) iex&gt; Nx . clip ( t , Nx . tensor ( 2.0 ) , Nx . max ( 1.0 , 3.0 ) ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 3 ] [ [ 2.0 , 2.0 , 3.0 ] , [ 3.0 , 3.0 , 3.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] , names : [ :x , :y ] ) iex&gt; Nx . clip ( t , 2 , 6.0 ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 3 ] [ [ 2.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] , type : :f32 , names : [ :x , :y ] ) iex&gt; Nx . clip ( t , 1 , 4 ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 3 ] [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 4.0 , 4.0 ] ] &gt;","ref":"Nx.html#clip/3","title":"Nx.clip/3","type":"function"},{"doc":"Checks if two tensors have the same shape, type, and compatible names. The data in the tensor is ignored. For convenience, this function accepts tensors and any container (such as maps and tuples as defined by the Nx.Container protocol) and recursively compares them, observing their container data structures are also the same. Examples iex&gt; Nx . compatible? ( Nx . iota ( { 3 , 2 } ) , Nx . iota ( { 3 , 2 } ) ) true iex&gt; Nx . compatible? ( Nx . iota ( { 3 , 2 } ) , Nx . iota ( { 3 , 2 } , names : [ :rows , :columns ] ) ) true iex&gt; Nx . compatible? ( ...&gt; Nx . iota ( { 3 , 2 } , names : [ :rows , nil ] ) , ...&gt; Nx . iota ( { 3 , 2 } , names : [ nil , :columns ] ) ...&gt; ) true iex&gt; Nx . compatible? ( ...&gt; Nx . iota ( { 3 , 2 } , names : [ :foo , :bar ] ) , ...&gt; Nx . iota ( { 3 , 2 } , names : [ :rows , :columns ] ) ...&gt; ) false iex&gt; Nx . compatible? ( Nx . iota ( { 3 , 2 } ) , Nx . iota ( { 2 , 3 } ) ) false iex&gt; Nx . compatible? ( Nx . iota ( { 2 , 2 } ) , Nx . iota ( { 2 , 2 } , type : :f32 ) ) false Using collections: iex&gt; Nx . compatible? ( { Nx . iota ( { 3 , 2 } ) , { 1 , 2 } } , { Nx . iota ( { 3 , 2 } ) , { 3 , 4 } } ) true iex&gt; Nx . compatible? ( %{ foo : Nx . iota ( { 3 , 2 } ) } , %{ foo : Nx . iota ( { 3 , 2 } ) } ) true iex&gt; Nx . compatible? ( %{ foo : Nx . iota ( { 3 , 2 } ) } , %{ bar : Nx . iota ( { 3 , 2 } ) } ) false","ref":"Nx.html#compatible?/2","title":"Nx.compatible?/2","type":"function"},{"doc":"Constructs a complex tensor from two equally-shaped tensors. Does not accept complex tensors as inputs. Examples iex&gt; Nx . complex ( Nx . tensor ( 1 ) , Nx . tensor ( 2 ) ) # Nx.Tensor &lt; c64 1.0 + 2.0 i &gt; iex&gt; Nx . complex ( Nx . tensor ( [ 1 , 2 ] ) , Nx . tensor ( [ 3 , 4 ] ) ) # Nx.Tensor &lt; c64 [ 2 ] [ 1.0 + 3.0 i , 2.0 + 4.0 i ] &gt;","ref":"Nx.html#complex/2","title":"Nx.complex/2","type":"function"},{"doc":"Concatenates tensors along the given axis. If no axis is provided, defaults to 0. All tensors must have the same rank and all of their dimension sizes but the concatenated dimension must match. If tensors are named, the names must be able to be merged. If tensors with mixed types are given, the types will be merged to a higher type and all of the tensors will be cast to the higher type before concatenating. Examples iex&gt; Nx . concatenate ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . concatenate ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 4 , 5 , 6 ] ) ] ) # Nx.Tensor &lt; s64 [ 6 ] [ 1 , 2 , 3 , 4 , 5 , 6 ] &gt; iex&gt; t1 = Nx . iota ( { 2 , 2 , 2 } , names : [ :x , :y , :z ] , type : :f32 ) iex&gt; t2 = Nx . iota ( { 1 , 2 , 2 } , names : [ :x , :y , :z ] , type : :u8 ) iex&gt; t3 = Nx . iota ( { 1 , 2 , 2 } , names : [ :x , :y , :z ] , type : :s64 ) iex&gt; Nx . concatenate ( [ t1 , t2 , t3 ] , axis : :x ) # Nx.Tensor &lt; f32 [ x : 4 ] [ y : 2 ] [ z : 2 ] [ [ [ 0.0 , 1.0 ] , [ 2.0 , 3.0 ] ] , [ [ 4.0 , 5.0 ] , [ 6.0 , 7.0 ] ] , [ [ 0.0 , 1.0 ] , [ 2.0 , 3.0 ] ] , [ [ 0.0 , 1.0 ] , [ 2.0 , 3.0 ] ] ] &gt; iex&gt; t1 = Nx . iota ( { 1 , 3 , 2 } , names : [ :x , :y , :z ] ) iex&gt; t2 = Nx . iota ( { 1 , 1 , 2 } , names : [ :x , :y , :z ] ) iex&gt; t3 = Nx . iota ( { 1 , 2 , 2 } , names : [ :x , :y , :z ] ) iex&gt; Nx . concatenate ( [ t1 , t2 , t3 ] , axis : :y ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 6 ] [ z : 2 ] [ [ [ 0 , 1 ] , [ 2 , 3 ] , [ 4 , 5 ] , [ 0 , 1 ] , [ 0 , 1 ] , [ 2 , 3 ] ] ] &gt; iex&gt; t1 = Nx . iota ( { 2 , 1 , 4 } , names : [ :x , :y , :z ] ) iex&gt; t2 = Nx . iota ( { 2 , 1 , 1 } , names : [ :x , :y , :z ] ) iex&gt; t3 = Nx . iota ( { 2 , 1 , 3 } , names : [ :x , :y , :z ] ) iex&gt; Nx . concatenate ( [ t1 , t2 , t3 ] , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 1 ] [ z : 8 ] [ [ [ 0 , 1 , 2 , 3 , 0 , 0 , 1 , 2 ] ] , [ [ 4 , 5 , 6 , 7 , 1 , 3 , 4 , 5 ] ] ] &gt; iex&gt; t1 = Nx . iota ( { 2 , 1 , 4 } , names : [ :x , :y , :z ] ) iex&gt; Nx . concatenate ( [ t1 ] , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 1 ] [ z : 4 ] [ [ [ 0 , 1 , 2 , 3 ] ] , [ [ 4 , 5 , 6 , 7 ] ] ] &gt;","ref":"Nx.html#concatenate/2","title":"Nx.concatenate/2","type":"function"},{"doc":"Calculates the complex conjugate of each element in the tensor. If $$z = a + bi = r e^\\theta$$, $$conjugate(z) = z^* = a - bi = r e^{-\\theta}$$ Examples iex&gt; Nx . conjugate ( Complex . new ( 1 , 2 ) ) # Nx.Tensor &lt; c64 1.0 - 2.0 i &gt; iex&gt; Nx . conjugate ( 1 ) # Nx.Tensor &lt; c64 1.0 + 0.0 i &gt; iex&gt; Nx . conjugate ( Nx . tensor ( [ Complex . new ( 1 , 2 ) , Complex . new ( 2 , - 4 ) ] ) ) # Nx.Tensor &lt; c64 [ 2 ] [ 1.0 - 2.0 i , 2.0 + 4.0 i ] &gt;","ref":"Nx.html#conjugate/1","title":"Nx.conjugate/1","type":"function"},{"doc":"Computes an n-D convolution (where n &gt;= 3 ) as used in neural networks. This function can be thought of as sliding an n-D kernel across the input, producing a new tensor that has the same number of elements as the number of valid windows in the input tensor. Each element is the result of summing the element-wise products in the window across each input channel. The ranks of both input and kernel must match. By default, both input and kernel are expected to have shapes of the following form: input - {batch_size, input_channels, input_d0, ..., input_dn} kernel - {output_channels, input_channels, kernel_d0, ..., kernel_dn} Where input_d0...input_dn and kernel_d0...kernel_dn represent an arbitrary number of spatial dimensions. You can alter this configuration using one of the *_permutation configuration options. Permutations are input, kernel, and output specifications for the layout of the convolution. For example, if your input tensor is configured with &quot;channels last&quot;, you can specify the input permutation with: Nx . conv ( img , kernel , input_permutation : [ 0 , 3 , 1 , 2 ] ) Permutations expect configurations that specify the location of dimensions in the following orders: input_permutation - [batch_dim, input_channel_dim, ...spatial_dims...] kernel_permutation - [output_channel_dim, input_channel_dim, ...spatial_dims...] output_permutation - [batch_dim, output_channel_dim, ...spatial_dims...] Using named tensors, it's a bit easier to see how permutations help you configure the convolution. Given input tensor with names [:batch, :height, :width, :channels] (channels last) and kernel tensor with names [:input, :output, :height, :width] , you can configure the convolution with the following permutations: Nx . conv ( img , kernel , input_permutation : [ :batch , :channels , :height , :width ] , kernel_permutation : [ :output , :input , :height , :width ] , output_permutation : [ :batch , :channels , :height , :width ] ) Notice that output_permutation is normalized with respect to the input permutation names. We cannot guarantee that every permutation is supported in every backend or compiler. To configure how the window slides along the input tensor, you can specify :strides . :strides must be a positive integer or tuple of positive integers for each spatial dimension in the input and kernel. For each spatial dimension, the window will slide by the configuration specified in :strides . As an example, for a 2-D convolution with strides: [2, 1] , the window will slide 2 positions along the first spatial dimension until it reaches the end of the dimension and then 1 position along the second spatial dimension. You may specify a padding configuration using :padding , which will zero-pad the input tensor. Acceptable padding configurations are: :valid - no padding :same - pad input spatial dimensions such that they will remain unchanged in the output tensor [{d0_hi, d0_lo}, ..., {dn_hi, dn_lo}] - a general padding configuration of edge high and edge low padding values. You may only specify padding for the edges of spatial dimensions of the input tensor. Padding values may be negative. You can dilate convolutions by setting :input_dilation or :kernel_dilation . Both :input_dilation and :kernel_dilation must either be positive integers or tuples of positive integers for each spatial dimension in the input and kernel tensors. Dilations can be thought of as applying dilation - 1 interior padding to the input or kernel tensor. You can split both the input and kernel tensor into feature groups using :feature_group_size . This will split both the input and kernel tensor channels and compute a grouped convolution. The size of the kernel input feature channels times the size of the feature group must match the size of the input tensor feature channels. Additionally, the size of the kernel output feature channels must be evenly divisible by the group size. You can also split the input tensor along the batch dimension by specifying :batch_group_size . This will compute a grouped convolution in the same way as with :feature_group_size , however, the input tensor will be split into groups along the batch dimension. Examples iex&gt; left = Nx . iota ( { 9 } ) iex&gt; left = Nx . reshape ( left , { 1 , 1 , 3 , 3 } ) iex&gt; right = Nx . iota ( { 4 } ) iex&gt; right = Nx . reshape ( right , { 4 , 1 , 1 , 1 } ) iex&gt; Nx . conv ( left , right , strides : [ 1 , 1 ] ) # Nx.Tensor &lt; f32 [ 1 ] [ 4 ] [ 3 ] [ 3 ] [ [ [ [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] , [ [ 0.0 , 1.0 , 2.0 ] , [ 3.0 , 4.0 , 5.0 ] , [ 6.0 , 7.0 , 8.0 ] ] , [ [ 0.0 , 2.0 , 4.0 ] , [ 6.0 , 8.0 , 10.0 ] , [ 12.0 , 14.0 , 16.0 ] ] , [ [ 0.0 , 3.0 , 6.0 ] , [ 9.0 , 12.0 , 15.0 ] , [ 18.0 , 21.0 , 24.0 ] ] ] ] &gt; iex&gt; left = Nx . iota ( { 9 } ) iex&gt; left = Nx . reshape ( left , { 1 , 1 , 3 , 3 } ) iex&gt; right = Nx . iota ( { 8 } ) iex&gt; right = Nx . reshape ( right , { 4 , 1 , 2 , 1 } ) iex&gt; Nx . conv ( left , right , strides : 2 , padding : :same , kernel_dilation : [ 2 , 1 ] ) # Nx.Tensor &lt; f32 [ 1 ] [ 4 ] [ 2 ] [ 2 ] [ [ [ [ 3.0 , 5.0 ] , [ 0.0 , 0.0 ] ] , [ [ 9.0 , 15.0 ] , [ 6.0 , 10.0 ] ] , [ [ 15.0 , 25.0 ] , [ 12.0 , 20.0 ] ] , [ [ 21.0 , 35.0 ] , [ 18.0 , 30.0 ] ] ] ] &gt; Complex tensors are also supported: iex&gt; left = Nx . tensor ( [ [ [ Complex . new ( 1 , 1 ) , 2 , Complex . new ( 3 , - 3 ) ] ] ] ) iex&gt; right = Nx . tensor ( [ [ [ 1 , Complex . new ( 0 , 2 ) , Complex . new ( 0 , 3 ) ] ] ] ) iex&gt; Nx . conv ( left , right , padding : [ { 2 , 2 } ] ) # Nx.Tensor &lt; c64 [ 1 ] [ 1 ] [ 5 ] [ [ [ - 3.0 + 3.0 i , - 2.0 + 8.0 i , 10.0 + 14.0 i , 8.0 + 6.0 i , 3.0 - 3.0 i ] ] ] &gt;","ref":"Nx.html#conv/3","title":"Nx.conv/3","type":"function"},{"doc":"Calculates the cosine of each element in the tensor. It is equivalent to: $$cos(z) = \\frac{e^{iz} + e^{-iz}}{2}$$ Examples iex&gt; Nx . cos ( 1 ) # Nx.Tensor &lt; f32 0.5403022766113281 &gt; iex&gt; Nx . cos ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.5403022766113281 , - 0.416146844625473 , - 0.9899924993515015 ] &gt;","ref":"Nx.html#cos/1","title":"Nx.cos/1","type":"function"},{"doc":"Calculates the hyperbolic cosine of each element in the tensor. It is equivalent to: $$cosh(z) = \\frac{e^z + e^{-z}}{2}$$ Examples iex&gt; Nx . cosh ( 1 ) # Nx.Tensor &lt; f32 1.5430806875228882 &gt; iex&gt; Nx . cosh ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.5430806875228882 , 3.762195587158203 , 10.067662239074707 ] &gt;","ref":"Nx.html#cosh/1","title":"Nx.cosh/1","type":"function"},{"doc":"Counts the number of leading zeros of each element in the tensor. Examples iex&gt; Nx . count_leading_zeros ( 1 ) # Nx.Tensor &lt; s64 63 &gt; iex&gt; Nx . count_leading_zeros ( - 1 ) # Nx.Tensor &lt; s64 0 &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0 , 0xF , 0xFF , 0xFFFF ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 4 ] [ 64 , 60 , 56 , 48 ] &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0xF000000000000000 , 0x0F00000000000000 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 0 , 4 ] &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0 , 0xF , 0xFF , 0xFFFF ] , type : :s32 , names : [ :x ] ) ) # Nx.Tensor &lt; s32 [ x : 4 ] [ 32 , 28 , 24 , 16 ] &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0 , 0xF , 0xFF , 0xFFFF ] , type : :s16 , names : [ :x ] ) ) # Nx.Tensor &lt; s16 [ x : 4 ] [ 16 , 12 , 8 , 0 ] &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0 , 1 , 2 , 4 , 8 , 16 , 32 , 64 , - 1 , - 128 ] , type : :s8 , names : [ :x ] ) ) # Nx.Tensor &lt; s8 [ x : 10 ] [ 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 , 0 , 0 ] &gt; iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0 , 1 , 2 , 4 , 8 , 16 , 32 , 64 , 128 ] , type : :u8 , names : [ :x ] ) ) # Nx.Tensor &lt; u8 [ x : 9 ] [ 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 , 0 ] &gt; Error cases iex&gt; Nx . count_leading_zeros ( Nx . tensor ( [ 0.0 , 1.0 ] ) ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#count_leading_zeros/1","title":"Nx.count_leading_zeros/1","type":"function"},{"doc":"Returns the cumulative maximum of elements along an axis. Options :axis - the axis to compare elements along. Defaults to 0 :reverse - whether to perform accumulation in the opposite direction. Defaults to false Examples iex&gt; Nx . cumulative_max ( Nx . tensor ( [ 3 , 4 , 2 , 1 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 , 4 , 4 , 4 ] &gt; iex&gt; Nx . cumulative_max ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 2 , 3 , 1 ] , [ 2 , 3 , 2 ] , [ 2 , 3 , 3 ] ] &gt; iex&gt; Nx . cumulative_max ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 2 , 3 , 3 ] , [ 1 , 3 , 3 ] , [ 2 , 2 , 3 ] ] &gt; iex&gt; Nx . cumulative_max ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 0 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 2 , 3 , 3 ] , [ 2 , 3 , 3 ] , [ 2 , 1 , 3 ] ] &gt; iex&gt; Nx . cumulative_max ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 1 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 3 , 3 , 1 ] , [ 3 , 3 , 2 ] , [ 3 , 3 , 3 ] ] &gt;","ref":"Nx.html#cumulative_max/2","title":"Nx.cumulative_max/2","type":"function"},{"doc":"Returns the cumulative minimum of elements along an axis. Options :axis - the axis to compare elements along. Defaults to 0 :reverse - whether to perform accumulation in the opposite direction. Defaults to false Examples iex&gt; Nx . cumulative_min ( Nx . tensor ( [ 3 , 4 , 2 , 1 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 , 3 , 2 , 1 ] &gt; iex&gt; Nx . cumulative_min ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 2 , 3 , 1 ] , [ 1 , 3 , 1 ] , [ 1 , 1 , 1 ] ] &gt; iex&gt; Nx . cumulative_min ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 2 , 2 , 1 ] , [ 1 , 1 , 1 ] , [ 2 , 1 , 1 ] ] &gt; iex&gt; Nx . cumulative_min ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 0 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 1 , 2 ] , [ 2 , 1 , 3 ] ] &gt; iex&gt; Nx . cumulative_min ( Nx . tensor ( [ [ 2 , 3 , 1 ] , [ 1 , 3 , 2 ] , [ 2 , 1 , 3 ] ] ) , axis : 1 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 2 , 2 ] , [ 1 , 1 , 3 ] ] &gt;","ref":"Nx.html#cumulative_min/2","title":"Nx.cumulative_min/2","type":"function"},{"doc":"Returns the cumulative product of elements along an axis. Options :axis - the axis to multiply elements along. Defaults to 0 :reverse - whether to perform accumulation in the opposite direction. Defaults to false Examples iex&gt; Nx . cumulative_product ( Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 1 , 2 , 6 , 24 ] &gt; iex&gt; Nx . cumulative_product ( Nx . iota ( { 3 , 3 } ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 1 , 2 ] , [ 0 , 4 , 10 ] , [ 0 , 28 , 80 ] ] &gt; iex&gt; Nx . cumulative_product ( Nx . iota ( { 3 , 3 } ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 0 , 0 ] , [ 3 , 12 , 60 ] , [ 6 , 42 , 336 ] ] &gt; iex&gt; Nx . cumulative_product ( Nx . iota ( { 3 , 3 } ) , axis : 0 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 28 , 80 ] , [ 18 , 28 , 40 ] , [ 6 , 7 , 8 ] ] &gt; iex&gt; Nx . cumulative_product ( Nx . iota ( { 3 , 3 } ) , axis : 1 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 2 , 2 ] , [ 60 , 20 , 5 ] , [ 336 , 56 , 8 ] ] &gt;","ref":"Nx.html#cumulative_product/2","title":"Nx.cumulative_product/2","type":"function"},{"doc":"Returns the cumulative sum of elements along an axis. Options :axis - the axis to sum elements along. Defaults to 0 :reverse - whether to perform accumulation in the opposite direction. Defaults to false Examples iex&gt; Nx . cumulative_sum ( Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 1 , 3 , 6 , 10 ] &gt; iex&gt; Nx . cumulative_sum ( Nx . iota ( { 3 , 3 } ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 1 , 2 ] , [ 3 , 5 , 7 ] , [ 9 , 12 , 15 ] ] &gt; iex&gt; Nx . cumulative_sum ( Nx . iota ( { 3 , 3 } ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 1 , 3 ] , [ 3 , 7 , 12 ] , [ 6 , 13 , 21 ] ] &gt; iex&gt; Nx . cumulative_sum ( Nx . iota ( { 3 , 3 } ) , axis : 0 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 9 , 12 , 15 ] , [ 9 , 11 , 13 ] , [ 6 , 7 , 8 ] ] &gt; iex&gt; Nx . cumulative_sum ( Nx . iota ( { 3 , 3 } ) , axis : 1 , reverse : true ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 3 , 3 , 2 ] , [ 12 , 9 , 5 ] , [ 21 , 15 , 8 ] ] &gt;","ref":"Nx.html#cumulative_sum/2","title":"Nx.cumulative_sum/2","type":"function"},{"doc":"Gets the default backend for the current process.","ref":"Nx.html#default_backend/0","title":"Nx.default_backend/0","type":"function"},{"doc":"Sets the given backend as default in the current process . The default backend is stored only in the process dictionary. This means if you start a separate process, such as Task , the default backend must be set on the new process too. This function is mostly used for scripting and testing. In your applications, you must prefer to set the backend in your config files: config :nx , :default_backend , { EXLA.Backend , device : :cuda } In your notebooks and on Mix.install/2 , you might: Mix . install ( [ { :nx , &quot;&gt;= 0.0.0&quot; } ] , config : [ nx : [ default_backend : { EXLA.Backend , device : :cuda } ] ] ) Or use Nx.global_default_backend/1 as it changes the default backend on all processes. Examples iex&gt; Nx . default_backend ( { EXLA.Backend , device : :cuda } ) { Nx.BinaryBackend , [ ] } iex&gt; Nx . default_backend ( ) { EXLA.Backend , device : :cuda }","ref":"Nx.html#default_backend/1","title":"Nx.default_backend/1","type":"function"},{"doc":"Deserializes a serialized representation of a tensor or a container with the given options. It is the opposite of Nx.serialize/2 . Examples iex&gt; a = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; serialized_a = Nx . serialize ( a ) iex&gt; Nx . deserialize ( serialized_a ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; container = { Nx . tensor ( [ 1 , 2 , 3 ] ) , %{ b : Nx . tensor ( [ 4 , 5 , 6 ] ) } } iex&gt; serialized_container = Nx . serialize ( container ) iex&gt; { a , %{ b : b } } = Nx . deserialize ( serialized_container ) iex&gt; a # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; b # Nx.Tensor &lt; s64 [ 3 ] [ 4 , 5 , 6 ] &gt;","ref":"Nx.html#deserialize/2","title":"Nx.deserialize/2","type":"function"},{"doc":"Element-wise division of two tensors. If a number is given, it is converted to a tensor. It always returns a float tensor. If any of the input tensors are not float, they are converted to f32. Division by zero raises, but it may trigger undefined behaviour on some compilers. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the / operator in place of this function: left / right . Examples Dividing scalars iex&gt; Nx . divide ( 1 , 2 ) # Nx.Tensor &lt; f32 0.5 &gt; Dividing tensors and scalars iex&gt; Nx . divide ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; iex&gt; Nx . divide ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 1.0 , 0.5 , 0.3333333432674408 ] &gt; Dividing tensors iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ nil , :y ] ) iex&gt; Nx . divide ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 0.10000000149011612 , 0.05000000074505806 ] , [ 0.20000000298023224 , 0.10000000149011612 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 , names : [ :x , :y ] ) iex&gt; Nx . divide ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 0.10000000149011612 , 0.05000000074505806 ] , [ 0.20000000298023224 , 0.10000000149011612 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :f32 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :f32 , names : [ nil , :y ] ) iex&gt; Nx . divide ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 0.10000000149011612 , 0.05000000074505806 ] , [ 0.20000000298023224 , 0.10000000149011612 ] ] &gt;","ref":"Nx.html#divide/2","title":"Nx.divide/2","type":"function"},{"doc":"Returns the dot product of two tensors. Given a and b , computes the dot product according to the following rules: If both a and b are scalars, it is equivalent to a * b . If a is a scalar and b is a tensor, it is equivalent to Nx.multiply(a, b) . If a is a tensor and b is a scalar, it is equivalent to Nx.multiply(a, b) . If both a and b are 1-D tensors (vectors), it is the sum of the element-wise product between a and b . The lengths of a and b must be equal. If both a and b are 2-D tensors (matrices), it is equivalent to matrix-multiplication. If either a or b is a 1-D tensor, and the other is an n-D tensor, it is the sum of the element-wise product along the last axis of a or b . The length of the 1-D tensor must match the last dimension of the n-D tensor. If a is an n-D tensor and b is an m-D tensor, it is the sum of the element-wise product along the last axis of a and the second-to-last axis of b . The last dimension of a must match the second-to-last dimension of b . For a more general dot function where you control which axes contract, see dot/4 . Examples Dot product of scalars iex&gt; Nx . dot ( 5 , 5 ) # Nx.Tensor &lt; s64 25 &gt; iex&gt; Nx . dot ( - 2.0 , 5.0 ) # Nx.Tensor &lt; f32 - 10.0 &gt; iex&gt; Nx . dot ( 2 , 2.0 ) # Nx.Tensor &lt; f32 4.0 &gt; Dot product of vectors iex&gt; Nx . dot ( Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 4 , 5 , 6 ] ) ) # Nx.Tensor &lt; s64 32 &gt; iex&gt; Nx . dot ( Nx . tensor ( [ 2.0 , 4.0 , 3.0 , 5.0 ] ) , Nx . tensor ( [ 1.0 , 2.0 , 3.0 , 4.0 ] ) ) # Nx.Tensor &lt; f32 39.0 &gt; iex&gt; Nx . dot ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) , Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; f32 14.0 &gt; Dot product of matrices iex&gt; left = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :i , :j ] ) iex&gt; right = Nx . tensor ( [ [ 7 , 8 ] , [ 9 , 10 ] , [ 11 , 12 ] ] , names : [ :x , :y ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; s64 [ i : 2 ] [ y : 2 ] [ [ 58 , 64 ] , [ 139 , 154 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 10.0 , 13.0 , 14.0 , 15.0 ] , [ 59.0 , 20.0 , 10.0 , 30.0 ] ] , names : [ :i , :j ] ) iex&gt; right = Nx . tensor ( [ [ 2.0 , 4.0 ] , [ 5.0 , 1.0 ] , [ 6.0 , 8.0 ] , [ 9.0 , 10.0 ] ] , names : [ :x , :y ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; f32 [ i : 2 ] [ y : 2 ] [ [ 304.0 , 315.0 ] , [ 548.0 , 636.0 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :i , :j ] ) iex&gt; right = Nx . tensor ( [ [ 7.0 , 8.0 ] , [ 9.0 , 10.0 ] , [ 11.0 , 12.0 ] ] , names : [ :x , :y ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; f32 [ i : 2 ] [ y : 2 ] [ [ 58.0 , 64.0 ] , [ 139.0 , 154.0 ] ] &gt; Dot product of vector and n-d tensor iex&gt; left = Nx . tensor ( [ [ [ 1 , 2 ] , [ 3 , 4 ] ] , [ [ 5 , 6 ] , [ 7 , 8 ] ] ] , names : [ :i , :j , :k ] ) iex&gt; right = Nx . tensor ( [ 5 , 10 ] , names : [ :x ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; s64 [ i : 2 ] [ j : 2 ] [ [ 25 , 55 ] , [ 85 , 115 ] ] &gt; iex&gt; left = Nx . tensor ( [ 5 , 10 ] , names : [ :x ] ) iex&gt; right = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :i , :j ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; s64 [ j : 3 ] [ 45 , 60 , 75 ] &gt; iex&gt; left = Nx . tensor ( [ [ [ [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] , [ [ 5.0 , 6.0 ] , [ 7.0 , 8.0 ] ] ] ] ] , names : [ :shard , :batch , :x , :y , :z ] ) iex&gt; right = Nx . tensor ( [ 2.0 , 2.0 ] , names : [ :data ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; f32 [ shard : 1 ] [ batch : 1 ] [ x : 2 ] [ y : 2 ] [ [ [ [ 6.0 , 14.0 ] , [ 22.0 , 30.0 ] ] ] ] &gt; Dot product of n-D and m-D tensor iex&gt; left = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; right = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 3 , 4 , 5 ] , [ 5 , 6 , 7 ] ] ] , names : [ :i , :j , :k ] ) iex&gt; Nx . dot ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ i : 1 ] [ k : 3 ] [ [ [ [ 22 , 28 , 34 ] ] , [ [ 49 , 64 , 79 ] ] , [ [ 76 , 100 , 124 ] ] ] , [ [ [ 22 , 28 , 34 ] ] , [ [ 49 , 64 , 79 ] ] , [ [ 76 , 100 , 124 ] ] ] ] &gt; Error Cases iex&gt; Nx . dot ( Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 1 , 2 ] ) ) ** (ArgumentError) dot/zip expects shapes to be compatible, dimension 0 of left-side (3) does not equal dimension 0 of right-side (2)","ref":"Nx.html#dot/2","title":"Nx.dot/2","type":"function"},{"doc":"Computes the generalized dot product between two tensors, given the contracting axes. This is equivalent to calling Nx.dot/6 with no batching dimensions: Nx . dot ( t1 , contract_axes1 , [ ] , t2 , contract_axes2 , [ ] ) Examples iex&gt; t1 = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) iex&gt; t2 = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , names : [ :height , :width ] ) iex&gt; Nx . dot ( t1 , [ 0 ] , t2 , [ 0 ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ width : 2 ] [ [ 100 , 140 ] , [ 140 , 200 ] ] &gt; iex&gt; t1 = Nx . tensor ( [ [ 0.0 , 1.0 , 2.0 ] , [ 3.0 , 4.0 , 5.0 ] ] ) iex&gt; t2 = Nx . tensor ( [ [ 0.0 , 1.0 ] , [ 2.0 , 3.0 ] , [ 4.0 , 5.0 ] ] ) iex&gt; Nx . dot ( t1 , [ 0 , 1 ] , t2 , [ 1 , 0 ] ) # Nx.Tensor &lt; f32 50.0 &gt;","ref":"Nx.html#dot/4","title":"Nx.dot/4","type":"function"},{"doc":"Computes the generalized dot product between two tensors, given the contracting and batch axes. The dot product is computed by multiplying the values from t1 given by contract_axes1 against the values from t2 given by contract_axes2 , considering batch axes of batch_axes1 and batch_axes2 . For instance, the first axis in contract_axes1 will be matched against the first axis in contract_axes2 and so on. The axes given by contract_axes1 and contract_axes2 are effectively removed from the final tensor, which is why they are often called the contraction axes. If no contracting axes are given, the final product works like Nx.outer/2 . Specifying batch axes will compute a vectorized dot product along the given batch dimensions. The length of batch_axes1 and batch_axes2 must match. Additionally, batch_axes1 and batch_axes2 must be a list of successive dimension numbers, where each batch axis matches the dimension of the corresponding batch axis in the other input. The contracting axes must be dot-product compatible and the batch dimensions must always have the same number of elements. Examples Contracting along axes iex&gt; t1 = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) iex&gt; t2 = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , names : [ :height , :width ] ) iex&gt; Nx . dot ( t1 , [ 0 ] , [ ] , t2 , [ 0 ] , [ ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ width : 2 ] [ [ 100 , 140 ] , [ 140 , 200 ] ] &gt; iex&gt; Nx . dot ( t1 , [ 0 ] , [ ] , t2 , [ 1 ] , [ ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ height : 2 ] [ [ 70 , 150 ] , [ 100 , 220 ] ] &gt; iex&gt; Nx . dot ( t1 , [ 1 ] , [ ] , t2 , [ 0 ] , [ ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ width : 2 ] [ [ 70 , 100 ] , [ 150 , 220 ] ] &gt; iex&gt; Nx . dot ( t1 , [ 1 ] , [ ] , t2 , [ 1 ] , [ ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ height : 2 ] [ [ 50 , 110 ] , [ 110 , 250 ] ] &gt; iex&gt; Nx . dot ( t1 , [ 0 , 1 ] , [ ] , t2 , [ 0 , 1 ] , [ ] ) # Nx.Tensor &lt; s64 300 &gt; If no axes are given, it works like outer/2 : iex&gt; t1 = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t2 = Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] ) iex&gt; Nx . dot ( t1 , [ ] , [ ] , t2 , [ ] , [ ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ 2 ] [ [ [ [ 10 , 20 ] , [ 30 , 40 ] ] , [ [ 20 , 40 ] , [ 60 , 80 ] ] ] , [ [ [ 30 , 60 ] , [ 90 , 120 ] ] , [ [ 40 , 80 ] , [ 120 , 160 ] ] ] ] &gt; Dot product between two batched tensors iex&gt; u = Nx . tensor ( [ [ [ 1 ] ] , [ [ 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] ] , [ [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 0 ] , v , [ 2 ] , [ 0 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 1 ] [ [ [ 3 ] ] , [ [ 8 ] ] ] &gt; iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 0 ] , v , [ 1 ] , [ 0 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 1 ] [ [ [ 6 ] ] , [ [ 16 ] ] ] &gt; Error cases iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 0 ] , v , [ 1 ] , [ ] ) ** (ArgumentError) right tensor must be batched if left tensor is batched iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ ] , v , [ 1 ] , [ 0 ] ) ** (ArgumentError) left tensor must be batched if right tensor is batched iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 1 ] , v , [ 1 ] , [ 0 ] ) ** (ArgumentError) invalid dot batch axis for the left tensor, batch axes must be successive dimensions starting from 0, got [1] iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 0 ] , v , [ 1 ] , [ 1 ] ) ** (ArgumentError) invalid dot batch axis for the right tensor, batch axes must be successive dimensions starting from 0, got [1] iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 0 ] , [ 0 ] , v , [ 1 ] , [ 0 ] ) ** (ArgumentError) dot batch axes for left tensor ([0]) cannot be in contract axes ([0]) iex&gt; u = Nx . tensor ( [ [ [ 1 , 1 ] ] , [ [ 2 , 2 ] ] ] ) iex&gt; v = Nx . tensor ( [ [ [ 3 ] , [ 3 ] ] , [ [ 4 ] , [ 4 ] ] ] ) iex&gt; Nx . dot ( u , [ 2 ] , [ 0 ] , v , [ 0 ] , [ 0 ] ) ** (ArgumentError) dot batch axes for right tensor ([0]) cannot be in contract axes ([0])","ref":"Nx.html#dot/6","title":"Nx.dot/6","type":"function"},{"doc":"Element-wise equality comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the == operator in place of this function: left == right . Examples Comparison of scalars iex&gt; Nx . equal ( 1 , 2 ) # Nx.Tensor &lt; u8 0 &gt; Comparison of tensors and scalars iex&gt; Nx . equal ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 0 , 0 ] &gt; Comparison of tensors iex&gt; left = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 5 ] ) iex&gt; Nx . equal ( left , right ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 1 , 0 ] &gt; iex&gt; left = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; Nx . equal ( left , right ) # Nx.Tensor &lt; u8 [ x : 2 ] [ 3 ] [ [ 1 , 1 , 1 ] , [ 0 , 0 , 0 ] ] &gt;","ref":"Nx.html#equal/2","title":"Nx.equal/2","type":"function"},{"doc":"Calculates the error function of each element in the tensor. It is equivalent to: $$erf(z) = \\frac{2}{\\sqrt{\\pi}} \\int_{0}^{z} e^{-t^2}dt$$ Examples iex&gt; Nx . erf ( 1 ) # Nx.Tensor &lt; f32 0.8427007794380188 &gt; iex&gt; Nx . erf ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.8427007794380188 , 0.9953222870826721 , 0.9999778866767883 ] &gt;","ref":"Nx.html#erf/1","title":"Nx.erf/1","type":"function"},{"doc":"Calculates the inverse error function of each element in the tensor. It is equivalent to: $$erf\\text{\\textunderscore}inv(erf(z)) = z$$ Examples iex&gt; Nx . erf_inv ( 0.10000000149011612 ) # Nx.Tensor &lt; f32 0.08885598927736282 &gt; iex&gt; Nx . erf_inv ( Nx . tensor ( [ 0.10000000149011612 , 0.5 , 0.8999999761581421 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.08885598927736282 , 0.4769362807273865 , 1.163087010383606 ] &gt;","ref":"Nx.html#erf_inv/1","title":"Nx.erf_inv/1","type":"function"},{"doc":"Calculates the one minus error function of each element in the tensor. It is equivalent to: $$erfc(z) = 1 - erf(z)$$ Examples iex&gt; Nx . erfc ( 1 ) # Nx.Tensor &lt; f32 0.15729920566082 &gt; iex&gt; Nx . erfc ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.15729920566082 , 0.004677734803408384 , 2.2090496713644825e-5 ] &gt;","ref":"Nx.html#erfc/1","title":"Nx.erfc/1","type":"function"},{"doc":"Calculates the exponential of each element in the tensor. It is equivalent to: $$exp(z) = e^z$$ Examples iex&gt; Nx . exp ( 1 ) # Nx.Tensor &lt; f32 2.7182817459106445 &gt; iex&gt; Nx . exp ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 2.7182817459106445 , 7.389056205749512 , 20.08553695678711 ] &gt;","ref":"Nx.html#exp/1","title":"Nx.exp/1","type":"function"},{"doc":"Calculates the exponential minus one of each element in the tensor. It is equivalent to: $$expm1(z) = e^z - 1$$ Examples iex&gt; Nx . expm1 ( 1 ) # Nx.Tensor &lt; f32 1.718281865119934 &gt; iex&gt; Nx . expm1 ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.718281865119934 , 6.389056205749512 , 19.08553695678711 ] &gt;","ref":"Nx.html#expm1/1","title":"Nx.expm1/1","type":"function"},{"doc":"Creates the identity matrix of size n . Examples iex&gt; Nx . eye ( 2 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 0 ] , [ 0 , 1 ] ] &gt; iex&gt; Nx . eye ( 3 , type : :f32 , names : [ :height , :width ] ) # Nx.Tensor &lt; f32 [ height : 3 ] [ width : 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] &gt; The first argument can also be a shape of a matrix: iex&gt; Nx . eye ( { 1 , 2 } ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ [ 1 , 0 ] ] &gt; The shape can also represent a tensor batch. In this case, the last two axes will represent the same identity matrix. iex&gt; Nx . eye ( { 2 , 4 , 3 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 4 ] [ 3 ] [ [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 0 , 0 , 1 ] , [ 0 , 0 , 0 ] ] , [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 0 , 0 , 1 ] , [ 0 , 0 , 0 ] ] ] &gt; Options :type - the type of the tensor :names - the names of the tensor dimensions :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#eye/2","title":"Nx.eye/2","type":"function"},{"doc":"Calculates the DFT of the given tensor. Options :eps - Threshold which backends can use for cleaning-up results. Defaults to 1.0e-10 . :length - Either a positive integer or :power_of_two . Will pad or slice the tensor accordingly. :power_of_two will automatically pad to the next power of two. Examples iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 , 0 , 0 ] ) ) # Nx.Tensor &lt; c64 [ 4 ] [ 2.0 + 0.0 i , 1.0 - 1.0 i , 0.0 + 0.0 i , 1.0 + 1.0 i ] &gt; iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 , 0 , 0 , 0 ] ) ) # Nx.Tensor &lt; c64 [ 5 ] [ 2.0 + 0.0 i , 1.3090169429779053 - 0.9510565400123596 i , 0.19098301231861115 - 0.5877852439880371 i , 0.19098301231861115 + 0.5877852439880371 i , 1.3090169429779053 + 0.9510565400123596 i ] &gt; iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 , 1 , 0 , 1 , 1 ] ) ) # Nx.Tensor &lt; c64 [ 6 ] [ 5.0 + 0.0 i , 1.0 + 0.0 i , - 1.0 + 0.0 i , 1.0 + 0.0 i , - 1.0 + 0.0 i , 1.0 + 0.0 i ] &gt; Padding and slicing can be introduced through :length : iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 4 ] [ 2.0 + 0.0 i , 1.0 - 1.0 i , 0.0 + 0.0 i , 1.0 + 1.0 i ] &gt; iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 , 0 ] ) , length : :power_of_two ) # Nx.Tensor &lt; c64 [ 4 ] [ 2.0 + 0.0 i , 1.0 - 1.0 i , 0.0 + 0.0 i , 1.0 + 1.0 i ] &gt; iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 , 0 , 0 , 2 , 3 ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 4 ] [ 2.0 + 0.0 i , 1.0 - 1.0 i , 0.0 + 0.0 i , 1.0 + 1.0 i ] &gt; If an N-dimensional tensor is passed, the DFT is applied to its last axis: iex&gt; Nx . fft ( Nx . tensor ( [ [ 1 , 1 , 0 , 0 , 2 , 3 ] , [ 1 , 0 , 0 , 0 , 2 , 3 ] ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 2 ] [ 4 ] [ [ 2.0 + 0.0 i , 1.0 - 1.0 i , 0.0 + 0.0 i , 1.0 + 1.0 i ] , [ 1.0 + 0.0 i , 1.0 + 0.0 i , 1.0 + 0.0 i , 1.0 + 0.0 i ] ] &gt; Error Cases iex&gt; Nx . fft ( Nx . tensor ( [ 1 , 1 ] ) , length : :invalid ) ** (RuntimeError) expected an integer or :power_of_two as length, got: :invalid","ref":"Nx.html#fft/2","title":"Nx.fft/2","type":"function"},{"doc":"Flattens a n-dimensional tensor to a 1-dimensional tensor. Flattening only changes the tensor metadata, it doesn't copy the underlying structure. Flatten is a destructive operation with respect to names. Examples iex&gt; t = Nx . iota ( { 2 , 2 , 2 , 2 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ 2 ] [ [ [ [ 0 , 1 ] , [ 2 , 3 ] ] , [ [ 4 , 5 ] , [ 6 , 7 ] ] ] , [ [ [ 8 , 9 ] , [ 10 , 11 ] ] , [ [ 12 , 13 ] , [ 14 , 15 ] ] ] ] &gt; iex&gt; Nx . flatten ( t ) # Nx.Tensor &lt; s64 [ 16 ] [ 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 ] &gt; And if the tensor is already 1-dimensional: iex&gt; t = Nx . iota ( { 16 } ) # Nx.Tensor &lt; s64 [ 16 ] [ 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 ] &gt; iex&gt; Nx . flatten ( t ) # Nx.Tensor &lt; s64 [ 16 ] [ 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 ] &gt;","ref":"Nx.html#flatten/1","title":"Nx.flatten/1","type":"function"},{"doc":"Calculates the floor of each element in the tensor. If a non-floating tensor is given, it is returned as is. If a floating tensor is given, then we apply the operation, but keep its type. Examples iex&gt; Nx . floor ( Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ - 1 , 0 , 1 ] &gt; iex&gt; Nx . floor ( Nx . tensor ( [ - 1.5 , - 0.5 , 0.5 , 1.5 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 4 ] [ - 2.0 , - 1.0 , 0.0 , 1.0 ] &gt;","ref":"Nx.html#floor/1","title":"Nx.floor/1","type":"function"},{"doc":"Creates a one-dimensional tensor from a binary with the given type . If the binary size does not match its type, an error is raised. Examples iex&gt; Nx . from_binary ( &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , :s8 ) # Nx.Tensor &lt; s8 [ 4 ] [ 1 , 2 , 3 , 4 ] &gt; The atom notation for types is also supported: iex&gt; Nx . from_binary ( &lt;&lt; 12.3 :: float - 64 - native &gt;&gt; , :f64 ) # Nx.Tensor &lt; f64 [ 1 ] [ 12.3 ] &gt; An error is raised for incompatible sizes: iex&gt; Nx . from_binary ( &lt;&lt; 1 , 2 , 3 , 4 &gt;&gt; , :f64 ) ** (ArgumentError) binary does not match the given size Options :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#from_binary/3","title":"Nx.from_binary/3","type":"function"},{"doc":"Loads a .npy file into a tensor. An .npy file stores a single array created from Python's NumPy library. This function can be useful for loading data originally created or intended to be loaded from NumPy into Elixir.","ref":"Nx.html#from_numpy/1","title":"Nx.from_numpy/1","type":"function"},{"doc":"Loads a .npz archive into a list of tensors. An .npz file is a zipped, possibly compressed archive containing multiple .npy files.","ref":"Nx.html#from_numpy_archive/1","title":"Nx.from_numpy_archive/1","type":"function"},{"doc":"Builds a new tensor by taking individual values from the original tensor at the given indices. The last dimension in indices must have the same size as the tensor rank, think of it as one value per axis. Examples iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . gather ( t , Nx . tensor ( [ [ 1 , 1 ] , [ 0 , 1 ] , [ 1 , 0 ] ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 4 , 2 , 3 ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . gather ( t , Nx . tensor ( [ [ [ 1 , 1 ] , [ 0 , 0 ] ] , [ [ 1 , 0 ] , [ 0 , 1 ] ] ] ) ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 4 , 1 ] , [ 3 , 2 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 11 , 12 ] ] , [ [ 101 , 102 ] , [ 111 , 112 ] ] ] ) iex&gt; Nx . gather ( t , Nx . tensor ( [ [ 0 , 0 , 0 ] , [ 0 , 1 , 1 ] , [ 1 , 1 , 1 ] ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 12 , 112 ] &gt; Error cases iex&gt; Nx . gather ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , Nx . tensor ( [ [ 0 , 0 ] ] , type : :f32 ) ) ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#gather/2","title":"Nx.gather/2","type":"function"},{"doc":"Sets the default backend globally. You must avoid calling this function at runtime. It is mostly useful during scripts or code notebooks to set a default. If you need to configure a global default backend in your applications, it is generally preferred to do so in your config/*.exs files: config :nx , :default_backend , { EXLA.Backend , [ ] } In your notebooks and on Mix.install/2 , you might: Mix . install ( [ { :nx , &quot;&gt;= 0.0.0&quot; } ] , config : [ nx : [ default_backend : { EXLA.Backend , device : :cuda } ] ] )","ref":"Nx.html#global_default_backend/1","title":"Nx.global_default_backend/1","type":"function"},{"doc":"Element-wise greater than comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the &gt; operator in place of this function: left &gt; right . Examples Comparison of scalars iex&gt; Nx . greater ( 1 , 2 ) # Nx.Tensor &lt; u8 0 &gt; Comparison of tensors and scalars iex&gt; Nx . greater ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 0 , 0 ] &gt; Comparison of tensors iex&gt; left = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 2 ] ) iex&gt; Nx . greater ( left , right ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 0 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; Nx . greater ( left , right ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 3 ] [ [ 0 , 0 , 0 ] , [ 1 , 1 , 1 ] ] &gt;","ref":"Nx.html#greater/2","title":"Nx.greater/2","type":"function"},{"doc":"Element-wise greater than or equal comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the &gt;= operator in place of this function: left &gt;= right . Examples Comparison of scalars iex&gt; Nx . greater_equal ( 1 , 2 ) # Nx.Tensor &lt; u8 0 &gt; Comparison of tensors and scalars iex&gt; Nx . greater_equal ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 0 , 0 ] &gt; Comparison of tensors iex&gt; left = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 2 ] ) iex&gt; Nx . greater_equal ( left , right ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 1 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; Nx . greater_equal ( left , right ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 1 , 1 ] ] &gt;","ref":"Nx.html#greater_equal/2","title":"Nx.greater_equal/2","type":"function"},{"doc":"Calculates the Inverse DFT of the given tensor. Options :eps - Threshold which backends can use for cleaning-up results. Defaults to 1.0e-10 . :length - Either a positive integer or :power_of_two . Will pad or slice the tensor accordingly. :power_of_two will automatically pad to the next power of two. Examples iex&gt; Nx . ifft ( Nx . tensor ( [ 2 , Complex . new ( 1 , - 1 ) , 0 , Complex . new ( 1 , 1 ) ] ) ) # Nx.Tensor &lt; c64 [ 4 ] [ 1.0 + 0.0 i , 1.0 + 0.0 i , 0.0 + 0.0 i , 0.0 + 0.0 i ] &gt; iex&gt; Nx . ifft ( Nx . tensor ( [ 5 , 1 , - 1 , 1 , - 1 , 1 ] ) ) # Nx.Tensor &lt; c64 [ 6 ] [ 1.0 + 0.0 i , 1.0 + 0.0 i , 1.0 + 0.0 i , 0.0 + 0.0 i , 1.0 + 0.0 i , 1.0 + 0.0 i ] &gt; Padding and slicing can be introduced through :length : iex&gt; Nx . ifft ( Nx . tensor ( [ 1 , 1 ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 4 ] [ 0.5 + 0.0 i , 0.25 + 0.25 i , 0.0 + 0.0 i , 0.25 - 0.25 i ] &gt; iex&gt; Nx . ifft ( Nx . tensor ( [ 1 , 1 , 0 ] ) , length : :power_of_two ) # Nx.Tensor &lt; c64 [ 4 ] [ 0.5 + 0.0 i , 0.25 + 0.25 i , 0.0 + 0.0 i , 0.25 - 0.25 i ] &gt; iex&gt; Nx . ifft ( Nx . tensor ( [ 1 , 1 , 0 , 0 , 2 , 3 ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 4 ] [ 0.5 + 0.0 i , 0.25 + 0.25 i , 0.0 + 0.0 i , 0.25 - 0.25 i ] &gt; If an N-dimensional tensor is passed, the Inverse DFT is applied to its last axis: iex&gt; Nx . ifft ( Nx . tensor ( [ [ 1 , 1 , 0 , 0 , 2 , 3 ] , [ 1 , 0 , 0 , 0 , 2 , 3 ] ] ) , length : 4 ) # Nx.Tensor &lt; c64 [ 2 ] [ 4 ] [ [ 0.5 + 0.0 i , 0.25 + 0.25 i , 0.0 + 0.0 i , 0.25 - 0.25 i ] , [ 0.25 + 0.0 i , 0.25 + 0.0 i , 0.25 + 0.0 i , 0.25 + 0.0 i ] ] &gt; Error Cases iex&gt; Nx . ifft ( Nx . tensor ( [ 1 , 1 ] ) , length : :invalid ) ** (RuntimeError) expected an integer or :power_of_two as length, got: :invalid","ref":"Nx.html#ifft/2","title":"Nx.ifft/2","type":"function"},{"doc":"Returns the imaginary component of each entry in a complex tensor as a floating point tensor. Examples iex&gt; Nx . imag ( Complex . new ( 1 , 2 ) ) # Nx.Tensor &lt; f32 2.0 &gt; iex&gt; Nx . imag ( Nx . tensor ( 1 ) ) # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; Nx . imag ( Nx . tensor ( 1 , type : :bf16 ) ) # Nx.Tensor &lt; bf16 0.0 &gt; iex&gt; Nx . imag ( Nx . tensor ( [ Complex . new ( 1 , 2 ) , Complex . new ( 2 , - 4 ) ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 2.0 , - 4.0 ] &gt;","ref":"Nx.html#imag/1","title":"Nx.imag/1","type":"function"},{"doc":"Performs an indexed add operation on the target tensor, adding the updates into the corresponding indices positions. This operation is the grad for gather/2 and gather-like operations such as take/3 and take_along_axis/3 . indices must be a fully qualified tensor of shape {n, Nx.rank(target)} , with n being an arbitrary number of indices, while updates must have a compatible {n} shape. See also: indexed_add/3 , gather/2 , take/3 , take_along_axis/3 Examples iex&gt; t = Nx . iota ( { 1 , 2 , 3 } ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] ] &gt; iex&gt; indices = Nx . tensor ( [ [ 0 , 0 , 0 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 2 ] , [ 0 , 1 , 2 ] ] ) iex&gt; updates = Nx . tensor ( [ 1 , 3 , 1 , - 2 , 5 ] ) iex&gt; Nx . indexed_add ( t , indices , updates ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 2 , 1 , 0 ] , [ 3 , 7 , 10 ] ] ] &gt; Type promotions should happen automatically, with the resulting type being the combination of the target type and the updates type. iex&gt; Nx . indexed_add ( Nx . tensor ( [ 1.0 ] ) , Nx . tensor ( [ [ 0 ] , [ 0 ] ] ) , Nx . tensor ( [ 1 , 1 ] ) ) # Nx.Tensor &lt; f32 [ 1 ] [ 3.0 ] &gt; iex&gt; Nx . indexed_add ( Nx . tensor ( [ 1 ] ) , Nx . tensor ( [ [ 0 ] , [ 0 ] ] ) , Nx . tensor ( [ 1.0 , 1.0 ] ) ) # Nx.Tensor &lt; f32 [ 1 ] [ 3.0 ] &gt; iex&gt; Nx . indexed_add ( Nx . tensor ( [ 1 ] , type : :s32 ) , Nx . tensor ( [ [ 0 ] , [ 0 ] ] ) , Nx . tensor ( [ 1 , 1 ] , type : :s64 ) ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] &gt; Error cases iex&gt; Nx . indexed_add ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ [ 1 , 2 , 3 ] ] ] ) , Nx . tensor ( [ 0 ] ) ) ** (ArgumentError) indices must be a rank 2 tensor, got: 3 iex&gt; Nx . indexed_add ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 ] ] ) , Nx . tensor ( [ [ 0 ] ] ) ) ** (ArgumentError) updates must be a rank 1 tensor, got: 2 iex&gt; Nx . indexed_add ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 , 3 ] ] ) , Nx . tensor ( [ 0 ] ) ) ** (ArgumentError) expected indices to have shape {*, 2}, got: {1, 3} iex&gt; Nx . indexed_add ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 ] ] ) , Nx . tensor ( [ 0 , 1 ] ) ) ** (ArgumentError) expected updates tensor to match the first axis of indices tensor with shape {1, 2}, got {2}","ref":"Nx.html#indexed_add/3","title":"Nx.indexed_add/3","type":"function"},{"doc":"Puts individual values from updates into the given tensor at the corresponding indices . indices must be a fully qualified tensor of shape {n, Nx.rank(target)} , with n being an arbitrary number of indices, while updates must have a compatible {n} shape. In case of repeating indices, the result is non-determinstic, since the operation happens in parallel when running on devices such as the GPU. See also: indexed_add/3 , put_slice/3 . Examples iex&gt; Nx . indexed_put ( Nx . tensor ( [ 0 , 0 , 0 ] ) , Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ 2 , 4 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 0 , 2 , 4 ] &gt; iex&gt; Nx . indexed_put ( Nx . tensor ( [ 0 , 0 , 0 ] ) , Nx . tensor ( [ [ 1 ] , [ 2 ] , [ 1 ] ] ) , Nx . tensor ( [ 3 , 4 , 2 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 0 , 2 , 4 ] &gt; iex&gt; t = Nx . iota ( { 1 , 2 , 3 } ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] ] &gt; iex&gt; indices = Nx . tensor ( [ [ 0 , 0 , 0 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 2 ] ] ) iex&gt; updates = Nx . tensor ( [ 1 , 3 , - 2 ] ) iex&gt; Nx . indexed_put ( t , indices , updates ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 1 , 1 , - 2 ] , [ 3 , 3 , 5 ] ] ] &gt; Type promotions should happen automatically, with the resulting type being the combination of the target type and the updates type. iex&gt; Nx . indexed_put ( Nx . tensor ( [ 1.0 ] ) , Nx . tensor ( [ [ 0 ] ] ) , Nx . tensor ( [ 3 ] ) ) # Nx.Tensor &lt; f32 [ 1 ] [ 3.0 ] &gt; iex&gt; Nx . indexed_put ( Nx . tensor ( [ 1 ] ) , Nx . tensor ( [ [ 0 ] ] ) , Nx . tensor ( [ 3.0 ] ) ) # Nx.Tensor &lt; f32 [ 1 ] [ 3.0 ] &gt; iex&gt; Nx . indexed_put ( Nx . tensor ( [ 1 ] , type : :s32 ) , Nx . tensor ( [ [ 0 ] ] ) , Nx . tensor ( [ 3 ] , type : :s64 ) ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] &gt; Error cases iex&gt; Nx . indexed_put ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ [ 1 , 2 , 3 ] ] ] ) , Nx . tensor ( [ 0 ] ) ) ** (ArgumentError) indices must be a rank 2 tensor, got: 3 iex&gt; Nx . indexed_put ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 ] ] ) , Nx . tensor ( [ [ 0 ] ] ) ) ** (ArgumentError) updates must be a rank 1 tensor, got: 2 iex&gt; Nx . indexed_put ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 , 3 ] ] ) , Nx . tensor ( [ 0 ] ) ) ** (ArgumentError) expected indices to have shape {*, 2}, got: {1, 3} iex&gt; Nx . indexed_put ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] ) , Nx . tensor ( [ [ 1 , 2 ] ] ) , Nx . tensor ( [ 0 , 1 ] ) ) ** (ArgumentError) expected updates tensor to match the first axis of indices tensor with shape {1, 2}, got {2}","ref":"Nx.html#indexed_put/3","title":"Nx.indexed_put/3","type":"function"},{"doc":"Creates a tensor with the given shape which increments along the provided axis. You may optionally provide dimension names. If no axis is provided, index counts up at each element. If a tensor or a number are given, the shape and names are taken from the tensor. Examples iex&gt; Nx . iota ( { } ) # Nx.Tensor &lt; s64 0 &gt; iex&gt; Nx . iota ( { 5 } ) # Nx.Tensor &lt; s64 [ 5 ] [ 0 , 1 , 2 , 3 , 4 ] &gt; iex&gt; Nx . iota ( { 3 , 2 , 3 } , names : [ :batch , :height , :width ] ) # Nx.Tensor &lt; s64 [ batch : 3 ] [ height : 2 ] [ width : 3 ] [ [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] , [ [ 6 , 7 , 8 ] , [ 9 , 10 , 11 ] ] , [ [ 12 , 13 , 14 ] , [ 15 , 16 , 17 ] ] ] &gt; iex&gt; Nx . iota ( { 3 , 3 } , axis : 1 , names : [ :batch , nil ] ) # Nx.Tensor &lt; s64 [ batch : 3 ] [ 3 ] [ [ 0 , 1 , 2 ] , [ 0 , 1 , 2 ] , [ 0 , 1 , 2 ] ] &gt; iex&gt; Nx . iota ( { 3 , 3 } , axis : - 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 1 , 2 ] , [ 0 , 1 , 2 ] , [ 0 , 1 , 2 ] ] &gt; iex&gt; Nx . iota ( { 3 , 4 , 3 } , axis : 0 , type : :f64 ) # Nx.Tensor &lt; f64 [ 3 ] [ 4 ] [ 3 ] [ [ [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] , [ [ 1.0 , 1.0 , 1.0 ] , [ 1.0 , 1.0 , 1.0 ] , [ 1.0 , 1.0 , 1.0 ] , [ 1.0 , 1.0 , 1.0 ] ] , [ [ 2.0 , 2.0 , 2.0 ] , [ 2.0 , 2.0 , 2.0 ] , [ 2.0 , 2.0 , 2.0 ] , [ 2.0 , 2.0 , 2.0 ] ] ] &gt; iex&gt; Nx . iota ( { 1 , 3 , 2 } , axis : 2 ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 0 , 1 ] , [ 0 , 1 ] , [ 0 , 1 ] ] ] &gt; Options :type - the type of the tensor :axis - an axis to repeat the iota over :names - the names of the tensor dimensions :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#iota/2","title":"Nx.iota/2","type":"function"},{"doc":"Determines if each element in tensor is Inf or -Inf . For complex tensors, if either of the components is infinity, the entry is deemed infinity as well. Examples iex&gt; Nx . is_infinity ( Nx . tensor ( [ :infinity , :nan , :neg_infinity , 1 , 0 ] ) ) # Nx.Tensor &lt; u8 [ 5 ] [ 1 , 0 , 1 , 0 , 0 ] &gt; iex&gt; Nx . is_infinity ( Nx . tensor ( [ :infinity , 1 , Complex . new ( 0 , :infinity ) , :neg_infinity ] ) ) # Nx.Tensor &lt; u8 [ 4 ] [ 1 , 0 , 1 , 1 ] &gt; iex&gt; Nx . is_infinity ( Nx . tensor ( [ 1 , 0 ] ) ) # Nx.Tensor &lt; u8 [ 2 ] [ 0 , 0 ] &gt;","ref":"Nx.html#is_infinity/1","title":"Nx.is_infinity/1","type":"function"},{"doc":"Determines if each element in tensor is a NaN . For complex tensors, if either of the components is NaN , the entry is deemed NaN as well. Examples iex&gt; Nx . is_nan ( Nx . tensor ( [ :nan , 1 , 0 ] ) ) # Nx.Tensor &lt; u8 [ 3 ] [ 1 , 0 , 0 ] &gt; iex&gt; Nx . is_nan ( Nx . tensor ( [ :nan , :infinity , Complex . new ( 0 , :nan ) ] ) ) # Nx.Tensor &lt; u8 [ 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; Nx . is_nan ( Nx . tensor ( [ 1 , 0 ] ) ) # Nx.Tensor &lt; u8 [ 2 ] [ 0 , 0 ] &gt;","ref":"Nx.html#is_nan/1","title":"Nx.is_nan/1","type":"function"},{"doc":"Checks whether the value is a valid numerical value. Returns true if the value is a number , a Complex number or an Nx.Tensor . See also: t/0","ref":"Nx.html#is_tensor/1","title":"Nx.is_tensor/1","type":"macro"},{"doc":"Element-wise left shift of two tensors. Only integer tensors are supported. If a float or complex tensor is given, an error is raised. If the right side is negative, it will raise, but it may trigger undefined behaviour on some compilers. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If the number of shifts are negative, Nx's default backend will raise, but it may trigger undefined behaviour in other backends. If you're using Nx.Defn.defn/2 , you can use the &lt;&lt;&lt; operator in place of this function: left &lt;&lt;&lt; right . Examples Left shift between scalars iex&gt; Nx . left_shift ( 1 , 0 ) # Nx.Tensor &lt; s64 1 &gt; Left shift between tensors and scalars iex&gt; Nx . left_shift ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 4 , 8 , 12 ] &gt; Left shift between tensors iex&gt; left = Nx . tensor ( [ 1 , 1 , - 1 , - 1 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 , 4 ] , names : [ :data ] ) iex&gt; Nx . left_shift ( left , right ) # Nx.Tensor &lt; s64 [ data : 4 ] [ 2 , 4 , - 8 , - 16 ] &gt; Error cases iex&gt; Nx . left_shift ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , 1.0 ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#left_shift/2","title":"Nx.left_shift/2","type":"function"},{"doc":"Element-wise less than comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the &lt; operator in place of this function: left &lt; right . Examples Comparison of scalars iex&gt; Nx . less ( 1 , 2 ) # Nx.Tensor &lt; u8 1 &gt; Comparison of tensors and scalars iex&gt; Nx . less ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 1 , 1 ] &gt; Comparison of tensors iex&gt; Nx . less ( Nx . tensor ( [ 1 , 2 , 1 ] ) , Nx . tensor ( [ 1 , 2 , 2 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 0 , 1 ] &gt; iex&gt; Nx . less ( Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 2.0 , 1.0 ] ] , names : [ :x , :y ] ) , Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 3 ] [ [ 0 , 0 , 0 ] , [ 0 , 0 , 1 ] ] &gt;","ref":"Nx.html#less/2","title":"Nx.less/2","type":"function"},{"doc":"Element-wise less than or equal comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the &lt;= operator in place of this function: left &lt;= right . Examples Comparison of scalars iex&gt; Nx . less_equal ( 1 , 2 ) # Nx.Tensor &lt; u8 1 &gt; Comparison of tensors and scalars iex&gt; Nx . less_equal ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 1 , 1 ] &gt; Comparison of tensors iex&gt; left = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 2 ] ) iex&gt; Nx . less_equal ( left , right ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 1 , 0 ] &gt; iex&gt; left = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :y ] ) iex&gt; Nx . less_equal ( left , right ) # Nx.Tensor &lt; u8 [ 2 ] [ y : 3 ] [ [ 1 , 1 , 1 ] , [ 0 , 0 , 0 ] ] &gt;","ref":"Nx.html#less_equal/2","title":"Nx.less_equal/2","type":"function"},{"doc":"Calculates the natural log plus one of each element in the tensor. It is equivalent to: $$log1p(z) = log(z + 1)$$ Examples iex&gt; Nx . log1p ( 1 ) # Nx.Tensor &lt; f32 0.6931471824645996 &gt; iex&gt; Nx . log1p ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.6931471824645996 , 1.0986123085021973 , 1.3862943649291992 ] &gt;","ref":"Nx.html#log1p/1","title":"Nx.log1p/1","type":"function"},{"doc":"Calculates the natural log of each element in the tensor. It is equivalent to: $$log(z) = ln(z),\\quad \\text{if z} \\in \\Reals$$ $$log(z) = ln(r) + i\\theta,\\quad\\text{if }z = re^{i\\theta} \\in \\Complex$$ Examples iex&gt; Nx . log ( 1 ) # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; Nx . log ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.0 , 0.6931471824645996 , 1.0986123085021973 ] &gt;","ref":"Nx.html#log/1","title":"Nx.log/1","type":"function"},{"doc":"Element-wise logical and of two tensors. Zero is considered false, any other number is considered true. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the and operator in place of this function: left and right . Examples iex&gt; Nx . logical_and ( 1 , Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_and ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 1 , 0 , 1 ] , [ 0 , 0 , 0 ] , [ 1 , 0 , 1 ] ] &gt; iex&gt; left = Nx . tensor ( [ - 1.0 , 0.0 , 1.0 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_and ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 1 , 0 , 1 ] , [ 0 , 0 , 0 ] , [ 1 , 0 , 1 ] ] &gt;","ref":"Nx.html#logical_and/2","title":"Nx.logical_and/2","type":"function"},{"doc":"Element-wise logical not a tensor. Zero is considered false, any other number is considered true. If you're using Nx.Defn.defn/2 , you can use the not operator in place of this function: not tensor . Examples iex&gt; Nx . logical_not ( Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 1 , 0 ] &gt; iex&gt; Nx . logical_not ( Nx . tensor ( [ - 1.0 , 0.0 , 1.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 1 , 0 ] &gt;","ref":"Nx.html#logical_not/1","title":"Nx.logical_not/1","type":"function"},{"doc":"Element-wise logical or of two tensors. Zero is considered false, any other number is considered true. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the or operator in place of this function: left or right . Examples iex&gt; Nx . logical_or ( 0 , Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_or ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 0 , 1 ] , [ 1 , 1 , 1 ] ] &gt; iex&gt; left = Nx . tensor ( [ - 1.0 , 0.0 , 1.0 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_or ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 1 , 1 , 1 ] , [ 1 , 0 , 1 ] , [ 1 , 1 , 1 ] ] &gt;","ref":"Nx.html#logical_or/2","title":"Nx.logical_or/2","type":"function"},{"doc":"Element-wise logical xor of two tensors. Zero is considered false, any other number is considered true. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. Examples iex&gt; Nx . logical_xor ( 0 , Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_xor ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 0 , 1 , 0 ] , [ 1 , 0 , 1 ] , [ 0 , 1 , 0 ] ] &gt; iex&gt; left = Nx . tensor ( [ - 1.0 , 0.0 , 1.0 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ [ - 1 ] , [ 0 ] , [ 1 ] ] ) iex&gt; Nx . logical_xor ( left , right ) # Nx.Tensor &lt; u8 [ 3 ] [ data : 3 ] [ [ 0 , 1 , 0 ] , [ 1 , 0 , 1 ] , [ 0 , 1 , 0 ] ] &gt;","ref":"Nx.html#logical_xor/2","title":"Nx.logical_xor/2","type":"function"},{"doc":"Creates a diagonal tensor from a 1D tensor. Converse of take_diagonal/2 . The returned tensor will be a square matrix of dimensions equal to the size of the tensor. If an offset is given, the absolute value of the offset is added to the matrix dimensions sizes. Examples Given a 1D tensor: iex&gt; Nx . make_diagonal ( Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 1 , 0 , 0 , 0 ] , [ 0 , 2 , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] , [ 0 , 0 , 0 , 4 ] ] &gt; Given a 1D tensor with an offset: iex&gt; Nx . make_diagonal ( Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 0 , 1 , 0 , 0 ] , [ 0 , 0 , 2 , 0 ] , [ 0 , 0 , 0 , 3 ] , [ 0 , 0 , 0 , 0 ] ] &gt; iex&gt; Nx . make_diagonal ( Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : - 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 0 , 0 , 0 , 0 ] , [ 1 , 0 , 0 , 0 ] , [ 0 , 2 , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] ] &gt; You can also have offsets with an abs greater than the tensor length: iex&gt; Nx . make_diagonal ( Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : - 4 ) # Nx.Tensor &lt; s64 [ 7 ] [ 7 ] [ [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 1 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 2 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 3 , 0 , 0 , 0 , 0 ] ] &gt; iex&gt; Nx . make_diagonal ( Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : 4 ) # Nx.Tensor &lt; s64 [ 7 ] [ 7 ] [ [ 0 , 0 , 0 , 0 , 1 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 2 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 3 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 0 , 0 ] ] &gt; Options :offset - offset used for making the diagonal. Use offset &gt; 0 for diagonals above the main diagonal, and offset &lt; 0 for diagonals below the main diagonal. Defaults to 0. Error cases iex&gt; Nx . make_diagonal ( Nx . tensor ( [ [ 0 , 0 ] , [ 0 , 1 ] ] ) ) ** (ArgumentError) make_diagonal/2 expects tensor of rank 1, got tensor of rank: 2","ref":"Nx.html#make_diagonal/2","title":"Nx.make_diagonal/2","type":"function"},{"doc":"Maps the given scalar function over the entire tensor. The type of the returned tensor will be of the same type as the input tensor, unless the :type option is given. Therefore, you may need to explicitly cast the tensor to avoid errors. For example, if you have an integer tensor and you convert it to a float, as below, it will fail: tensor = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , Nx . map ( tensor , fn x -&gt; Nx . multiply ( x , 1.0 ) end ) You need to explicitly pass the output type in such cases: iex&gt; tensor = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . map ( tensor , [ type : :f32 ] , fn x -&gt; Nx . multiply ( x , 1.0 ) end ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] &gt; Limitations Given this function relies on anonymous functions, it may not be available or efficient on all Nx backends. Therefore, you should avoid using map/2 whenever possible and use other functions in the Nx module to achieve the desired result. Examples iex&gt; Nx . map ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , fn x -&gt; Nx . add ( x , 1 ) end ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 2 , 3 , 4 ] , [ 5 , 6 , 7 ] ] &gt; iex&gt; Nx . map ( Nx . tensor ( 1 ) , fn x -&gt; Nx . add ( x , 1 ) end ) # Nx.Tensor &lt; s64 2 &gt; iex&gt; Nx . map ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , [ type : :f64 ] , fn x -&gt; Nx . add ( x , 1 ) end ) # Nx.Tensor &lt; f64 [ 2 ] [ 3 ] [ [ 2.0 , 3.0 , 4.0 ] , [ 5.0 , 6.0 , 7.0 ] ] &gt;","ref":"Nx.html#map/3","title":"Nx.map/3","type":"function"},{"doc":"Element-wise maximum of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the max/2 function in place of this function: max(left, right) . Examples Max between scalars iex&gt; Nx . max ( 1 , 2 ) # Nx.Tensor &lt; s64 2 &gt; Max between tensors and scalars iex&gt; Nx . max ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . max ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; Max between tensors iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] ) iex&gt; Nx . max ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 10 , 20 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 ) iex&gt; Nx . max ( left , right ) # Nx.Tensor &lt; s8 [ x : 2 ] [ 2 ] [ [ 10 , 20 ] , [ 10 , 20 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :f32 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :f32 , names : [ nil , :y ] ) iex&gt; Nx . max ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 10.0 , 20.0 ] , [ 10.0 , 20.0 ] ] &gt;","ref":"Nx.html#max/2","title":"Nx.max/2","type":"function"},{"doc":"Returns the mean for the tensor. If the :axes option is given, it aggregates over that dimension, effectively removing it. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the averaged axes to size 1. Examples iex&gt; Nx . mean ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; f32 42.0 &gt; iex&gt; Nx . mean ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; f32 2.0 &gt; Aggregating over an axis iex&gt; Nx . mean ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; f32 2.0 &gt; iex&gt; Nx . mean ( Nx . tensor ( [ 1 , 2 , 3 ] , type : :u8 , names : [ :x ] ) , axes : [ :x ] ) # Nx.Tensor &lt; f32 2.0 &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . mean ( t , axes : [ :x ] ) # Nx.Tensor &lt; f32 [ y : 2 ] [ z : 3 ] [ [ 4.0 , 5.0 , 6.0 ] , [ 7.0 , 8.0 , 9.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . mean ( t , axes : [ :x , :z ] ) # Nx.Tensor &lt; f32 [ y : 2 ] [ 5.0 , 8.0 ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . mean ( t , axes : [ - 1 ] ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 2.0 , 5.0 ] , [ 8.0 , 11.0 ] ] &gt; Keeping axes iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . mean ( t , axes : [ - 1 ] , keep_axes : true ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ z : 1 ] [ [ [ 2.0 ] , [ 5.0 ] ] , [ [ 8.0 ] , [ 11.0 ] ] ] &gt;","ref":"Nx.html#mean/2","title":"Nx.mean/2","type":"function"},{"doc":"Returns the median for the tensor. If the :axis option is given, it aggregates over that dimension, effectively removing it. axis: 0 implies aggregating over the highest order dimension and so forth. If the axis is negative, then the axis will be counted from the back. For example, axis: -1 will always aggregate over the last dimension. You may optionally set :keep_axis to true, which will retain the rank of the input tensor by setting the reduced axis to size 1. Examples iex&gt; Nx . median ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . median ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; s64 2 &gt; iex&gt; Nx . median ( Nx . tensor ( [ 1 , 2 ] ) ) # Nx.Tensor &lt; f32 1.5 &gt; Aggregating over an axis iex&gt; Nx . median ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) , axis : 0 ) # Nx.Tensor &lt; f32 [ y : 3 ] [ 2.5 , 3.5 , 4.5 ] &gt; iex&gt; Nx . median ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 2 , 5 ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; weights = Nx . tensor ( [ [ [ 0 , 1 , 2 ] , [ 1 , 1 , 0 ] ] , [ [ - 1 , 1 , - 1 ] , [ 1 , 1 , - 1 ] ] ] ) iex&gt; Nx . weighted_mean ( t , weights , axis : :x ) # Nx.Tensor &lt; f32 [ y : 2 ] [ z : 3 ] [ [ 7.0 , 5.0 , - 3.0 ] , [ 7.0 , 8.0 , 12.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 2 ] , [ 3 , 4 , 2 ] ] , [ [ 4 , 5 , 2 ] , [ 7 , 9 , 2 ] ] ] ) iex&gt; Nx . median ( t , axis : - 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 2 , 3 ] , [ 4 , 7 ] ] &gt; Keeping axis iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 2 ] , [ 3 , 4 , 2 ] ] , [ [ 4 , 5 , 2 ] , [ 7 , 9 , 2 ] ] ] ) iex&gt; Nx . median ( t , axis : - 1 , keep_axis : true ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 1 ] [ [ [ 2 ] , [ 3 ] ] , [ [ 4 ] , [ 7 ] ] ] &gt;","ref":"Nx.html#median/2","title":"Nx.median/2","type":"function"},{"doc":"Element-wise minimum of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the min/2 function in place of this function: min(left, right) . Examples Min between scalars iex&gt; Nx . min ( 1 , 2 ) # Nx.Tensor &lt; s64 1 &gt; Min between tensors and scalars iex&gt; Nx . min ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 1 , 1 ] &gt; iex&gt; Nx . min ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 1.0 , 1.0 , 1.0 ] &gt; Min between tensors iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] ) iex&gt; Nx . min ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 2 ] [ [ 1 , 1 ] , [ 2 , 2 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 ) iex&gt; Nx . min ( left , right ) # Nx.Tensor &lt; s8 [ x : 2 ] [ y : 2 ] [ [ 1 , 1 ] , [ 2 , 2 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :f32 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :f32 , names : [ nil , :y ] ) iex&gt; Nx . min ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 1.0 , 1.0 ] , [ 2.0 , 2.0 ] ] &gt;","ref":"Nx.html#min/2","title":"Nx.min/2","type":"function"},{"doc":"Returns the mode of a tensor (the value that appears most often). If the :axis option is given, it aggregates over that dimension, effectively removing it. axis: 0 implies aggregating over the highest order dimension and so forth. If the axis is negative, then the axis will be counted from the back. For example, axis: -1 will always aggregate over the last dimension. You may optionally set :keep_axis to true, which will retain the rank of the input tensor by setting the reduced axis to size 1. Examples iex&gt; Nx . mode ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ 1 ] ] ) ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . mode ( Nx . tensor ( [ 1 , 2 , 2 , 3 , 5 ] ) ) # Nx.Tensor &lt; s64 2 &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ 1 , 2 , 2 , 3 , 5 ] , [ 1 , 1 , 76 , 8 , 1 ] ] ) ) # Nx.Tensor &lt; s64 1 &gt; Aggregating over an axis iex&gt; Nx . mode ( Nx . tensor ( [ [ 1 , 2 , 2 , 3 , 5 ] , [ 1 , 1 , 76 , 8 , 1 ] ] ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 5 ] [ 1 , 1 , 2 , 3 , 1 ] &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ 1 , 2 , 2 , 3 , 5 ] , [ 1 , 1 , 76 , 8 , 1 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 , 1 ] &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ [ [ 1 ] ] ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] [ 1 ] [ [ [ 1 ] ] ] &gt; Keeping axis iex&gt; Nx . mode ( Nx . tensor ( [ [ 1 , 2 , 2 , 3 , 5 ] , [ 1 , 1 , 76 , 8 , 1 ] ] ) , axis : 1 , keep_axis : true ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ [ 2 ] , [ 1 ] ] &gt; iex&gt; Nx . mode ( Nx . tensor ( 1 ) , keep_axis : true ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ [ 1 ] ] ] ) , keep_axis : true ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] [ 1 ] [ [ [ 1 ] ] ] &gt; iex&gt; Nx . mode ( Nx . tensor ( [ [ [ [ 1 ] ] ] ] ) , axis : 1 , keep_axis : true ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] [ 1 ] [ 1 ] [ [ [ [ 1 ] ] ] ] &gt;","ref":"Nx.html#mode/2","title":"Nx.mode/2","type":"function"},{"doc":"Element-wise multiplication of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the * operator operator in place of this function as left * right . Examples Multiplying scalars iex&gt; Nx . multiply ( 1 , 2 ) # Nx.Tensor &lt; s64 2 &gt; Multiplying tensors and scalars iex&gt; Nx . multiply ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . multiply ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; Multiplying tensors iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ :x , :y ] ) iex&gt; Nx . multiply ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 20 , 40 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 , names : [ nil , :y ] ) iex&gt; Nx . multiply ( left , right ) # Nx.Tensor &lt; s8 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 20 , 40 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :f32 , names : [ nil , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :f32 , names : [ :x , nil ] ) iex&gt; Nx . multiply ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 10.0 , 20.0 ] , [ 20.0 , 40.0 ] ] &gt;","ref":"Nx.html#multiply/2","title":"Nx.multiply/2","type":"function"},{"doc":"Returns all of the names in a tensor. Examples iex&gt; Nx . names ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :batch , :data ] ) ) [ :batch , :data ] iex&gt; Nx . names ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) [ nil ] iex&gt; Nx . names ( 5 ) [ ]","ref":"Nx.html#names/1","title":"Nx.names/1","type":"function"},{"doc":"Negates each element in the tensor. If you're using Nx.Defn.defn/2 , you can use the - unary operator in place of this function: -tensor . Examples iex&gt; Nx . negate ( 1 ) # Nx.Tensor &lt; s64 - 1 &gt; iex&gt; Nx . negate ( Nx . tensor ( [ - 1 , 0 , 1 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 0 , - 1 ] &gt; iex&gt; Nx . negate ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , type : :f32 ) ) # Nx.Tensor &lt; f32 [ 3 ] [ - 1.0 , - 2.0 , - 3.0 ] &gt; If an unsigned tensor is given, it works as bitwise_not : iex&gt; Nx . negate ( Nx . tensor ( [ 0 , 1 , 2 ] , type : :u8 , names : [ :x ] ) ) # Nx.Tensor &lt; u8 [ x : 3 ] [ 0 , 255 , 254 ] &gt;","ref":"Nx.html#negate/1","title":"Nx.negate/1","type":"function"},{"doc":"Adds a new axis of size 1 with optional name . Examples iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . new_axis ( t , 0 , :new ) # Nx.Tensor &lt; s64 [ new : 1 ] [ 2 ] [ 3 ] [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] &gt; iex&gt; Nx . new_axis ( t , 1 , :new ) # Nx.Tensor &lt; s64 [ 2 ] [ new : 1 ] [ 3 ] [ [ [ 1 , 2 , 3 ] ] , [ [ 4 , 5 , 6 ] ] ] &gt; iex&gt; Nx . new_axis ( t , 2 , :new ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ new : 1 ] [ [ [ 1 ] , [ 2 ] , [ 3 ] ] , [ [ 4 ] , [ 5 ] , [ 6 ] ] ] &gt; Axis can also be negative, which will start from the back: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . new_axis ( t , - 1 , :new ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ new : 1 ] [ [ [ 1 ] , [ 2 ] , [ 3 ] ] , [ [ 4 ] , [ 5 ] , [ 6 ] ] ] &gt;","ref":"Nx.html#new_axis/3","title":"Nx.new_axis/3","type":"function"},{"doc":"Element-wise not-equal comparison of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the != operator in place of this function: left != right . Examples Comparison of scalars iex&gt; Nx . not_equal ( 1 , 2 ) # Nx.Tensor &lt; u8 1 &gt; Comparison of tensor and scalar iex&gt; Nx . not_equal ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , Nx . tensor ( 1 ) ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 1 , 1 ] &gt; Comparison of tensors iex&gt; left = Nx . tensor ( [ 1 , 1 , 2 ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) iex&gt; Nx . not_equal ( left , right ) # Nx.Tensor &lt; u8 [ data : 3 ] [ 0 , 1 , 1 ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 , 4 , 2 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 1 , 3 , 2 ] , [ 4 , 2 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx . not_equal ( left , right ) # Nx.Tensor &lt; u8 [ x : 2 ] [ y : 3 ] [ [ 0 , 1 , 0 ] , [ 0 , 1 , 1 ] ] &gt;","ref":"Nx.html#not_equal/2","title":"Nx.not_equal/2","type":"function"},{"doc":"Computes the outer product of two tensors. The output is always a two-dimensional tensor. Examples iex&gt; Nx . outer ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , 100 ) # Nx.Tensor &lt; s64 [ x : 3 ] [ 1 ] [ [ 100 ] , [ 200 ] , [ 300 ] ] &gt; iex&gt; Nx . outer ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , Nx . tensor ( [ 10 , 20 ] , names : [ :y ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ y : 2 ] [ [ 10 , 20 ] , [ 20 , 40 ] , [ 30 , 60 ] ] &gt; iex&gt; Nx . outer ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) , Nx . tensor ( [ 10 , 20 , 30 ] , names : [ :z ] ) ) # Nx.Tensor &lt; s64 [ x : 4 ] [ z : 3 ] [ [ 10 , 20 , 30 ] , [ 20 , 40 , 60 ] , [ 30 , 60 , 90 ] , [ 40 , 80 , 120 ] ] &gt;","ref":"Nx.html#outer/2","title":"Nx.outer/2","type":"function"},{"doc":"Pads a tensor with a given value. You must specify a padding configuration. A padding configuration is a list of tuples consisting of {pad_width_low, pad_width_high, pad_width_interior} for each dimension in the input tensor. The padding configuration must be of the same length as the tensor shape. Padding widths can be negative. If they are negative, the tensor is clipped on either end according to the padding width. Interior padding widths cannot be negative. Examples iex&gt; Nx . pad ( Nx . tensor ( 1 ) , 0 , [ ] ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . pad ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 0 , [ { 1 , 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ data : 5 ] [ 0 , 1 , 2 , 3 , 0 ] &gt; iex&gt; Nx . pad ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , 0 , [ { 0 , 0 , 1 } , { 0 , 0 , 1 } ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 5 ] [ [ 1 , 0 , 2 , 0 , 3 ] , [ 0 , 0 , 0 , 0 , 0 ] , [ 4 , 0 , 5 , 0 , 6 ] ] &gt; iex&gt; Nx . pad ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , 0 , [ { 1 , 1 , 0 } , { 1 , 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 5 ] [ [ 0 , 0 , 0 , 0 , 0 ] , [ 0 , 1 , 2 , 3 , 0 ] , [ 0 , 4 , 5 , 6 , 0 ] , [ 0 , 0 , 0 , 0 , 0 ] ] &gt; iex&gt; tensor = Nx . tensor ( [ [ [ 1 , 2 ] , [ 3 , 4 ] ] , [ [ 5 , 6 ] , [ 7 , 8 ] ] ] ) iex&gt; Nx . pad ( tensor , 0 , [ { 0 , 2 , 0 } , { 1 , 1 , 0 } , { 1 , 0 , 0 } ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ 3 ] [ [ [ 0 , 0 , 0 ] , [ 0 , 1 , 2 ] , [ 0 , 3 , 4 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 0 , 5 , 6 ] , [ 0 , 7 , 8 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] ] &gt; iex&gt; tensor = Nx . tensor ( [ [ [ 1 , 2 ] , [ 3 , 4 ] ] , [ [ 5 , 6 ] , [ 7 , 8 ] ] ] ) iex&gt; Nx . pad ( tensor , 0 , [ { 1 , 0 , 0 } , { 1 , 1 , 0 } , { 0 , 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 4 ] [ 3 ] [ [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 1 , 2 , 0 ] , [ 3 , 4 , 0 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 5 , 6 , 0 ] , [ 7 , 8 , 0 ] , [ 0 , 0 , 0 ] ] ] &gt; iex&gt; tensor = Nx . tensor ( [ [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] , [ [ 5.0 , 6.0 ] , [ 7.0 , 8.0 ] ] ] ) iex&gt; Nx . pad ( tensor , 0.0 , [ { 1 , 2 , 0 } , { 1 , 0 , 0 } , { 0 , 1 , 0 } ] ) # Nx.Tensor &lt; f32 [ 5 ] [ 3 ] [ 3 ] [ [ [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] , [ [ 0.0 , 0.0 , 0.0 ] , [ 1.0 , 2.0 , 0.0 ] , [ 3.0 , 4.0 , 0.0 ] ] , [ [ 0.0 , 0.0 , 0.0 ] , [ 5.0 , 6.0 , 0.0 ] , [ 7.0 , 8.0 , 0.0 ] ] , [ [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] , [ [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] ] &gt; iex&gt; Nx . pad ( Nx . tensor ( [ 0 , 1 , 2 , 3 , 0 ] ) , 0 , [ { - 1 , - 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; tensor = Nx . tensor ( [ ...&gt; [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] , ...&gt; [ [ 0 , 0 , 0 ] , [ 1 , 2 , 0 ] , [ 3 , 4 , 0 ] , [ 0 , 0 , 0 ] ] , ...&gt; [ [ 0 , 0 , 0 ] , [ 5 , 6 , 0 ] , [ 7 , 8 , 0 ] , [ 0 , 0 , 0 ] ] ...&gt; ] ) iex&gt; Nx . pad ( tensor , 0 , [ { - 1 , 0 , 0 } , { - 1 , - 1 , 0 } , { 0 , - 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1 , 2 ] , [ 3 , 4 ] ] , [ [ 5 , 6 ] , [ 7 , 8 ] ] ] &gt; iex&gt; tensor = Nx . tensor ( [ [ 0 , 1 , 2 , 3 ] , [ 0 , 4 , 5 , 6 ] ] ) iex&gt; Nx . pad ( tensor , 0 , [ { 0 , 0 , 0 } , { - 1 , 1 , 0 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 4 ] [ [ 1 , 2 , 3 , 0 ] , [ 4 , 5 , 6 , 0 ] ] &gt; iex&gt; tensor = Nx . tensor ( [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] , type : :f32 ) iex&gt; Nx . pad ( tensor , 0 , [ { - 1 , 2 , 0 } , { 1 , - 1 , 0 } ] ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 0.0 , 3.0 , 4.0 ] , [ 0.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] &gt;","ref":"Nx.html#pad/3","title":"Nx.pad/3","type":"function"},{"doc":"Calculates the complex phase angle of each element in the tensor. $$phase(z) = atan2(b, a), z = a + bi \\in \\Complex$$ Examples iex&gt; Nx . phase ( Complex . new ( 1 , 2 ) ) # Nx.Tensor &lt; f32 1.1071487665176392 &gt; iex&gt; Nx . phase ( 1 ) # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; import Nx , only : [ sigil_V : 2 ] iex&gt; Nx . phase ( ~V[1+2i -2+1i] ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.1071487665176392 , 2.677945137023926 ] &gt;","ref":"Nx.html#phase/1","title":"Nx.phase/1","type":"function"},{"doc":"Computes the bitwise population count of each element in the tensor. Examples iex&gt; Nx . population_count ( 1 ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . population_count ( - 128 ) # Nx.Tensor &lt; s64 57 &gt; iex&gt; Nx . population_count ( Nx . tensor ( [ 0 , 1 , 254 , 255 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 4 ] [ 0 , 1 , 7 , 8 ] &gt; iex&gt; Nx . population_count ( Nx . tensor ( [ 0 , 1 , 126 , 127 , - 1 , - 127 , - 128 ] , type : :s8 , names : [ :x ] ) ) # Nx.Tensor &lt; s8 [ x : 7 ] [ 0 , 1 , 6 , 7 , 8 , 2 , 1 ] &gt; Error cases iex&gt; Nx . population_count ( Nx . tensor ( [ 0.0 , 1.0 ] ) ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#population_count/1","title":"Nx.population_count/1","type":"function"},{"doc":"Element-wise power of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If both tensors are integers and the exponent is negative, it will raise, but it may trigger undefined behaviour on some compilers. Examples Power of scalars iex&gt; Nx . power ( 2 , 4 ) # Nx.Tensor &lt; s64 16 &gt; Power of tensors and scalars iex&gt; Nx . power ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 4 , 9 ] &gt; iex&gt; Nx . power ( 2 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 2.0 , 4.0 , 8.0 ] &gt; Power of tensors iex&gt; Nx . power ( Nx . tensor ( [ [ 2 ] , [ 3 ] ] , names : [ :x , nil ] ) , Nx . tensor ( [ [ 4 , 5 ] ] , names : [ nil , :y ] ) ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 16 , 32 ] , [ 81 , 243 ] ] &gt;","ref":"Nx.html#power/2","title":"Nx.power/2","type":"function"},{"doc":"Returns the product for the tensor. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the multiplied axes to size 1. Examples iex&gt; Nx . product ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . product ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 6 &gt; iex&gt; Nx . product ( Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] , names : [ :x , :y ] ) ) # Nx.Tensor &lt; f32 24.0 &gt; Giving a tensor with low precision casts it to a higher precision to make sure the sum does not overflow: iex&gt; Nx . product ( Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , type : :u8 , names : [ :x , :y ] ) ) # Nx.Tensor &lt; u64 240000 &gt; iex&gt; Nx . product ( Nx . tensor ( [ [ 10 , 20 ] , [ 30 , 40 ] ] , type : :s8 , names : [ :x , :y ] ) ) # Nx.Tensor &lt; s64 240000 &gt; Aggregating over an axis iex&gt; Nx . product ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; s64 6 &gt; Same tensor over different axes combinations: iex&gt; t = Nx . tensor ( ...&gt; [ ...&gt; [ ...&gt; [ 1 , 2 , 3 ] , ...&gt; [ 4 , 5 , 6 ] ...&gt; ] , ...&gt; [ ...&gt; [ 7 , 8 , 9 ] , ...&gt; [ 10 , 11 , 12 ] ...&gt; ] ...&gt; ] , ...&gt; names : [ :x , :y , :z ] ...&gt; ) iex&gt; Nx . product ( t , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 7 , 16 , 27 ] , [ 40 , 55 , 72 ] ] &gt; iex&gt; Nx . product ( t , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 4 , 10 , 18 ] , [ 70 , 88 , 108 ] ] &gt; iex&gt; Nx . product ( t , axes : [ :x , :z ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ 3024 , 158400 ] &gt; iex&gt; Nx . product ( t , axes : [ :z ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 6 , 120 ] , [ 504 , 1320 ] ] &gt; iex&gt; Nx . product ( t , axes : [ - 3 ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 7 , 16 , 27 ] , [ 40 , 55 , 72 ] ] &gt; Keeping axes iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . product ( t , axes : [ :z ] , keep_axes : true ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ z : 1 ] [ [ [ 6 ] , [ 120 ] ] , [ [ 504 ] , [ 1320 ] ] ] &gt; Errors iex&gt; Nx . product ( Nx . tensor ( [ [ 1 , 2 ] ] ) , axes : [ 2 ] ) ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#product/2","title":"Nx.product/2","type":"function"},{"doc":"Puts the individual values from a 1D diagonal into the diagonal indices of the given 2D tensor. See also: take_diagonal/2 , make_diagonal/2 . Examples Given a 2D tensor and a 1D diagonal: iex&gt; t = Nx . broadcast ( 0 , { 4 , 4 } ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] ] &gt; iex&gt; Nx . put_diagonal ( t , Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 1 , 0 , 0 , 0 ] , [ 0 , 2 , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] , [ 0 , 0 , 0 , 4 ] ] &gt; iex&gt; t = Nx . broadcast ( 0 , { 4 , 3 } ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 ] [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] &gt; iex&gt; Nx . put_diagonal ( t , Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 ] [ [ 1 , 0 , 0 ] , [ 0 , 2 , 0 ] , [ 0 , 0 , 3 ] , [ 0 , 0 , 0 ] ] &gt; Given a 2D tensor and a 1D diagonal with a positive offset: iex&gt; Nx . put_diagonal ( Nx . broadcast ( 0 , { 4 , 4 } ) , Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 0 , 1 , 0 , 0 ] , [ 0 , 0 , 2 , 0 ] , [ 0 , 0 , 0 , 3 ] , [ 0 , 0 , 0 , 0 ] ] &gt; iex&gt; Nx . put_diagonal ( Nx . broadcast ( 0 , { 4 , 3 } ) , Nx . tensor ( [ 1 , 2 ] ) , offset : 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 ] [ [ 0 , 1 , 0 ] , [ 0 , 0 , 2 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] &gt; Given a 2D tensor and a 1D diagonal with a negative offset: iex&gt; Nx . put_diagonal ( Nx . broadcast ( 0 , { 4 , 4 } ) , Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : - 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 0 , 0 , 0 , 0 ] , [ 1 , 0 , 0 , 0 ] , [ 0 , 2 , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] ] &gt; iex&gt; Nx . put_diagonal ( Nx . broadcast ( 0 , { 4 , 3 } ) , Nx . tensor ( [ 1 , 2 , 3 ] ) , offset : - 1 ) # Nx.Tensor &lt; s64 [ 4 ] [ 3 ] [ [ 0 , 0 , 0 ] , [ 1 , 0 , 0 ] , [ 0 , 2 , 0 ] , [ 0 , 0 , 3 ] ] &gt; Options :offset - offset used for putting the diagonal. Use offset &gt; 0 for diagonals above the main diagonal, and offset &lt; 0 for diagonals below the main diagonal. Defaults to 0. Error cases Given an invalid tensor: iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 , 3 } ) , Nx . iota ( { 3 } ) ) ** (ArgumentError) put_diagonal/3 expects tensor of rank 2, got tensor of rank: 3 Given invalid diagonals: iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 } ) , Nx . iota ( { 3 , 3 } ) ) ** (ArgumentError) put_diagonal/3 expects diagonal of rank 1, got tensor of rank: 2 iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 } ) , Nx . iota ( { 2 } ) ) ** (ArgumentError) expected diagonal tensor of length: 3, got diagonal tensor of length: 2 iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 } ) , Nx . iota ( { 3 } ) , offset : 1 ) ** (ArgumentError) expected diagonal tensor of length: 2, got diagonal tensor of length: 3 Given invalid offsets: iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 } ) , Nx . iota ( { 3 } ) , offset : 4 ) ** (ArgumentError) offset must be less than length of axis 1 when positive, got: 4 iex&gt; Nx . put_diagonal ( Nx . iota ( { 3 , 3 } ) , Nx . iota ( { 3 } ) , offset : - 3 ) ** (ArgumentError) absolute value of offset must be less than length of axis 0 when negative, got: -3","ref":"Nx.html#put_diagonal/3","title":"Nx.put_diagonal/3","type":"function"},{"doc":"Puts the given slice into the given tensor at the given start_indices . The given slice must be of the same rank as tensor. Each axis must be less than or equal to the size to the equivalent axis in the tensor. The number of elements in start_indices should match the rank of the tensor. See also: indexed_add/3 , put_slice/3 . Examples iex&gt; t = Nx . tensor ( [ 0 , 1 , 2 , 3 , 4 ] ) iex&gt; Nx . put_slice ( t , [ 2 ] , Nx . tensor ( [ 5 , 6 ] ) ) # Nx.Tensor &lt; s64 [ 5 ] [ 0 , 1 , 5 , 6 , 4 ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . put_slice ( t , [ 0 , 1 ] , Nx . tensor ( [ [ 7 , 8 ] , [ 9 , 10 ] ] ) ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 1 , 7 , 8 ] , [ 4 , 9 , 10 ] ] &gt; Similar to slice/3 , dynamic start indexes are also supported: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . put_slice ( t , [ Nx . tensor ( 0 ) , Nx . tensor ( 1 ) ] , Nx . tensor ( [ [ 10.0 , 11.0 ] ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ [ 1.0 , 10.0 , 11.0 ] , [ 4.0 , 5.0 , 6.0 ] ] &gt; Also similar to slice/3 , if start_index + slice_dimension &gt; dimension , the start index will be clipped in order to put the whole slice: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . put_slice ( t , [ 1 , 1 ] , Nx . tensor ( [ [ 7 , 8 ] , [ 9 , 10 ] ] ) ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 1 , 7 , 8 ] , [ 4 , 9 , 10 ] ] &gt;","ref":"Nx.html#put_slice/3","title":"Nx.put_slice/3","type":"function"},{"doc":"Element-wise integer division of two tensors. If a number is given, it is converted to a tensor. It always returns an integer tensor. Input tensors and numbers must be integer types. Division by zero raises, but it may trigger undefined behaviour on some compilers. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. Caveat for grad The grad operation is not supported for quotient/2 . Since integer division is, by definition, a closed operation for the set of integers and grad involves floating points, grad is undefined. If you need to support gradients, you might consider using floor division, but beware of precision errors caused by floating points: a |&gt; Nx . divide ( b ) |&gt; Nx . floor ( ) Examples Integer dividing scalars iex&gt; Nx . quotient ( 11 , 2 ) # Nx.Tensor &lt; s64 5 &gt; Integer dividing tensors and scalars iex&gt; Nx . quotient ( Nx . tensor ( [ 2 , 4 , 5 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 2 , 2 ] &gt; iex&gt; Nx . quotient ( 10 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 10 , 5 , 3 ] &gt; Dividing tensors iex&gt; left = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ nil , :y ] ) iex&gt; right = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , nil ] ) iex&gt; Nx . quotient ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 5 , 10 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 ) iex&gt; Nx . quotient ( left , right ) # Nx.Tensor &lt; s8 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 5 , 10 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 10 , 20 ] ] , type : :u8 , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :u32 ) iex&gt; Nx . quotient ( left , right ) # Nx.Tensor &lt; u32 [ x : 2 ] [ y : 2 ] [ [ 10 , 20 ] , [ 5 , 10 ] ] &gt;","ref":"Nx.html#quotient/2","title":"Nx.quotient/2","type":"function"},{"doc":"Shortcut for random_normal(shape, 0.0, 1.0, opts) .","ref":"Nx.html#random_normal/2","title":"Nx.random_normal/2","type":"function"},{"doc":"Returns a normally-distributed random tensor with the given shape. The distribution has mean of mu and standard deviation of sigma . Return type is one of {:bf, 16} , {:f, 32} or {:f, 64} . If a tensor or a number are given, the shape is taken from the tensor. Examples iex&gt; t = Nx . random_normal ( { 10 } ) iex&gt; Nx . shape ( t ) { 10 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; t = Nx . random_normal ( { 5 , 5 } , 2.0 , 1.0 , type : :bf16 ) iex&gt; Nx . shape ( t ) { 5 , 5 } iex&gt; Nx . type ( t ) { :bf , 16 } iex&gt; t = Nx . random_normal ( { 3 , 3 , 3 } , - 1.0 , 1.0 , type : :f32 ) iex&gt; Nx . shape ( t ) { 3 , 3 , 3 } iex&gt; Nx . type ( t ) { :f , 32 } If given a tensor as a shape, it takes the shape, names, and default type from the tensor: iex&gt; t = Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] , names : [ :batch , :data ] ) iex&gt; t = Nx . random_normal ( t ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ :batch , :data ] iex&gt; t = Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] ) iex&gt; t = Nx . random_normal ( t , type : :f32 ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ nil , nil ] The same applies to numbers: iex&gt; t = Nx . random_normal ( 10.0 ) iex&gt; Nx . shape ( t ) { } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ ] If you pass the :names option, the resulting tensor will take on those names: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :batch , :data ] ) iex&gt; t = Nx . random_normal ( t , names : [ :batch , nil ] ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ :batch , nil ] Options :type - the type of the tensor :names - the names of the tensor dimensions :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#random_normal/4","title":"Nx.random_normal/4","type":"function"},{"doc":"Shortcut for random_uniform(shape, 0.0, 1.0, opts) .","ref":"Nx.html#random_uniform/2","title":"Nx.random_uniform/2","type":"function"},{"doc":"Returns a uniformly-distributed random tensor with the given shape. The distribution is bounded on the semi-open interval [min, max) . If min and max are integers, then the tensor has type {:s, 64} . Otherwise, a {:f, 64} tensor is returned. You can also pass any valid type via the :type option. If a tensor or a number are given, the shape and default type are taken from them. Examples Generating Floats iex&gt; t = Nx . random_uniform ( { 10 } ) iex&gt; for &lt;&lt; x :: float - 32 - native &lt;- Nx . to_binary ( t ) &gt;&gt; do ...&gt; true = x &gt;= 0.0 and x &lt; 1.0 ...&gt; end iex&gt; Nx . shape ( t ) { 10 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; t = Nx . random_uniform ( { 5 , 5 } , type : :bf16 ) iex&gt; byte_size ( Nx . to_binary ( t ) ) 50 iex&gt; Nx . shape ( t ) { 5 , 5 } iex&gt; Nx . type ( t ) { :bf , 16 } iex&gt; t = Nx . random_uniform ( { 5 , 5 } , - 1.0 , 1.0 , type : :f64 ) iex&gt; for &lt;&lt; x :: float - 64 - native &lt;- Nx . to_binary ( t ) &gt;&gt; do ...&gt; true = x &gt;= - 1.0 and x &lt; 1.0 ...&gt; end iex&gt; Nx . shape ( t ) { 5 , 5 } iex&gt; Nx . type ( t ) { :f , 64 } Generating Integers iex&gt; t = Nx . random_uniform ( { 10 } , 5 , 10 , type : :u8 ) iex&gt; for &lt;&lt; x :: 8 - unsigned - native &lt;- Nx . to_binary ( t ) &gt;&gt; do ...&gt; true = x &gt;= 5 and x &lt; 10 ...&gt; end iex&gt; Nx . shape ( t ) { 10 } iex&gt; Nx . type ( t ) { :u , 8 } iex&gt; t = Nx . random_uniform ( { 5 , 5 } , - 5 , 5 , type : :s64 ) iex&gt; for &lt;&lt; x :: 64 - signed - native &lt;- Nx . to_binary ( t ) &gt;&gt; do ...&gt; true = x &gt;= - 5 and x &lt; 5 ...&gt; end iex&gt; Nx . shape ( t ) { 5 , 5 } iex&gt; Nx . type ( t ) { :s , 64 } Tensors as shapes If given a tensor as a shape, it takes the shape and names from the tensor: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :batch , :data ] ) iex&gt; t = Nx . random_uniform ( t ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ :batch , :data ] iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; t = Nx . random_uniform ( t , type : :f32 ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ nil , nil ] The same applies to numbers: iex&gt; t = Nx . random_uniform ( 10 ) iex&gt; Nx . shape ( t ) { } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; t = Nx . random_uniform ( 10.0 ) iex&gt; Nx . shape ( t ) { } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ ] If you pass :names as an option, the resulting tensor will take on those names: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :batch , :data ] ) iex&gt; t = Nx . random_uniform ( t , names : [ :batch , nil ] ) iex&gt; Nx . shape ( t ) { 2 , 2 } iex&gt; Nx . type ( t ) { :f , 32 } iex&gt; Nx . names ( t ) [ :batch , nil ] Options :type - the type of the tensor :names - the names of the tensor dimensions :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#random_uniform/4","title":"Nx.random_uniform/4","type":"function"},{"doc":"Returns the rank of a tensor. If a tuple is given as a shape, it computes the rank of the given tuple. Examples iex&gt; Nx . rank ( Nx . tensor ( 1 ) ) 0 iex&gt; Nx . rank ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) 2 iex&gt; Nx . rank ( 1 ) 0 iex&gt; Nx . rank ( { 1 , 2 , 3 } ) 3","ref":"Nx.html#rank/1","title":"Nx.rank/1","type":"function"},{"doc":"Returns the real component of each entry in a complex tensor as a floating point tensor. Examples iex&gt; Nx . real ( Complex . new ( 1 , 2 ) ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx . real ( Nx . tensor ( 1 ) ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx . real ( Nx . tensor ( 1 , type : :bf16 ) ) # Nx.Tensor &lt; bf16 1.0 &gt; iex&gt; Nx . real ( Nx . tensor ( [ Complex . new ( 1 , 2 ) , Complex . new ( 2 , - 4 ) ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.0 , 2.0 ] &gt;","ref":"Nx.html#real/1","title":"Nx.real/1","type":"function"},{"doc":"Reduces over a tensor with the given accumulator. The given fun will receive two tensors and it must return the reduced value. The tensor may be reduced in parallel and the reducer function can be called with arguments in any order, the initial accumulator may be given multiples, and it may be non-deterministic. Therefore, the reduction function should be associative (or as close as possible to associativity considered floats themselves are not strictly associative). By default, it reduces all dimensions of the tensor and return a scalar. If the :axes option is given, it aggregates over multiple dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. The type of the returned tensor will be computed based on the given tensor and the initial value. For example, a tensor of integers with a float accumulator will be cast to float, as done by most binary operators. You can also pass a :type option to change this behaviour. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the reduced axes to size 1. Limitations Given this function relies on anonymous functions, it may not be available or efficient on all Nx backends. Therefore, you should avoid using reduce/4 whenever possible. Instead, use functions sum/2 , reduce_max/2 , all/1 , and so forth. Examples iex&gt; Nx . reduce ( Nx . tensor ( 42 ) , 0 , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . reduce ( Nx . tensor ( [ 1 , 2 , 3 ] ) , 0 , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 6 &gt; iex&gt; Nx . reduce ( Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] ) , 0 , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; f32 10.0 &gt; Aggregating over axes iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :x ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 6 &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :x ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 8 , 10 , 12 ] , [ 14 , 16 , 18 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :y ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 5 , 7 , 9 ] , [ 17 , 19 , 21 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :x , 2 ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ y : 2 ] [ 30 , 48 ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ - 1 ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 6 , 15 ] , [ 24 , 33 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :x ] ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 8 , 10 , 12 ] , [ 14 , 16 , 18 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce ( t , 0 , [ axes : [ :x ] , keep_axes : true ] , fn x , y -&gt; Nx . add ( x , y ) end ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 2 ] [ z : 3 ] [ [ [ 8 , 10 , 12 ] , [ 14 , 16 , 18 ] ] ] &gt;","ref":"Nx.html#reduce/4","title":"Nx.reduce/4","type":"function"},{"doc":"Returns the maximum values of the tensor. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the reduced axes to size 1. Examples iex&gt; Nx . reduce_max ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . reduce_max ( Nx . tensor ( 42.0 ) ) # Nx.Tensor &lt; f32 42.0 &gt; iex&gt; Nx . reduce_max ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; s64 3 &gt; Aggregating over an axis iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 4 ] , [ 2 , 1 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx . reduce_max ( t , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ y : 3 ] [ 3 , 1 , 4 ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 4 ] , [ 2 , 1 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx . reduce_max ( t , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 4 , 2 ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 4 , 5 ] ] , [ [ 2 , 4 ] , [ 3 , 8 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce_max ( t , axes : [ :x , :z ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ 4 , 8 ] &gt; Keeping axes iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 4 , 5 ] ] , [ [ 2 , 4 ] , [ 3 , 8 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce_max ( t , axes : [ :x , :z ] , keep_axes : true ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 2 ] [ z : 1 ] [ [ [ 4 ] , [ 8 ] ] ] &gt;","ref":"Nx.html#reduce_max/2","title":"Nx.reduce_max/2","type":"function"},{"doc":"Returns the minimum values of the tensor. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the reduced axes to size 1. Examples iex&gt; Nx . reduce_min ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . reduce_min ( Nx . tensor ( 42.0 ) ) # Nx.Tensor &lt; f32 42.0 &gt; iex&gt; Nx . reduce_min ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; s64 1 &gt; Aggregating over an axis iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 4 ] , [ 2 , 1 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx . reduce_min ( t , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ y : 3 ] [ 2 , 1 , 1 ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 4 ] , [ 2 , 1 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx . reduce_min ( t , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 1 , 1 ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 4 , 5 ] ] , [ [ 2 , 4 ] , [ 3 , 8 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce_min ( t , axes : [ :x , :z ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ 1 , 3 ] &gt; Keeping axes iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 4 , 5 ] ] , [ [ 2 , 4 ] , [ 3 , 8 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . reduce_min ( t , axes : [ :x , :z ] , keep_axes : true ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 2 ] [ z : 1 ] [ [ [ 1 ] , [ 3 ] ] ] &gt;","ref":"Nx.html#reduce_min/2","title":"Nx.reduce_min/2","type":"function"},{"doc":"Element-wise remainder of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the rem/2 function in place of this function: rem(left, right) . Examples Remainder of scalars iex&gt; Nx . remainder ( 1 , 2 ) # Nx.Tensor &lt; s64 1 &gt; Remainder of tensors and scalars iex&gt; Nx . remainder ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 1 , 0 , 1 ] &gt; iex&gt; Nx . remainder ( 2 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.0 , 0.0 , 2.0 ] &gt; Remainder of tensors iex&gt; left = Nx . tensor ( [ [ 10 ] , [ 20 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 3 , 4 ] ] , names : [ nil , :y ] ) iex&gt; Nx . remainder ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 1 , 2 ] , [ 2 , 0 ] ] &gt; Remainder involving negative values If given a negative value as the right operand, the operation will return the negative image of the remainder. For the example below, note that in modulo-10, adding 20 shouldn't change the result, but in this case it does because the sign changes. iex&gt; left = Nx . tensor ( - 11 , type : :s8 ) iex&gt; right = Nx . tensor ( 10 , type : :u8 ) iex&gt; Nx . remainder ( left , right ) # Nx.Tensor &lt; s16 - 1 &gt; iex&gt; Nx . remainder ( Nx . add ( left , Nx . tensor ( 20 , type : :s8 ) ) , right ) # Nx.Tensor &lt; s16 9 &gt; iex&gt; positive_left = Nx . tensor ( 9 , type : :u8 ) iex&gt; Nx . remainder ( positive_left , right ) # Nx.Tensor &lt; u8 9 &gt; iex&gt; Nx . remainder ( Nx . add ( positive_left , Nx . tensor ( 20 , type : :u8 ) ) , right ) # Nx.Tensor &lt; u8 9 &gt;","ref":"Nx.html#remainder/2","title":"Nx.remainder/2","type":"function"},{"doc":"Adds (or overrides) the given names to the tensor. Examples iex&gt; Nx . rename ( Nx . iota ( { 2 , 3 } ) , [ :foo , :bar ] ) # Nx.Tensor &lt; s64 [ foo : 2 ] [ bar : 3 ] [ [ 0 , 1 , 2 ] , [ 3 , 4 , 5 ] ] &gt;","ref":"Nx.html#rename/2","title":"Nx.rename/2","type":"function"},{"doc":"Changes the shape of a tensor. The new shape is either a tuple or a tensor which we will retrieve the current shape from. The shapes must be compatible: the product of each dimension in the shape must be equal. You may specify one of the dimensions as :auto . Nx will compute the size of the dimension based on the original shape and new shape. Reshaping only changes the tensor metadata, it doesn't copy the underlying structure. Reshape is a destructive operation with respect to names. You can optionally provide :names for each of the dimensions in the reshaped tensor. If you do not provide :names , they will be taken from the tensor the shape is taken from or all of the dimension names will be set to nil . Examples iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 , 4 ] , names : [ :x ] ) iex&gt; Nx . reshape ( t , { 2 , 2 } , names : [ :x , :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] ] &gt; The shape can also be an existing tensor: iex&gt; shape = Nx . tensor ( [ [ 0 ] , [ 0 ] , [ 0 ] , [ 0 ] ] , names : [ :x , :y ] ) iex&gt; Nx . reshape ( Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) , shape ) # Nx.Tensor &lt; s64 [ x : 4 ] [ y : 1 ] [ [ 1 ] , [ 2 ] , [ 3 ] , [ 4 ] ] &gt; Even a scalar can be transformed into a 3-dimensional tensor: iex&gt; t = Nx . tensor ( 1 ) iex&gt; Nx . reshape ( t , { 1 , 1 , 1 } , names : [ :x , :y , :z ] ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 1 ] [ z : 1 ] [ [ [ 1 ] ] ] &gt; You can use :auto to infer dimension sizes. This is useful when you don't know the size some dimension should be ahead of time: iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . reshape ( t , { :auto , 2 } , names : [ :x , :y ] ) # Nx.Tensor &lt; s64 [ x : 3 ] [ y : 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] &gt;","ref":"Nx.html#reshape/3","title":"Nx.reshape/3","type":"function"},{"doc":"Reverses the tensor in the given dimensions. If no axes are provided, reverses every axis. You can pass either names or numbers for the reverse dimensions. Dimensions must be unique, but they do not have to be successive. Examples iex&gt; Nx . reverse ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 , 2 , 1 ] &gt; iex&gt; Nx . reverse ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 6 , 5 , 4 ] , [ 3 , 2 , 1 ] ] &gt; iex&gt; Nx . reverse ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ x : 3 ] [ 3 , 2 , 1 ] &gt; iex&gt; Nx . reverse ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 4 , 5 , 6 ] , [ 1 , 2 , 3 ] ] &gt; iex&gt; Nx . reverse ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 3 , 2 , 1 ] , [ 6 , 5 , 4 ] ] &gt; iex&gt; Nx . reverse ( Nx . iota ( { 2 , 2 , 2 } , type : :f32 , names : [ :x , :y , :z ] ) , axes : [ :x , :z ] ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ z : 2 ] [ [ [ 5.0 , 4.0 ] , [ 7.0 , 6.0 ] ] , [ [ 1.0 , 0.0 ] , [ 3.0 , 2.0 ] ] ] &gt;","ref":"Nx.html#reverse/2","title":"Nx.reverse/2","type":"function"},{"doc":"Element-wise right shift of two tensors. Only integer tensors are supported. If a float or complex tensor is given, an error is raised. If the right side is negative, it will raise, but it may trigger undefined behaviour on some compilers. It performs an arithmetic shift if the tensor is made of signed integers, it performs a logical shift otherwise. In other words, it preserves the sign for signed integers. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If the number of shifts are negative, Nx's default backend will raise, but it may trigger undefined behaviour in other backends. If you're using Nx.Defn.defn/2 , you can use the &gt;&gt;&gt; operator in place of this function: left &gt;&gt;&gt; right . Examples Right shift between scalars iex&gt; Nx . right_shift ( 1 , 0 ) # Nx.Tensor &lt; s64 1 &gt; Right shift between tensors and scalars iex&gt; Nx . right_shift ( Nx . tensor ( [ 2 , 4 , 8 ] , names : [ :data ] ) , 2 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 0 , 1 , 2 ] &gt; Right shift between tensors iex&gt; left = Nx . tensor ( [ 16 , 32 , - 64 , - 128 ] , names : [ :data ] ) iex&gt; right = Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) iex&gt; Nx . right_shift ( left , right ) # Nx.Tensor &lt; s64 [ data : 4 ] [ 8 , 8 , - 8 , - 8 ] &gt; Error cases iex&gt; Nx . right_shift ( Nx . tensor ( [ 0 , 0 , 1 , 1 ] ) , 1.0 ) ** (ArgumentError) bitwise operators expect integer tensors as inputs and outputs an integer tensor, got: {:f, 32}","ref":"Nx.html#right_shift/2","title":"Nx.right_shift/2","type":"function"},{"doc":"Calculates the round (away from zero) of each element in the tensor. If a non-floating tensor is given, it is returned as is. If a floating tensor is given, then we apply the operation, but keep its type. Examples iex&gt; Nx . round ( Nx . tensor ( [ - 1 , 0 , 1 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ - 1 , 0 , 1 ] &gt; iex&gt; Nx . round ( Nx . tensor ( [ - 1.5 , - 0.5 , 0.5 , 1.5 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 4 ] [ - 2.0 , - 1.0 , 1.0 , 2.0 ] &gt;","ref":"Nx.html#round/1","title":"Nx.round/1","type":"function"},{"doc":"Calculates the reverse square root of each element in the tensor. It is equivalent to: $$rsqrt(z) = \\frac{1}{\\sqrt{z}}$$ Examples iex&gt; Nx . rsqrt ( 1 ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx . rsqrt ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.0 , 0.7071067690849304 , 0.5773502588272095 ] &gt;","ref":"Nx.html#rsqrt/1","title":"Nx.rsqrt/1","type":"function"},{"doc":"Constructs a tensor from two tensors, based on a predicate. The resulting tensor is built by evaluating each element of pred and returning either the corresponding element from on_true or on_false . pred must either be 1 or 0 or a tensor of predicates with a shape that matches the largest shape between s1 or s2 . If the shape of on_true or on_false do not match the shape of pred , attempts to broadcast both so they match the shape of pred . Examples When the first argument is a scalar: iex&gt; Nx . select ( 1 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , Nx . tensor ( [ 4 , 5 , 6 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . select ( 0 , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :y ] ) , Nx . tensor ( [ 4 , 5 , 6 ] , names : [ :y ] ) ) # Nx.Tensor &lt; s64 [ y : 3 ] [ 4 , 5 , 6 ] &gt; iex&gt; Nx . select ( 0 , Nx . tensor ( [ [ 1 , 2 ] ] , names : [ :x , :y ] ) , Nx . tensor ( [ [ 3 ] , [ 4 ] ] , names : [ :x , :y ] ) ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 3 , 3 ] , [ 4 , 4 ] ] &gt; When the first argument is a tensor: iex&gt; Nx . select ( Nx . tensor ( [ 0 , 1 , 0 ] , names : [ :x ] ) , Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :y ] ) , Nx . tensor ( [ 4 , 5 , 6 ] , names : [ :z ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ 4 , 2 , 6 ] &gt; iex&gt; x = Nx . tensor ( [ 2 , 4 , 6 ] , names : [ :x ] ) iex&gt; y = Nx . tensor ( [ 3 , 2 , 1 ] ) iex&gt; Nx . select ( Nx . greater ( x , y ) , Nx . tensor ( [ 2 , 4 , 6 ] , names : [ :i ] ) , Nx . tensor ( [ 1 , 3 , 5 ] , names : [ :j ] ) ) # Nx.Tensor &lt; s64 [ x : 3 ] [ 1 , 4 , 6 ] &gt; iex&gt; x = Nx . tensor ( [ 2 , 4 , 6 , 8 , 10 ] , names : [ :x ] ) iex&gt; y = Nx . tensor ( [ 1 , 6 , 2 , 11 , 2 ] , names : [ :x ] ) iex&gt; Nx . select ( Nx . greater ( x , y ) , Nx . tensor ( 2 ) , Nx . tensor ( [ 1 , 3 , 5 , 7 , 9 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 5 ] [ 2 , 3 , 2 , 7 , 2 ] &gt; If the tensor has other values, any non-zero value is considered true: iex&gt; Nx . select ( Nx . tensor ( [ 0 , 1 , 2 ] , type : :u8 ) , Nx . tensor ( [ 0 , 0 , 0 ] ) , Nx . tensor ( [ 1 , 1 , 1 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 0 , 0 ] &gt; iex&gt; Nx . select ( Nx . tensor ( [ 0 , 1 , 0 ] ) , Nx . tensor ( [ 1 , 1 , 1 ] ) , Nx . tensor ( [ 2.0 , 2.0 , 2.0 ] ) ) # Nx.Tensor &lt; f32 [ 3 ] [ 2.0 , 1.0 , 2.0 ] &gt;","ref":"Nx.html#select/3","title":"Nx.select/3","type":"function"},{"doc":"Serializes the given tensor or container of tensors to iodata. You may pass a tensor, tuple, or map to serialize. opts controls the serialization options. For example, you can choose to compress the given tensor or container of tensors by passing a compression level: Nx . serialize ( tensor , compressed : 9 ) Compression level corresponds to compression options in :erlang.term_to_iovec/2 . iodata is a list of binaries that can be written to any io device, such as a file or a socket. You can ensure the result is a binary by calling IO.iodata_to_binary/1 . Examples iex&gt; a = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; serialized_a = Nx . serialize ( a ) iex&gt; Nx . deserialize ( serialized_a ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; container = { Nx . tensor ( [ 1 , 2 , 3 ] ) , %{ b : Nx . tensor ( [ 4 , 5 , 6 ] ) } } iex&gt; serialized_container = Nx . serialize ( container ) iex&gt; { a , %{ b : b } } = Nx . deserialize ( serialized_container ) iex&gt; a # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; b # Nx.Tensor &lt; s64 [ 3 ] [ 4 , 5 , 6 ] &gt;","ref":"Nx.html#serialize/2","title":"Nx.serialize/2","type":"function"},{"doc":"Returns the shape of the tensor as a tuple. The size of this tuple gives the rank of the tensor. If a shape as a tuple is given, it returns the shape itself. Examples iex&gt; Nx . shape ( Nx . tensor ( 1 ) ) { } iex&gt; Nx . shape ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) { 2 , 3 } iex&gt; Nx . shape ( 1 ) { } iex&gt; Nx . shape ( { 1 , 2 , 3 } ) { 1 , 2 , 3 }","ref":"Nx.html#shape/1","title":"Nx.shape/1","type":"function"},{"doc":"Shuffles tensor elements. By default, shuffles elements within the whole tensor. When :axis is given, shuffles the tensor along the specific axis instead. Options :axis - the axis to shuffle along Examples Shuffling all elements: t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) Nx . shuffle ( t ) #=&gt; # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 5 , 1 ] , [ 2 , 3 ] , [ 6 , 4 ] ] &gt; Shuffling rows in a two-dimensional tensor: t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) Nx . shuffle ( t , axis : 0 ) #=&gt; # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 5 , 6 ] , [ 1 , 2 ] , [ 3 , 4 ] ] &gt;","ref":"Nx.html#shuffle/2","title":"Nx.shuffle/2","type":"function"},{"doc":"A convenient ~M sigil for building matrices (two-dimensional tensors). Examples Before using sigils, you must first import them: import Nx , only : :sigils Then you use the sigil to create matrices. The sigil: ~M&lt; -1 0 0 1 0 2 0 0 0 0 3 0 0 0 0 4 &gt; Is equivalent to: Nx . tensor ( [ [ - 1 , 0 , 0 , 1 ] , [ 0 , 2 , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] , [ 0 , 0 , 0 , 4 ] ] ) If the tensor has any complex type, it defaults to c64. If the tensor has any float type, it defaults to f32. Otherwise, it is s64. You can specify the tensor type as a sigil modifier: iex&gt; import Nx , only : :sigils iex&gt; ~M[0.1 0.2 0.3 0.4]f16 # Nx.Tensor &lt; f16 [ 1 ] [ 4 ] [ [ 0.0999755859375 , 0.199951171875 , 0.300048828125 , 0.39990234375 ] ] &gt; iex&gt; ~M[1+1i 2-2.0i -3] # Nx.Tensor &lt; c64 [ 1 ] [ 3 ] [ [ 1.0 + 1.0 i , 2.0 - 2.0 i , - 3.0 + 0.0 i ] ] &gt; iex&gt; ~M[1 Inf NaN] # Nx.Tensor &lt; f32 [ 1 ] [ 3 ] [ [ 1.0 , Inf , NaN ] ] &gt; iex&gt; ~M[1i Inf NaN] # Nx.Tensor &lt; c64 [ 1 ] [ 3 ] [ [ 0.0 + 1.0 i , Inf + 0.0 i , NaN + 0.0 i ] ] &gt; iex&gt; ~M[1i Inf+2i NaN-Infi] # Nx.Tensor &lt; c64 [ 1 ] [ 3 ] [ [ 0.0 + 1.0 i , Inf + 2.0 i , NaN - Infi ] ] &gt;","ref":"Nx.html#sigil_M/2","title":"Nx.sigil_M/2","type":"macro"},{"doc":"A convenient ~V sigil for building vectors (one-dimensional tensors). Examples Before using sigils, you must first import them: import Nx , only : :sigils Then you use the sigil to create vectors. The sigil: ~V[-1 0 0 1] Is equivalent to: Nx . tensor ( [ - 1 , 0 , 0 , 1 ] ) If the tensor has any complex type, it defaults to c64. If the tensor has any float type, it defaults to f32. Otherwise, it is s64. You can specify the tensor type as a sigil modifier: iex&gt; import Nx , only : :sigils iex&gt; ~V[0.1 0.2 0.3 0.4]f16 # Nx.Tensor &lt; f16 [ 4 ] [ 0.0999755859375 , 0.199951171875 , 0.300048828125 , 0.39990234375 ] &gt; iex&gt; ~V[1+1i 2-2.0i -3] # Nx.Tensor &lt; c64 [ 3 ] [ 1.0 + 1.0 i , 2.0 - 2.0 i , - 3.0 + 0.0 i ] &gt; iex&gt; ~V[1 Inf NaN] # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , Inf , NaN ] &gt; iex&gt; ~V[1i Inf NaN] # Nx.Tensor &lt; c64 [ 3 ] [ 0.0 + 1.0 i , Inf + 0.0 i , NaN + 0.0 i ] &gt; iex&gt; ~V[1i Inf+2i NaN-Infi] # Nx.Tensor &lt; c64 [ 3 ] [ 0.0 + 1.0 i , Inf + 2.0 i , NaN - Infi ] &gt;","ref":"Nx.html#sigil_V/2","title":"Nx.sigil_V/2","type":"macro"},{"doc":"Calculates the sigmoid of each element in the tensor. It is equivalent to: $$sigmoid(z) = \\frac{1}{1 + e^{-z}}$$ Examples iex&gt; Nx . sigmoid ( 1 ) # Nx.Tensor &lt; f32 0.7310585975646973 &gt; iex&gt; Nx . sigmoid ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.7310585975646973 , 0.8807970881462097 , 0.9525741338729858 ] &gt;","ref":"Nx.html#sigmoid/1","title":"Nx.sigmoid/1","type":"function"},{"doc":"Computes the sign of each element in the tensor. If a number is less than zero, it returns -1. If a number is more than zero, it returns 1. Otherwise it returns zero (which may either be positive or negative for floats). Examples iex&gt; Nx . sign ( Nx . tensor ( [ - 2 , - 1 , 0 , 1 , 2 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 [ x : 5 ] [ - 1 , - 1 , 0 , 1 , 1 ] &gt;","ref":"Nx.html#sign/1","title":"Nx.sign/1","type":"function"},{"doc":"Calculates the sine of each element in the tensor. It is equivalent to: $$sin(z) = \\frac{e^{iz} - e^{-iz}}{2i}$$ Examples iex&gt; Nx . sin ( 1 ) # Nx.Tensor &lt; f32 0.8414709568023682 &gt; iex&gt; Nx . sin ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.8414709568023682 , 0.9092974066734314 , 0.14112000167369843 ] &gt;","ref":"Nx.html#sin/1","title":"Nx.sin/1","type":"function"},{"doc":"Calculates the hyperbolic sine of each element in the tensor. It is equivalent to: $$sinh(z) = \\frac{e^z - e^{-z}}{2}$$ Examples iex&gt; Nx . sinh ( 1 ) # Nx.Tensor &lt; f32 1.175201177597046 &gt; iex&gt; Nx . sinh ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.175201177597046 , 3.6268603801727295 , 10.017874717712402 ] &gt;","ref":"Nx.html#sinh/1","title":"Nx.sinh/1","type":"function"},{"doc":"Returns the number of elements in the tensor. If a tuple is given, it returns the number of elements in a tensor with that shape. Examples iex&gt; Nx . size ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) 6 iex&gt; Nx . size ( 1 ) 1 iex&gt; Nx . size ( { 1 , 2 , 3 , 2 } ) 12","ref":"Nx.html#size/1","title":"Nx.size/1","type":"function"},{"doc":"Slices a tensor from start_indices with lengths . You can optionally provide a stride to specify the amount of stride in each dimension. Both start indices and lengths must match the rank of the input tensor shape. All start indexes must be greater than or equal to zero. All lengths must be strictly greater than zero. If start_index + length exceeds the tensor dimension, the start_index will be clipped in order to guarantee the length is the requested one. See the &quot;Clipping&quot; section below. It is possible for start_indices to be a list of tensors. However, lengths must always be a list of integers. If you want to specify a tensor as the list of indices, see take/3 . If the :strides is given, it must be strictly greater than zero. The resulting tensor will have the shape of length unless :strides are given. It is not possible to slice in reverse. See gather/2 , slice_along_axis/4 , take/3 , and take_along_axis/3 for other ways to retrieve values from a tensor. Examples iex&gt; Nx . slice ( Nx . tensor ( [ 1 , 2 , 3 , 4 , 5 , 6 ] ) , [ 0 ] , [ 3 ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . slice ( Nx . tensor ( [ 1 , 2 , 3 , 4 , 5 , 6 ] ) , [ 0 ] , [ 6 ] , strides : [ 2 ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 3 , 5 ] &gt; iex&gt; Nx . slice ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) , [ 0 , 0 ] , [ 3 , 2 ] , strides : [ 2 , 1 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 2 ] , [ 5 , 6 ] ] &gt; Strides can also be a number that applies to all dimensions: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) iex&gt; Nx . slice ( t , [ 0 , 0 ] , [ 3 , 2 ] , strides : 2 ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ [ 1 ] , [ 5 ] ] &gt; A more complex example: iex&gt; t = Nx . iota ( { 900 } ) iex&gt; t = Nx . reshape ( t , { 2 , 15 , 30 } ) iex&gt; Nx . slice ( t , [ 0 , 4 , 11 ] , [ 2 , 3 , 9 ] , strides : [ 2 , 1 , 3 ] ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 3 ] [ [ [ 131 , 134 , 137 ] , [ 161 , 164 , 167 ] , [ 191 , 194 , 197 ] ] ] &gt; Tensors as start_indices The start_indices list can be made of scalar tensors: iex&gt; Nx . slice ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) ] , [ 1 , 1 ] ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] [ [ 6 ] ] &gt; iex&gt; t = Nx . tensor ( [ ...&gt; [ 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 ] , ...&gt; [ 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 ] , ...&gt; [ 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 ] , ...&gt; [ 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 ] , ...&gt; [ 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 ] , ...&gt; [ 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 , 1.0 ] ...&gt; ] ) iex&gt; Nx . slice ( t , [ Nx . tensor ( 0 ) , Nx . tensor ( 0 ) ] , [ 6 , 7 ] , strides : [ 5 , 3 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ [ 0.0 , 0.0 , 0.0 ] , [ 1.0 , 1.0 , 1.0 ] ] &gt; Clipping slice/3 will always guarantee the return tensor has the given lengths . See the following example: iex&gt; Nx . slice ( Nx . iota ( { 3 , 3 } ) , [ 2 , 2 ] , [ 1 , 1 ] ) # Nx.Tensor &lt; s64 [ 1 ] [ 1 ] [ [ 8 ] ] &gt; In the example above, start_index + length &lt;= dimension , so there is no clipping. However, if the start_index + length is to exceed the dimension, the index will be clipped in order to guarantee the given lengths: iex&gt; Nx . slice ( Nx . iota ( { 3 , 3 } ) , [ 2 , 2 ] , [ 2 , 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 4 , 5 ] , [ 7 , 8 ] ] &gt; This also applies when the start index is given by tensors: iex&gt; Nx . slice ( Nx . iota ( { 3 , 3 } ) , [ Nx . tensor ( 2 ) , Nx . tensor ( 2 ) ] , [ 2 , 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 4 , 5 ] , [ 7 , 8 ] ] &gt; Error cases iex&gt; Nx . slice ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , [ Nx . tensor ( [ 1 , 2 ] ) , Nx . tensor ( 1 ) ] , [ 1 , 1 ] ) ** (ArgumentError) index must be scalar, got shape {2} for axis 0 iex&gt; Nx . slice ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , [ Nx . tensor ( 1.0 ) , Nx . tensor ( 0 ) ] , [ 1 , 1 ] ) ** (ArgumentError) index must be integer type, got {:f, 32} for axis 0","ref":"Nx.html#slice/4","title":"Nx.slice/4","type":"function"},{"doc":"Slices a tensor along the given axis. You can optionally provide a stride to specify the amount of stride in along the given dimension. Start index must be greater than or equal to zero. It can be an integer or a scalar tensor. Length must be strictly greater than zero. start_index + length must not exceed the respective tensor dimension. The axis will be normalized with the dimensions and names of the given tensor. If the :strides is given, it must be strictly greater than zero. It is not possible to slice in reverse. See gather/2 , slice/3 , take/3 , and take_along_axis/3 for other ways to retrieve values from a tensor. Options :axis - The axis along which to take the values from. Defaults to 0 . :strides - The stride to slice the axis along of. Defaults to 1 . Examples iex&gt; Nx . slice_along_axis ( Nx . iota ( { 5 , 2 } ) , 1 , 2 , axis : 0 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 2 , 3 ] , [ 4 , 5 ] ] &gt; iex&gt; Nx . slice_along_axis ( Nx . iota ( { 2 , 5 } ) , 1 , 2 , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 2 ] , [ 6 , 7 ] ] &gt; iex&gt; Nx . slice_along_axis ( Nx . iota ( { 2 , 5 } , names : [ :x , :y ] ) , 0 , 1 , axis : :x ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 5 ] [ [ 0 , 1 , 2 , 3 , 4 ] ] &gt; iex&gt; Nx . slice_along_axis ( Nx . iota ( { 2 , 5 } , names : [ :x , :y ] ) , Nx . tensor ( 0 ) , 1 , axis : :x ) # Nx.Tensor &lt; s64 [ x : 1 ] [ y : 5 ] [ [ 0 , 1 , 2 , 3 , 4 ] ] &gt; iex&gt; Nx . slice_along_axis ( Nx . iota ( { 2 , 5 } ) , 0 , 3 , axis : - 1 , strides : 2 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 0 , 2 ] , [ 5 , 7 ] ] &gt;","ref":"Nx.html#slice_along_axis/4","title":"Nx.slice_along_axis/4","type":"function"},{"doc":"Sorts the tensor along the given axis according to the given direction. If no axis is given, defaults to 0 . Options :axis - The name or number of the corresponding axis on which the sort should be applied :direction - Can be :asc or :desc . Defaults to :asc Examples iex&gt; Nx . sort ( Nx . tensor ( [ 16 , 23 , 42 , 4 , 8 , 15 ] ) ) # Nx.Tensor &lt; s64 [ 6 ] [ 4 , 8 , 15 , 16 , 23 , 42 ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . sort ( t , axis : :x ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 2 , 1 , 4 ] , [ 3 , 5 , 7 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . sort ( t , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 3 , 7 ] , [ 2 , 4 , 5 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 1 , 7 ] , [ 2 , 5 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . sort ( t , axis : :y , direction : :asc ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 3 , 7 ] , [ 2 , 4 , 5 ] ] &gt; iex&gt; t = Nx . tensor ( ...&gt; [ ...&gt; [ [ 4 , 5 ] , [ 2 , 5 ] , [ 5 , 0 ] ] , ...&gt; [ [ 1 , 9 ] , [ 2 , 1 ] , [ 2 , 1 ] ] , ...&gt; [ [ 0 , - 1 ] , [ - 1 , 0 ] , [ 0 , - 1 ] ] , ...&gt; [ [ - 1 , 0 ] , [ 0 , - 1 ] , [ - 1 , 0 ] ] ...&gt; ] , ...&gt; names : [ :x , :y , :z ] ...&gt; ) iex&gt; Nx . sort ( t , axis : :x ) # Nx.Tensor &lt; s64 [ x : 4 ] [ y : 3 ] [ z : 2 ] [ [ [ - 1 , - 1 ] , [ - 1 , - 1 ] , [ - 1 , - 1 ] ] , [ [ 0 , 0 ] , [ 0 , 0 ] , [ 0 , 0 ] ] , [ [ 1 , 5 ] , [ 2 , 1 ] , [ 2 , 0 ] ] , [ [ 4 , 9 ] , [ 2 , 5 ] , [ 5 , 1 ] ] ] &gt; Same tensor sorted over different axes: iex&gt; t = Nx . tensor ( ...&gt; [ ...&gt; [ ...&gt; [ 4 , 5 , 2 ] , ...&gt; [ 2 , 5 , 3 ] , ...&gt; [ 5 , 0 , 2 ] ...&gt; ] , ...&gt; [ ...&gt; [ 1 , 9 , 8 ] , ...&gt; [ 2 , 1 , 3 ] , ...&gt; [ 2 , 1 , 4 ] ...&gt; ] ...&gt; ] , ...&gt; names : [ :x , :y , :z ] ...&gt; ) iex&gt; Nx . sort ( t , axis : :x ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 1 , 5 , 2 ] , [ 2 , 1 , 3 ] , [ 2 , 0 , 2 ] ] , [ [ 4 , 9 , 8 ] , [ 2 , 5 , 3 ] , [ 5 , 1 , 4 ] ] ] &gt; iex&gt; Nx . sort ( t , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 2 , 0 , 2 ] , [ 4 , 5 , 2 ] , [ 5 , 5 , 3 ] ] , [ [ 1 , 1 , 3 ] , [ 2 , 1 , 4 ] , [ 2 , 9 , 8 ] ] ] &gt; iex&gt; Nx . sort ( t , axis : :z ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ z : 3 ] [ [ [ 2 , 4 , 5 ] , [ 2 , 3 , 5 ] , [ 0 , 2 , 5 ] ] , [ [ 1 , 8 , 9 ] , [ 1 , 2 , 3 ] , [ 1 , 2 , 4 ] ] ] &gt;","ref":"Nx.html#sort/2","title":"Nx.sort/2","type":"function"},{"doc":"Calculates the square root of each element in the tensor. It is equivalent to: $$sqrt(z) = \\sqrt{z}$$ Examples iex&gt; Nx . sqrt ( 1 ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx . sqrt ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.0 , 1.4142135381698608 , 1.7320507764816284 ] &gt;","ref":"Nx.html#sqrt/1","title":"Nx.sqrt/1","type":"function"},{"doc":"Squeezes the given size 1 dimensions out of the tensor. If no axes are given, squeezes all size 1 dimensions from the tensor. While this is equivalent to a reshape which eliminates the size 1 axes, squeeze preserves important information about which axes were squeezed out which can then be used later on in transformations. Examples iex&gt; Nx . squeeze ( Nx . tensor ( [ [ [ [ [ 1 ] ] ] ] ] ) ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . squeeze ( Nx . tensor ( [ [ [ [ 1 ] ] ] , [ [ [ 2 ] ] ] ] , names : [ :x , :y , :z , :i ] ) ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 1 , 2 ] &gt; iex&gt; Nx . squeeze ( Nx . tensor ( [ [ 1 , 2 , 3 ] ] , names : [ :x , :y ] ) , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ y : 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . squeeze ( Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , :y ] ) , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 1 , 2 ] &gt; Error cases iex&gt; Nx . squeeze ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) , axes : [ 1 ] ) ** (ArgumentError) cannot squeeze dimensions whose sizes are not 1, got 3 for dimension 1 iex&gt; Nx . squeeze ( Nx . tensor ( [ [ [ [ [ 1 ] ] ] ] ] ) , axes : [ 0 , 0 ] ) ** (ArgumentError) axes [0, 0] must be unique integers between 0 and 4","ref":"Nx.html#squeeze/2","title":"Nx.squeeze/2","type":"function"},{"doc":"Joins a list of tensors with the same shape along a new axis. Options :axis - optional index of the axis along which the tensors are stacked. Defaults to 0. :name - optional name for the added dimension. Defaults to an unnamed axis. Examples iex&gt; Nx . stack ( [ 1 , 2 , 3 ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . stack ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 4 , 5 , 6 ] ) ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] &gt; iex&gt; t1 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; t2 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; t3 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; Nx . stack ( [ t1 , t2 , t3 ] , axis : - 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 4 ] [ 3 ] [ [ [ [ 0 , 0 , 0 ] , [ 1 , 1 , 1 ] , [ 2 , 2 , 2 ] , [ 3 , 3 , 3 ] ] ] , [ [ [ 4 , 4 , 4 ] , [ 5 , 5 , 5 ] , [ 6 , 6 , 6 ] , [ 7 , 7 , 7 ] ] ] ] &gt; iex&gt; t1 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; t2 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; t3 = Nx . iota ( { 2 , 1 , 4 } ) iex&gt; Nx . stack ( [ t1 , t2 , t3 ] , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 1 ] [ 4 ] [ [ [ [ 0 , 1 , 2 , 3 ] ] , [ [ 0 , 1 , 2 , 3 ] ] , [ [ 0 , 1 , 2 , 3 ] ] ] , [ [ [ 4 , 5 , 6 , 7 ] ] , [ [ 4 , 5 , 6 , 7 ] ] , [ [ 4 , 5 , 6 , 7 ] ] ] ] &gt; iex&gt; Nx . stack ( [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) ] , name : :x ) # Nx.Tensor &lt; s64 [ x : 2 ] [ 1 , 2 ] &gt;","ref":"Nx.html#stack/2","title":"Nx.stack/2","type":"function"},{"doc":"Finds the standard deviation of a tensor. The standard deviation is taken as the square root of the variance. If the :ddof (delta degrees of freedom) option is given, the divisor n - ddof is used to calculate the variance. See variance/2 . Examples iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) ) # Nx.Tensor &lt; f32 1.1180340051651 &gt; iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , ddof : 1 ) # Nx.Tensor &lt; f32 1.29099440574646 &gt; iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.0 , 1.0 ] &gt; iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 1 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 0.5 , 0.5 ] &gt; iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 0 ] , ddof : 1 ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.4142135381698608 , 1.4142135381698608 ] &gt; iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 1 ] , ddof : 1 ) # Nx.Tensor &lt; f32 [ 2 ] [ 0.7071067690849304 , 0.7071067690849304 ] &gt; Keeping axes iex&gt; Nx . standard_deviation ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , keep_axes : true ) # Nx.Tensor &lt; f32 [ 1 ] [ 1 ] [ [ 1.1180340051651 ] ] &gt;","ref":"Nx.html#standard_deviation/2","title":"Nx.standard_deviation/2","type":"function"},{"doc":"Element-wise subtraction of two tensors. If a number is given, it is converted to a tensor. It will broadcast tensors whenever the dimensions do not match and broadcasting is possible. If you're using Nx.Defn.defn/2 , you can use the - operator in place of this function: left - right . Examples Subtracting scalars iex&gt; Nx . subtract ( 1 , 2 ) # Nx.Tensor &lt; s64 - 1 &gt; Subtracting tensors and scalars iex&gt; Nx . subtract ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :data ] ) , 1 ) # Nx.Tensor &lt; s64 [ data : 3 ] [ 0 , 1 , 2 ] &gt; iex&gt; Nx . subtract ( 1 , Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] , names : [ :data ] ) ) # Nx.Tensor &lt; f32 [ data : 3 ] [ 0.0 , - 1.0 , - 2.0 ] &gt; Subtracting tensors iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , names : [ :x , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , names : [ :x , :y ] ) iex&gt; Nx . subtract ( left , right ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ - 9 , - 19 ] , [ - 8 , - 18 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :s8 , names : [ :x , nil ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :s8 , names : [ nil , :y ] ) iex&gt; Nx . subtract ( left , right ) # Nx.Tensor &lt; s8 [ x : 2 ] [ y : 2 ] [ [ - 9 , - 19 ] , [ - 8 , - 18 ] ] &gt; iex&gt; left = Nx . tensor ( [ [ 1 ] , [ 2 ] ] , type : :f32 , names : [ nil , :y ] ) iex&gt; right = Nx . tensor ( [ [ 10 , 20 ] ] , type : :f32 , names : [ :x , nil ] ) iex&gt; Nx . subtract ( left , right ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ - 9.0 , - 19.0 ] , [ - 8.0 , - 18.0 ] ] &gt;","ref":"Nx.html#subtract/2","title":"Nx.subtract/2","type":"function"},{"doc":"Returns the sum for the tensor. If the :axes option is given, it aggregates over the given dimensions, effectively removing them. axes: [0] implies aggregating over the highest order dimension and so forth. If the axis is negative, then counts the axis from the back. For example, axes: [-1] will always aggregate all rows. You may optionally set :keep_axes to true, which will retain the rank of the input tensor by setting the summed axes to size 1. Examples iex&gt; Nx . sum ( Nx . tensor ( 42 ) ) # Nx.Tensor &lt; s64 42 &gt; iex&gt; Nx . sum ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; s64 6 &gt; iex&gt; Nx . sum ( Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] , names : [ :x , :y ] ) ) # Nx.Tensor &lt; f32 10.0 &gt; Giving a tensor with low precision casts it to a higher precision to make sure the sum does not overflow: iex&gt; Nx . sum ( Nx . tensor ( [ [ 101 , 102 ] , [ 103 , 104 ] ] , type : :s8 , names : [ :x , :y ] ) ) # Nx.Tensor &lt; s64 410 &gt; iex&gt; Nx . sum ( Nx . tensor ( [ [ 101 , 102 ] , [ 103 , 104 ] ] , type : :s16 , names : [ :x , :y ] ) ) # Nx.Tensor &lt; s64 410 &gt; Aggregating over an axis iex&gt; Nx . sum ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; s64 6 &gt; Same tensor over different axes combinations: iex&gt; t = Nx . tensor ( ...&gt; [ ...&gt; [ ...&gt; [ 1 , 2 , 3 ] , ...&gt; [ 4 , 5 , 6 ] ...&gt; ] , ...&gt; [ ...&gt; [ 7 , 8 , 9 ] , ...&gt; [ 10 , 11 , 12 ] ...&gt; ] ...&gt; ] , ...&gt; names : [ :x , :y , :z ] ...&gt; ) iex&gt; Nx . sum ( t , axes : [ :x ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 8 , 10 , 12 ] , [ 14 , 16 , 18 ] ] &gt; iex&gt; Nx . sum ( t , axes : [ :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ z : 3 ] [ [ 5 , 7 , 9 ] , [ 17 , 19 , 21 ] ] &gt; iex&gt; Nx . sum ( t , axes : [ :z ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 6 , 15 ] , [ 24 , 33 ] ] &gt; iex&gt; Nx . sum ( t , axes : [ :x , :z ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ 30 , 48 ] &gt; iex&gt; Nx . sum ( t , axes : [ :z ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 6 , 15 ] , [ 24 , 33 ] ] &gt; iex&gt; Nx . sum ( t , axes : [ - 3 ] ) # Nx.Tensor &lt; s64 [ y : 2 ] [ z : 3 ] [ [ 8 , 10 , 12 ] , [ 14 , 16 , 18 ] ] &gt; Keeping axes iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; Nx . sum ( t , axes : [ :z ] , keep_axes : true ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ z : 1 ] [ [ [ 6 ] , [ 15 ] ] , [ [ 24 ] , [ 33 ] ] ] &gt; Errors iex&gt; Nx . sum ( Nx . tensor ( [ [ 1 , 2 ] ] ) , axes : [ 2 ] ) ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#sum/2","title":"Nx.sum/2","type":"function"},{"doc":"Takes and concatenates slices along an axis. Intuitively speaking, take/3 reorders tensor slices along the given axis based on the given indices, possibly duplicating and removing slices. Passing a multi-dimensional indices tensor only affects the resulting shape. Specifically, the given axis in the input shape gets replaced with the indices shape. See gather/2 , slice/3 , slice_along_axis/4 , and take_along_axis/3 for other ways to retrieve values from a tensor. Options :axis - an axis to take tensor slices over. Defaults to 0. Examples iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ 1 , 0 , 1 ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 3 , 4 ] , [ 1 , 2 ] , [ 3 , 4 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ 1 , 0 , 1 ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 2 , 1 , 2 ] , [ 4 , 3 , 4 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ 1 , 0 , 1 ] ) , axis : :y ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 2 , 1 , 2 ] , [ 4 , 3 , 4 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 11 , 12 ] ] , [ [ 101 , 102 ] , [ 111 , 112 ] ] ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ 1 , 0 , 1 ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 2 ] [ [ [ 11 , 12 ] , [ 1 , 2 ] , [ 11 , 12 ] ] , [ [ 111 , 112 ] , [ 101 , 102 ] , [ 111 , 112 ] ] ] &gt; Multi-dimensional indices tensor: iex&gt; t = Nx . tensor ( [ [ 1 , 2 ] , [ 11 , 12 ] ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ [ 0 , 0 ] , [ 1 , 1 ] , [ 0 , 0 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 2 ] [ [ [ 1 , 1 ] , [ 2 , 2 ] , [ 1 , 1 ] ] , [ [ 11 , 11 ] , [ 12 , 12 ] , [ 11 , 11 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 ] , [ 11 , 12 ] ] , [ [ 101 , 102 ] , [ 111 , 112 ] ] ] ) iex&gt; Nx . take ( t , Nx . tensor ( [ [ 0 , 0 , 0 ] , [ 1 , 1 , 1 ] , [ 0 , 0 , 0 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 3 ] [ 2 ] [ [ [ [ 1 , 2 ] , [ 1 , 2 ] , [ 1 , 2 ] ] , [ [ 11 , 12 ] , [ 11 , 12 ] , [ 11 , 12 ] ] , [ [ 1 , 2 ] , [ 1 , 2 ] , [ 1 , 2 ] ] ] , [ [ [ 101 , 102 ] , [ 101 , 102 ] , [ 101 , 102 ] ] , [ [ 111 , 112 ] , [ 111 , 112 ] , [ 111 , 112 ] ] , [ [ 101 , 102 ] , [ 101 , 102 ] , [ 101 , 102 ] ] ] ] &gt; Error cases iex&gt; Nx . take ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , Nx . tensor ( [ 1 , 0 , 1 ] , type : :f32 ) ) ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#take/3","title":"Nx.take/3","type":"function"},{"doc":"Takes the values from a tensor given an indices tensor, along the specified axis. The indices shape must be the same as the tensor 's shape, with the exception for the axis dimension, which can have arbitrary size. The returned tensor will have the same shape as the indices tensor. See gather/2 , slice/3 , slice_along_axis/4 , and take/3 for other ways to retrieve values from a tensor. Options :axis - The axis along which to take the values from. Defaults to 0 . Examples iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . take_along_axis ( t , Nx . tensor ( [ [ 0 , 0 , 2 , 2 , 1 , 1 ] , [ 2 , 2 , 1 , 1 , 0 , 0 ] ] ) , axis : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 6 ] [ [ 1 , 1 , 3 , 3 , 2 , 2 ] , [ 6 , 6 , 5 , 5 , 4 , 4 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; Nx . take_along_axis ( t , Nx . tensor ( [ [ 0 , 1 , 1 ] , [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] ] ) , axis : 0 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 1 , 5 , 6 ] , [ 4 , 2 , 3 ] , [ 1 , 5 , 3 ] ] &gt; The indices returned from Nx.argsort/2 can be used with Nx.take_along_axis/3 to produce the sorted tensor (or to sort more tensors according to the same criteria). iex&gt; tensor = Nx . tensor ( [ [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ] ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ] &gt; iex&gt; idx1 = Nx . argsort ( tensor , axis : 1 , direction : :desc ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 2 , 2 ] , [ 1 , 1 ] , [ 0 , 0 ] ] ] &gt; iex&gt; Nx . take_along_axis ( tensor , idx1 , axis : 1 ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 5 , 6 ] , [ 3 , 4 ] , [ 1 , 2 ] ] ] &gt; iex&gt; idx2 = Nx . argsort ( tensor , axis : 2 , direction : :desc ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 1 , 0 ] , [ 1 , 0 ] , [ 1 , 0 ] ] ] &gt; iex&gt; Nx . take_along_axis ( tensor , idx2 , axis : 2 ) # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ 2 ] [ [ [ 2 , 1 ] , [ 4 , 3 ] , [ 6 , 5 ] ] ] &gt; Error cases iex&gt; tensor = Nx . iota ( { 3 , 3 } ) iex&gt; idx = Nx . tensor ( [ [ 2.0 ] , [ 1.0 ] , [ 2.0 ] ] , type : :f32 ) iex&gt; Nx . take_along_axis ( tensor , idx , axis : 1 ) ** (ArgumentError) indices must be an integer tensor, got {:f, 32}","ref":"Nx.html#take_along_axis/3","title":"Nx.take_along_axis/3","type":"function"},{"doc":"Extracts the diagonal of batched matrices. Converse of make_diagonal/2 . Examples Given a matrix without offset: iex&gt; Nx . take_diagonal ( Nx . tensor ( [ ...&gt; [ 0 , 1 , 2 ] , ...&gt; [ 3 , 4 , 5 ] , ...&gt; [ 6 , 7 , 8 ] ...&gt; ] ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 0 , 4 , 8 ] &gt; And if given a matrix along with an offset: iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 3 } ) , offset : 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 , 5 ] &gt; iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 3 } ) , offset : - 1 ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 , 7 ] &gt; Given batched matrix: iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 2 , 2 } ) ) # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 0 , 3 ] , [ 4 , 7 ] , [ 8 , 11 ] ] &gt; iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 2 , 2 } ) , offset : - 1 ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 ] [ [ 2 ] , [ 6 ] , [ 10 ] ] &gt; Options :offset - offset used for extracting the diagonal. Use offset &gt; 0 for diagonals above the main diagonal, and offset &lt; 0 for diagonals below the main diagonal. Defaults to 0. Error cases iex&gt; Nx . take_diagonal ( Nx . tensor ( [ 0 , 1 , 2 ] ) ) ** (ArgumentError) take_diagonal/2 expects tensor of rank 2 or higher, got tensor of rank: 1 iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 3 } ) , offset : 3 ) ** (ArgumentError) offset must be less than length of axis 1 when positive, got: 3 iex&gt; Nx . take_diagonal ( Nx . iota ( { 3 , 3 } ) , offset : - 4 ) ** (ArgumentError) absolute value of offset must be less than length of axis 0 when negative, got: -4","ref":"Nx.html#take_diagonal/2","title":"Nx.take_diagonal/2","type":"function"},{"doc":"Calculates the tangent of each element in the tensor. It is equivalent to: $$tan(z) = \\frac{sin(z)}{cos(z)}$$ Examples iex&gt; Nx . tan ( 1 ) # Nx.Tensor &lt; f32 1.5574077367782593 &gt; iex&gt; Nx . tan ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 1.5574077367782593 , - 2.185039758682251 , - 0.14254654943943024 ] &gt;","ref":"Nx.html#tan/1","title":"Nx.tan/1","type":"function"},{"doc":"Calculates the hyperbolic tangent of each element in the tensor. It is equivalent to: $$sinh(z) = \\frac{e^z - e^{-z}}{e^z + e^{-z}}$$ Examples iex&gt; Nx . tanh ( 1 ) # Nx.Tensor &lt; f32 0.7615941762924194 &gt; iex&gt; Nx . tanh ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) ) # Nx.Tensor &lt; f32 [ x : 3 ] [ 0.7615941762924194 , 0.9640275835990906 , 0.9950547814369202 ] &gt;","ref":"Nx.html#tanh/1","title":"Nx.tanh/1","type":"function"},{"doc":"Creates a tensor template. You can't perform any operation on this tensor. It exists exclusively to define APIs that say a tensor with a certain type, shape, and names is expected in the future. Examples iex&gt; Nx . template ( { 2 , 3 } , :f32 ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] Nx.TemplateBackend &gt; iex&gt; Nx . template ( { 2 , 3 } , { :f , 32 } , names : [ :rows , :columns ] ) # Nx.Tensor &lt; f32 [ rows : 2 ] [ columns : 3 ] Nx.TemplateBackend &gt; Although note it is impossible to perform any operation on a tensor template: iex&gt; t = Nx . template ( { 2 , 3 } , { :f , 32 } , names : [ :rows , :columns ] ) iex&gt; Nx . abs ( t ) ** (RuntimeError) cannot perform operations on a Nx.TemplateBackend tensor To convert existing tensors to templates, use to_template/1 .","ref":"Nx.html#template/3","title":"Nx.template/3","type":"function"},{"doc":"Builds a tensor. The argument is either a number, which means the tensor is a scalar (zero-dimensions), a list of those (the tensor is a vector) or a list of n-lists of those, leading to n-dimensional tensors. The tensor will be allocated in Nx.default_backend/0 , unless the :backend option is given, which overrides the default one. Examples A number returns a tensor of zero dimensions: iex&gt; Nx . tensor ( 0 ) # Nx.Tensor &lt; s64 0 &gt; iex&gt; Nx . tensor ( 1.0 ) # Nx.Tensor &lt; f32 1.0 &gt; Giving a list returns a vector (a one-dimensional tensor): iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; Nx . tensor ( [ 1.2 , 2.3 , 3.4 , 4.5 ] ) # Nx.Tensor &lt; f32 [ 4 ] [ 1.2000000476837158 , 2.299999952316284 , 3.4000000953674316 , 4.5 ] &gt; The type can be explicitly given. Integers and floats bigger than the given size overflow: iex&gt; Nx . tensor ( [ 300 , 301 , 302 ] , type : :s8 ) # Nx.Tensor &lt; s8 [ 3 ] [ 44 , 45 , 46 ] &gt; Mixed types give higher priority to floats: iex&gt; Nx . tensor ( [ 1 , 2 , 3.0 ] ) # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; Boolean values are also accepted, where true is converted to 1 and false to 0 , with the type being inferred as {:u, 8} iex&gt; Nx . tensor ( true ) # Nx.Tensor &lt; u8 1 &gt; iex&gt; Nx . tensor ( false ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx . tensor ( [ true , false ] ) # Nx.Tensor &lt; u8 [ 2 ] [ 1 , 0 ] &gt; Multi-dimensional tensors are also possible: iex&gt; Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] &gt; iex&gt; Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] &gt; iex&gt; Nx . tensor ( [ [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] , [ [ - 1 , - 2 ] , [ - 3 , - 4 ] , [ - 5 , - 6 ] ] ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 2 ] [ [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] , [ [ - 1 , - 2 ] , [ - 3 , - 4 ] , [ - 5 , - 6 ] ] ] &gt; Floats and complex numbers Besides single-precision (32 bits), floats can also have half-precision (16) or double-precision (64): iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] , type : :f16 ) # Nx.Tensor &lt; f16 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] , type : :f64 ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; Brain-floating points are also supported: iex&gt; Nx . tensor ( [ 1 , 2 , 3 ] , type : :bf16 ) # Nx.Tensor &lt; bf16 [ 3 ] [ 1.0 , 2.0 , 3.0 ] &gt; In all cases, the non-finite values negative infinity (-Inf), infinity (Inf), and &quot;not a number&quot; (NaN) can be represented by the atoms :neg_infinity , :infinity , and :nan respectively: iex&gt; Nx . tensor ( [ :neg_infinity , :nan , :infinity ] ) # Nx.Tensor &lt; f32 [ 3 ] [ - Inf , NaN , Inf ] &gt; Finally, complex numbers are also supported in tensors: iex&gt; Nx . tensor ( Complex . new ( 1 , - 1 ) ) # Nx.Tensor &lt; c64 1.0 - 1.0 i &gt; Naming dimensions You can provide names for tensor dimensions. Names are atoms: iex&gt; Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , names : [ :x , :y ] ) # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] &gt; Names make your code more expressive: iex&gt; Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] , names : [ :batch , :height , :width ] ) # Nx.Tensor &lt; s64 [ batch : 1 ] [ height : 3 ] [ width : 3 ] [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] &gt; You can also leave dimension names as nil : iex&gt; Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] , names : [ :batch , nil , nil ] ) # Nx.Tensor &lt; s64 [ batch : 1 ] [ 3 ] [ 3 ] [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] &gt; However, you must provide a name for every dimension in the tensor: iex&gt; Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ] , names : [ :batch ] ) ** (ArgumentError) invalid names for tensor of rank 3, when specifying names every dimension must have a name or be nil Options :type - sets the type of the tensor. If one is not given, one is automatically inferred based on the input. :names - dimension names. If you wish to specify dimension names you must specify a name for every dimension in the tensor. Only nil and atoms are supported as dimension names. :backend - the backend to allocate the tensor on. It is either an atom or a tuple in the shape {backend, options} . This option is ignored inside defn","ref":"Nx.html#tensor/2","title":"Nx.tensor/2","type":"function"},{"doc":"Creates a new tensor by repeating the input tensor along the given axes. If the tensor has less dimensions than the repetitions given, the tensor will grow in dimensionality. If the tensor has more dimensions than the repetitions given, tiling is done from the rightmost dimensions (i.e. if the input shape is {1,2,3} and repetitions = [2] , the result is the same as if repetitions = [1,1,2] ). Examples iex&gt; a = Nx . tensor ( [ 0 , 1 , 2 ] ) iex&gt; Nx . tile ( a , [ 2 ] ) # Nx.Tensor &lt; s64 [ 6 ] [ 0 , 1 , 2 , 0 , 1 , 2 ] &gt; iex&gt; Nx . tile ( a , [ 1 , 2 ] ) # Nx.Tensor &lt; s64 [ 1 ] [ 6 ] [ [ 0 , 1 , 2 , 0 , 1 , 2 ] ] &gt; iex&gt; Nx . tile ( a , [ 2 , 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 6 ] [ [ 0 , 1 , 2 , 0 , 1 , 2 ] , [ 0 , 1 , 2 , 0 , 1 , 2 ] ] &gt; iex&gt; Nx . tile ( a , [ 2 , 1 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 0 , 1 , 2 ] , [ 0 , 1 , 2 ] ] &gt; iex&gt; Nx . tile ( a , [ 2 , 1 , 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 6 ] [ [ [ 0 , 1 , 2 , 0 , 1 , 2 ] ] , [ [ 0 , 1 , 2 , 0 , 1 , 2 ] ] ] &gt; iex&gt; b = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) iex&gt; Nx . tile ( b , [ 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 4 ] [ [ 1 , 2 , 1 , 2 ] , [ 3 , 4 , 3 , 4 ] ] &gt; iex&gt; Nx . tile ( b , [ 2 , 1 ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] , [ 1 , 2 ] , [ 3 , 4 ] ] &gt; iex&gt; Nx . tile ( b , [ 1 , 2 ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 4 ] [ [ 1 , 2 , 1 , 2 ] , [ 3 , 4 , 3 , 4 ] ] &gt; iex&gt; c = Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) iex&gt; Nx . tile ( c , [ 4 , 1 ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 4 ] [ [ 1 , 2 , 3 , 4 ] , [ 1 , 2 , 3 , 4 ] , [ 1 , 2 , 3 , 4 ] , [ 1 , 2 , 3 , 4 ] ] &gt; Error cases iex&gt; Nx . tile ( Nx . tensor ( [ 1 , 2 ] ) , 1.0 ) ** (ArgumentError) repetitions must be a list of integers, got: 1.0 iex&gt; Nx . tile ( Nx . tensor ( [ 1 , 2 ] ) , [ 1 , 1.0 ] ) ** (ArgumentError) repetitions must be a list of integers, got: [1, 1.0] iex&gt; Nx . tile ( Nx . tensor ( [ 1 , 2 ] ) , nil ) ** (ArgumentError) repetitions must be a list of integers, got: nil","ref":"Nx.html#tile/2","title":"Nx.tile/2","type":"function"},{"doc":"Converts the underlying tensor to a stream of tensor batches. The first dimension (axis 0) is divided by batch_size . In case the dimension cannot be evenly divided by batch_size , you may specify what to do with leftover data using :leftover . :leftover must be one of :repeat or :discard . :repeat repeats the first n values to make the last batch match the desired batch size. :discard discards excess elements. Examples In the examples below we immediately pipe to Enum.to_list/1 for convenience, but in practice you want to lazily traverse the batches to avoid allocating multiple tensors at once in certain backends: iex&gt; [ first , second ] = Nx . to_batched ( Nx . iota ( { 2 , 2 , 2 } ) , 1 ) |&gt; Enum . to_list ( ) iex&gt; first # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 0 , 1 ] , [ 2 , 3 ] ] ] &gt; iex&gt; second # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 4 , 5 ] , [ 6 , 7 ] ] ] &gt; If the batch size would result in uneven batches, you can repeat or discard excess data. By default, we repeat: iex&gt; [ first , second , third ] = Nx . to_batched ( Nx . iota ( { 5 , 2 } , names : [ :x , :y ] ) , 2 ) |&gt; Enum . to_list ( ) iex&gt; first # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 0 , 1 ] , [ 2 , 3 ] ] &gt; iex&gt; second # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 4 , 5 ] , [ 6 , 7 ] ] &gt; iex&gt; third # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 8 , 9 ] , [ 0 , 1 ] ] &gt; But you can also discard: iex&gt; [ first , second ] = Nx . to_batched ( Nx . iota ( { 5 , 2 } , names : [ :x , :y ] ) , 2 , leftover : :discard ) |&gt; Enum . to_list ( ) iex&gt; first # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 0 , 1 ] , [ 2 , 3 ] ] &gt; iex&gt; second # Nx.Tensor &lt; s64 [ x : 2 ] [ y : 2 ] [ [ 4 , 5 ] , [ 6 , 7 ] ] &gt;","ref":"Nx.html#to_batched/3","title":"Nx.to_batched/3","type":"function"},{"doc":"","ref":"Nx.html#to_batched_list/3","title":"Nx.to_batched_list/3","type":"function"},{"doc":"Returns the underlying tensor as a binary. Warning : converting a tensor to a binary can potentially be a very expensive operation, as it may copy a GPU tensor fully to the machine memory. It returns the in-memory binary representation of the tensor in a row-major fashion. The binary is in the system endianness, which has to be taken into account if the binary is meant to be serialized to other systems. Options :limit - limit the number of entries represented in the binary Examples iex&gt; Nx . to_binary ( 1 ) &lt;&lt; 1 :: 64 - native &gt;&gt; iex&gt; Nx . to_binary ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) ) &lt;&lt; 1.0 :: float - 32 - native , 2.0 :: float - 32 - native , 3.0 :: float - 32 - native &gt;&gt; iex&gt; Nx . to_binary ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) , limit : 2 ) &lt;&lt; 1.0 :: float - 32 - native , 2.0 :: float - 32 - native &gt;&gt;","ref":"Nx.html#to_binary/2","title":"Nx.to_binary/2","type":"function"},{"doc":"Returns the underlying tensor as a flat list. Negative infinity (-Inf), infinity (Inf), and &quot;not a number&quot; (NaN) will be represented by the atoms :neg_infinity , :infinity , and :nan respectively. Examples iex&gt; Nx . to_flat_list ( 1 ) [ 1 ] iex&gt; Nx . to_flat_list ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) ) [ 1.0 , 2.0 , 3.0 ] iex&gt; Nx . to_flat_list ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) , limit : 2 ) [ 1.0 , 2.0 ] Non-finite numbers are returned as atoms: iex&gt; t = Nx . tensor ( [ :neg_infinity , :nan , :infinity ] ) iex&gt; Nx . to_flat_list ( t ) [ :neg_infinity , :nan , :infinity ]","ref":"Nx.html#to_flat_list/2","title":"Nx.to_flat_list/2","type":"function"},{"doc":"Returns a heatmap struct with the tensor data. On terminals, coloring is done via ANSI colors. If ANSI is not enabled, the tensor is normalized to show numbers between 0 and 9. Terminal coloring Coloring is enabled by default on most Unix terminals. It is also available on Windows consoles from Windows 10, although it must be explicitly enabled for the current user in the registry by running the following command: reg add HKCU \\ Console / v VirtualTerminalLevel / t REG_DWORD / d 1 After running the command above, you must restart your current console. Options :ansi_enabled - forces ansi to be enabled or disabled. Defaults to IO.ANSI.enabled?/0 :ansi_whitespace - which whitespace character to use when printing. By default it uses &quot;\\u3000&quot; , which is a full-width whitespace which often prints more precise shapes","ref":"Nx.html#to_heatmap/2","title":"Nx.to_heatmap/2","type":"function"},{"doc":"Returns the underlying tensor as a number. Negative infinity (-Inf), infinity (Inf), and &quot;not a number&quot; (NaN) will be represented by the atoms :neg_infinity , :infinity , and :nan respectively. If the tensor has a dimension, it raises. Examples iex&gt; Nx . to_number ( 1 ) 1 iex&gt; Nx . to_number ( Nx . tensor ( [ 1.0 , 2.0 , 3.0 ] ) ) ** (ArgumentError) cannot convert tensor of shape {3} to number","ref":"Nx.html#to_number/1","title":"Nx.to_number/1","type":"function"},{"doc":"Converts a tensor (or tuples and maps of tensors) to tensor templates. Templates are useful when you need to pass types and shapes to operations and the data is not yet available. For convenience, this function accepts tensors and any container (such as maps and tuples as defined by the Nx.Container protocol) and recursively converts all tensors to templates. Examples iex&gt; Nx . iota ( { 2 , 3 } ) |&gt; Nx . to_template ( ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] Nx.TemplateBackend &gt; iex&gt; { int , float } = Nx . to_template ( { 1 , 2.0 } ) iex&gt; int # Nx.Tensor &lt; s64 Nx.TemplateBackend &gt; iex&gt; float # Nx.Tensor &lt; f32 Nx.TemplateBackend &gt; Although note it is impossible to perform any operation on a tensor template: iex&gt; t = Nx . iota ( { 2 , 3 } ) |&gt; Nx . to_template ( ) iex&gt; Nx . abs ( t ) ** (RuntimeError) cannot perform operations on a Nx.TemplateBackend tensor To build a template from scratch, use template/3 .","ref":"Nx.html#to_template/1","title":"Nx.to_template/1","type":"function"},{"doc":"Converts the given number (or tensor) to a tensor. The Nx API works with numbers, complex numbers, and tensors. This function exists to normalize those values into tensors (i.e. Nx.Tensor structs). If your goal is to create tensors from lists, see tensor/2 . If you want to create a tensor from binary, see from_binary/3 .","ref":"Nx.html#to_tensor/1","title":"Nx.to_tensor/1","type":"function"},{"doc":"Transposes a tensor to the given axes . If no axes are given, the default behavior is to reverse the order of the original tensor's axes. The axes is a list of integers or dimension names containing how the new dimensions must be ordered. The highest dimension is zero. Examples iex&gt; Nx . transpose ( Nx . tensor ( 1 ) ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . transpose ( Nx . iota ( { 2 , 3 , 4 } , names : [ :x , :y , :z ] ) ) # Nx.Tensor &lt; s64 [ z : 4 ] [ y : 3 ] [ x : 2 ] [ [ [ 0 , 12 ] , [ 4 , 16 ] , [ 8 , 20 ] ] , [ [ 1 , 13 ] , [ 5 , 17 ] , [ 9 , 21 ] ] , [ [ 2 , 14 ] , [ 6 , 18 ] , [ 10 , 22 ] ] , [ [ 3 , 15 ] , [ 7 , 19 ] , [ 11 , 23 ] ] ] &gt; iex&gt; Nx . transpose ( Nx . tensor ( 1 ) , axes : [ ] ) # Nx.Tensor &lt; s64 1 &gt; iex&gt; Nx . transpose ( Nx . iota ( { 2 , 3 , 4 } , names : [ :batch , :x , :y ] ) , axes : [ 2 , 1 , :batch ] ) # Nx.Tensor &lt; s64 [ y : 4 ] [ x : 3 ] [ batch : 2 ] [ [ [ 0 , 12 ] , [ 4 , 16 ] , [ 8 , 20 ] ] , [ [ 1 , 13 ] , [ 5 , 17 ] , [ 9 , 21 ] ] , [ [ 2 , 14 ] , [ 6 , 18 ] , [ 10 , 22 ] ] , [ [ 3 , 15 ] , [ 7 , 19 ] , [ 11 , 23 ] ] ] &gt; iex&gt; Nx . transpose ( Nx . iota ( { 2 , 3 , 4 } , names : [ :batch , :x , :y ] ) , axes : [ :y , :batch , :x ] ) # Nx.Tensor &lt; s64 [ y : 4 ] [ batch : 2 ] [ x : 3 ] [ [ [ 0 , 4 , 8 ] , [ 12 , 16 , 20 ] ] , [ [ 1 , 5 , 9 ] , [ 13 , 17 , 21 ] ] , [ [ 2 , 6 , 10 ] , [ 14 , 18 , 22 ] ] , [ [ 3 , 7 , 11 ] , [ 15 , 19 , 23 ] ] ] &gt; iex&gt; Nx . transpose ( Nx . iota ( { 2 , 3 , 4 } , names : [ :batch , :x , :y ] ) , axes : [ :batch , :y , :x ] ) # Nx.Tensor &lt; s64 [ batch : 2 ] [ y : 4 ] [ x : 3 ] [ [ [ 0 , 4 , 8 ] , [ 1 , 5 , 9 ] , [ 2 , 6 , 10 ] , [ 3 , 7 , 11 ] ] , [ [ 12 , 16 , 20 ] , [ 13 , 17 , 21 ] , [ 14 , 18 , 22 ] , [ 15 , 19 , 23 ] ] ] &gt; Errors iex&gt; Nx . transpose ( Nx . iota ( { 2 , 2 } , names : [ :batch , :x ] ) , axes : [ :batch ] ) ** (ArgumentError) expected length of permutation (1) to match rank of shape (2) iex&gt; Nx . transpose ( Nx . iota ( { 2 , 2 } ) , axes : [ 1 , 2 ] ) ** (ArgumentError) given axis (2) invalid for shape with rank 2","ref":"Nx.html#transpose/2","title":"Nx.transpose/2","type":"function"},{"doc":"Returns the type of the tensor. See Nx.Type for more information. Examples iex&gt; Nx . type ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) { :s , 64 } iex&gt; Nx . type ( Nx . tensor ( [ 1 , 2 , 3 ] , type : :f32 ) ) { :f , 32 } iex&gt; Nx . type ( 1 ) { :s , 64 } iex&gt; Nx . type ( 1.0 ) { :f , 32 }","ref":"Nx.html#type/1","title":"Nx.type/1","type":"function"},{"doc":"Finds the variance of a tensor. The variance is the average of the squared deviations from the mean. The mean is typically calculated as sum(tensor) / n , where n is the total of elements. If, however, :ddof (delta degrees of freedom) is specified, the divisor n - ddof is used instead. Examples iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) ) # Nx.Tensor &lt; f32 1.25 &gt; iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , ddof : 1 ) # Nx.Tensor &lt; f32 1.6666666269302368 &gt; iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.0 , 1.0 ] &gt; iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 1 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 0.25 , 0.25 ] &gt; iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 0 ] , ddof : 1 ) # Nx.Tensor &lt; f32 [ 2 ] [ 2.0 , 2.0 ] &gt; iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 1 ] , ddof : 1 ) # Nx.Tensor &lt; f32 [ 2 ] [ 0.5 , 0.5 ] &gt; Keeping axes iex&gt; Nx . variance ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , axes : [ 1 ] , keep_axes : true ) # Nx.Tensor &lt; f32 [ 2 ] [ 1 ] [ [ 0.25 ] , [ 0.25 ] ] &gt;","ref":"Nx.html#variance/2","title":"Nx.variance/2","type":"function"},{"doc":"Returns the weighted mean for the tensor and the weights. If the :axis option is given, it aggregates over that dimension, effectively removing it. axis: 0 implies aggregating over the highest order dimension and so forth. If the axis is negative, then the axis will be counted from the back. For example, axis: -1 will always aggregate over the last dimension. You may optionally set :keep_axis to true, which will retain the rank of the input tensor by setting the averaged axis to size 1. Examples iex&gt; Nx . weighted_mean ( Nx . tensor ( 42 ) , Nx . tensor ( 2 ) ) # Nx.Tensor &lt; f32 42.0 &gt; iex&gt; Nx . weighted_mean ( Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 3 , 2 , 1 ] ) ) # Nx.Tensor &lt; f32 1.6666666269302368 &gt; Aggregating over an axis iex&gt; Nx . weighted_mean ( Nx . tensor ( [ 1 , 2 , 3 ] , names : [ :x ] ) , Nx . tensor ( [ 4 , 5 , 6 ] ) , axis : 0 ) # Nx.Tensor &lt; f32 2.133333444595337 &gt; iex&gt; Nx . weighted_mean ( Nx . tensor ( [ 1 , 2 , 3 ] , type : :u8 , names : [ :x ] ) , Nx . tensor ( [ 1 , 3 , 5 ] ) , axis : :x ) # Nx.Tensor &lt; f32 2.444444417953491 &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; weights = Nx . tensor ( [ [ [ 0 , 1 , 2 ] , [ 1 , 1 , 0 ] ] , [ [ - 1 , 1 , - 1 ] , [ 1 , 1 , - 1 ] ] ] ) iex&gt; Nx . weighted_mean ( t , weights , axis : :x ) # Nx.Tensor &lt; f32 [ y : 2 ] [ z : 3 ] [ [ 7.0 , 5.0 , - 3.0 ] , [ 7.0 , 8.0 , 12.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; weights = Nx . tensor ( [ [ [ 0 , 1 , 2 ] , [ 1 , 1 , 0 ] ] , [ [ - 1 , 1 , - 1 ] , [ 1 , 1 , - 1 ] ] ] ) iex&gt; Nx . weighted_mean ( t , weights , axis : - 1 ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ [ 2.6666667461395264 , 4.5 ] , [ 8.0 , 9.0 ] ] &gt; iex&gt; t = Nx . iota ( { 3 , 4 } ) iex&gt; weights = Nx . tensor ( [ 1 , 2 , 3 , 4 ] ) iex&gt; Nx . weighted_mean ( t , weights , axis : 1 ) # Nx.Tensor &lt; f32 [ 3 ] [ 2.0 , 6.0 , 10.0 ] &gt; Keeping axis iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 7 , 8 , 9 ] , [ 10 , 11 , 12 ] ] ] , names : [ :x , :y , :z ] ) iex&gt; weights = Nx . tensor ( [ [ [ 0 , 1 , 2 ] , [ 1 , 1 , 0 ] ] , [ [ - 1 , 1 , - 1 ] , [ 1 , 1 , - 1 ] ] ] ) iex&gt; Nx . weighted_mean ( t , weights , axis : - 1 , keep_axis : true ) # Nx.Tensor &lt; f32 [ x : 2 ] [ y : 2 ] [ z : 1 ] [ [ [ 2.6666667461395264 ] , [ 4.5 ] ] , [ [ 8.0 ] , [ 9.0 ] ] ] &gt;","ref":"Nx.html#weighted_mean/3","title":"Nx.weighted_mean/3","type":"function"},{"doc":"Returns the maximum over each window of size window_dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. Pads with the minimum value for the type of the given tensor. Examples iex&gt; Nx . window_max ( Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) , { 1 , 2 , 1 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 3 ] [ [ [ 4 , 5 , 6 ] ] , [ [ 4 , 5 , 6 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_max ( t , { 2 , 2 , 1 } , strides : [ 1 , 2 , 3 ] , padding : [ { 0 , 1 } , { 2 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ - 9223372036854775808 , - 9223372036854775808 ] , [ - 9223372036854775808 , 6 ] ] , [ [ - 9223372036854775808 , - 9223372036854775808 ] , [ - 9223372036854775808 , 6 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4.0 , 2.0 , 3.0 ] , [ 2.0 , 5.0 , 6.5 ] ] , [ [ 1.2 , 2.2 , 3.2 ] , [ 4.0 , 5.0 , 6.2 ] ] ] ) iex&gt; Nx . window_max ( t , { 2 , 1 , 1 } , strides : [ 2 , 1 , 1 ] , padding : [ { 1 , 1 } , { 0 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 5 ] [ [ [ - Inf , 4.0 , 2.0 , 3.0 , - Inf ] , [ - Inf , 2.0 , 5.0 , 6.5 , - Inf ] ] , [ [ - Inf , 1.2000000476837158 , 2.200000047683716 , 3.200000047683716 , - Inf ] , [ - Inf , 4.0 , 5.0 , 6.199999809265137 , - Inf ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_max ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 4 , 3 ] , [ 4 , 7 ] ] ] &gt;","ref":"Nx.html#window_max/3","title":"Nx.window_max/3","type":"function"},{"doc":"Averages over each window of size window_dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. Pads with 0 . Examples iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_mean ( t , { 1 , 2 , 1 } ) # Nx.Tensor &lt; f32 [ 2 ] [ 1 ] [ 3 ] [ [ [ 2.5 , 3.5 , 4.5 ] ] , [ [ 2.5 , 3.5 , 4.5 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_mean ( t , { 2 , 2 , 1 } , strides : [ 1 , 2 , 3 ] , padding : [ { 0 , 1 } , { 2 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 0.0 , 0.0 ] , [ 0.0 , 4.5 ] ] , [ [ 0.0 , 0.0 ] , [ 0.0 , 2.25 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4.0 , 2.0 , 3.0 ] , [ 2.0 , 5.0 , 6.5 ] ] , [ [ 1.2 , 2.2 , 3.2 ] , [ 4.0 , 5.0 , 6.2 ] ] ] ) iex&gt; Nx . window_mean ( t , { 2 , 1 , 1 } , strides : [ 2 , 1 , 1 ] , padding : [ { 1 , 1 } , { 0 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 5 ] [ [ [ 0.0 , 2.0 , 1.0 , 1.5 , 0.0 ] , [ 0.0 , 1.0 , 2.5 , 3.25 , 0.0 ] ] , [ [ 0.0 , 0.6000000238418579 , 1.100000023841858 , 1.600000023841858 , 0.0 ] , [ 0.0 , 2.0 , 2.5 , 3.0999999046325684 , 0.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 1 ] ] iex&gt; Nx . window_mean ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; f32 [ 1 ] [ 2 ] [ 3 ] [ [ [ 3.0 , 1.5 , 2.0 ] , [ 3.0 , 1.5 , 4.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_mean ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; f32 [ 1 ] [ 2 ] [ 2 ] [ [ [ 2.5 , 2.5 ] , [ 2.5 , 4.5 ] ] ] &gt;","ref":"Nx.html#window_mean/3","title":"Nx.window_mean/3","type":"function"},{"doc":"Returns the minimum over each window of size window_dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. Pads with the maximum value for the type of the given tensor. Examples iex&gt; Nx . window_min ( Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) , { 1 , 2 , 1 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 3 ] [ [ [ 1 , 2 , 3 ] ] , [ [ 1 , 2 , 3 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_min ( t , { 2 , 2 , 1 } , strides : [ 1 , 2 , 3 ] , padding : [ { 0 , 1 } , { 2 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ 9223372036854775807 , 9223372036854775807 ] , [ 9223372036854775807 , 3 ] ] , [ [ 9223372036854775807 , 9223372036854775807 ] , [ 9223372036854775807 , 3 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4.0 , 2.0 , 3.0 ] , [ 2.0 , 5.0 , 6.5 ] ] , [ [ 1.2 , 2.2 , 3.2 ] , [ 4.0 , 5.0 , 6.2 ] ] ] ) iex&gt; Nx . window_min ( t , { 2 , 1 , 1 } , strides : [ 2 , 1 , 1 ] , padding : [ { 1 , 1 } , { 0 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 5 ] [ [ [ Inf , 4.0 , 2.0 , 3.0 , Inf ] , [ Inf , 2.0 , 5.0 , 6.5 , Inf ] ] , [ [ Inf , 1.2000000476837158 , 2.200000047683716 , 3.200000047683716 , Inf ] , [ Inf , 4.0 , 5.0 , 6.199999809265137 , Inf ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_min ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 1 , 2 ] , [ 1 , 2 ] ] ] &gt;","ref":"Nx.html#window_min/3","title":"Nx.window_min/3","type":"function"},{"doc":"Returns the product over each window of size window_dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. The rank of the input tensor and the window dimensions must match. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. Pads with 1. Examples iex&gt; Nx . window_product ( Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) , { 1 , 2 , 1 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 3 ] [ [ [ 4 , 10 , 18 ] ] , [ [ 4 , 10 , 18 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_product ( t , { 2 , 2 , 1 } , strides : [ 1 , 2 , 3 ] , padding : [ { 0 , 1 } , { 2 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1 , 1 ] , [ 1 , 324 ] ] , [ [ 1 , 1 ] , [ 1 , 18 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4.0 , 2.0 , 3.0 ] , [ 2.0 , 5.0 , 6.5 ] ] , [ [ 1.2 , 2.2 , 3.2 ] , [ 4.0 , 5.0 , 6.2 ] ] ] ) iex&gt; Nx . window_product ( t , { 2 , 1 , 1 } , strides : [ 2 , 1 , 1 ] , padding : [ { 1 , 1 } , { 0 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 5 ] [ [ [ 1.0 , 4.0 , 2.0 , 3.0 , 1.0 ] , [ 1.0 , 2.0 , 5.0 , 6.5 , 1.0 ] ] , [ [ 1.0 , 1.2000000476837158 , 2.200000047683716 , 3.200000047683716 , 1.0 ] , [ 1.0 , 4.0 , 5.0 , 6.199999809265137 , 1.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_product ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 4 , 6 ] , [ 4 , 14 ] ] ] &gt;","ref":"Nx.html#window_product/3","title":"Nx.window_product/3","type":"function"},{"doc":"Reduces over each window of size dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. The rank of the input tensor and the window dimensions must match. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. The padding value is equal to the initial value passed to acc . The initial value must be a number or a scalar shaped tensor. Examples iex&gt; init_value = Nx.Constants . min_finite ( :s64 ) iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 , 4 ] , [ 4 , 5 , 6 , 7 ] , [ 7 , 8 , 9 , 10 ] , [ 11 , 12 , 13 , 14 ] ] ) iex&gt; Nx . window_reduce ( t , init_value , { 2 , 2 } , fn x , acc -&gt; Nx . max ( x , acc ) end ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 5 , 6 , 7 ] , [ 8 , 9 , 10 ] , [ 12 , 13 , 14 ] ] &gt; iex&gt; init_value = Nx.Constants . min_finite ( :s64 ) iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ) iex&gt; opts = [ padding : :same , strides : [ 1 , 1 ] ] iex&gt; Nx . window_reduce ( t , init_value , { 2 , 2 } , opts , fn x , acc -&gt; Nx . max ( x , acc ) end ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 5 , 6 , 6 ] , [ 8 , 9 , 9 ] , [ 8 , 9 , 9 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) iex&gt; opts = [ padding : :same , strides : [ 1 , 1 ] ] iex&gt; Nx . window_reduce ( t , 0 , { 1 , 2 } , opts , fn x , acc -&gt; Nx . add ( x , acc ) end ) # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 3 , 5 , 3 ] , [ 9 , 11 , 6 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ padding : :valid , strides : [ 2 , 1 , 1 ] , window_dilations : [ 1 , 1 , 2 ] ] iex&gt; Nx . window_reduce ( t , 0 , { 1 , 1 , 2 } , opts , fn x , acc -&gt; Nx . add ( x , acc ) end ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 5 , 5 ] , [ 5 , 9 ] ] ] &gt;","ref":"Nx.html#window_reduce/5","title":"Nx.window_reduce/5","type":"function"},{"doc":"Performs a window_reduce to select the maximum index in each window of the input tensor according to and scatters source tensor to corresponding maximum indices in the output tensor. Output tensor is initialized as a full tensor with values init_value . If indices overlap, adds overlapping source values. The shape of the source tensor must match the valid windows in the input tensor. This means the shape of the source tensor must match the shape of the input tensor after a window_reduce op with padding padding and strides strides . This function is the gradient of window_max . Examples iex&gt; t = Nx . tensor ( [ ...&gt; [ 7 , 2 , 5 , 3 , 10 , 2 ] , ...&gt; [ 3 , 8 , 9 , 3 , 4 , 2 ] , ...&gt; [ 1 , 5 , 7 , 5 , 6 , 1 ] , ...&gt; [ 0 , 6 , 2 , 7 , 2 , 8 ] ...&gt; ] ) iex&gt; opts = [ strides : [ 2 , 3 ] , padding : :valid ] iex&gt; Nx . window_scatter_max ( t , Nx . tensor ( [ [ 2 , 6 ] , [ 3 , 1 ] ] ) , 0 , { 2 , 3 } , opts ) # Nx.Tensor &lt; s64 [ 4 ] [ 6 ] [ [ 0 , 0 , 0 , 0 , 6 , 0 ] , [ 0 , 0 , 2 , 0 , 0 , 0 ] , [ 0 , 0 , 3 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 1 ] ] &gt; iex&gt; t = Nx . tensor ( [ ...&gt; [ 7 , 2 , 5 , 3 , 8 ] , ...&gt; [ 3 , 8 , 9 , 3 , 4 ] , ...&gt; [ 1 , 5 , 7 , 5 , 6 ] , ...&gt; [ 0 , 6 , 2 , 10 , 2 ] ...&gt; ] ) iex&gt; opts = [ strides : [ 2 , 2 ] , padding : :valid ] iex&gt; Nx . window_scatter_max ( t , Nx . tensor ( [ [ 2 , 6 ] , [ 3 , 1 ] ] ) , 0 , { 2 , 3 } , opts ) # Nx.Tensor &lt; s64 [ 4 ] [ 5 ] [ [ 0 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 8 , 0 , 0 ] , [ 0 , 0 , 3 , 0 , 0 ] , [ 0 , 0 , 0 , 1 , 0 ] ] &gt;","ref":"Nx.html#window_scatter_max/5","title":"Nx.window_scatter_max/5","type":"function"},{"doc":"Performs a window_reduce to select the minimum index in each window of the input tensor according to and scatters source tensor to corresponding minimum indices in the output tensor. Output tensor is initialized as a full tensor with values init_value . If indices overlap, adds overlapping source values. The shape of the source tensor must match the valid windows in the input tensor. This means the shape of the source tensor must match the shape of the input tensor after a window_reduce op with padding padding and strides strides . This function is the gradient of window_min . Examples iex&gt; t = Nx . tensor ( [ ...&gt; [ 7 , 2 , 5 , 3 , 10 , 2 ] , ...&gt; [ 3 , 8 , 9 , 3 , 4 , 2 ] , ...&gt; [ 1 , 5 , 7 , 5 , 6 , 1 ] , ...&gt; [ 0 , 6 , 2 , 7 , 2 , 8 ] ...&gt; ] ) iex&gt; opts = [ strides : [ 2 , 3 ] , padding : :valid ] iex&gt; Nx . window_scatter_min ( t , Nx . tensor ( [ [ 2 , 6 ] , [ 3 , 1 ] ] ) , 0 , { 2 , 3 } , opts ) # Nx.Tensor &lt; s64 [ 4 ] [ 6 ] [ [ 0 , 2 , 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 , 0 , 6 ] , [ 0 , 0 , 0 , 0 , 0 , 1 ] , [ 3 , 0 , 0 , 0 , 0 , 0 ] ] &gt; iex&gt; t = Nx . tensor ( [ ...&gt; [ 7 , 2 , 5 , 3 , 8 ] , ...&gt; [ 3 , 8 , 9 , 3 , 4 ] , ...&gt; [ 1 , 5 , 7 , 5 , 6 ] , ...&gt; [ 0 , 6 , 2 , 10 , 2 ] ...&gt; ] ) iex&gt; opts = [ strides : [ 2 , 2 ] , padding : :valid ] iex&gt; Nx . window_scatter_min ( t , Nx . tensor ( [ [ 2 , 6 ] , [ 3 , 1 ] ] ) , 0 , { 2 , 3 } , opts ) # Nx.Tensor &lt; s64 [ 4 ] [ 5 ] [ [ 0 , 2 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 6 , 0 ] , [ 0 , 0 , 0 , 0 , 0 ] , [ 3 , 0 , 0 , 0 , 1 ] ] &gt;","ref":"Nx.html#window_scatter_min/5","title":"Nx.window_scatter_min/5","type":"function"},{"doc":"Sums over each window of size window_dimensions in the given tensor, producing a tensor that contains the same number of elements as valid positions of the window. You may optionally specify :strides which is a tuple of non-zero steps to take along each axis between each window. You may also optionally specify :padding which is either one of :valid (no padding) or :same (pad so output shape is the same as input shape) or a general padding configuration for each dimension in the input tensor. Your padding configuration cannot include any negative pad values. You may only specify padding for the high and low edges of the given dimension. Pads with 0 . Examples iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_sum ( t , { 1 , 2 , 1 } ) # Nx.Tensor &lt; s64 [ 2 ] [ 1 ] [ 3 ] [ [ [ 5 , 7 , 9 ] ] , [ [ 5 , 7 , 9 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] , [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ] ) iex&gt; Nx . window_sum ( t , { 2 , 2 , 1 } , strides : [ 1 , 2 , 3 ] , padding : [ { 0 , 1 } , { 2 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ 0 , 0 ] , [ 0 , 18 ] ] , [ [ 0 , 0 ] , [ 0 , 9 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4.0 , 2.0 , 3.0 ] , [ 2.0 , 5.0 , 6.5 ] ] , [ [ 1.2 , 2.2 , 3.2 ] , [ 4.0 , 5.0 , 6.2 ] ] ] ) iex&gt; Nx . window_sum ( t , { 2 , 1 , 1 } , strides : [ 2 , 1 , 1 ] , padding : [ { 1 , 1 } , { 0 , 0 } , { 1 , 1 } ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 5 ] [ [ [ 0.0 , 4.0 , 2.0 , 3.0 , 0.0 ] , [ 0.0 , 2.0 , 5.0 , 6.5 , 0.0 ] ] , [ [ 0.0 , 1.2000000476837158 , 2.200000047683716 , 3.200000047683716 , 0.0 ] , [ 0.0 , 4.0 , 5.0 , 6.199999809265137 , 0.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 1 ] ] iex&gt; Nx . window_sum ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 3 ] [ [ [ 6 , 3 , 4 ] , [ 6 , 3 , 8 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : :valid , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_sum ( t , { 1 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 1 ] [ 2 ] [ 2 ] [ [ [ 5 , 5 ] , [ 5 , 9 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ [ 4 , 2 , 1 , 3 ] , [ 4 , 2 , 1 , 7 ] ] , [ [ 1 , 2 , 5 , 7 ] , [ 1 , 8 , 9 , 2 ] ] ] ) iex&gt; opts = [ strides : [ 2 , 1 , 1 ] , padding : [ { 2 , 1 } , { 3 , 1 } , { 1 , 0 } ] , window_dilations : [ 1 , 2 , 2 ] ] iex&gt; Nx . window_sum ( t , { 2 , 1 , 2 } , opts ) # Nx.Tensor &lt; s64 [ 2 ] [ 6 ] [ 3 ] [ [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] ] , [ [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 0 , 0 , 0 ] , [ 4 , 11 , 14 ] , [ 10 , 15 , 19 ] , [ 0 , 0 , 0 ] ] ] &gt;","ref":"Nx.html#window_sum/3","title":"Nx.window_sum/3","type":"function"},{"doc":"","ref":"Nx.html#t:axes/0","title":"Nx.axes/0","type":"type"},{"doc":"","ref":"Nx.html#t:axis/0","title":"Nx.axis/0","type":"type"},{"doc":"","ref":"Nx.html#t:shape/0","title":"Nx.shape/0","type":"type"},{"doc":"Represents a numerical value. Can be a plain number, a Complex number or an Nx.Tensor . See also: is_tensor/1","ref":"Nx.html#t:t/0","title":"Nx.t/0","type":"type"},{"doc":"","ref":"Nx.html#t:template/0","title":"Nx.template/0","type":"type"},{"doc":"Common constants used in computations.","ref":"Nx.Constants.html","title":"Nx.Constants","type":"module"},{"doc":"Returns the imaginary constant. Accepts the same options as Nx.tensor/2 Examples iex&gt; Nx.Constants . i ( ) # Nx.Tensor &lt; c64 0.0 + 1.0 i &gt; iex&gt; Nx.Constants . i ( type : { :c , 128 } ) # Nx.Tensor &lt; c128 0.0 + 1.0 i &gt; Error cases iex&gt; Nx.Constants . i ( type : { :f , 32 } ) ** (ArgumentError) invalid type for complex number. Expected {:c, 64} or {:c, 128}, got: {:f, 32}","ref":"Nx.Constants.html#i/1","title":"Nx.Constants.i/1","type":"function"},{"doc":"Returns infinity in f32.","ref":"Nx.Constants.html#infinity/0","title":"Nx.Constants.infinity/0","type":"function"},{"doc":"Returns infinity. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . infinity ( { :bf , 16 } ) # Nx.Tensor &lt; bf16 Inf &gt; iex&gt; Nx.Constants . infinity ( { :f , 16 } ) # Nx.Tensor &lt; f16 Inf &gt; iex&gt; Nx.Constants . infinity ( { :f , 32 } ) # Nx.Tensor &lt; f32 Inf &gt; iex&gt; Nx.Constants . infinity ( { :f , 64 } ) # Nx.Tensor &lt; f64 Inf &gt;","ref":"Nx.Constants.html#infinity/2","title":"Nx.Constants.infinity/2","type":"function"},{"doc":"Returns a scalar tensor with the maximum value for the given type. It is infinity for floating point tensors. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . max ( { :u , 8 } ) # Nx.Tensor &lt; u8 255 &gt; iex&gt; Nx.Constants . max ( { :f , 32 } ) # Nx.Tensor &lt; f32 Inf &gt;","ref":"Nx.Constants.html#max/2","title":"Nx.Constants.max/2","type":"function"},{"doc":"Returns a scalar tensor with the maximum finite value for the given type. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . max_finite ( { :u , 8 } ) # Nx.Tensor &lt; u8 255 &gt; iex&gt; Nx.Constants . max_finite ( { :s , 16 } ) # Nx.Tensor &lt; s16 32677 &gt; iex&gt; Nx.Constants . max_finite ( { :f , 32 } ) # Nx.Tensor &lt; f32 3.4028234663852886e38 &gt;","ref":"Nx.Constants.html#max_finite/2","title":"Nx.Constants.max_finite/2","type":"function"},{"doc":"Returns a scalar tensor with the minimum value for the given type. It is negative infinity for floating point tensors. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . min ( { :u , 8 } ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx.Constants . min ( { :f , 32 } ) # Nx.Tensor &lt; f32 - Inf &gt;","ref":"Nx.Constants.html#min/2","title":"Nx.Constants.min/2","type":"function"},{"doc":"Returns a scalar tensor with the minimum finite value for the given type. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . min_finite ( { :u , 8 } ) # Nx.Tensor &lt; u8 0 &gt; iex&gt; Nx.Constants . min_finite ( { :s , 16 } ) # Nx.Tensor &lt; s16 - 32678 &gt; iex&gt; Nx.Constants . min_finite ( { :f , 32 } ) # Nx.Tensor &lt; f32 - 3.4028234663852886e38 &gt;","ref":"Nx.Constants.html#min_finite/2","title":"Nx.Constants.min_finite/2","type":"function"},{"doc":"Returns infinity in f32.","ref":"Nx.Constants.html#nan/0","title":"Nx.Constants.nan/0","type":"function"},{"doc":"Returns NaN (Not a Number). Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . nan ( { :bf , 16 } ) # Nx.Tensor &lt; bf16 NaN &gt; iex&gt; Nx.Constants . nan ( { :f , 16 } ) # Nx.Tensor &lt; f16 NaN &gt; iex&gt; Nx.Constants . nan ( { :f , 32 } ) # Nx.Tensor &lt; f32 NaN &gt; iex&gt; Nx.Constants . nan ( { :f , 64 } ) # Nx.Tensor &lt; f64 NaN &gt;","ref":"Nx.Constants.html#nan/2","title":"Nx.Constants.nan/2","type":"function"},{"doc":"Returns infinity in f32.","ref":"Nx.Constants.html#neg_infinity/0","title":"Nx.Constants.neg_infinity/0","type":"function"},{"doc":"Returns negative infinity. Options :backend - a backend to allocate the tensor on. Examples iex&gt; Nx.Constants . neg_infinity ( { :bf , 16 } ) # Nx.Tensor &lt; bf16 - Inf &gt; iex&gt; Nx.Constants . neg_infinity ( { :f , 16 } ) # Nx.Tensor &lt; f16 - Inf &gt; iex&gt; Nx.Constants . neg_infinity ( { :f , 32 } ) # Nx.Tensor &lt; f32 - Inf &gt; iex&gt; Nx.Constants . neg_infinity ( { :f , 64 } ) # Nx.Tensor &lt; f64 - Inf &gt;","ref":"Nx.Constants.html#neg_infinity/2","title":"Nx.Constants.neg_infinity/2","type":"function"},{"doc":"Numerical functions. A numerical function is a subset of Elixir tailored for numerical computations. For example, the following function: defmodule MyModule do import Nx.Defn defn softmax ( t ) do Nx . exp ( t ) / Nx . sum ( Nx . exp ( t ) ) end end will work with scalars, vector, matrices, and n-dimensional tensors. Depending on your compiler of choice, the code can even be JIT-compiled and run either on the CPU or GPU. To support these features, defn is a subset of Elixir. It replaces Elixir's Kernel by Nx.Defn.Kernel . Nx.Defn.Kernel provides tensor-aware operators, such as + , - , etc, while also preserving many high-level constructs known to Elixir developers, such as pipe operator, aliases, conditionals, pattern-matching, the access syntax, and more: For example, the code above can also be written as: defmodule MyModule do import Nx.Defn defn softmax ( t ) do t |&gt; Nx . exp ( t ) |&gt; then ( &amp; &amp;1 / Nx . sum ( &amp;1 ) ) end end Please consult Nx.Defn.Kernel for a complete reference. Operators defn attempts to keep as close to the Elixir semantics as possible but that's not achievable. For example, mathematical and bitwise operators ( + , - , &amp;&amp;&amp; , &lt;&lt;&lt; , etc.) in Elixir work on numbers, which means mapping them to tensors is straight-forward and they largely preserve the same semantics, except they are now multi-dimensional. On the other hand, the logical operators and , or , and not work with booleans in Elixir ( true and false ), which map to 0 and 1 in defn . Therefore, when working with logical operators inside defn , 0 is considered false and all other numbers are considered true , which is represented as the number 1 . For example, in defn , 0 and 1 as well as 0 and 2 return 0 , while 1 and 1 or 1 and -1 will return 1 . The same semantics apply to conditional expressions inside defn , such as if , while , etc. JIT compilers The power of Nx.Defn is given by its compilers. The default compiler is Nx.Defn.Evaluator , which evalutes the code. You can use jit/3 to compile a function on the fly using a different compiler, such as EXLA : fun = Nx.Defn . jit ( &amp; MyModule . softmax / 1 , compiler : EXLA ) fun . ( my_tensor ) The above will return an anonymous function that optimizes, compiles, and run softmax on the fly on the CPU (or the GPU) if available. You can also change the default compiler for all numerical definitions ( defn ) by setting the default options. This can be done in your config/*.exs files as follows: config :nx , :default_defn_options , compiler : EXLA Now calling MyModule.softmax(my_tensor) will use EXLA even without wrapping it in jit/2 . However, note that compilation may be quite time consuming on the first invocation, that's why it is often preferred to use the compiler: EXLA option when calling the functions in this module instead. EXLA, in particular, also exports a EXLA.jit/2 function for convenience. defn functions are compiled when they are invoked, based on the type and shapes of the tensors given as arguments. The compilation is then cached based on the tensors shapes and types. Calling the same function with a tensor of different values but same shape and type means no recompilation is performed. For those interested in writing custom compilers, see Nx.Defn.Compiler . Invoking custom Elixir code Inside defn you can only call other defn functions and the functions in the Nx module. However, it is possible to use transforms, defined with either deftransform or deftransformp to invoke any Elixir code. You can call code which was defined with deftransform from another module: defmodule MyRemoteModule do import Nx.Defn deftransform remote_elixir_code ( value ) do IO . inspect ( value ) end end defn add_and_mult ( a , b , c ) do res = a * b + c MyRemoteModule . remote_elixir_code ( res ) end You can also define and call a private transform defined through deftransformp : defn add_and_mult ( a , b , c ) do res = a * b + c custom_elixir_code ( res ) end deftransformp custom_elixir_code ( value ) , do : IO . inspect ( value ) For example, the two code snippets invoke IO.inspect/1 , which is not a defn function, with the value of res . This is useful as it allows developers to transform defn code to optimize, add new properties, and so on. The only difference between using deftransform and deftransformp is wether you want to expose and share the code with other modules, just like def and defp . Transforms can also be used to manipulate Elixir data structures, such as options. defn expects all inputs to be tensors, with the exception of a default argument (declared with \\\\ ) which will be treated as options. For example, imagine you want to support options where the :axis key is required. While you can't invoke Keyword directly, you can do it via a transform: defn sum_axis ( t , opts \\\\ [ ] ) do opts = keyword! ( opts , [ :axis ] ) axis = get_axis ( opts ) Nx . sum ( t , axes : [ axis ] ) end deftransformp get_axis ( opts ) , do : Keyword . fetch! ( opts , :axis ) Inputs and outputs types Nx and defn expect the arguments to be numbers, tensors, or one composite data type that implements Nx.LazyContainer . Tuples and maps implement Nx.LazyContainer by default. As previously described, defn are cached based on the shape, type, and names of the input tensors, but not their values. defn also accepts two special arguments: functions (or tuples of functions) and lists (most commonly as keyword lists). Those values are passed as is to numerical definitions and cached as a whole. For this reason, you must never capture tensors in functions or pass tensors in keyword lists. When numbers are given as arguments, they are always immediately converted to tensors on invocation. If you want to keep numbers as is or if you want to pass any other value to numerical definitions, they must be given as keyword lists. Default arguments defn functions support default arguments. They are typically used as options. For example, imagine you want to create a function named zeros, which returns a tensor of zeroes with a given type and shape. It could be implemented like this: defn zeros ( opts \\\\ [ ] ) do opts = keyword! ( opts , type : { :f , 32 } , shape : { } ) Nx . broadcast ( Nx . tensor ( 0 , type : opts [ :type ] ) , opts [ :shape ] ) end The function above accepts opts which are then validated and given default values via the keyword!/2 function. Note that while it is possible to access options via the Access syntax, such as opts[:shape] , it is not possible to directly call functions in the Keyword module inside defn . To freely manipulate any Elixir value inside defn , you have to use transforms, as described in the &quot;Invoking custom Elixir code&quot; section. Important! When it comes to JIT compilation, each different set of options (as well as anonymous functions) will lead to a different compilation of the numerical function. Furthermore, if tensors are given through keyword lists, they won't be cached effectively. Tensors in defn are cached based on their shape and type, not their value, but this is not true if the tensor is given via a default argument or captured by an anonymous function. For this reason, it is extremely discouraged to pass tensors through anonymous functions and default arguments . Working with maps and structs While Nx supports maps in defn , you must be careful if your numerical definitions are receiving maps and returning maps. For example, imagine this code: defn update_a ( map ) do %{ map | a : Nx . add ( map . a , 1 ) } end The following code increments the value under the key :a by 1. However, because the function receives the whole map and returns the whole map, it means if the map has 120 keys, the whole map will be copied to the CPU/GPU, and then brought back. However, if you do this instead: defn update_a ( map ) do Nx . add ( map . a , 1 ) end And then update the map on Elixir, outside of defn : %{ map | a : update_a ( map ) } Nx will only send the parts of the map that matters.","ref":"Nx.Defn.html","title":"Nx.Defn","type":"module"},{"doc":"Compiles the given anonymous function with the given tensor shapes. While jit/2 compiles a function just-in time based on the input shapes, this function precompiles the given anonymous function based on the input shapes. This can be beneficial for large numerical definitions, where the cache mechanism in jit/2 may take miliseconds. For example, take the following definition: defn softmax ( t ) , do : Nx . exp ( t ) / Nx . sum ( Nx . exp ( t ) ) You can jit and then apply it as: fun = Nx.Defn . compile ( &amp; softmax / 1 , [ Nx . template ( { 3 } , { :s , 64 } ) ] , compiler : EXLA ) fun . ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) You can also pass a mixture of templates and options when compiling a function. In such cases, you must only pass the inputs when invoking the compiled function, as the options will already be embedded in its compiled value: fun = Nx.Defn . compile ( &amp; Nx . sum / 2 , [ Nx . template ( { 2 , 2 } , { :s , 64 } ) , [ axes : [ 1 ] ] ] ) fun . ( Nx . iota ( { 2 , 2 } ) ) If the input tensors do not match the shape of the tensors given on compilation, it will raise. Options :compiler - the compiler for the JIT compilation :hooks - a map of hooks to execute. See Nx.Defn.Kernel.hook/3","ref":"Nx.Defn.html#compile/3","title":"Nx.Defn.compile/3","type":"function"},{"doc":"Wraps an anonymous function to return its underlying defn expression. Warning This function must be invoked for debugging purposes only. Options :hooks - a map of hooks to execute. See Nx.Defn.Kernel.hook/3","ref":"Nx.Defn.html#debug_expr/2","title":"Nx.Defn.debug_expr/2","type":"function"},{"doc":"Invokes the anonymous function to return its underlying defn expression. Warning This function must be invoked for debugging purposes only. It accepts the same options as debug_expr/2 .","ref":"Nx.Defn.html#debug_expr_apply/3","title":"Nx.Defn.debug_expr_apply/3","type":"function"},{"doc":"Gets the default options for the current process.","ref":"Nx.Defn.html#default_options/0","title":"Nx.Defn.default_options/0","type":"function"},{"doc":"Sets the default options for defn in the current process. The options defined here apply to all future invocations of defn done by the current process. It also applies to calls to the jit/3 and stream/3 functions in this module. The default options are stored only in the process dictionary and override any global options. This means if you start a separate process, such as Task , the default options must be set on the new process too. This function is mostly used for scripting and testing. In your applications, you typically set the default options in your config files: config :nx , :default_defn_options , [ compiler : EXLA , client : :cuda ]","ref":"Nx.Defn.html#default_options/1","title":"Nx.Defn.default_options/1","type":"function"},{"doc":"Defines a public numerical function.","ref":"Nx.Defn.html#defn/2","title":"Nx.Defn.defn/2","type":"macro"},{"doc":"Defines a private numerical function. Private numerical functions are always inlined by their callers at compilation time. This happens to all local function calls within defn .","ref":"Nx.Defn.html#defnp/2","title":"Nx.Defn.defnp/2","type":"macro"},{"doc":"Can be used to define bodiless clauses for multi-clause transforms. See also: deftransform/2 Examples deftransform foo ( bar , baz \\ 1 ) deftransform foo ( bar , 1 ) , do : bar deftransform foo ( bar , baz ) , do : bar + baz","ref":"Nx.Defn.html#deftransform/1","title":"Nx.Defn.deftransform/1","type":"macro"},{"doc":"Defines a transform that executes the given fun with arg when building defn expressions. Example Take the following defn expression: defn tanh_power ( a , b ) do Nx . tanh ( a ) + Nx . power ( b , 2 ) end Let's see a trivial example, which is to use IO.inspect/1 to print a tensor expression at definition time: defn tanh_power ( a , b ) do Nx . tanh ( a ) + Nx . power ( b , 2 ) |&gt; my_inspect ( ) end deftransformp my_inspect ( expr ) , do : IO . inspect ( expr ) Or: defn tanh_power ( a , b ) do res = Nx . tanh ( a ) + Nx . power ( b , 2 ) my_inspect ( res ) res end When invoked in both cases, it will print the expression being built by defn : # Nx.Defn.Expr &lt; parameter a parameter c b = tanh [ a ] ( ) d = power [ c , 2 ] ( ) e = add [ b , d ] ( ) &gt; Although, for convenience, you might use print_expr/2 instead.","ref":"Nx.Defn.html#deftransform/2","title":"Nx.Defn.deftransform/2","type":"macro"},{"doc":"Private function version for deftransform/1","ref":"Nx.Defn.html#deftransformp/1","title":"Nx.Defn.deftransformp/1","type":"macro"},{"doc":"Private function version for deftransform/2","ref":"Nx.Defn.html#deftransformp/2","title":"Nx.Defn.deftransformp/2","type":"macro"},{"doc":"Sets the default options globally. The options defined here apply to all future invocations of defn . It also applies to calls to the jit/3 and stream/3 functions in this module. You must avoid calling this function at runtime. It is mostly useful during scripts or code notebooks to set a default. If you need to configure a global default options in your applications, you can do so in your config/*.exs files: config :nx , :default_defn_options , [ compiler : EXLA , client : :cuda ]","ref":"Nx.Defn.html#global_default_options/1","title":"Nx.Defn.global_default_options/1","type":"function"},{"doc":"Receives an anonymous function and returns a new anonymous function that returns the gradient of the input function when invoked. Examples iex&gt; fun = Nx.Defn . grad ( fn x -&gt; Nx . sin ( x ) end ) iex&gt; fun . ( Nx . tensor ( 0 ) ) # Nx.Tensor &lt; f32 1.0 &gt;","ref":"Nx.Defn.html#grad/1","title":"Nx.Defn.grad/1","type":"function"},{"doc":"Computes the gradient of the given var on fun . The result of the grad function must be a scalar tensor. If a non-scalar tensor is given, it is assumed the additional dimensions are batch dimensions. Examples defn tanh_grad ( t ) do grad ( t , &amp; Nx . tanh / &amp;1 ) end To differentiate on multiple vars, pass a tuple as first argument: defn tanh_power_grad ( a , b ) do grad ( { a , b } , fn { a , b } -&gt; Nx . tanh ( a ) + Nx . power ( b , 2 ) end ) end var_or_vars can be any Nx.Container with one or multiple tensors.","ref":"Nx.Defn.html#grad/2","title":"Nx.Defn.grad/2","type":"function"},{"doc":"Wraps an anonymous function with just-in-time compilation. Once invoked, the wrapped anonymous function will perform just in time compilation with the configured compiler. For example, take the following definition: defn softmax ( t ) , do : Nx . exp ( t ) / Nx . sum ( Nx . exp ( t ) ) You can jit and then apply it as: fun = Nx.Defn . jit ( &amp; softmax / 1 , compiler : EXLA ) fun . ( Nx . tensor ( [ 1 , 2 , 3 ] ) ) Options :compiler - the compiler for the JIT compilation :hooks - a map of hooks to execute. See Nx.Defn.Kernel.hook/3 :on_conflict - what to do if a JIT compilation is already in place. It may be :raise (the default), :force (forces a new JIT compilation), or :reuse (reuses the exiting JIT compilation). It is not recommended to set the :compiler option when reusing.","ref":"Nx.Defn.html#jit/2","title":"Nx.Defn.jit/2","type":"function"},{"doc":"","ref":"Nx.Defn.html#jit/3","title":"Nx.Defn.jit/3","type":"function"},{"doc":"Invokes the anonymous function with just-in-time compilation. This function is equivalent to calling jit/2 and then applying the given arguments to the anonymous function. For example, take the following definition: defn softmax ( t ) , do : Nx . exp ( t ) / Nx . sum ( Nx . exp ( t ) ) You can jit_apply/3 it as: Nx.Defn . jit_apply ( &amp; softmax / 1 , [ Nx . tensor ( [ 1 , 2 , 3 ] ) ] , compiler : EXLA ) It accepts the same options as jit/2 .","ref":"Nx.Defn.html#jit_apply/3","title":"Nx.Defn.jit_apply/3","type":"function"},{"doc":"","ref":"Nx.Defn.html#jit_or_apply/3","title":"Nx.Defn.jit_or_apply/3","type":"function"},{"doc":"Starts streaming the given anonymous function with just-in-time compilation. At least two arguments are expected: The first argument is a tensor template of the data to be streamed in The second argument is a tensor with the stream initial state The streaming function must return a two element tuple, the first element is the data to be sent and the second is the accumulator. For each streamed chunk, you must call Nx.Stream.send/2 and Nx.Stream.recv/1 . You don't need to call recv immediately after send , but doing so can be a useful mechanism to provide backpressure. Once all chunks are sent, you must use Nx.Stream.done/1 to receive the accumulated result. Let's see an example: defmodule Streamed do import Nx.Defn defn sum ( tensor , acc ) do { acc , tensor + acc } end end Now let's invoke it: stream = Nx.Defn . stream ( &amp; Streamed . sum / 2 , [ Nx . template ( { } , { :s , 64 } ) , 0 ] ) for i &lt;- 1 .. 5 do Nx.Stream . send ( stream , i ) IO . inspect { :chunk , Nx.Stream . recv ( stream ) } end IO . inspect { :result , Nx.Stream . done ( stream ) } It will print: { :chunk , 0 } { :chunk , 1 } { :chunk , 2 } { :chunk , 3 } { :chunk , 4 } { :result , 5 } Options :hooks - a map of hooks to execute. See Nx.Defn.Kernel.hook/3","ref":"Nx.Defn.html#stream/3","title":"Nx.Defn.stream/3","type":"function"},{"doc":"Receives an anonymous function and returns a new anonymous function that returns the value and gradient of the input function when invoked. Examples iex&gt; fun = Nx.Defn . value_and_grad ( fn x -&gt; Nx . sin ( x ) end ) iex&gt; { value , grad } = fun . ( Nx . tensor ( 0 ) ) iex&gt; value # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; grad # Nx.Tensor &lt; f32 1.0 &gt;","ref":"Nx.Defn.html#value_and_grad/1","title":"Nx.Defn.value_and_grad/1","type":"function"},{"doc":"Computes the value and gradient of the given var on fun with an optional data transformation. It returns a tuple with the value and the gradient. Examples defn tanh_grad ( t ) do value_and_grad ( t , &amp; Nx . tanh / &amp;1 ) end To differentiate on multiple vars, pass a tuple as first argument: defn tanh_power_grad ( a , b ) do value_and_grad ( { a , b } , fn { a , b } -&gt; Nx . tanh ( a ) + Nx . power ( b , 2 ) end ) end var_or_vars can be any Nx.Container with one or multiple tensors. transform allows you to transform the expression before the gradient is calculated. This enables optimizations that reuse parts of expressions. As an example, consider the following objective function: defn objective ( predict_fn , loss_fn , params , inputs , targets ) do preds = predict_fn . ( params , inputs ) loss = loss_fn . ( preds , targets ) { preds , loss } end You can compute the gradient with respect to just the loss function by applying a transform: { { preds , loss } , gradient } = value_and_grad ( params , &amp; objective ( predict_fn , loss_fn , &amp;1 , inputs , targets ) , &amp; elem ( &amp;1 , 1 ) ) preds can be re-used to compute other metrics such as accuracy, absolute error, etc. without having to do another forward pass.","ref":"Nx.Defn.html#value_and_grad/3","title":"Nx.Defn.value_and_grad/3","type":"function"},{"doc":"All imported functionality available inside defn blocks.","ref":"Nx.Defn.Kernel.html","title":"Nx.Defn.Kernel","type":"module"},{"doc":"Element-wise bitwise AND operation. Only integer tensors are supported. It delegates to Nx.bitwise_and/2 (supports broadcasting). Examples defn and_or ( a , b ) do { a &amp;&amp;&amp; b , a ||| b } end","ref":"Nx.Defn.Kernel.html#&&&/2","title":"Nx.Defn.Kernel.&&&/2","type":"function"},{"doc":"Element-wise multiplication operator. It delegates to Nx.power/2 (supports broadcasting). Examples defn power ( a , b ) do a * * b end","ref":"Nx.Defn.Kernel.html#**/2","title":"Nx.Defn.Kernel.**/2","type":"function"},{"doc":"Element-wise multiplication operator. It delegates to Nx.multiply/2 (supports broadcasting). Examples defn multiply ( a , b ) do a * b end","ref":"Nx.Defn.Kernel.html#*/2","title":"Nx.Defn.Kernel.*/2","type":"function"},{"doc":"Element-wise unary plus operator. Simply returns the given argument. Examples defn plus_and_minus ( a ) do { + a , - a } end","ref":"Nx.Defn.Kernel.html#+/1","title":"Nx.Defn.Kernel.+/1","type":"macro"},{"doc":"Element-wise addition operator. It delegates to Nx.add/2 (supports broadcasting). Examples defn add ( a , b ) do a + b end","ref":"Nx.Defn.Kernel.html#+/2","title":"Nx.Defn.Kernel.+/2","type":"function"},{"doc":"Element-wise unary plus operator. It delegates to Nx.negate/1 . Examples defn plus_and_minus ( a ) do { + a , - a } end","ref":"Nx.Defn.Kernel.html#-/1","title":"Nx.Defn.Kernel.-/1","type":"macro"},{"doc":"Element-wise subtraction operator. It delegates to Nx.subtract/2 (supports broadcasting). Examples defn subtract ( a , b ) do a - b end","ref":"Nx.Defn.Kernel.html#-/2","title":"Nx.Defn.Kernel.-/2","type":"function"},{"doc":"Builds a range. Ranges are inclusive and both sides must be integers. The step of the range is computed based on the first and last values of the range. Examples iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; t [ 1 .. 2 ] # Nx.Tensor &lt; s64 [ 2 ] [ 2 , 3 ] &gt;","ref":"Nx.Defn.Kernel.html#../2","title":"Nx.Defn.Kernel.../2","type":"function"},{"doc":"Builds a range with step. Ranges are inclusive and both sides must be integers. Examples iex&gt; t = Nx . tensor ( [ 1 , 2 , 3 ] ) iex&gt; t [ 1 .. 2 // 1 ] # Nx.Tensor &lt; s64 [ 2 ] [ 2 , 3 ] &gt;","ref":"Nx.Defn.Kernel.html#..///3","title":"Nx.Defn.Kernel...///3","type":"function"},{"doc":"Element-wise division operator. It delegates to Nx.divide/2 (supports broadcasting). Examples defn divide ( a , b ) do a / b end","ref":"Nx.Defn.Kernel.html#//2","title":"Nx.Defn.Kernel.//2","type":"function"},{"doc":"Element-wise inequality operation. It delegates to Nx.not_equal/2 . Examples defn check_inequality ( a , b ) do a != b end","ref":"Nx.Defn.Kernel.html#!=/2","title":"Nx.Defn.Kernel.!=/2","type":"macro"},{"doc":"Element-wise less than operation. It delegates to Nx.less/2 . Examples defn check_less_than ( a , b ) do a &lt; b end","ref":"Nx.Defn.Kernel.html#%3C/2","title":"Nx.Defn.Kernel.</2","type":"macro"},{"doc":"Element-wise left shift operation. Only integer tensors are supported. It delegates to Nx.left_shift/2 (supports broadcasting). Examples defn shift_left_and_right ( a , b ) do { a &lt;&lt;&lt; b , a &gt;&gt;&gt; b } end","ref":"Nx.Defn.Kernel.html#%3C%3C%3C/2","title":"Nx.Defn.Kernel.<<</2","type":"function"},{"doc":"Element-wise less-equal operation. It delegates to Nx.less_equal/2 . Examples defn check_less_equal ( a , b ) do a &lt;= b end","ref":"Nx.Defn.Kernel.html#%3C=/2","title":"Nx.Defn.Kernel.<=/2","type":"macro"},{"doc":"Concatenates two strings. Equivalent to Kernel.&lt;&gt;/2 .","ref":"Nx.Defn.Kernel.html#%3C%3E/2","title":"Nx.Defn.Kernel.<>/2","type":"macro"},{"doc":"Element-wise equality operation. It delegates to Nx.equal/2 . Examples defn check_equality ( a , b ) do a == b end","ref":"Nx.Defn.Kernel.html#==/2","title":"Nx.Defn.Kernel.==/2","type":"macro"},{"doc":"Element-wise greater than operation. It delegates to Nx.greater/2 . Examples defn check_greater_than ( a , b ) do a &gt; b end","ref":"Nx.Defn.Kernel.html#%3E/2","title":"Nx.Defn.Kernel.>/2","type":"macro"},{"doc":"Element-wise greater-equal operation. It delegates to Nx.greater_equal/2 . Examples defn check_greater_equal ( a , b ) do a &gt;= b end","ref":"Nx.Defn.Kernel.html#%3E=/2","title":"Nx.Defn.Kernel.>=/2","type":"macro"},{"doc":"Element-wise right shift operation. Only integer tensors are supported. It delegates to Nx.right_shift/2 (supports broadcasting). Examples defn shift_left_and_right ( a , b ) do { a &lt;&lt;&lt; b , a &gt;&gt;&gt; b } end","ref":"Nx.Defn.Kernel.html#%3E%3E%3E/2","title":"Nx.Defn.Kernel.>>>/2","type":"function"},{"doc":"Reads a module attribute at compilation time. It is useful to inject code constants into defn . It delegates to Kernel.@/1 . Examples @two_per_two Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) defn add_2x2_attribute ( t ) , do : t + @two_per_two","ref":"Nx.Defn.Kernel.html#@/1","title":"Nx.Defn.Kernel.@/1","type":"macro"},{"doc":"Defines an alias, as in Kernel.SpecialForms.alias/2 . An alias allows you to refer to a module using its aliased name. For example: defn some_fun ( t ) do alias Math.Helpers , as : MH MH . fft ( t ) end If the :as option is not given, the alias defaults to the last part of the given alias. For example, alias Math.Helpers is equivalent to: alias Math.Helpers , as : Helpers Finally, note that aliases define outside of a function also apply to the function, as they have lexical scope: alias Math.Helpers , as : MH defn some_fun ( t ) do MH . fft ( t ) end","ref":"Nx.Defn.Kernel.html#alias/2","title":"Nx.Defn.Kernel.alias/2","type":"macro"},{"doc":"Element-wise logical AND operation. Zero is considered false, all other numbers are considered true. It delegates to Nx.logical_and/2 (supports broadcasting). Examples defn and_or ( a , b ) do { a and b , a or b } end","ref":"Nx.Defn.Kernel.html#and/2","title":"Nx.Defn.Kernel.and/2","type":"macro"},{"doc":"Asserts the keyword list has the given keys. If it succeeds, it returns the given keyword list. Raises an error otherwise. Examples To assert the tensor is a scalar, you can pass the empty tuple shape : iex&gt; assert_keys ( [ one : 1 , two : 2 ] , [ :one , :two ] ) [ one : 1 , two : 2 ] If the keys are not available, an error is raised: iex&gt; assert_keys ( [ one : 1 , two : 2 ] , [ :three ] ) ** (ArgumentError) expected key :three in keyword list, got: [one: 1, two: 2]","ref":"Nx.Defn.Kernel.html#assert_keys/2","title":"Nx.Defn.Kernel.assert_keys/2","type":"function"},{"doc":"Attaches a token to an expression. See hook/3 .","ref":"Nx.Defn.Kernel.html#attach_token/2","title":"Nx.Defn.Kernel.attach_token/2","type":"function"},{"doc":"Pattern matches the result of expr against the given clauses. For example: case Nx . shape ( tensor ) do { _ } -&gt; implementation_for_rank_one ( tensor ) { _ , _ } -&gt; implementation_for_rank_two ( tensor ) _ -&gt; implementation_for_rank_n ( tensor ) end Opposite to cond/2 and if/2 , which can execute the branching in the device, case s are always expanded when building the expression, and never on the device. This allows case/2 to work very similarly to Elixir's own Kernel.SpecialForms.case/2 , with only the following restrictions in place: case inside defn only accepts structs, atoms, integers, and tuples as arguments case can match on struct names but not on its fields guards in case inside defn can only access variables defined within the pattern Here is an example of case with guards: case Nx . shape ( tensor ) do { x , y } when x &gt; y -&gt; implementation_for_tall ( tensor ) { x , y } when x &lt; y -&gt; implementation_for_wide ( tensor ) { x , x } -&gt; implementation_for_square ( tensor ) end","ref":"Nx.Defn.Kernel.html#case/2","title":"Nx.Defn.Kernel.case/2","type":"macro"},{"doc":"Evaluates the expression corresponding to the first clause that evaluates to a truthy value. It has the format of: cond do condition1 -&gt; expr1 condition2 -&gt; expr2 true -&gt; expr3 end The conditions must be a scalar. Zero is considered false, any other number is considered true. The booleans false and true are supported, but any other value will raise. All clauses are normalized to the same type and are broadcast to the same shape. The last condition must always evaluate to true. All clauses are executed in the device, unless they can be determined to always be true/false while building the numerical expression. Examples cond do Nx . all ( Nx . greater ( a , 0 ) ) -&gt; b * c Nx . all ( Nx . less ( a , 0 ) ) -&gt; b + c true -&gt; b - c end","ref":"Nx.Defn.Kernel.html#cond/1","title":"Nx.Defn.Kernel.cond/1","type":"macro"},{"doc":"Creates a token for hooks. See hook/3 .","ref":"Nx.Defn.Kernel.html#create_token/0","title":"Nx.Defn.Kernel.create_token/0","type":"function"},{"doc":"Defines a custom gradient for the given expression. It expects a fun to compute the gradient. The function will be called with the expression itself and the current gradient. It must return a list of arguments and their updated gradient to continue applying grad on. Examples For example, if the gradient of cos(t) were to be implemented by hand: def cos ( t ) do custom_grad ( Nx . cos ( t ) , fn _ans , g -&gt; [ { t , - g * Nx . sin ( t ) } ] end ) end","ref":"Nx.Defn.Kernel.html#custom_grad/2","title":"Nx.Defn.Kernel.custom_grad/2","type":"function"},{"doc":"Element-wise quotient operator. It delegates to Nx.quotient/2 (supports broadcasting). Examples defn quotient ( a , b ) do div ( a , b ) end","ref":"Nx.Defn.Kernel.html#div/2","title":"Nx.Defn.Kernel.div/2","type":"function"},{"doc":"Gets the element at the zero-based index in tuple. It raises ArgumentError when index is negative or it is out of range of the tuple elements. Examples iex&gt; tuple = { 1 , 2 , 3 } iex&gt; elem ( tuple , 0 ) 1","ref":"Nx.Defn.Kernel.html#elem/2","title":"Nx.Defn.Kernel.elem/2","type":"function"},{"doc":"Shortcut for hook/3 .","ref":"Nx.Defn.Kernel.html#hook/2","title":"Nx.Defn.Kernel.hook/2","type":"function"},{"doc":"Defines a hook. Hooks are a mechanism to execute an anonymous function for side-effects with runtime tensor values. Let's see an example: defmodule Hooks do import Nx.Defn defn add_and_mult ( a , b ) do add = hook ( a + b , fn tensor -&gt; IO . inspect ( { :add , tensor } ) end ) mult = hook ( a * b , fn tensor -&gt; IO . inspect ( { :mult , tensor } ) end ) { add , mult } end end Note a hook can only access the variables passed as arguments to the hook. It cannot access any other variable defined in defn outside of the hook. The defn above defines two hooks, one is called with the value of a + b and another with a * b . Once you invoke the function above, you should see this printed: Hooks . add_and_mult ( 2 , 3 ) { :add , # Nx.Tensor &lt; s64 5 &gt; } { :mult , # Nx.Tensor &lt; s64 6 &gt; } In other words, the hook function accepts a tensor expression as argument and it will invoke a custom function with a tensor value at runtime. hook returns the result of the given expression. The expression can be any tensor or a Nx.Container . Note you must return the result of the hook call . For example, the code below won't inspect the :add tuple, because the hook is not returned from defn : defn add_and_mult ( a , b ) do _add = hook ( a + b , fn tensor -&gt; IO . inspect ( { :add , tensor } ) end ) mult = hook ( a * b , fn tensor -&gt; IO . inspect ( { :mult , tensor } ) end ) mult end We will learn how to hook into a value that is not part of the result in the &quot;Hooks and tokens&quot; section. Named hooks It is possible to give names to the hooks. This allows them to be defined or overridden by calling Nx.Defn.jit/2 or Nx.Defn.stream/2 . Let's see an example: defmodule Hooks do import Nx.Defn defn add_and_mult ( a , b ) do add = hook ( a + b , :hooks_add ) mult = hook ( a * b , :hooks_mult ) { add , mult } end end Now you can pass the hook as argument as follows: hooks = %{ hooks_add : fn tensor -&gt; IO . inspect { :add , tensor } end } fun = Nx.Defn . jit ( &amp; Hooks . add_and_mult / 2 , hooks : hooks ) fun . ( Nx . tensor ( 2 ) , Nx . tensor ( 3 ) ) Important! We recommend to prefix your hook names by the name of your project to avoid conflicts. If a named hook is not given, compilers can optimize that away and not transfer the tensor from the device in the first place. You can also mix named hooks with callbacks: defn add_and_mult ( a , b ) do add = hook ( a + b , :hooks_add , fn tensor -&gt; IO . inspect ( { :add , tensor } ) end ) mult = hook ( a * b , :hooks_mult , fn tensor -&gt; IO . inspect ( { :mult , tensor } ) end ) { add , mult } end If a hook with the same name is given to Nx.Defn.jit/2 or Nx.Defn.stream/2 , then it will override the default callback. Hooks and tokens So far, we have always returned the result of the hook call. However, what happens if the values we want to hook are not part of the return value, such as below? defn add_and_mult ( a , b ) do _add = hook ( a + b , :hooks_add , &amp; IO . inspect ( { :add , &amp;1 } ) ) mult = hook ( a * b , :hooks_mult , &amp; IO . inspect ( { :mult , &amp;1 } ) ) mult end In such cases, you must use tokens. Tokens are used to create an ordering over hooks, ensuring hooks execute in a certain sequence: defn add_and_mult ( a , b ) do token = create_token ( ) { token , _add } = hook_token ( token , a + b , :hooks_add , &amp; IO . inspect ( { :add , &amp;1 } ) ) { token , mult } = hook_token ( token , a * b , :hooks_mult , &amp; IO . inspect ( { :mult , &amp;1 } ) ) attach_token ( token , mult ) end The example above creates a token and uses hook_token/4 to create hooks attached to their respective tokens. By using a token, we guarantee that those hooks will be invoked in the order in which they were defined. Then, at the end of the function, we attach the token (and its associated hooks) to the result mult . In fact, the hook/3 function is implemented roughly like this: def hook ( tensor_expr , name , function ) do { token , result } = hook_token ( create_token ( ) , tensor_expr , name , function ) attach_token ( token , result ) end Note you must attach the token at the end, otherwise the hooks will be &quot;lost&quot;, as if they were not defined. This also applies to conditionals and loops. The token must be attached within the branch they are used. For example, this won't work: token = create_token ( ) { token , result } = if Nx . any ( value ) do hook_token ( token , some_value ) else hook_token ( token , another_value ) end attach_token ( result ) Instead, you must write: token = create_token ( ) if Nx . any ( value ) do { token , result } = hook_token ( token , some_value ) attach_token ( token , result ) else { token , result } = hook_token ( token , another_value ) attach_token ( token , result ) end","ref":"Nx.Defn.Kernel.html#hook/3","title":"Nx.Defn.Kernel.hook/3","type":"function"},{"doc":"Shortcut for hook_token/4 .","ref":"Nx.Defn.Kernel.html#hook_token/3","title":"Nx.Defn.Kernel.hook_token/3","type":"function"},{"doc":"Defines a hook with an existing token. See hook/3 .","ref":"Nx.Defn.Kernel.html#hook_token/4","title":"Nx.Defn.Kernel.hook_token/4","type":"function"},{"doc":"Provides if/else expressions. The first argument must be a scalar. Zero is considered false, any other number is considered true. The booleans false and true are supported, but any other value will raise. The second argument is a keyword list with do and else blocks. The sides are broadcast to return the same shape and normalized to return the same type. Examples if Nx . any ( Nx . equal ( t , 0 ) ) do 0.0 else 1 / t end In case else is not given, it is assumed to be 0 with the same as the do clause. If you want to nest multiple conditionals, see cond/1 instead.","ref":"Nx.Defn.Kernel.html#if/2","title":"Nx.Defn.Kernel.if/2","type":"macro"},{"doc":"Imports functions and macros into the current scope, as in Kernel.SpecialForms.import/2 . Imports are typically discouraged in favor of alias/2 . Examples defn some_fun ( t ) do import Math.Helpers fft ( t ) end","ref":"Nx.Defn.Kernel.html#import/2","title":"Nx.Defn.Kernel.import/2","type":"macro"},{"doc":"Converts the given expression into a string. inspect/2 is used to convert expressions into strings, typically to be used as part of error messages. If you want to inspect for debugging, consider using print_expr/2 , to print the underlying expression, or print_value/2 to print the value during execution. defn square_shape ( tensor ) do case Nx . shape ( tensor ) do { n , n } -&gt; n shape -&gt; raise ArgumentError , &quot;expected a square tensor: \#{ inspect ( shape ) } &quot; end end","ref":"Nx.Defn.Kernel.html#inspect/2","title":"Nx.Defn.Kernel.inspect/2","type":"function"},{"doc":"Ensures the first argument is a keyword with the given keys and default values. The second argument must be a list of atoms, specifying a given key, or tuples specifying a key and a default value. If any of the keys in the keyword is not defined on values , it raises an error. This does not validate required keys. For such, use assert_keys/2 instead. This is equivalent to Elixir's Keyword.validate!/2 . Examples iex&gt; keyword! ( [ ] , [ one : 1 , two : 2 ] ) |&gt; Enum . sort ( ) [ one : 1 , two : 2 ] iex&gt; keyword! ( [ two : 3 ] , [ one : 1 , two : 2 ] ) |&gt; Enum . sort ( ) [ one : 1 , two : 3 ] If atoms are given, they are supported as keys but do not provide a default value: iex&gt; keyword! ( [ ] , [ :one , two : 2 ] ) |&gt; Enum . sort ( ) [ two : 2 ] iex&gt; keyword! ( [ one : 1 ] , [ :one , two : 2 ] ) |&gt; Enum . sort ( ) [ one : 1 , two : 2 ] Passing an unknown key raises: iex&gt; keyword! ( [ three : 3 ] , [ one : 1 , two : 2 ] ) ** (ArgumentError) unknown key :three in [three: 3], expected one of [:one, :two]","ref":"Nx.Defn.Kernel.html#keyword!/2","title":"Nx.Defn.Kernel.keyword!/2","type":"function"},{"doc":"Element-wise maximum operation. It delegates to Nx.max/2 (supports broadcasting). Examples defn min_max ( a , b ) do { min ( a , b ) , max ( a , b ) } end","ref":"Nx.Defn.Kernel.html#max/2","title":"Nx.Defn.Kernel.max/2","type":"function"},{"doc":"Element-wise minimum operation. It delegates to Nx.min/2 (supports broadcasting). Examples defn min_max ( a , b ) do { min ( a , b ) , max ( a , b ) } end","ref":"Nx.Defn.Kernel.html#min/2","title":"Nx.Defn.Kernel.min/2","type":"function"},{"doc":"Element-wise logical NOT operation. Zero is considered false, all other numbers are considered true. It delegates to Nx.logical_not/1 . Examples defn logical_not ( a ) , do : not a","ref":"Nx.Defn.Kernel.html#not/1","title":"Nx.Defn.Kernel.not/1","type":"macro"},{"doc":"Element-wise logical OR operation. Zero is considered false, all other numbers are considered true. It delegates to Nx.logical_or/2 (supports broadcasting). Examples defn and_or ( a , b ) do { a and b , a or b } end","ref":"Nx.Defn.Kernel.html#or/2","title":"Nx.Defn.Kernel.or/2","type":"macro"},{"doc":"Prints the given expression to the terminal. It returns the given expressions. Examples defn tanh_grad ( t ) do grad ( t , &amp; Nx . tanh / 1 ) |&gt; print_expr ( ) end When invoked, it will print the expression being built by defn : # Nx.Tensor &lt; Nx.Defn.Expr parameter a s64 parameter c s64 b = tanh [ a ] f64 d = power [ c , 2 ] s64 e = add [ b , d ] f64 &gt;","ref":"Nx.Defn.Kernel.html#print_expr/2","title":"Nx.Defn.Kernel.print_expr/2","type":"function"},{"doc":"Prints the value at runtime to the terminal. This function is implemented on top of hook/3 and therefore has the following restrictions: It can only inspect tensors and Nx.Container The return value of this function must be part of the output All options are passed to IO.inspect/2 . Examples defn tanh_grad ( t ) do grad ( t , fn t -&gt; t |&gt; Nx . tanh ( ) |&gt; print_value ( ) end ) end defn tanh_grad ( t ) do grad ( t , fn t -&gt; t |&gt; Nx . tanh ( ) |&gt; print_value ( label : &quot;tanh&quot; ) end ) end","ref":"Nx.Defn.Kernel.html#print_value/2","title":"Nx.Defn.Kernel.print_value/2","type":"function"},{"doc":"Raises a runtime exception with the given message . See raise/2 for more information on exceptions inside defn .","ref":"Nx.Defn.Kernel.html#raise/1","title":"Nx.Defn.Kernel.raise/1","type":"macro"},{"doc":"Raises an exception with the given arguments . raise/2 is invoked while building the numerical expression, not inside the device. This means that raise may be invoked on unexpected situations, as we build the numerical expression. To better understand those cases, let's see some examples. First, let's start with a valid use case for raise/2 : raise on mismatched shapes. Inside defn , we know the tensor shapes and types, but not their values, so we can assert on the shape while building the numerical expression: defn square_shape ( tensor ) do case Nx . shape ( tensor ) do { n , n } -&gt; n shape -&gt; raise ArgumentError , &quot;expected a square tensor: \#{ inspect ( shape ) } &quot; end end In the example above, only the matching branch of the case is executed, so if you give it a 2x2 tensor, it will return 2. However, if you give it a non-square tensor, it will raise. Now consider this code: defn some_check ( a , b ) do if a != b do a * b else raise &quot;expected different tensors, got: \#{ inspect ( a ) } and \#{ inspect ( b ) } &quot; end end In this case, both a and b are tensors and we are comparing their values. However, their values are unknown, which means we need to convert the whole if to a numerical expression and run it on the device. However, once we convert the else branch, it will execute raise/2 , making it so the code above always raises! In such cases, there are no alternatives. We can't execute exceptions in the CPU/GPU, so you need to approach the problem under a different perspective.","ref":"Nx.Defn.Kernel.html#raise/2","title":"Nx.Defn.Kernel.raise/2","type":"macro"},{"doc":"Element-wise remainder operation. It delegates to Nx.remainder/2 (supports broadcasting). Examples defn divides_by_5? ( a ) do rem ( a , 5 ) |&gt; Nx . any ( ) |&gt; Nx . equal ( Nx . tensor ( 1 ) ) end","ref":"Nx.Defn.Kernel.html#rem/2","title":"Nx.Defn.Kernel.rem/2","type":"function"},{"doc":"Requires a module in order to use its macros, as in Kernel.SpecialForms.require/2 . Examples defn some_fun ( t ) do require NumericalMacros NumericalMacros . some_macro t do ... end end","ref":"Nx.Defn.Kernel.html#require/2","title":"Nx.Defn.Kernel.require/2","type":"macro"},{"doc":"Rewrites the types of expr recursively according to opts Options :max_unsigned_type - replaces all signed tensors with size equal to or greater then the given type by the given type :max_signed_type - replaces all signed tensors with size equal to or greater then the given type by the given type :max_float_type - replaces all float tensors with size equal to or greater then the given type by the given type Examples rewrite_types ( expr , max_float_type : { :f , 32 } )","ref":"Nx.Defn.Kernel.html#rewrite_types/2","title":"Nx.Defn.Kernel.rewrite_types/2","type":"function"},{"doc":"Stops computing the gradient for the given expression. It effectively annotates the gradient for the given expression is 1.0. Examples expr = stop_grad ( expr )","ref":"Nx.Defn.Kernel.html#stop_grad/1","title":"Nx.Defn.Kernel.stop_grad/1","type":"function"},{"doc":"Pipes value to the given fun and returns the value itself. Useful for running synchronous side effects in a pipeline. Examples Let's suppose you want to inspect an expression in the middle of a pipeline. You could write: a |&gt; Nx . add ( b ) |&gt; tap ( &amp; print_expr / 1 ) |&gt; Nx . multiply ( c )","ref":"Nx.Defn.Kernel.html#tap/2","title":"Nx.Defn.Kernel.tap/2","type":"macro"},{"doc":"Pipes value into the given fun . In other words, it invokes fun with value as argument. This is most commonly used in pipelines, allowing you to pipe a value to a function outside of its first argument. Examples a |&gt; Nx . add ( b ) |&gt; then ( &amp; Nx . subtract ( c , &amp;1 ) )","ref":"Nx.Defn.Kernel.html#then/2","title":"Nx.Defn.Kernel.then/2","type":"macro"},{"doc":"Defines a while loop. It expects the initial arguments, a condition expression, and a block : while initial , condition do block end condition must return a scalar tensor where 0 is false and any other number is true. The given block will be executed while condition is true. Each invocation of block must return a value in the same shape as initial arguments. while will return the value of the last execution of block . If block is never executed because the initial condition is false, it returns initial . Note: you must prefer to use the operations in the Nx module, whenever available, instead of writing your own loops. Examples A simple loop that increments x until it is 10 can be written as: while x = 0 , Nx . less ( x , 10 ) do x + 1 end However, it is important to note that all variables you intend to use inside the &quot;while&quot; must be explicitly given as argument to &quot;while&quot;. For example, imagine the amount we want to increment by in the example above is given by a variable y . The following example is invalid: while x = 0 , Nx . less ( x , 10 ) do x + y end Instead, both x and y must be passed as variables to while : while { x = 0 , y } , Nx . less ( x , 10 ) do { x + y , y } end Similarly, to compute the factorial of x using while : defn factorial ( x ) do { factorial , _ } = while { factorial = 1 , x } , Nx . greater ( x , 1 ) do { factorial * x , x - 1 } end factorial end Generators Inspired by Elixir's for-comprehensions , while in defn supports generators. Generators may be tensors or ranges. Tensor generators When the generator is a tensor, Nx will traverse its highest dimension. For example, you could sum a one dimensional tensor as follows: while acc = 0 , i &lt;- tensor do acc + i end Note: implementing sum using while , as above, is done as an example. In practice, you must prefer to use the operations in the Nx module, whenever available, instead of writing your own loops. One advantage of using generators is that you can also unroll the loop for performance: while acc = 0 , i &lt;- tensor , unroll : true do acc + i end Or unroll it in batches: while acc = 0 , i &lt;- tensor , unroll : 4 do acc + i end Unrolling means that the the while body is automatically duplicated a certain amount of times, as if you wrote all iterations by hand. This makes the final expression larger, which causes a longer compilation time, however it enables additional compile-time optimizations (such as fusion), improving the runtime efficiency. Range generators A range can also be given as a generator. The range may be increasing or decreasing. Also remember that ranges in Elixir are inclusive on both begin and end. The sum example from the previous section could also be written with ranges: while { tensor , acc = 0 } , i &lt;- 0 .. Nx . axis_size ( tensor , 0 ) - 1 do acc + tensor [ i ] end","ref":"Nx.Defn.Kernel.html#while/4","title":"Nx.Defn.Kernel.while/4","type":"macro"},{"doc":"Pipes the argument on the left to the function call on the right. It delegates to Kernel.|&gt;/2 . Examples defn exp_sum ( t ) do t |&gt; Nx . exp ( ) |&gt; Nx . sum ( ) end","ref":"Nx.Defn.Kernel.html#%7C%3E/2","title":"Nx.Defn.Kernel.|>/2","type":"macro"},{"doc":"Element-wise bitwise OR operation. Only integer tensors are supported. It delegates to Nx.bitwise_or/2 (supports broadcasting). Examples defn and_or ( a , b ) do { a &amp;&amp;&amp; b , a ||| b } end","ref":"Nx.Defn.Kernel.html#%7C%7C%7C/2","title":"Nx.Defn.Kernel.|||/2","type":"function"},{"doc":"Element-wise bitwise not operation. Only integer tensors are supported. It delegates to Nx.bitwise_not/1 . Examples defn bnot ( a ) , do : ~~~ a","ref":"Nx.Defn.Kernel.html#~~~/1","title":"Nx.Defn.Kernel.~~~/1","type":"function"},{"doc":"Nx conveniences for linear algebra.","ref":"Nx.LinAlg.html","title":"Nx.LinAlg","type":"module"},{"doc":"Returns the adjoint of a given tensor. If the input tensor is real it transposes it's two inner-most axes. If the input tensor is complex, it additionally applies Nx.conjugate/1 to it. Examples iex&gt; Nx.LinAlg . adjoint ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 3 ] , [ 2 , 4 ] ] &gt; iex&gt; Nx.LinAlg . adjoint ( Nx . tensor ( [ [ 1 , Complex . new ( 0 , 2 ) ] , [ 3 , Complex . new ( 0 , - 4 ) ] ] ) ) # Nx.Tensor &lt; c64 [ 2 ] [ 2 ] [ [ 1.0 + 0.0 i , 3.0 + 0.0 i ] , [ 0.0 - 2.0 i , 0.0 + 4.0 i ] ] &gt;","ref":"Nx.LinAlg.html#adjoint/1","title":"Nx.LinAlg.adjoint/1","type":"function"},{"doc":"Performs a Cholesky decomposition of a batch of square matrices. The matrices must be positive-definite and either Hermitian if complex or symmetric if real. An error is raised by the default backend if those conditions are not met. Other backends may emit undefined behaviour. Examples iex&gt; Nx.LinAlg . cholesky ( Nx . tensor ( [ [ 20.0 , 17.6 ] , [ 17.6 , 16.0 ] ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ [ 4.4721360206604 , 0.0 ] , [ 3.9354796409606934 , 0.7155413031578064 ] ] &gt; iex&gt; Nx.LinAlg . cholesky ( Nx . tensor ( [ [ [ 2.0 , 3.0 ] , [ 3.0 , 5.0 ] ] , [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1.4142135381698608 , 0.0 ] , [ 2.1213202476501465 , 0.7071067690849304 ] ] , [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ ...&gt; [ 6.0 , 3.0 , 4.0 , 8.0 ] , ...&gt; [ 3.0 , 6.0 , 5.0 , 1.0 ] , ...&gt; [ 4.0 , 5.0 , 10.0 , 7.0 ] , ...&gt; [ 8.0 , 1.0 , 7.0 , 25.0 ] ...&gt; ] ) iex&gt; Nx.LinAlg . cholesky ( t ) # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 2.4494898319244385 , 0.0 , 0.0 , 0.0 ] , [ 1.2247449159622192 , 2.1213202476501465 , 0.0 , 0.0 ] , [ 1.632993221282959 , 1.4142135381698608 , 2.309401035308838 , 0.0 ] , [ 3.265986442565918 , - 1.4142135381698608 , 1.5877132415771484 , 3.132491111755371 ] ] &gt; iex&gt; Nx.LinAlg . cholesky ( Nx . tensor ( [ [ 1.0 , Complex . new ( 0 , - 2 ) ] , [ Complex . new ( 0 , 2 ) , 5.0 ] ] ) ) # Nx.Tensor &lt; c64 [ 2 ] [ 2 ] [ [ 1.0 + 0.0 i , 0.0 + 0.0 i ] , [ 0.0 + 2.0 i , 1.0 + 0.0 i ] ] &gt; Error cases iex&gt; Nx.LinAlg . cholesky ( Nx . tensor ( [ [ 1.0 , 2.0 ] , [ 3.0 , 4.0 ] ] ) ) ** (ArgumentError) matrix must be hermitian, a matrix is hermitian iff X = adjoint(X)","ref":"Nx.LinAlg.html#cholesky/1","title":"Nx.LinAlg.cholesky/1","type":"function"},{"doc":"Calculates the determinant of batched square matrices. Examples For 2x2 and 3x3, the results are given by the closed formulas: iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) ) # Nx.Tensor &lt; f32 - 2.0 &gt; iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 1.0 , - 2.0 , 3.0 ] , [ 7.0 , 8.0 , 9.0 ] ] ) ) # Nx.Tensor &lt; f32 48.0 &gt; When there are linearly dependent rows or columns, the determinant is 0: iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ [ 1.0 , 0.0 ] , [ 3.0 , 0.0 ] ] ) ) # Nx.Tensor &lt; f32 0.0 &gt; iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ - 1.0 , - 2.0 , - 3.0 ] , [ 4.0 , 5.0 , 6.0 ] ] ) ) # Nx.Tensor &lt; f32 0.0 &gt; The determinant can also be calculated when the axes are bigger than 3: iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ ...&gt; [ 1 , 0 , 0 , 0 ] , ...&gt; [ 0 , 1 , 2 , 3 ] , ...&gt; [ 0 , 1 , - 2 , 3 ] , ...&gt; [ 0 , 7 , 8 , 9.0 ] ...&gt; ] ) ) # Nx.Tensor &lt; f32 48.0 &gt; iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ ...&gt; [ 0 , 0 , 0 , 0 , - 1 ] , ...&gt; [ 0 , 1 , 2 , 3 , 0 ] , ...&gt; [ 0 , 1 , - 2 , 3 , 0 ] , ...&gt; [ 0 , 7 , 8 , 9 , 0 ] , ...&gt; [ 1 , 0 , 0 , 0 , 0 ] ...&gt; ] ) ) # Nx.Tensor &lt; f32 48.0 &gt; iex&gt; Nx.LinAlg . determinant ( Nx . tensor ( [ ...&gt; [ [ 2 , 4 , 6 , 7 ] , [ 5 , 1 , 8 , 8 ] , [ 1 , 7 , 3 , 1 ] , [ 3 , 9 , 2 , 4 ] ] , ...&gt; [ [ 2 , 5 , 1 , 3 ] , [ 4 , 1 , 7 , 9 ] , [ 6 , 8 , 3 , 2 ] , [ 7 , 8 , 1 , 4 ] ] ...&gt; ] ) ) # Nx.Tensor &lt; f32 [ 2 ] [ 630.0 , 630.0 ] &gt; If the axes are named, their names are not preserved in the output: iex&gt; two_by_two = Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , names : [ :x , :y ] ) iex&gt; Nx.LinAlg . determinant ( two_by_two ) # Nx.Tensor &lt; f32 - 2.0 &gt; iex&gt; three_by_three = Nx . tensor ( [ [ 1.0 , 2.0 , 3.0 ] , [ 1.0 , - 2.0 , 3.0 ] , [ 7.0 , 8.0 , 9.0 ] ] , names : [ :x , :y ] ) iex&gt; Nx.LinAlg . determinant ( three_by_three ) # Nx.Tensor &lt; f32 48.0 &gt; Also supports complex inputs: iex&gt; t = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 0 , Complex . new ( 0 , 2 ) , 0 ] , [ 0 , 0 , 3 ] ] ) iex&gt; Nx.LinAlg . determinant ( t ) # Nx.Tensor &lt; c64 0.0 + 6.0 i &gt; iex&gt; t = Nx . tensor ( [ [ 0 , 0 , 0 , 1 ] , [ 0 , Complex . new ( 0 , 2 ) , 0 , 0 ] , [ 0 , 0 , 3 , 0 ] , [ 1 , 0 , 0 , 0 ] ] ) iex&gt; Nx.LinAlg . determinant ( t ) # Nx.Tensor &lt; c64 0.0 - 6.0 i &gt;","ref":"Nx.LinAlg.html#determinant/1","title":"Nx.LinAlg.determinant/1","type":"function"},{"doc":"Calculates the Eigenvalues and Eigenvectors of batched Hermitian 2-D matrices. It returns {eigenvals, eigenvecs} . Options :max_iter - integer . Defaults to 50_000 Number of maximum iterations before stopping the decomposition :eps - float . Defaults to 1.0e-10 Tolerance applied during the decomposition Note not all options apply to all backends, as backends may have specific optimizations that render these mechanisms unnecessary. Examples iex&gt; { eigenvals , eigenvecs } = Nx.LinAlg . eigh ( Nx . tensor ( [ [ 1 , 0 ] , [ 0 , 2 ] ] ) ) iex&gt; Nx . round ( eigenvals ) # Nx.Tensor &lt; f32 [ 2 ] [ 1.0 , 2.0 ] &gt; iex&gt; eigenvecs # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] &gt; iex&gt; { eigenvals , eigenvecs } = Nx.LinAlg . eigh ( Nx . tensor ( [ [ 0 , 1 , 2 ] , [ 1 , 0 , 2 ] , [ 2 , 2 , 3 ] ] ) ) iex&gt; Nx . round ( eigenvals ) # Nx.Tensor &lt; f32 [ 3 ] [ 5.0 , - 1.0 , - 1.0 ] &gt; iex&gt; eigenvecs # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 0.4082472324371338 , 0.9128734469413757 , 0.0 ] , [ 0.40824851393699646 , - 0.18257413804531097 , 0.8944271802902222 ] , [ 0.8164970278739929 , - 0.36514827609062195 , - 0.4472135901451111 ] ] &gt; iex&gt; { eigenvals , eigenvecs } = Nx.LinAlg . eigh ( Nx . tensor ( [ [ [ 2 , 5 ] , [ 5 , 6 ] ] , [ [ 1 , 0 ] , [ 0 , 4 ] ] ] ) ) iex&gt; Nx . round ( eigenvals ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ [ 9.0 , - 1.0 ] , [ 1.0 , 4.0 ] ] &gt; iex&gt; eigenvecs # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 0.5606290698051453 , - 0.828070342540741 ] , [ 0.8280670642852783 , 0.5606313347816467 ] ] , [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] ] &gt; Error cases iex&gt; Nx.LinAlg . eigh ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] ) ) ** (ArgumentError) tensor must be a square matrix or a batch of square matrices, got shape: {2, 3} iex&gt; Nx.LinAlg . eigh ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) ) ** (ArgumentError) matrix must be hermitian, a matrix is hermitian iff X = adjoint(X)","ref":"Nx.LinAlg.html#eigh/2","title":"Nx.LinAlg.eigh/2","type":"function"},{"doc":"Inverts a batch of square matrices. For non-square matrices, use svd/2 for pseudo-inverse calculations. Examples iex&gt; a = Nx . tensor ( [ [ 1 , 2 , 1 , 1 ] , [ 0 , 1 , 0 , 1 ] , [ 0 , 0 , 1 , 1 ] , [ 0 , 0 , 0 , 1 ] ] ) iex&gt; a_inv = Nx.LinAlg . invert ( a ) # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 1.0 , - 2.0 , - 1.0 , 2.0 ] , [ 0.0 , 1.0 , 0.0 , - 1.0 ] , [ 0.0 , 0.0 , 1.0 , - 1.0 ] , [ 0.0 , 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; Nx . dot ( a , a_inv ) # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 1.0 , 0.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; Nx . dot ( a_inv , a ) # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 1.0 , 0.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ [ [ 1 , 2 ] , [ 0 , 1 ] ] , [ [ 1 , 1 ] , [ 0 , 1 ] ] ] ) iex&gt; a_inv = Nx.LinAlg . invert ( a ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1.0 , - 2.0 ] , [ 0.0 , 1.0 ] ] , [ [ 1.0 , - 1.0 ] , [ 0.0 , 1.0 ] ] ] &gt; iex&gt; Nx . dot ( a , [ 2 ] , [ 0 ] , a_inv , [ 1 ] , [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] , [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] ] &gt; iex&gt; Nx . dot ( a_inv , [ 2 ] , [ 0 ] , a , [ 1 ] , [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] , [ [ 1.0 , 0.0 ] , [ 0.0 , 1.0 ] ] ] &gt; Error cases iex&gt; Nx.LinAlg . invert ( Nx . tensor ( [ [ 3 , 0 , 0 , 0 ] , [ 2 , 1 , 0 , 0 ] ] ) ) ** (ArgumentError) invert/1 expects a square matrix or a batch of square matrices, got tensor with shape: {2, 4} iex&gt; Nx.LinAlg . invert ( Nx . tensor ( [ [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 1 , 1 , 1 , 1 ] ] ) ) ** (ArgumentError) can&#39;t solve for singular matrix","ref":"Nx.LinAlg.html#invert/1","title":"Nx.LinAlg.invert/1","type":"function"},{"doc":"Calculates the A = PLU decomposition of batched square 2-D matrices A. Options :eps - Rounding error threshold that can be applied during the factorization Examples iex&gt; { p , l , u } = Nx.LinAlg . lu ( Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] , [ 7 , 8 , 9 ] ] ) ) iex&gt; p # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 0 , 0 , 1 ] , [ 0 , 1 , 0 ] , [ 1 , 0 , 0 ] ] &gt; iex&gt; l # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.5714285969734192 , 1.0 , 0.0 ] , [ 0.1428571492433548 , 2.0 , 1.0 ] ] &gt; iex&gt; u # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 7.0 , 8.0 , 9.0 ] , [ 0.0 , 0.4285714328289032 , 0.8571428656578064 ] , [ 0.0 , 0.0 , 0.0 ] ] &gt; iex&gt; p |&gt; Nx . dot ( l ) |&gt; Nx . dot ( u ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 2.0 , 3.0 ] , [ 4.0 , 5.0 , 6.0 ] , [ 7.0 , 8.0 , 9.0 ] ] &gt; iex&gt; { p , l , u } = Nx.LinAlg . lu ( Nx . tensor ( [ [ 1 , 0 , 1 ] , [ - 1 , 0 , - 1 ] , [ 1 , 1 , 1 ] ] ) ) iex&gt; p # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 1 , 0 , 0 ] , [ 0 , 0 , 1 ] , [ 0 , 1 , 0 ] ] &gt; iex&gt; l # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 1.0 , 1.0 , 0.0 ] , [ - 1.0 , 0.0 , 1.0 ] ] &gt; iex&gt; u # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 1.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] &gt; iex&gt; p |&gt; Nx . dot ( l ) |&gt; Nx . dot ( u ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 1.0 ] , [ - 1.0 , 0.0 , - 1.0 ] , [ 1.0 , 1.0 , 1.0 ] ] &gt; iex&gt; { p , l , u } = Nx.LinAlg . lu ( Nx . tensor ( [ [ [ 9 , 8 , 7 ] , [ 6 , 5 , 4 ] , [ 3 , 2 , 1 ] ] , [ [ - 1 , 0 , - 1 ] , [ 1 , 0 , 1 ] , [ 1 , 1 , 1 ] ] ] ) ) iex&gt; p # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ 3 ] [ [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 0 , 0 , 1 ] ] , [ [ 1 , 0 , 0 ] , [ 0 , 0 , 1 ] , [ 0 , 1 , 0 ] ] ] &gt; iex&gt; l # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ 3 ] [ [ [ 1.0 , 0.0 , 0.0 ] , [ 0.6666666865348816 , 1.0 , 0.0 ] , [ 0.3333333432674408 , 2.0 , 1.0 ] ] , [ [ 1.0 , 0.0 , 0.0 ] , [ - 1.0 , 1.0 , 0.0 ] , [ - 1.0 , 0.0 , 1.0 ] ] ] &gt; iex&gt; u # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ 3 ] [ [ [ 9.0 , 8.0 , 7.0 ] , [ 0.0 , - 0.3333333432674408 , - 0.6666666865348816 ] , [ 0.0 , 0.0 , 0.0 ] ] , [ [ - 1.0 , 0.0 , - 1.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 ] ] ] &gt; iex&gt; p |&gt; Nx . dot ( [ 2 ] , [ 0 ] , l , [ 1 ] , [ 0 ] ) |&gt; Nx . dot ( [ 2 ] , [ 0 ] , u , [ 1 ] , [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ 3 ] [ [ [ 9.0 , 8.0 , 7.0 ] , [ 6.0 , 5.0 , 4.0 ] , [ 3.0 , 2.0 , 1.0 ] ] , [ [ - 1.0 , 0.0 , - 1.0 ] , [ 1.0 , 0.0 , 1.0 ] , [ 1.0 , 1.0 , 1.0 ] ] ] &gt; Error cases iex&gt; Nx.LinAlg . lu ( Nx . tensor ( [ [ 1 , 1 , 1 , 1 ] , [ - 1 , 4 , 4 , - 1 ] , [ 4 , - 2 , 2 , 0 ] ] ) ) ** (ArgumentError) tensor must be a square matrix or a batch of square matrices, got shape: {3, 4}","ref":"Nx.LinAlg.html#lu/2","title":"Nx.LinAlg.lu/2","type":"function"},{"doc":"Produces the tensor taken to the given power by matrix dot-product. The input is always a tensor of batched square matrices and an integer, and the output is a tensor of the same dimensions as the input tensor. The dot-products are unrolled inside defn . Examples iex&gt; Nx.LinAlg . matrix_power ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , 0 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 0 ] , [ 0 , 1 ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , 6 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 5743 , 8370 ] , [ 12555 , 18298 ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . eye ( 3 ) , 65535 ) # Nx.Tensor &lt; s64 [ 3 ] [ 3 ] [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 0 , 0 , 1 ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] ) , - 1 ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ [ - 2.0 , 1.0 ] , [ 1.5 , - 0.5 ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . iota ( { 2 , 2 , 2 } ) , 3 ) # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ 2 ] [ [ [ 6 , 11 ] , [ 22 , 39 ] ] , [ [ 514 , 615 ] , [ 738 , 883 ] ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . iota ( { 2 , 2 , 2 } ) , - 3 ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ - 4.875 , 1.375 ] , [ 2.75 , - 0.75 ] ] , [ [ - 110.375 , 76.875 ] , [ 92.25 , - 64.25 ] ] ] &gt; iex&gt; Nx.LinAlg . matrix_power ( Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] , [ 5 , 6 ] ] ) , 1 ) ** (ArgumentError) matrix_power/2 expects a square matrix or a batch of square matrices, got tensor with shape: {3, 2}","ref":"Nx.LinAlg.html#matrix_power/2","title":"Nx.LinAlg.matrix_power/2","type":"function"},{"doc":"Calculates the p-norm of a tensor. For the 0-norm, the norm is the number of non-zero elements in the tensor. Options :axes - defines the axes upon which the norm will be calculated. Applies only on 2-norm for 2-D tensors. Default: nil . :ord - defines which norm will be calculated according to the table below. Default: 2 ord 2-D 1-D nil Frobenius norm 2-norm :nuclear Nuclear norm - :frobenius Frobenius norm - :inf max(sum(abs(x), axes: [1])) max(abs(x)) :neg_inf min(sum(abs(x), axes: [1])) min(abs(x)) 0 - Number of non-zero elements 1 max(sum(abs(x), axes: [0])) as below -1 min(sum(abs(x), axes: [0])) as below 2 2-norm as below -2 smallest singular value as below other - power(sum(power(abs(x), p)), 1/p) Examples Vector norms iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , 4 ] ) ) # Nx.Tensor &lt; f32 5.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , 4 ] ) , ord : 1 ) # Nx.Tensor &lt; f32 7.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , - 4 ] ) , ord : :inf ) # Nx.Tensor &lt; f32 4.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , - 4 ] ) , ord : :neg_inf ) # Nx.Tensor &lt; f32 3.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , - 4 , 0 , 0 ] ) , ord : 0 ) # Nx.Tensor &lt; f32 2.0 &gt; Matrix norms iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , - 1 ] , [ 2 , - 4 ] ] ) , ord : - 1 ) # Nx.Tensor &lt; f32 5.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , - 2 ] , [ 2 , - 4 ] ] ) , ord : 1 ) # Nx.Tensor &lt; f32 6.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , - 2 ] , [ 2 , - 4 ] ] ) , ord : :neg_inf ) # Nx.Tensor &lt; f32 5.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , - 2 ] , [ 2 , - 4 ] ] ) , ord : :inf ) # Nx.Tensor &lt; f32 6.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , 0 ] , [ 0 , - 4 ] ] ) , ord : :frobenius ) # Nx.Tensor &lt; f32 5.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 0 , - 4 , 0 ] , [ 0 , 0 , 9 ] ] ) , ord : :nuclear ) # Nx.Tensor &lt; f32 14.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 0 , - 4 , 0 ] , [ 0 , 0 , 9 ] ] ) , ord : - 2 ) # Nx.Tensor &lt; f32 1.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , 0 ] , [ 0 , - 4 ] ] ) ) # Nx.Tensor &lt; f32 5.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 3 , 4 ] , [ 0 , - 4 ] ] ) , axes : [ 1 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 5.0 , 4.0 ] &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ Complex . new ( 0 , 3 ) , 4 ] , [ 4 , 0 ] ] ) , axes : [ 0 ] ) # Nx.Tensor &lt; f32 [ 2 ] [ 5.0 , 4.0 ] &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ Complex . new ( 0 , 3 ) , 0 ] , [ 4 , 0 ] ] ) , ord : :neg_inf ) # Nx.Tensor &lt; f32 3.0 &gt; iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ [ 0 , 0 ] , [ 0 , 0 ] ] ) ) # Nx.Tensor &lt; f32 0.0 &gt; Error cases iex&gt; Nx.LinAlg . norm ( Nx . tensor ( [ 3 , 4 ] ) , ord : :frobenius ) ** (ArgumentError) expected a 2-D tensor for ord: :frobenius, got a 1-D tensor","ref":"Nx.LinAlg.html#norm/2","title":"Nx.LinAlg.norm/2","type":"function"},{"doc":"Calculates the QR decomposition of a tensor with shape {..., M, N} . Options :mode - Can be one of :reduced , :complete . Defaults to :reduced For the following, K = min(M, N) :reduced - returns q and r with shapes {..., M, K} and {..., K, N} :complete - returns q and r with shapes {..., M, M} and {..., M, N} :eps - Rounding error threshold that can be applied during the triangularization Examples iex&gt; { q , r } = Nx.LinAlg . qr ( Nx . tensor ( [ [ - 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , - 1 ] ] ) ) iex&gt; q # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; r # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ - 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , - 1.0 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 1 ] ] ) iex&gt; { q , r } = Nx.LinAlg . qr ( t ) iex&gt; q # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; r # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; { qs , rs } = Nx.LinAlg . qr ( Nx . tensor ( [ [ [ - 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , - 1 ] ] , [ [ 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 1 ] ] ] ) ) iex&gt; qs # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ 3 ] [ [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] , [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] ] &gt; iex&gt; rs # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ 3 ] [ [ [ - 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , - 1.0 ] ] , [ [ 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , 1.0 ] ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 1 ] , [ 0 , 0 , 1 ] ] , type : :f32 ) iex&gt; { q , r } = Nx.LinAlg . qr ( t , mode : :reduced ) iex&gt; q # Nx.Tensor &lt; f32 [ 4 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.7071067690849304 ] , [ 0.0 , 0.0 , 0.7071067690849304 ] ] &gt; iex&gt; r # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , 1.4142135381698608 ] ] &gt; iex&gt; t = Nx . tensor ( [ [ 3 , 2 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 1 ] , [ 0 , 0 , 0 ] ] , type : :f32 ) iex&gt; { q , r } = Nx.LinAlg . qr ( t , mode : :complete ) iex&gt; q # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 1.0 , 0.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; r # Nx.Tensor &lt; f32 [ 4 ] [ 3 ] [ [ 3.0 , 2.0 , 1.0 ] , [ 0.0 , 1.0 , 1.0 ] , [ 0.0 , 0.0 , 1.0 ] , [ 0.0 , 0.0 , 0.0 ] ] &gt; Error cases iex&gt; Nx.LinAlg . qr ( Nx . tensor ( [ [ [ 1 , 1 , 1 , 1 ] , [ - 1 , 4 , 4 , - 1 ] , [ 4 , - 2 , 2 , 0 ] ] ] ) ) ** (ArgumentError) tensor must have at least as many rows as columns in the last two axes, got 3 rows and 4 columns iex&gt; Nx.LinAlg . qr ( Nx . tensor ( [ 1 , 2 , 3 , 4 , 5 ] ) ) ** (ArgumentError) tensor must have at least rank 2, got rank 1 with shape {5}","ref":"Nx.LinAlg.html#qr/2","title":"Nx.LinAlg.qr/2","type":"function"},{"doc":"Solves the system AX = B . A must have shape {..., n, n} and B must have shape {..., n, m} or {..., n} . X has the same shape as B . Examples iex&gt; a = Nx . tensor ( [ [ 1 , 3 , 2 , 1 ] , [ 2 , 1 , 0 , 0 ] , [ 1 , 0 , 1 , 0 ] , [ 1 , 1 , 1 , 1 ] ] ) iex&gt; Nx.LinAlg . solve ( a , Nx . tensor ( [ - 3 , 0 , 4 , - 2 ] ) ) |&gt; Nx . round ( ) # Nx.Tensor &lt; f32 [ 4 ] [ 1.0 , - 2.0 , 3.0 , - 4.0 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 1 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . solve ( a , Nx . tensor ( [ 0 , 2 , 1 ] ) ) |&gt; Nx . round ( ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 1.0 , - 1.0 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 1 ] , [ 1 , 1 , 0 ] , [ 0 , 1 , 1 ] ] ) iex&gt; b = Nx . tensor ( [ [ 2 , 2 , 3 ] , [ 2 , 2 , 4 ] , [ 2 , 0 , 1 ] ] ) iex&gt; Nx.LinAlg . solve ( a , b ) |&gt; Nx . round ( ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 2.0 , 3.0 ] , [ 1.0 , 0.0 , 1.0 ] , [ 1.0 , 0.0 , 0.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ [ [ 14 , 10 ] , [ 9 , 9 ] ] , [ [ 4 , 11 ] , [ 2 , 3 ] ] ] ) iex&gt; b = Nx . tensor ( [ [ [ 2 , 4 ] , [ 3 , 2 ] ] , [ [ 1 , 5 ] , [ - 3 , - 1 ] ] ] ) iex&gt; Nx.LinAlg . solve ( a , b ) |&gt; Nx . round ( ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 0.0 , 0.0 ] , [ 1.0 , 0.0 ] ] , [ [ - 4.0 , - 3.0 ] , [ 1.0 , 1.0 ] ] ] &gt; If the axes are named, their names are not preserved in the output: iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 1 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , names : [ :x , :y ] ) iex&gt; Nx.LinAlg . solve ( a , Nx . tensor ( [ 0 , 2 , 1 ] , names : [ :z ] ) ) |&gt; Nx . round ( ) # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 1.0 , - 1.0 ] &gt; Error cases iex&gt; Nx.LinAlg . solve ( Nx . tensor ( [ [ 1 , 0 ] , [ 0 , 1 ] ] ) , Nx . tensor ( [ 4 , 2 , 4 , 2 ] ) ) ** (ArgumentError) `b` tensor has incompatible dimensions, expected {2, 2} or {2}, got: {4} iex&gt; Nx.LinAlg . solve ( Nx . tensor ( [ [ 3 , 0 , 0 , 0 ] , [ 2 , 1 , 0 , 0 ] , [ 1 , 1 , 1 , 1 ] ] ) , Nx . tensor ( [ 4 ] ) ) ** (ArgumentError) `a` tensor has incompatible dimensions, expected a square matrix or a batch of square matrices, got: {3, 4}","ref":"Nx.LinAlg.html#solve/2","title":"Nx.LinAlg.solve/2","type":"function"},{"doc":"Calculates the Singular Value Decomposition of batched 2-D matrices. It returns {u, s, vt} where the elements of s are sorted from highest to lowest. Options :max_iter - integer . Defaults to 1000 Number of maximum iterations before stopping the decomposition :eps - float . Defaults to 1.0e-10 Tolerance applied during the decomposition Note not all options apply to all backends, as backends may have specific optimizations that render these mechanisms unnecessary. Examples iex&gt; { u , s , vt } = Nx.LinAlg . svd ( Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 0 , 1 , 0 ] , [ 0 , 0 , - 1 ] ] ) ) iex&gt; u # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; s # Nx.Tensor &lt; f32 [ 3 ] [ 1.0 , 1.0 , 1.0 ] &gt; iex&gt; vt # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , - 1.0 ] ] &gt; iex&gt; { u , s , vt } = Nx.LinAlg . svd ( Nx . tensor ( [ [ 2 , 0 , 0 ] , [ 0 , 3 , 0 ] , [ 0 , 0 , - 1 ] , [ 0 , 0 , 0 ] ] ) ) iex&gt; u # Nx.Tensor &lt; f32 [ 4 ] [ 4 ] [ [ 1.0 , 0.0 , 0.0 , 0.0 ] , [ 0.0 , 1.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , 1.0 , 0.0 ] , [ 0.0 , 0.0 , 0.0 , 1.0 ] ] &gt; iex&gt; s # Nx.Tensor &lt; f32 [ 3 ] [ 3.0 , 2.0 , 1.0 ] &gt; iex&gt; vt # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 0.0 , 1.0 , 0.0 ] , [ 1.0 , 0.0 , 0.0 ] , [ 0.0 , 0.0 , - 1.0 ] ] &gt;","ref":"Nx.LinAlg.html#svd/2","title":"Nx.LinAlg.svd/2","type":"function"},{"doc":"Solve the equation a x = b for x, assuming a is a batch of triangular matrices. Can also solve x a = b for x. See the :left_side option below. b must either be a batch of square matrices with the same dimensions as a or a batch of 1-D tensors with as many rows as a . Batch dimensions of a and b must be the same. Options The following options are defined in order of precedence :transform_a - Defines op(a) , depending on its value. Can be one of: :none -&gt; op(a) = a :transpose -&gt; op(a) = transpose(a) Defaults to :none :lower - When true , defines the a matrix as lower triangular. If false, a is upper triangular. Defaults to true :left_side - When true , solves the system as op(A).X = B . Otherwise, solves X.op(A) = B . Defaults to true . Examples iex&gt; a = Nx . tensor ( [ [ 3 , 0 , 0 , 0 ] , [ 2 , 1 , 0 , 0 ] , [ 1 , 0 , 1 , 0 ] , [ 1 , 1 , 1 , 1 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 4 , 2 , 4 , 2 ] ) ) # Nx.Tensor &lt; f32 [ 4 ] [ 1.3333333730697632 , - 0.6666666865348816 , 2.6666667461395264 , - 1.3333333730697632 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 1 , 2 , 1 ] ) ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 1.0 , - 1.0 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 0 , 1 , 1 ] ] ) iex&gt; b = Nx . tensor ( [ [ 1 , 2 , 3 ] , [ 2 , 2 , 4 ] , [ 2 , 0 , 1 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ 1.0 , 2.0 , 3.0 ] , [ 1.0 , 0.0 , 1.0 ] , [ 1.0 , 0.0 , 0.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 1 , 1 , 1 ] , [ 0 , 1 , 0 , 1 ] , [ 0 , 0 , 1 , 2 ] , [ 0 , 0 , 0 , 3 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 2 , 4 , 2 , 4 ] ) , lower : false ) # Nx.Tensor &lt; f32 [ 4 ] [ - 1.3333333730697632 , 2.6666667461395264 , - 0.6666666865348816 , 1.3333333730697632 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 2 , 1 ] ] ) iex&gt; b = Nx . tensor ( [ [ 0 , 2 , 1 ] , [ 1 , 1 , 0 ] , [ 3 , 3 , 1 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b , left_side : false ) # Nx.Tensor &lt; f32 [ 3 ] [ 3 ] [ [ - 1.0 , 0.0 , 1.0 ] , [ 0.0 , 1.0 , 0.0 ] , [ 1.0 , 1.0 , 1.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 1 , 1 ] , [ 0 , 1 , 1 ] , [ 0 , 0 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 1 , 2 , 1 ] ) , transform_a : :transpose , lower : false ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 1.0 , - 1.0 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 1 , 2 , 1 ] ) , transform_a : :none ) # Nx.Tensor &lt; f64 [ 3 ] [ 1.0 , 1.0 , - 1.0 ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 2 , 1 ] ] ) iex&gt; b = Nx . tensor ( [ [ 0 , 1 , 3 ] , [ 2 , 1 , 3 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b , left_side : false ) # Nx.Tensor &lt; f32 [ 2 ] [ 3 ] [ [ 2.0 , - 5.0 , 3.0 ] , [ 4.0 , - 5.0 , 3.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 2 , 1 ] ] ) iex&gt; b = Nx . tensor ( [ [ 0 , 2 ] , [ 3 , 0 ] , [ 0 , 0 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b , left_side : true ) # Nx.Tensor &lt; f32 [ 3 ] [ 2 ] [ [ 0.0 , 2.0 ] , [ 3.0 , - 2.0 ] , [ - 6.0 , 2.0 ] ] &gt; iex&gt; a = Nx . tensor ( [ ...&gt; [ 1 , 0 , 0 ] , ...&gt; [ 1 , Complex . new ( 0 , 1 ) , 0 ] , ...&gt; [ Complex . new ( 0 , 1 ) , 1 , 1 ] ...&gt; ] ) iex&gt; b = Nx . tensor ( [ 1 , - 1 , Complex . new ( 3 , 3 ) ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b ) # Nx.Tensor &lt; c64 [ 3 ] [ 1.0 + 0.0 i , 0.0 + 2.0 i , 3.0 + 0.0 i ] &gt; iex&gt; a = Nx . tensor ( [ [ [ 1 , 0 ] , [ 2 , 3 ] ] , [ [ 4 , 0 ] , [ 5 , 6 ] ] ] ) iex&gt; b = Nx . tensor ( [ [ 2 , - 1 ] , [ 3 , 7 ] ] ) iex&gt; Nx.LinAlg . triangular_solve ( a , b ) # Nx.Tensor &lt; f32 [ 2 ] [ 2 ] [ [ 2.0 , - 1.6666666269302368 ] , [ 0.75 , 0.5416666865348816 ] ] &gt; Error cases iex&gt; Nx.LinAlg . triangular_solve ( Nx . tensor ( [ [ 3 , 0 , 0 , 0 ] , [ 2 , 1 , 0 , 0 ] ] ) , Nx . tensor ( [ 4 , 2 , 4 , 2 ] ) ) ** (ArgumentError) triangular_solve/3 expected a square matrix or a batch of square matrices, got tensor with shape: {2, 4} iex&gt; Nx.LinAlg . triangular_solve ( Nx . tensor ( [ [ 3 , 0 , 0 , 0 ] , [ 2 , 1 , 0 , 0 ] , [ 1 , 1 , 1 , 1 ] , [ 1 , 1 , 1 , 1 ] ] ) , Nx . tensor ( [ 4 ] ) ) ** (ArgumentError) incompatible dimensions for a and b on triangular solve iex&gt; Nx.LinAlg . triangular_solve ( Nx . tensor ( [ [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 0 , 0 , 0 , 0 ] , [ 1 , 1 , 1 , 1 ] ] ) , Nx . tensor ( [ 4 , 2 , 4 , 2 ] ) ) ** (ArgumentError) can&#39;t solve for singular matrix iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 1 , 2 , 1 ] ) , transform_a : :conjugate ) ** (ArgumentError) complex numbers not supported yet iex&gt; a = Nx . tensor ( [ [ 1 , 0 , 0 ] , [ 1 , 1 , 0 ] , [ 1 , 1 , 1 ] ] , type : :f64 ) iex&gt; Nx.LinAlg . triangular_solve ( a , Nx . tensor ( [ 1 , 2 , 1 ] ) , transform_a : :other ) ** (ArgumentError) invalid value for :transform_a option, expected :none, :transpose, or :conjugate, got: :other","ref":"Nx.LinAlg.html#triangular_solve/3","title":"Nx.LinAlg.triangular_solve/3","type":"function"},{"doc":"Pseudo-random number generators. Unlike the stateful pseudo-random number generators (PRNGs) that users of most programming languages and numerical libraries may be accustomed to, Nx random functions require an explicit PRNG key to be passed as a first argument. That key is defined by an Nx.Tensor composed of 2 unsigned 32-bit integers, usually generated by the Nx.Random.key/1 function: iex&gt; Nx.Random . key ( 12 ) # Nx.Tensor &lt; u32 [ 2 ] [ 0 , 12 ] &gt; This key can then be used in any of Nx’s random number generation routines: iex&gt; key = Nx.Random . key ( 12 ) iex&gt; { uniform , _new_key } = Nx.Random . uniform ( key ) iex&gt; uniform # Nx.Tensor &lt; f32 0.7691127061843872 &gt; Now, when generating a new random number, you pass the new_key to get a different number. The function in this module also have a *_split variant, which is used when the key has been split before hand. Design and Context In short, Nx's PRNGs are based on a Threefry counter PRNG associated to a functional array-oriented splitting model. To summarize, among other requirements, Nx's PRNG aims to: Ensure reproducibility Parallelize well, both in terms of vectorization (generating array values) and multi-replica, multi-core computation. In particular it should not use sequencing constraints between random function calls.","ref":"Nx.Random.html","title":"Nx.Random","type":"module"},{"doc":"Folds in new data to a PRNG key. Examples iex&gt; key = Nx.Random . key ( 42 ) iex&gt; Nx.Random . fold_in ( key , 99 ) # Nx.Tensor &lt; u32 [ 2 ] [ 2015327502 , 1351855566 ] &gt; iex&gt; key = Nx.Random . key ( 42 ) iex&gt; Nx.Random . fold_in ( key , 1234 ) # Nx.Tensor &lt; u32 [ 2 ] [ 1356445167 , 2917756949 ] &gt; iex&gt; key = Nx.Random . key ( 42 ) iex&gt; Nx.Random . fold_in ( key , Nx . tensor ( [ [ 1 , 99 ] , [ 1234 , 13 ] ] ) ) # Nx.Tensor &lt; u32 [ 2 ] [ 2 ] [ 2 ] [ [ [ 64467757 , 2916123636 ] , [ 2015327502 , 1351855566 ] ] , [ [ 1356445167 , 2917756949 ] , [ 3514951389 , 229662949 ] ] ] &gt;","ref":"Nx.Random.html#fold_in/2","title":"Nx.Random.fold_in/2","type":"function"},{"doc":"Create a pseudo-random number generator (PRNG) key given an integer seed. Examples iex&gt; Nx.Random . key ( 12 ) # Nx.Tensor &lt; u32 [ 2 ] [ 0 , 12 ] &gt; iex&gt; Nx.Random . key ( 999999999999 ) # Nx.Tensor &lt; u32 [ 2 ] [ 232 , 3567587327 ] &gt;","ref":"Nx.Random.html#key/1","title":"Nx.Random.key/1","type":"function"},{"doc":"Shortcut for normal(key, 0.0, 1.0, opts) .","ref":"Nx.Random.html#normal/2","title":"Nx.Random.normal/2","type":"function"},{"doc":"Returns a normal distribution with the given mean and standard_deviation . Options :type - a float or complex type for the returned tensor :shape - shape of the returned tensor :names - the names of the returned tensor Examples iex&gt; key = Nx.Random . key ( 42 ) iex&gt; { normal , _new_key } = Nx.Random . normal ( key ) iex&gt; normal # Nx.Tensor &lt; f32 1.3694695234298706 &gt; iex&gt; key = Nx.Random . key ( 42 ) iex&gt; { normal , _new_key } = Nx.Random . normal ( key , 0 , 1 , shape : { 3 , 2 } , type : :f16 ) iex&gt; normal # Nx.Tensor &lt; f16 [ 3 ] [ 2 ] [ [ - 0.32568359375 , - 0.77197265625 ] , [ 0.39208984375 , 0.5341796875 ] , [ 0.270751953125 , - 2.080078125 ] ] &gt; iex&gt; key = Nx.Random . key ( 42 ) iex&gt; { normal , _new_key } = Nx.Random . normal ( key , 0 , 1 , shape : { 2 , 2 } , type : :c64 ) iex&gt; normal # Nx.Tensor &lt; c64 [ 2 ] [ 2 ] [ [ - 0.7632761001586914 + 0.8661127686500549 i , - 0.14282889664173126 - 0.7384796142578125 i ] , [ 0.678461492061615 + 0.4118310809135437 i , - 2.269538402557373 - 0.3689095079898834 i ] ] &gt; iex&gt; key = Nx.Random . key ( 1337 ) iex&gt; { normal , _new_key } = Nx.Random . normal ( key , 10 , 5 , shape : { 1_000 } ) iex&gt; Nx . mean ( normal ) # Nx.Tensor &lt; f32 9.70022201538086 &gt; iex&gt; Nx . standard_deviation ( normal ) # Nx.Tensor &lt; f32 5.051416397094727 &gt;","ref":"Nx.Random.html#normal/4","title":"Nx.Random.normal/4","type":"function"},{"doc":"Same as normal/4 but assumes the key has already been split.","ref":"Nx.Random.html#normal_split/4","title":"Nx.Random.normal_split/4","type":"function"},{"doc":"Sample uniform random integer values in [min_value, max_value) . Options :type - the integer type for the returned tensor :shape - shape of the returned tensor :names - the names of the returned tensor Examples iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; { randint , _new_key } = Nx.Random . randint ( key , 1 , 100 ) iex&gt; randint # Nx.Tensor &lt; s64 66 &gt; iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; { randint , _new_key } = Nx.Random . randint ( key , 1 , 100 , shape : { 3 , 2 } , type : :u32 ) iex&gt; randint # Nx.Tensor &lt; u32 [ 3 ] [ 2 ] [ [ 9 , 20 ] , [ 19 , 6 ] , [ 71 , 15 ] ] &gt;","ref":"Nx.Random.html#randint/4","title":"Nx.Random.randint/4","type":"function"},{"doc":"Same as randint/4 but assumes the key has already been split.","ref":"Nx.Random.html#randint_split/4","title":"Nx.Random.randint_split/4","type":"function"},{"doc":"Splits a PRNG key into num new keys by adding a leading axis. Examples iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; Nx.Random . split ( key ) # Nx.Tensor &lt; u32 [ 2 ] [ 2 ] [ [ 56197195 , 1801093307 ] , [ 961309823 , 1704866707 ] ] &gt; iex&gt; key = Nx.Random . key ( 999999999999 ) iex&gt; Nx.Random . split ( key , parts : 4 ) # Nx.Tensor &lt; u32 [ 4 ] [ 2 ] [ [ 3959978897 , 4079927650 ] , [ 3769699049 , 3585271160 ] , [ 3182829676 , 333122445 ] , [ 3185556048 , 1258545461 ] ] &gt;","ref":"Nx.Random.html#split/2","title":"Nx.Random.split/2","type":"function"},{"doc":"","ref":"Nx.Random.html#threefry2x32_20_concat/2","title":"Nx.Random.threefry2x32_20_concat/2","type":"function"},{"doc":"Shortcut for uniform(key, 0.0, 1.0, opts) .","ref":"Nx.Random.html#uniform/2","title":"Nx.Random.uniform/2","type":"function"},{"doc":"Sample uniform float values in [min_val, max_val) . Options :type - a float type for the returned tensor :shape - shape of the returned tensor :names - the names of the returned tensor Examples iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; { uniform , _new_key } = Nx.Random . uniform ( key ) iex&gt; uniform # Nx.Tensor &lt; f32 0.9728643894195557 &gt; iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; { uniform , _new_key } = Nx.Random . uniform ( key , shape : { 3 , 2 } , type : :f16 ) iex&gt; uniform # Nx.Tensor &lt; f16 [ 3 ] [ 2 ] [ [ 0.75390625 , 0.6484375 ] , [ 0.7294921875 , 0.21484375 ] , [ 0.09765625 , 0.0693359375 ] ] &gt; iex&gt; key = Nx.Random . key ( 1701 ) iex&gt; { uniform , _new_key } = Nx.Random . uniform ( key , shape : { 2 , 2 } , type : :c64 ) iex&gt; uniform # Nx.Tensor &lt; c64 [ 2 ] [ 2 ] [ [ 0.18404805660247803 + 0.6546461582183838 i , 0.5525915622711182 + 0.11568140983581543 i ] , [ 0.6074584722518921 + 0.8104375600814819 i , 0.247686505317688 + 0.21975469589233398 i ] ] &gt;","ref":"Nx.Random.html#uniform/4","title":"Nx.Random.uniform/4","type":"function"},{"doc":"Same as uniform/4 but assumes the key has already been split.","ref":"Nx.Random.html#uniform_split/4","title":"Nx.Random.uniform_split/4","type":"function"},{"doc":"Serving encapsulates client and server work to perform batched requests. Serving can be executed on the fly, without starting a server, but most often it is used to run servers that batch requests until a given size or timeout is reached. Inline/serverless workflow First, let's define a simple numerical definition function: defmodule MyDefn do import Nx.Defn defnp print_and_multiply ( x ) do print_value ( { :debug , x } ) x * 2 end end The function prints the given tensor and double its contents. We can use new/1 to create a serving that will return a JITted or compiled function to execute on batches of tensors: iex&gt; serving = Nx.Serving . new ( fn -&gt; Nx.Defn . jit ( &amp; print_and_multiply / 1 ) end ) iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) ] ) iex&gt; Nx.Serving . run ( serving , batch ) { :debug , # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ [ 1 , 2 , 3 ] ] &gt; } # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ [ 2 , 4 , 6 ] ] &gt; You should see two values printed. The first is the result of Nx.Defn.Kernel.print_value/1 , which shows the tensor that was actually part of the computation and how it was batched. Then we see the result of the computation. When defining a Nx.Serving , we can also customize how the data is batched by using the client_preprocessing as well as the result by using client_postprocessing hooks. Let's give it another try: iex&gt; serving = ( ...&gt; Nx.Serving . new ( fn -&gt; Nx.Defn . jit ( &amp; print_and_multiply / 1 ) end ) ...&gt; |&gt; Nx.Serving . client_preprocessing ( fn input -&gt; { Nx.Batch . stack ( input ) , :client_info } end ) ...&gt; |&gt; Nx.Serving . client_postprocessing ( &amp; { &amp;1 , &amp;2 , &amp;3 } ) ...&gt; ) iex&gt; Nx.Serving . run ( serving , [ Nx . tensor ( [ 1 , 2 ] ) , Nx . tensor ( [ 3 , 4 ] ) ] ) { :debug , # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 1 , 2 ] , [ 3 , 4 ] ] &gt; } { # Nx.Tensor &lt; s64 [ 2 ] [ 2 ] [ [ 2 , 4 ] , [ 6 , 8 ] ] &gt; , :server_info , :client_info } You can see the results are a bit different now. First of all, notice we were able to run the serving passing a list of tensors. Our custom client_preprocessing function stacks those tensors into a batch of two entries and returns a tuple with a Nx.Batch struct and additional client information which we represent as the atom :client_info . The default client preprocessing simply enforces a batch was given and returns no client information. Then the result is a {..., ..., ...} tuple, returned by the client postprocessing function, containing the result, the server information (which we will learn later how to customize it), and the client information. From this, we can infer the default implementation of client_postprocessing simply returns the result, discarding the server and client information. So far, Nx.Serving has not given us much. It has simply encapsulated the execution of a function. Its full power comes when we start running our own Nx.Serving process. That's when we will also learn why we have a client_ prefix in some of the function names. Stateful/process workflow Nx.Serving allows us to define a process that will batch requests up to a certain size or within a certain time. To do so, we need to start a Nx.Serving process with a serving inside a supervision tree: children = [ { Nx.Serving , serving : Nx.Serving . new ( Nx.Defn . jit ( &amp; print_and_multiply / 1 ) ) , name : MyServing , batch_size : 10 , batch_timeout : 100 } ] Supervisor . start_child ( children , strategy : :one_for_one ) Now you can send batched runs to said process: iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) , Nx . tensor ( [ 4 , 5 , 6 ] ) ] ) iex&gt; Nx.Serving . batched_run ( MyServing , batch ) { :debug , # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 1 , 2 , 3 ] , [ 4 , 5 , 6 ] ] &gt; } # Nx.Tensor &lt; s64 [ 2 ] [ 3 ] [ [ 2 , 4 , 6 ] , [ 8 , 10 , 12 ] ] &gt; In the example, we pushed a batch of 2 and eventually got a reply. The process will wait up for requests from other processes, up to 100 milliseconds or until it gets 10 entries. Then it merges all batches together and, once the result is computed, it slices and distributed those responses to each caller. If there is any client_preprocessing function, it will be executed before the batch is sent to the server. If there is any client_postprocessing function, it will be executed after getting the response from the server. Module-based serving In the examples so far, we have been using the default version of Nx.Serving , which executes the given function for each batch. However, we can also use new/2 to start a module-based version of Nx.Serving , which acts similar to an Elixir GenServer and gives us more control over both inline and process workflows. A simple module implementation of a Nx.Serving could look like this: defmodule MyServing do @behaviour Nx.Serving defnp print_and_multiply ( x ) do print_value ( { :debug , x } ) x * 2 end @impl true def init ( _inline_or_process , :unused_arg ) do { :ok , Nx.Defn . jit ( &amp; print_and_multiply / 1 ) } end @impl true def handle_batch ( batch , function ) do { :execute , fn -&gt; { function . ( batch ) , :server_info } end , function } end end It has two functions: init/2 , which receives the type of serving ( :inline or :process ) and the serving argument. In this step, we capture print_and_multiply/1 as a jitted function. The second function is called handle_batch/2 . This function receives a Nx.Batch and it must return a function to execute. The function itself must return a two element-tuple: the batched results and some server information. The server information can be any value and we set it to the atom :server_info . Now let's give it a try by defining a serving with our module and then running it on a batch: iex&gt; serving = Nx.Serving . new ( MyServing , :unused_arg ) iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( [ 1 , 2 , 3 ] ) ] ) iex&gt; Nx.Serving . run ( serving , batch ) { :debug , # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ [ 1 , 2 , 3 ] ] &gt; } # Nx.Tensor &lt; s64 [ 1 ] [ 3 ] [ [ 2 , 4 , 6 ] ] &gt; From here on, you use start_link/1 to start this serving in your supervision and even customize client_preprocessing/1 and client_postprocessing/1 callbacks to this serving, as seen in the previous sections.","ref":"Nx.Serving.html","title":"Nx.Serving","type":"behaviour"},{"doc":"Runs the given input on the process given by name . The process name will batch requests and send a response either when the batch is full or on timeout. See the module documentation for more information. Note you cannot batch an input larger than the configured :batch_size in the server.","ref":"Nx.Serving.html#batched_run/2","title":"Nx.Serving.batched_run/2","type":"function"},{"doc":"Sets the client postprocessing function. The default implementation returns the first element given to the function.","ref":"Nx.Serving.html#client_postprocessing/2","title":"Nx.Serving.client_postprocessing/2","type":"function"},{"doc":"Sets the client preprocessing function. The default implementation creates a single element batch with the given argument and is equivalent to &amp;Nx.Batch.stack([&amp;1]) .","ref":"Nx.Serving.html#client_preprocessing/2","title":"Nx.Serving.client_preprocessing/2","type":"function"},{"doc":"Receives a batch and returns a function to execute the batch. In case of serving processes, the function is executed is an separate process.","ref":"Nx.Serving.html#c:handle_batch/2","title":"Nx.Serving.handle_batch/2","type":"callback"},{"doc":"The callback used to initialize the serving. The first argument reveals if the serving is executed inline, such as by calling run/2 , by started with the process. The second argument is the serving argument given to new/2 . It must return {:ok, state} , where the state can be any term.","ref":"Nx.Serving.html#c:init/2","title":"Nx.Serving.init/2","type":"callback"},{"doc":"Creates a new function serving. It expects a function that returns a JITted (via Nx.Defn.jit/2 ) or compiled (via Nx.Defn.compile/3 ) one-arity function as argument. The JITted/compiled function will be called with the arguments returned by the client_preprocessing callback. A second argument called process_options , which is optional, can be given to customize the options when starting the serving under a process.","ref":"Nx.Serving.html#new/2","title":"Nx.Serving.new/2","type":"function"},{"doc":"Creates a new module-based serving. It expects a module and an argument that is given to its init callback. A third argument called process_options , which is optional, can be given to customize the options when starting the serving under a process.","ref":"Nx.Serving.html#new/3","title":"Nx.Serving.new/3","type":"function"},{"doc":"Runs serving with the given input inline with the current process.","ref":"Nx.Serving.html#run/2","title":"Nx.Serving.run/2","type":"function"},{"doc":"Starts a Nx.Serving process to batch requests to a given serving. Options :name - an atom with the name of the process :serving - a Nx.Serving struct with the serving configuration :batch_size - the maximum size to batch for. A value is first read from the Nx.Serving struct and then it falls back to this option (which defaults to 1 ) :batch_timeout - the maximum time to wait, in milliseconds, before executing the batch. A value is first read from the Nx.Serving struct and then it falls back to this option (which defaults to 100 ms)","ref":"Nx.Serving.html#start_link/1","title":"Nx.Serving.start_link/1","type":"function"},{"doc":"","ref":"Nx.Serving.html#t:client_info/0","title":"Nx.Serving.client_info/0","type":"type"},{"doc":"","ref":"Nx.Serving.html#t:client_postprocessing/0","title":"Nx.Serving.client_postprocessing/0","type":"type"},{"doc":"","ref":"Nx.Serving.html#t:client_preprocessing/0","title":"Nx.Serving.client_preprocessing/0","type":"type"},{"doc":"","ref":"Nx.Serving.html#t:metadata/0","title":"Nx.Serving.metadata/0","type":"type"},{"doc":"","ref":"Nx.Serving.html#t:t/0","title":"Nx.Serving.t/0","type":"type"},{"doc":"A protocol that teaches Nx how to traverse data structures non-recursively. Nx and defn expect the arguments to be numbers, tensors, or one of the following composite data types: tuples of numbers/tensors maps of any key with numbers/tensors as values any struct that implements Nx.Container If you need to pass additional values, you can implement or derive this protocol. For example: @derive { Nx.Container , containers : [ :field_name , :other_field ] } defstruct [ :field_name , :other_fields , ... ] The :containers option is required and it must specify a list of fields that contains tensors. Inside defn , the container fields will be automatically converted to tensor expressions. All other fields will be reset to their default value, unless you explicitly declare them to be kept: @derive { Nx.Container , containers : [ :field_name , :other_field ] , keep : [ :another_field ] } defstruct [ :field_name , :other_fields , ... ] Note the functions in this module are not recursive. If you want to deeply traverse and reduce containers, use the functions in Nx.Defn.Composite instead. Careful! : If you keep a field, its value will be part of the Nx.Defn compiler cache key (i.e. therefore if you give a struct with two different values for a kept field, Nx.Defn will have to compile and cache it twice). You must only keep fields that you are certain to be used inside defn during compilation time.","ref":"Nx.Container.html","title":"Nx.Container","type":"protocol"},{"doc":"Reduces non-recursively tensors in a data structure with acc and fun . fun is invoked with each tensor or tensor container in the data structure plus an accumulator. It must return the new accumulator. This function the final accumulator. Given fun may receive containers, it is not recursive by default. See Nx.Defn.Composite.reduce/3 for a recursive variant.","ref":"Nx.Container.html#reduce/3","title":"Nx.Container.reduce/3","type":"function"},{"doc":"Traverses non-recursively tensors in a data structure with acc and fun . fun is invoked with each tensor or tensor container in the data structure plus an accumulator. It must return a two element tuple with the updated value and accumulator. This function returns the updated container and the accumulator. Given fun may receive containers, it is not recursive by default. See Nx.Defn.Composite.traverse/3 for a recursive variant.","ref":"Nx.Container.html#traverse/3","title":"Nx.Container.traverse/3","type":"function"},{"doc":"","ref":"Nx.Container.html#t:t/0","title":"Nx.Container.t/0","type":"type"},{"doc":"Converts a data structure to a container lazily and recursively. Sometimes building tensors for a container is an expensive operation, so we want to allow that to happen lazily. This module provides a single traverse implementation that emits the tensor template and a function that computes the tensor as two distinct values. Then a tensor is only allocated if necessary. This protocol is used throughout Nx.Defn API. This means compilation, jitting, and streaming will only realize lazy tensors when necessary. If a data structure does not implement this protocol, a default implementation converts eager to lazy using Nx.Container . When a value is given to defn , it is first converted to tensors and containers via Nx.LazyContainer . Inside defn , there are no lazy containers, only containers.","ref":"Nx.LazyContainer.html","title":"Nx.LazyContainer","type":"protocol"},{"doc":"Traverses recursively tensors in a data structure with acc and fun . For each tensor in the container, fun receives a tensor template, an anonymous function to build the actual tensor, and the accumulator . It returns a two element tuple with a non-lazy Nx.Container and the accumulator. This function returns the updated container and the accumulator. Note this function is recursive by default. Therefore if you are implementing this function and one of your arguments may be containers, you must call Nx.LazyContainer.traverse/3 on said arguments so they are recursively traversed.","ref":"Nx.LazyContainer.html#traverse/3","title":"Nx.LazyContainer.traverse/3","type":"function"},{"doc":"","ref":"Nx.LazyContainer.html#t:t/0","title":"Nx.LazyContainer.t/0","type":"type"},{"doc":"The protocol for streaming data in and out of backends.","ref":"Nx.Stream.html","title":"Nx.Stream","type":"protocol"},{"doc":"Returns the output of the stream. It may be a tensor, a tuple of tensors, or a map of tensors.","ref":"Nx.Stream.html#done/1","title":"Nx.Stream.done/1","type":"function"},{"doc":"Receives data from the stream. It may be a tensor, a tuple of tensors, or a map of tensors.","ref":"Nx.Stream.html#recv/1","title":"Nx.Stream.recv/1","type":"function"},{"doc":"Sends a tensor. Returns the given tensor.","ref":"Nx.Stream.html#send/2","title":"Nx.Stream.send/2","type":"function"},{"doc":"","ref":"Nx.Stream.html#t:t/0","title":"Nx.Stream.t/0","type":"type"},{"doc":"Creates a batch of tensors (and containers). A batch is lazily traversed, concatenated, and padded upon defn invocation.","ref":"Nx.Batch.html","title":"Nx.Batch","type":"module"},{"doc":"A Nx.Batch struct. The :size field is public.","ref":"Nx.Batch.html#__struct__/0","title":"Nx.Batch.__struct__/0","type":"function"},{"doc":"Concatenates the given entries to the batch. Entries are concatenated based on their first axis. If the first axis has multiple entries, each entry is added to the size of the batch. You can either concatenate to an existing batch or skip the batch argument to create a new batch. See stack/2 if you want to stack entries instead of concatenating them. Examples If no batch is given, one is automatically created: iex&gt; batch = Nx.Batch . concatenate ( [ Nx . tensor ( [ 1 ] ) , Nx . tensor ( [ 2 ] ) , Nx . tensor ( [ 3 ] ) ] ) iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; But you can also concatenate to existing batches: iex&gt; batch = Nx.Batch . concatenate ( [ Nx . tensor ( [ 1 ] ) , Nx . tensor ( [ 2 ] ) ] ) iex&gt; batch = Nx.Batch . concatenate ( batch , [ Nx . tensor ( [ 3 ] ) , Nx . tensor ( [ 4 ] ) ] ) iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 1 , 2 , 3 , 4 ] &gt; If the first axis has multiple entries, each entry counts towards the size of the batch: iex&gt; batch = Nx.Batch . concatenate ( [ Nx . tensor ( [ 1 , 2 ] ) , Nx . tensor ( [ 3 , 4 , 5 ] ) ] ) iex&gt; batch . size 5 iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 5 ] [ 1 , 2 , 3 , 4 , 5 ] &gt; What makes batches powerful is that they can concatenate across containers: iex&gt; container1 = { Nx . tensor ( [ 11 ] ) , Nx . tensor ( [ 21 ] ) } iex&gt; container2 = { Nx . tensor ( [ 12 ] ) , Nx . tensor ( [ 22 ] ) } iex&gt; batch = Nx.Batch . concatenate ( [ container1 , container2 ] ) iex&gt; { batched1 , batched2 } = Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) iex&gt; batched1 # Nx.Tensor &lt; s64 [ 2 ] [ 11 , 12 ] &gt; iex&gt; batched2 # Nx.Tensor &lt; s64 [ 2 ] [ 21 , 22 ] &gt;","ref":"Nx.Batch.html#concatenate/2","title":"Nx.Batch.concatenate/2","type":"function"},{"doc":"Merges a list of batches. See merge/2 .","ref":"Nx.Batch.html#merge/1","title":"Nx.Batch.merge/1","type":"function"},{"doc":"Merges two batches. The tensors on the left will appear before the tensors on the right. The size and padding of both batches are summed. The padding still applies only at the end of batch. It will raise if the batch templates are incompatible. Examples iex&gt; batch1 = Nx.Batch . stack ( [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) , Nx . tensor ( 3 ) ] ) iex&gt; batch2 = Nx.Batch . concatenate ( [ Nx . tensor ( [ 4 , 5 ] ) , Nx . tensor ( [ 6 , 7 , 8 ] ) ] ) iex&gt; batch = Nx.Batch . merge ( batch1 , batch2 ) iex&gt; batch . size 8 iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 8 ] [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] &gt;","ref":"Nx.Batch.html#merge/2","title":"Nx.Batch.merge/2","type":"function"},{"doc":"Returns a new empty batch.","ref":"Nx.Batch.html#new/0","title":"Nx.Batch.new/0","type":"function"},{"doc":"Configures the batch with the given padding. The batch will be padded when consumed: iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) , Nx . tensor ( 3 ) ] ) iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ Nx.Batch . pad ( batch , 2 ) ] ) # Nx.Tensor &lt; s64 [ 5 ] [ 1 , 2 , 3 , 0 , 0 ] &gt;","ref":"Nx.Batch.html#pad/2","title":"Nx.Batch.pad/2","type":"function"},{"doc":"Splits a batch in two, where the first one has at most n elements. If there is any padding and the batch is not full, the amount of padding necessary will be moved to the first batch and the remaining stays in the second batch. Examples iex&gt; batch = Nx.Batch . concatenate ( [ Nx . tensor ( [ 1 , 2 ] ) , Nx . tensor ( [ 3 , 4 , 5 ] ) ] ) iex&gt; { left , right } = Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ Nx.Batch . split ( batch , 3 ) ] ) iex&gt; left # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; iex&gt; right # Nx.Tensor &lt; s64 [ 2 ] [ 4 , 5 ] &gt;","ref":"Nx.Batch.html#split/2","title":"Nx.Batch.split/2","type":"function"},{"doc":"Stacks the given entries to the batch. Each entry counts exactly as a single entry. You can either stack to an existing batch or skip the batch argument to create a new batch. See concatenate/2 if you want to concatenate entries instead of stacking them. Examples If no batch is given, one is automatically created: iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) , Nx . tensor ( 3 ) ] ) iex&gt; batch . size 3 iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 3 ] [ 1 , 2 , 3 ] &gt; But you can also stack an existing batch: iex&gt; batch = Nx.Batch . stack ( [ Nx . tensor ( 1 ) , Nx . tensor ( 2 ) ] ) iex&gt; batch = Nx.Batch . stack ( batch , [ Nx . tensor ( 3 ) , Nx . tensor ( 4 ) ] ) iex&gt; batch . size 4 iex&gt; Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) # Nx.Tensor &lt; s64 [ 4 ] [ 1 , 2 , 3 , 4 ] &gt; What makes batches powerful is that they can concatenate across containers: iex&gt; container1 = { Nx . tensor ( 11 ) , Nx . tensor ( 21 ) } iex&gt; container2 = { Nx . tensor ( 12 ) , Nx . tensor ( 22 ) } iex&gt; batch = Nx.Batch . stack ( [ container1 , container2 ] ) iex&gt; { batched1 , batched2 } = Nx.Defn . jit_apply ( &amp; Function . identity / 1 , [ batch ] ) iex&gt; batched1 # Nx.Tensor &lt; s64 [ 2 ] [ 11 , 12 ] &gt; iex&gt; batched2 # Nx.Tensor &lt; s64 [ 2 ] [ 21 , 22 ] &gt;","ref":"Nx.Batch.html#stack/2","title":"Nx.Batch.stack/2","type":"function"},{"doc":"","ref":"Nx.Batch.html#t:t/0","title":"Nx.Batch.t/0","type":"type"},{"doc":"Provides a heatmap that is printed using ANSI colors in the terminal.","ref":"Nx.Heatmap.html","title":"Nx.Heatmap","type":"module"},{"doc":"The tensor struct and the behaviour for backends. Nx.Tensor is a generic container for multidimensional data structures. It contains the tensor type, shape, and names. The data itself is a struct that points to a backend responsible for controlling the data. The backend behaviour is described in Nx.Backend . The tensor has the following fields: :data - the tensor backend and its data :shape - the tensor shape :type - the tensor type :names - the tensor names In general it is discouraged to access those fields directly. Use the functions in the Nx module instead. Backends have to access those fields but it cannot update them, except for the :data field itself.","ref":"Nx.Tensor.html","title":"Nx.Tensor","type":"module"},{"doc":"","ref":"Nx.Tensor.html#t:axes/0","title":"Nx.Tensor.axes/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:axis/0","title":"Nx.Tensor.axis/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:data/0","title":"Nx.Tensor.data/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:name/0","title":"Nx.Tensor.name/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:shape/0","title":"Nx.Tensor.shape/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:t/0","title":"Nx.Tensor.t/0","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:t/1","title":"Nx.Tensor.t/1","type":"type"},{"doc":"","ref":"Nx.Tensor.html#t:type/0","title":"Nx.Tensor.type/0","type":"type"},{"doc":"The behaviour for tensor backends. Each backend is module that defines a struct and implements the callbacks defined in this module. The callbacks are mostly implementations of the functions in the Nx module with the tensor output shape given as first argument. Nx backends come in two flavors: opaque backends, of which you should not access its data directly except through the functions in the Nx module, and public ones, of which its data can be directly accessed and traversed. The former typically have the Backend suffix. Nx ships with the following backends: Nx.BinaryBackend - an opaque backend written in pure Elixir that stores the data in Elixir's binaries. This is the default backend used by the Nx module. The backend itself (and its data) is private and must not be accessed directly. Nx.TemplateBackend - an opaque backend written that works as a template in APIs to declare the type, shape, and names of tensors to be expected in the future. Nx.Defn.Expr - a public backend used by defn to build expression trees that are traversed by custom compilers. This module also includes functions that are meant to be shared across backends.","ref":"Nx.Backend.html","title":"Nx.Backend","type":"behaviour"},{"doc":"","ref":"Nx.Backend.html#c:abs/2","title":"Nx.Backend.abs/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:acos/2","title":"Nx.Backend.acos/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:acosh/2","title":"Nx.Backend.acosh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:add/3","title":"Nx.Backend.add/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:all/3","title":"Nx.Backend.all/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:all_close/4","title":"Nx.Backend.all_close/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:any/3","title":"Nx.Backend.any/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:argmax/3","title":"Nx.Backend.argmax/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:argmin/3","title":"Nx.Backend.argmin/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:argsort/3","title":"Nx.Backend.argsort/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:as_type/2","title":"Nx.Backend.as_type/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:asin/2","title":"Nx.Backend.asin/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:asinh/2","title":"Nx.Backend.asinh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:atan2/3","title":"Nx.Backend.atan2/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:atan/2","title":"Nx.Backend.atan/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:atanh/2","title":"Nx.Backend.atanh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:backend_copy/3","title":"Nx.Backend.backend_copy/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:backend_deallocate/1","title":"Nx.Backend.backend_deallocate/1","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:backend_transfer/3","title":"Nx.Backend.backend_transfer/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:bitcast/2","title":"Nx.Backend.bitcast/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:bitwise_and/3","title":"Nx.Backend.bitwise_and/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:bitwise_not/2","title":"Nx.Backend.bitwise_not/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:bitwise_or/3","title":"Nx.Backend.bitwise_or/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:bitwise_xor/3","title":"Nx.Backend.bitwise_xor/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:broadcast/4","title":"Nx.Backend.broadcast/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cbrt/2","title":"Nx.Backend.cbrt/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:ceil/2","title":"Nx.Backend.ceil/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cholesky/2","title":"Nx.Backend.cholesky/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:clip/4","title":"Nx.Backend.clip/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:concatenate/3","title":"Nx.Backend.concatenate/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:conjugate/2","title":"Nx.Backend.conjugate/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:constant/3","title":"Nx.Backend.constant/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:conv/4","title":"Nx.Backend.conv/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cos/2","title":"Nx.Backend.cos/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cosh/2","title":"Nx.Backend.cosh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:count_leading_zeros/2","title":"Nx.Backend.count_leading_zeros/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cumulative_max/3","title":"Nx.Backend.cumulative_max/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cumulative_min/3","title":"Nx.Backend.cumulative_min/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cumulative_product/3","title":"Nx.Backend.cumulative_product/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:cumulative_sum/3","title":"Nx.Backend.cumulative_sum/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:determinant/2","title":"Nx.Backend.determinant/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:divide/3","title":"Nx.Backend.divide/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:dot/7","title":"Nx.Backend.dot/7","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:eigh/3","title":"Nx.Backend.eigh/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:equal/3","title":"Nx.Backend.equal/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:erf/2","title":"Nx.Backend.erf/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:erf_inv/2","title":"Nx.Backend.erf_inv/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:erfc/2","title":"Nx.Backend.erfc/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:exp/2","title":"Nx.Backend.exp/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:expm1/2","title":"Nx.Backend.expm1/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:eye/2","title":"Nx.Backend.eye/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:fft/3","title":"Nx.Backend.fft/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:floor/2","title":"Nx.Backend.floor/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:from_binary/3","title":"Nx.Backend.from_binary/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:gather/3","title":"Nx.Backend.gather/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:greater/3","title":"Nx.Backend.greater/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:greater_equal/3","title":"Nx.Backend.greater_equal/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:ifft/3","title":"Nx.Backend.ifft/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:imag/2","title":"Nx.Backend.imag/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:indexed_add/4","title":"Nx.Backend.indexed_add/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:indexed_put/4","title":"Nx.Backend.indexed_put/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:init/1","title":"Nx.Backend.init/1","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:inspect/2","title":"Nx.Backend.inspect/2","type":"callback"},{"doc":"Inspects the given tensor given by binary . Note the binary may have fewer elements than the tensor size but, in such cases, it must strictly have more elements than inspect_opts.limit","ref":"Nx.Backend.html#inspect/3","title":"Nx.Backend.inspect/3","type":"function"},{"doc":"","ref":"Nx.Backend.html#c:iota/3","title":"Nx.Backend.iota/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:is_infinity/2","title":"Nx.Backend.is_infinity/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:is_nan/2","title":"Nx.Backend.is_nan/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:left_shift/3","title":"Nx.Backend.left_shift/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:less/3","title":"Nx.Backend.less/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:less_equal/3","title":"Nx.Backend.less_equal/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:log1p/2","title":"Nx.Backend.log1p/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:log/2","title":"Nx.Backend.log/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:logical_and/3","title":"Nx.Backend.logical_and/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:logical_not/2","title":"Nx.Backend.logical_not/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:logical_or/3","title":"Nx.Backend.logical_or/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:logical_xor/3","title":"Nx.Backend.logical_xor/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:lu/3","title":"Nx.Backend.lu/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:map/4","title":"Nx.Backend.map/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:max/3","title":"Nx.Backend.max/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:min/3","title":"Nx.Backend.min/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:multiply/3","title":"Nx.Backend.multiply/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:negate/2","title":"Nx.Backend.negate/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:not_equal/3","title":"Nx.Backend.not_equal/3","type":"callback"},{"doc":"Invoked for execution of optional callbacks with a default implementation. First we will attempt to call the optional callback itself (one of the many callbacks defined below), then we attempt to call this callback (which is also optional), then we fallback to the default iomplementation.","ref":"Nx.Backend.html#c:optional/3","title":"Nx.Backend.optional/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:pad/4","title":"Nx.Backend.pad/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:phase/2","title":"Nx.Backend.phase/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:population_count/2","title":"Nx.Backend.population_count/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:power/3","title":"Nx.Backend.power/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:product/3","title":"Nx.Backend.product/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:put_slice/4","title":"Nx.Backend.put_slice/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:qr/3","title":"Nx.Backend.qr/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:quotient/3","title":"Nx.Backend.quotient/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:random_normal/4","title":"Nx.Backend.random_normal/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:random_uniform/4","title":"Nx.Backend.random_uniform/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:real/2","title":"Nx.Backend.real/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:reduce/5","title":"Nx.Backend.reduce/5","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:reduce_max/3","title":"Nx.Backend.reduce_max/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:reduce_min/3","title":"Nx.Backend.reduce_min/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:remainder/3","title":"Nx.Backend.remainder/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:reshape/2","title":"Nx.Backend.reshape/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:reverse/3","title":"Nx.Backend.reverse/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:right_shift/3","title":"Nx.Backend.right_shift/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:round/2","title":"Nx.Backend.round/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:rsqrt/2","title":"Nx.Backend.rsqrt/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:select/4","title":"Nx.Backend.select/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sigmoid/2","title":"Nx.Backend.sigmoid/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sign/2","title":"Nx.Backend.sign/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sin/2","title":"Nx.Backend.sin/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sinh/2","title":"Nx.Backend.sinh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:slice/5","title":"Nx.Backend.slice/5","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:solve/3","title":"Nx.Backend.solve/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sort/3","title":"Nx.Backend.sort/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sqrt/2","title":"Nx.Backend.sqrt/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:squeeze/3","title":"Nx.Backend.squeeze/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:subtract/3","title":"Nx.Backend.subtract/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:sum/3","title":"Nx.Backend.sum/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:svd/3","title":"Nx.Backend.svd/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:take/4","title":"Nx.Backend.take/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:take_along_axis/4","title":"Nx.Backend.take_along_axis/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:tan/2","title":"Nx.Backend.tan/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:tanh/2","title":"Nx.Backend.tanh/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:to_batched/3","title":"Nx.Backend.to_batched/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:to_binary/2","title":"Nx.Backend.to_binary/2","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:transpose/3","title":"Nx.Backend.transpose/3","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:triangular_solve/4","title":"Nx.Backend.triangular_solve/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_max/4","title":"Nx.Backend.window_max/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_min/4","title":"Nx.Backend.window_min/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_product/4","title":"Nx.Backend.window_product/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_reduce/6","title":"Nx.Backend.window_reduce/6","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_scatter_max/6","title":"Nx.Backend.window_scatter_max/6","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_scatter_min/6","title":"Nx.Backend.window_scatter_min/6","type":"callback"},{"doc":"","ref":"Nx.Backend.html#c:window_sum/4","title":"Nx.Backend.window_sum/4","type":"callback"},{"doc":"","ref":"Nx.Backend.html#t:axes/0","title":"Nx.Backend.axes/0","type":"type"},{"doc":"","ref":"Nx.Backend.html#t:axis/0","title":"Nx.Backend.axis/0","type":"type"},{"doc":"","ref":"Nx.Backend.html#t:backend_options/0","title":"Nx.Backend.backend_options/0","type":"type"},{"doc":"","ref":"Nx.Backend.html#t:shape/0","title":"Nx.Backend.shape/0","type":"type"},{"doc":"","ref":"Nx.Backend.html#t:t/0","title":"Nx.Backend.t/0","type":"type"},{"doc":"","ref":"Nx.Backend.html#t:tensor/0","title":"Nx.Backend.tensor/0","type":"type"},{"doc":"An opaque backend written in pure Elixir that stores the data in Elixir's binaries. This is the default backend used by the Nx module. The backend itself (and its data) is private and must not be accessed directly.","ref":"Nx.BinaryBackend.html","title":"Nx.BinaryBackend","type":"module"},{"doc":"An opaque backend written that is used as template to declare the type, shape, and names of tensors to be expected in the future. It doesn't perform any operation, it always raises.","ref":"Nx.TemplateBackend.html","title":"Nx.TemplateBackend","type":"module"},{"doc":"Conveniences for working with types. A type is a two-element tuple with the name and the size. The respective sizes for the types are the following: :s - signed integer (8, 16, 32, 64) :u - unsigned integer (8, 16, 32, 64) :f - float (16, 32, 64) :bf - a brain floating point (16) :c - a complex number, represented as a pair of floats (64, 128) Each type has an equivalent atom representation, for example {:s, 8} can be expressed as :s8 . When working with user-given types make sure to call normalize!/1 to get the canonical representation. Note: there is a special type used by the defn compiler which is {:tuple, size} , that represents a tuple. Said types do not appear on user code, only on compiler implementations, and therefore are not handled by the functions in this module.","ref":"Nx.Type.html","title":"Nx.Type","type":"module"},{"doc":"Casts the given number to type. It does not handle overflow/underflow, returning the number as is, but cast. Examples iex&gt; Nx.Type . cast_number! ( { :u , 8 } , 10 ) 10 iex&gt; Nx.Type . cast_number! ( { :s , 8 } , 10 ) 10 iex&gt; Nx.Type . cast_number! ( { :s , 8 } , - 10 ) - 10 iex&gt; Nx.Type . cast_number! ( { :f , 32 } , 10 ) 10.0 iex&gt; Nx.Type . cast_number! ( { :bf , 16 } , - 10 ) - 10.0 iex&gt; Nx.Type . cast_number! ( { :f , 32 } , 10.0 ) 10.0 iex&gt; Nx.Type . cast_number! ( { :bf , 16 } , - 10.0 ) - 10.0 iex&gt; Nx.Type . cast_number! ( { :c , 64 } , 10 ) % Complex { im : 0.0 , re : 10.0 } iex&gt; Nx.Type . cast_number! ( { :u , 8 } , - 10 ) ** (ArgumentError) cannot cast number -10 to {:u, 8} iex&gt; Nx.Type . cast_number! ( { :s , 8 } , 10.0 ) ** (ArgumentError) cannot cast number 10.0 to {:s, 8}","ref":"Nx.Type.html#cast_number!/2","title":"Nx.Type.cast_number!/2","type":"function"},{"doc":"Returns true if the type is a complex number. Examples iex&gt; Nx.Type . complex? ( { :c , 64 } ) true iex&gt; Nx.Type . complex? ( { :f , 64 } ) false","ref":"Nx.Type.html#complex?/1","title":"Nx.Type.complex?/1","type":"function"},{"doc":"Returns true if the type is a float in Elixir. Examples iex&gt; Nx.Type . float? ( { :f , 32 } ) true iex&gt; Nx.Type . float? ( { :bf , 16 } ) true iex&gt; Nx.Type . float? ( { :u , 64 } ) false","ref":"Nx.Type.html#float?/1","title":"Nx.Type.float?/1","type":"function"},{"doc":"Infers the type of the given number. Examples iex&gt; Nx.Type . infer ( 1 ) { :s , 64 } iex&gt; Nx.Type . infer ( 1.0 ) { :f , 32 } iex&gt; Nx.Type . infer ( Complex . new ( 1 ) ) { :c , 64 }","ref":"Nx.Type.html#infer/1","title":"Nx.Type.infer/1","type":"function"},{"doc":"Returns infinity as a binary for the given type.","ref":"Nx.Type.html#infinity_binary/1","title":"Nx.Type.infinity_binary/1","type":"function"},{"doc":"Returns true if the type is an integer in Elixir. Examples iex&gt; Nx.Type . integer? ( { :s , 8 } ) true iex&gt; Nx.Type . integer? ( { :u , 64 } ) true iex&gt; Nx.Type . integer? ( { :f , 64 } ) false","ref":"Nx.Type.html#integer?/1","title":"Nx.Type.integer?/1","type":"function"},{"doc":"Returns the maximum possible value for the given type.","ref":"Nx.Type.html#max_binary/1","title":"Nx.Type.max_binary/1","type":"function"},{"doc":"Returns the maximum possible finite value for the given type.","ref":"Nx.Type.html#max_finite_binary/1","title":"Nx.Type.max_finite_binary/1","type":"function"},{"doc":"Merges the given types finding a suitable representation for both. Types have the following precedence: c &gt; f &gt; bf &gt; s &gt; u If the types are the same, they are merged to the highest size. If they are different, the one with the highest precedence wins, as long as the size of the max(big, small * 2)) fits under 64 bits. Otherwise it casts to f64. In the case of complex numbers, the maximum bit size is 128 bits because they are composed of two floats. Examples iex&gt; Nx.Type . merge ( { :s , 8 } , { :s , 8 } ) { :s , 8 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :s , 64 } ) { :s , 64 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :u , 8 } ) { :s , 16 } iex&gt; Nx.Type . merge ( { :s , 16 } , { :u , 8 } ) { :s , 16 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :u , 16 } ) { :s , 32 } iex&gt; Nx.Type . merge ( { :s , 32 } , { :u , 8 } ) { :s , 32 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :u , 32 } ) { :s , 64 } iex&gt; Nx.Type . merge ( { :s , 64 } , { :u , 8 } ) { :s , 64 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :u , 64 } ) { :s , 64 } iex&gt; Nx.Type . merge ( { :u , 8 } , { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . merge ( { :u , 64 } , { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . merge ( { :s , 64 } , { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . merge ( { :u , 8 } , { :f , 64 } ) { :f , 64 } iex&gt; Nx.Type . merge ( { :u , 64 } , { :f , 64 } ) { :f , 64 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :f , 64 } ) { :f , 64 } iex&gt; Nx.Type . merge ( { :s , 64 } , { :f , 64 } ) { :f , 64 } iex&gt; Nx.Type . merge ( { :u , 8 } , { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . merge ( { :u , 64 } , { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . merge ( { :s , 8 } , { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . merge ( { :s , 64 } , { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . merge ( { :f , 32 } , { :bf , 16 } ) { :f , 32 } iex&gt; Nx.Type . merge ( { :f , 64 } , { :bf , 16 } ) { :f , 64 } iex&gt; Nx.Type . merge ( { :c , 64 } , { :f , 32 } ) { :c , 64 } iex&gt; Nx.Type . merge ( { :c , 64 } , { :c , 64 } ) { :c , 64 } iex&gt; Nx.Type . merge ( { :c , 128 } , { :c , 64 } ) { :c , 128 }","ref":"Nx.Type.html#merge/2","title":"Nx.Type.merge/2","type":"function"},{"doc":"Merges the given types with the type of a number. We attempt to keep the original type and its size as best as possible. Examples iex&gt; Nx.Type . merge_number ( { :u , 8 } , 0 ) { :u , 8 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , 255 ) { :u , 8 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , 256 ) { :u , 16 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , - 1 ) { :s , 16 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , - 32767 ) { :s , 16 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , - 32768 ) { :s , 16 } iex&gt; Nx.Type . merge_number ( { :u , 8 } , - 32769 ) { :s , 32 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , 0 ) { :s , 8 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , 127 ) { :s , 8 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , - 128 ) { :s , 8 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , 128 ) { :s , 16 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , - 129 ) { :s , 16 } iex&gt; Nx.Type . merge_number ( { :s , 8 } , 1.0 ) { :f , 32 } iex&gt; Nx.Type . merge_number ( { :u , 64 } , - 1337 ) { :s , 64 } iex&gt; Nx.Type . merge_number ( { :f , 32 } , 1 ) { :f , 32 } iex&gt; Nx.Type . merge_number ( { :f , 32 } , 1.0 ) { :f , 32 } iex&gt; Nx.Type . merge_number ( { :f , 64 } , 1.0 ) { :f , 64 }","ref":"Nx.Type.html#merge_number/2","title":"Nx.Type.merge_number/2","type":"function"},{"doc":"Returns the minimum possible value for the given type.","ref":"Nx.Type.html#min_binary/1","title":"Nx.Type.min_binary/1","type":"function"},{"doc":"Returns the minimum possible finite value for the given type.","ref":"Nx.Type.html#min_finite_binary/1","title":"Nx.Type.min_finite_binary/1","type":"function"},{"doc":"Returns infinity as a binary for the given type.","ref":"Nx.Type.html#nan_binary/1","title":"Nx.Type.nan_binary/1","type":"function"},{"doc":"Returns negative infinity as a binary for the given type.","ref":"Nx.Type.html#neg_infinity_binary/1","title":"Nx.Type.neg_infinity_binary/1","type":"function"},{"doc":"Validates and normalizes the given type tuple. It returns the type tuple or raises. Accepts both the tuple format and the short atom format. Examples iex&gt; Nx.Type . normalize! ( { :u , 8 } ) { :u , 8 } iex&gt; Nx.Type . normalize! ( :u8 ) { :u , 8 } iex&gt; Nx.Type . normalize! ( { :u , 0 } ) ** (ArgumentError) invalid numerical type: {:u, 0} (see Nx.Type docs for all supported types) iex&gt; Nx.Type . normalize! ( { :k , 8 } ) ** (ArgumentError) invalid numerical type: {:k, 8} (see Nx.Type docs for all supported types)","ref":"Nx.Type.html#normalize!/1","title":"Nx.Type.normalize!/1","type":"function"},{"doc":"Converts the given type to an aggregation precision. Examples iex&gt; Nx.Type . to_aggregate ( { :s , 8 } ) { :s , 64 } iex&gt; Nx.Type . to_aggregate ( { :u , 32 } ) { :u , 64 } iex&gt; Nx.Type . to_aggregate ( { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . to_aggregate ( { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . to_aggregate ( { :c , 64 } ) { :c , 64 }","ref":"Nx.Type.html#to_aggregate/1","title":"Nx.Type.to_aggregate/1","type":"function"},{"doc":"Converts the given type to a complex representation with the minimum size necessary. Examples iex&gt; Nx.Type . to_complex ( { :s , 64 } ) { :c , 64 } iex&gt; Nx.Type . to_complex ( { :bf , 16 } ) { :c , 64 } iex&gt; Nx.Type . to_complex ( { :f , 32 } ) { :c , 64 } iex&gt; Nx.Type . to_complex ( { :c , 64 } ) { :c , 64 } iex&gt; Nx.Type . to_complex ( { :f , 64 } ) { :c , 128 } iex&gt; Nx.Type . to_complex ( { :c , 128 } ) { :c , 128 }","ref":"Nx.Type.html#to_complex/1","title":"Nx.Type.to_complex/1","type":"function"},{"doc":"Converts the given type to a floating point representation with the minimum size necessary. Note both float and complex are floating point representations. Examples iex&gt; Nx.Type . to_floating ( { :s , 8 } ) { :f , 32 } iex&gt; Nx.Type . to_floating ( { :s , 32 } ) { :f , 32 } iex&gt; Nx.Type . to_floating ( { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . to_floating ( { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . to_floating ( { :c , 64 } ) { :c , 64 }","ref":"Nx.Type.html#to_floating/1","title":"Nx.Type.to_floating/1","type":"function"},{"doc":"Converts the given type to a real number representation with the minimum size necessary. Examples iex&gt; Nx.Type . to_real ( { :s , 8 } ) { :f , 32 } iex&gt; Nx.Type . to_real ( { :s , 64 } ) { :f , 32 } iex&gt; Nx.Type . to_real ( { :bf , 16 } ) { :bf , 16 } iex&gt; Nx.Type . to_real ( { :c , 64 } ) { :f , 32 } iex&gt; Nx.Type . to_real ( { :c , 128 } ) { :f , 64 } iex&gt; Nx.Type . to_real ( { :f , 32 } ) { :f , 32 } iex&gt; Nx.Type . to_real ( { :f , 64 } ) { :f , 64 }","ref":"Nx.Type.html#to_real/1","title":"Nx.Type.to_real/1","type":"function"},{"doc":"Returns a string representation of the given type. Examples iex&gt; Nx.Type . to_string ( { :s , 8 } ) &quot;s8&quot; iex&gt; Nx.Type . to_string ( { :s , 16 } ) &quot;s16&quot; iex&gt; Nx.Type . to_string ( { :s , 32 } ) &quot;s32&quot; iex&gt; Nx.Type . to_string ( { :s , 64 } ) &quot;s64&quot; iex&gt; Nx.Type . to_string ( { :u , 8 } ) &quot;u8&quot; iex&gt; Nx.Type . to_string ( { :u , 16 } ) &quot;u16&quot; iex&gt; Nx.Type . to_string ( { :u , 32 } ) &quot;u32&quot; iex&gt; Nx.Type . to_string ( { :u , 64 } ) &quot;u64&quot; iex&gt; Nx.Type . to_string ( { :f , 16 } ) &quot;f16&quot; iex&gt; Nx.Type . to_string ( { :bf , 16 } ) &quot;bf16&quot; iex&gt; Nx.Type . to_string ( { :f , 32 } ) &quot;f32&quot; iex&gt; Nx.Type . to_string ( { :f , 64 } ) &quot;f64&quot;","ref":"Nx.Type.html#to_string/1","title":"Nx.Type.to_string/1","type":"function"},{"doc":"","ref":"Nx.Type.html#t:short_t/0","title":"Nx.Type.short_t/0","type":"type"},{"doc":"","ref":"Nx.Type.html#t:t/0","title":"Nx.Type.t/0","type":"type"},{"doc":"The specification and helper functions for custom defn compilers.","ref":"Nx.Defn.Compiler.html","title":"Nx.Defn.Compiler","type":"behaviour"},{"doc":"Callback for compilation. It receives an opaque key used for caching, the function vars , the function fun which builds a defn expression, and the compiler options. It must call fun with the vars as arguments. It returns a function that receives a list of arguments and returns a list of results. The callback uses double underscores so it can be defined at root modules without affecting the module's main API.","ref":"Nx.Defn.Compiler.html#c:__compile__/4","title":"Nx.Defn.Compiler.__compile__/4","type":"callback"},{"doc":"Callback for compilation. It receives an opaque key used for caching, the function vars , the function fun which builds a defn expression, a list of argument list in args_list , and the compiler options. It must call fun with the vars as arguments. Note the key does not include the vars in its cache. Therefore, if you want to cache the result of fun.(vars) , you likely want to include the vars in the cache key. vars is a list of containers expressions. Once the expression is built and compiled, it must be invoked for each list of arguments in args_list . In a nutshell, vars are used to build the expression from fun which is then invoked for each list of arguments in args_list . All lists in args_list are guaranteed to be flat lists of the same length, containing zero-arity functions that return tensors of the same type, shape, and name. The callback uses double underscores so it can be defined at root modules without affecting the module's main API.","ref":"Nx.Defn.Compiler.html#c:__jit__/5","title":"Nx.Defn.Compiler.__jit__/5","type":"callback"},{"doc":"Callback for streaming (on top of JIT compilation). It receives the same arguments as __jit__/5 with the addition of the streaming input and accumulator templates. If the input and accumulator are containers, they are kept in their container shapes. As in __jit__/5 , both vars and args_list are flat lists of tensors (without their container shape). It must return a struct that implements the Nx.Stream protocol.","ref":"Nx.Defn.Compiler.html#c:__stream__/7","title":"Nx.Defn.Compiler.__stream__/7","type":"callback"},{"doc":"Returns the current compiler. Returns nil if we are not inside defn .","ref":"Nx.Defn.Compiler.html#current/0","title":"Nx.Defn.Compiler.current/0","type":"function"},{"doc":"Functions to deal with composite data types according to Nx.Container . The functions in this module can be used both inside and outside defn . Note the functions in this module traverses tensors, but it does not automatically convert values to tensors. For example, the tuple {1, 2, 3} once traversed will emit the numbers 1 , 2 , and 3 . If desired, you can invoke Nx.to_tensor/1 to normalize them. Note that, when a value is given to defn , it is first converted to tensors and containers via Nx.LazyContainer . Inside defn , there are no lazy containers, only containers.","ref":"Nx.Defn.Composite.html","title":"Nx.Defn.Composite","type":"module"},{"doc":"Traverses two composite types to see if they are compatible. For non-composite types, the given fun will be called to compare numbers/tensors pairwise.","ref":"Nx.Defn.Composite.html#compatible?/3","title":"Nx.Defn.Composite.compatible?/3","type":"function"},{"doc":"Counts the number of non-composite types in the composite type. Examples iex&gt; Nx.Defn.Composite . count ( 123 ) 1 iex&gt; Nx.Defn.Composite . count ( { 1 , { 2 , 3 } } ) 3 iex&gt; Nx.Defn.Composite . count ( { Complex . new ( 1 ) , { Nx . tensor ( 2 ) , 3 } } ) 3","ref":"Nx.Defn.Composite.html#count/1","title":"Nx.Defn.Composite.count/1","type":"function"},{"doc":"Flattens recursively the given list of composite types. Elements that are not tensors (i.e. numbers and Complex numbers) are kept as is unless a custom function is given. Examples iex&gt; Nx.Defn.Composite . flatten_list ( [ 1 , { 2 , 3 } ] ) [ 1 , 2 , 3 ] iex&gt; Nx.Defn.Composite . flatten_list ( [ 1 , { 2 , 3 } ] , [ Nx . tensor ( 4 ) ] ) [ 1 , 2 , 3 , Nx . tensor ( 4 ) ]","ref":"Nx.Defn.Composite.html#flatten_list/2","title":"Nx.Defn.Composite.flatten_list/2","type":"function"},{"doc":"Reduces recursively the given composite types with acc and fun . If composite tensor expressions are given, such as a tuple, the composite type is recursively traversed and returned. If a non-composite tensor expression is given, the function is invoked for it but not for its arguments.","ref":"Nx.Defn.Composite.html#reduce/3","title":"Nx.Defn.Composite.reduce/3","type":"function"},{"doc":"Traverses recursively the given composite types with fun . If a composite tensor is given, such as a tuple, the composite type is recursively traversed and returned. Otherwise the function is invoked with the tensor (be it a number, complex, or actual tensor).","ref":"Nx.Defn.Composite.html#traverse/2","title":"Nx.Defn.Composite.traverse/2","type":"function"},{"doc":"Traverses recursively the given composite types with acc and fun . If a composite tensor is given, such as a tuple, the composite type is recursively traversed and returned. Otherwise the function is invoked with the tensor (be it a number, complex, or actual tensor).","ref":"Nx.Defn.Composite.html#traverse/3","title":"Nx.Defn.Composite.traverse/3","type":"function"},{"doc":"The default implementation of a Nx.Defn.Compiler that evaluates the expression tree against the tensor backend.","ref":"Nx.Defn.Evaluator.html","title":"Nx.Defn.Evaluator","type":"module"},{"doc":"","ref":"Nx.Defn.Expr.html","title":"Nx.Defn.Expr","type":"module"},{"doc":"The expression used by Nx.Defn.Compiler . Nx.Defn.Compiler changes Nx default backend from Nx.BinaryBackend to Nx.Defn.Expr . It is a struct with the following fields: :id - a unique identifier :op - the operation name :args - the operation arguments :context - the context of the expression. The default context is :root . Convenience functions for traversing expressions and composite types can be found in Nx.Defn.Composite and Nx.Defn.Tree . Syntax nodes Most nodes are created directly via the Nx module and therefore map directly to Nx.Tensor callbacks. However the following syntax nodes exist: parameter(integer) constant(number) tensor(tensor) metadata(expr, metadata) elem(tuple, pos) - created automatically from expression that return tuples. Note it may return tuples too, which means we have nested tuples fun(parameters, t, mfa) - the mfa is used only for introspection purposes cond(clauses, otherwise) while(initial, condition, body) attach_token(token(%Nx.Defn.Token{}), expr) defn compilers must handle said nodes accordingly.","ref":"Nx.Defn.Expr.html#__struct__/0","title":"Nx.Defn.Expr.__struct__/0","type":"function"},{"doc":"Creates a cond tensor expression.","ref":"Nx.Defn.Expr.html#cond/2","title":"Nx.Defn.Expr.cond/2","type":"function"},{"doc":"Creates a tensor expression metadata node wrapping the given tensor expression. The metadata is map. If the inspect key is present, it will be used to annotate the metadata when inspected. Otherwise the metadata node does not appear during inspection.","ref":"Nx.Defn.Expr.html#metadata/2","title":"Nx.Defn.Expr.metadata/2","type":"function"},{"doc":"Creates a tensor expression parameter at pos based on the given tensor expression.","ref":"Nx.Defn.Expr.html#parameter/2","title":"Nx.Defn.Expr.parameter/2","type":"function"},{"doc":"Creates a tensor expression parameter at pos based on the given tensor and context .","ref":"Nx.Defn.Expr.html#parameter/3","title":"Nx.Defn.Expr.parameter/3","type":"function"},{"doc":"Creates a tensor expression parameter at pos with the given context , type , shape , and pos .","ref":"Nx.Defn.Expr.html#parameter/4","title":"Nx.Defn.Expr.parameter/4","type":"function"},{"doc":"Builds an tensor expression from the given tensor.","ref":"Nx.Defn.Expr.html#tensor/1","title":"Nx.Defn.Expr.tensor/1","type":"function"},{"doc":"Creates a tuple with elements in list that points to tuple expression expr . list must be a list of tensor expressions of the same size as the tuple expression.","ref":"Nx.Defn.Expr.html#tuple/2","title":"Nx.Defn.Expr.tuple/2","type":"function"},{"doc":"Creates a while tensor expression.","ref":"Nx.Defn.Expr.html#while/5","title":"Nx.Defn.Expr.while/5","type":"function"},{"doc":"A defn token used by hooks. Documentation for compilers The token has a hooks field as a list of maps of the shape: %{ expr : Nx.Tensor . t | Nx.Container . t , name : atom ( ) , callback : ( Nx.Tensor . t | Nx.Container . t -&gt; term ( ) ) | nil } The hooks field must only be accessed by defn compilers.","ref":"Nx.Defn.Token.html","title":"Nx.Defn.Token","type":"module"},{"doc":"Helper functions to traverse defn expressions, either as single nodes or recursively.","ref":"Nx.Defn.Tree.html","title":"Nx.Defn.Tree","type":"module"},{"doc":"Applies the given function to the arguments of the node, with the given accumulator as a starting value. By default, type is :all , which means all arguments are traversed. If type is :scope , only expressions that are in the same scope are traversed. Therefore, expressions such as while 's condition and body, optional 's default implementation, functions, and so forth are not traversed. Note cond s are always traversed because, while they introduce a new scope, they can also access its parents directly, so you must take cond s into account accordingly. Warning: be very careful when using this function to traverse the expression recursively. If you plan to do so, you should consider also storing the visited nodes to avoid multiple traversals by using tensor.data.expr.id as cache key.","ref":"Nx.Defn.Tree.html#apply_args/4","title":"Nx.Defn.Tree.apply_args/4","type":"function"},{"doc":"Check if the given tree has any of the given hooks in it.","ref":"Nx.Defn.Tree.html#has_hooks?/2","title":"Nx.Defn.Tree.has_hooks?/2","type":"function"},{"doc":"Puts new args in the given tensor expression and gives it a new id.","ref":"Nx.Defn.Tree.html#put_args/2","title":"Nx.Defn.Tree.put_args/2","type":"function"},{"doc":"Rewrites the types of the given tensor expressions according to the given options. Options :max_float_type - set the max float type :max_signed_type - set the max signed integer type :max_unsigned_type - set the max unsigned integer type","ref":"Nx.Defn.Tree.html#rewrite_types/2","title":"Nx.Defn.Tree.rewrite_types/2","type":"function"},{"doc":"Gets all IDs of all elements in the same scope. while 's condition and body, fun 's body and similar are considered different scopes. When it comes to cond , an ID will only be considered if it is used outside of the cond or used in several distinct conds. Constants are also ignored, as they have global IDs based on the constants themselves. An existing maps of ids can be given to accumulate on top of it.","ref":"Nx.Defn.Tree.html#scope_ids/2","title":"Nx.Defn.Tree.scope_ids/2","type":"function"},{"doc":"Mix . install ( [ { :nx , &quot;~&gt; 0.2&quot; } ] )","ref":"intro-to-nx.html","title":"Introduction to Nx","type":"extras"},{"doc":"Elixir's primary numerical datatypes and structures are not optimized for numerical programming. Nx is a library built to bridge that gap. Elixir Nx is a numerical computing library to smoothly integrate to typed, multidimensional data implemented on other platforms (called tensors). This support extends to the compilers and libraries that support those tensors. Nx has three primary capabilities: In Nx, tensors hold typed data in multiple, named dimensions. Numerical definitions, known as defn , support custom code with tensor-aware operators and functions. Automatic differentiation , also known as autograd or autodiff, supports common computational scenarios such as machine learning, simulations, curve fitting, and probabilistic models. Here's more about each of those capabilities. Nx tensors can hold unsigned integers (u8, u16, u32, u64), signed integers (s8, s16, s32, s64), floats (f32, f64), brain floats (bf16), and complex (c64, c128). Tensors support backends implemented outside of Elixir, including Google's Accelerated Linear Algebra (XLA) and LibTorch. Numerical definitions have compiler support to allow just-in-time compilation that support specialized processors to speed up numeric computation including TPUs and GPUs. To know Nx, we'll get to know tensors first. This rapid overview will touch on the major libraries. Then, future notebooks will take a deep dive into working with tensors in detail, autograd, and backends. Then, we'll dive into specific problem spaces like Axon, the machine learning library.","ref":"intro-to-nx.html#numerical-elixir","title":"Introduction to Nx - Numerical Elixir","type":"extras"},{"doc":"Systems of equations are a central theme in numerical computing. These equations are often expressed and solved with multidimensional arrays. For example, this is a two dimensional array: $$ \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} $$ Elixir programmers typically express a similar data structure using a list of lists, like this: [ [ 1 , 2 ] , [ 3 , 4 ] ] This data structure works fine within many functional programming algorithms, but breaks down with deep nesting and random access. On top of that, Elixir numeric types lack optimization for many numerical applications. They work fine when programs need hundreds or even thousands of calculations. They tend to break down with traditional STEM applications when a typical problem needs millions of calculations. In Nx, we express multi-dimensional data using typed tensors. Simply put, a tensor is a multi-dimensional array with a predetermined shape and type. To interact with them, Nx relies on tensor-aware operators rather than Enum.map/2 and Enum.reduce/3 . In this section, we'll look at some of the various tools for creating and interacting with tensors. The IEx helpers will assist our exploration of the core tensor concepts. import IEx.Helpers Now, everything is set up, so we're ready to create some tensors. Creating tensors Start out by getting a feel for Nx through its documentation. Do so through the IEx helpers, like this: h Nx Immediately, you can see that tensors are at the center of the API. The main API for creating tensors is Nx.tensor/2 : h Nx . tensor We use it to create tensors from raw Elixir lists of numbers, like this: tensor = 1 .. 4 |&gt; Enum . chunk_every ( 2 ) |&gt; Nx . tensor ( names : [ :y , :x ] ) The result shows all of the major fields that make up a tensor: The data, presented as the list of lists [[1, 2], [3, 4]] . The type of the tensor, a signed integer 64 bits long, with the type s64 . The shape of the tensor, going left to right, with the outside dimensions listed first. The names of each dimension. We can easily convert it to a binary: binary = Nx . to_binary ( tensor ) A tensor of type s64 uses four bytes for each integer. The binary shows the individual bytes that make up the tensor, so you can see the integers 1..4 interspersed among the zeros that make up the tensor. If all of our data only uses positive numbers from 0..255 , we could save space with a different type: Nx . tensor ( [ [ 1 , 2 ] , [ 3 , 4 ] ] , type : :u8 ) |&gt; Nx . to_binary ( ) If you have already have binary, you can directly convert it to a tensor by passing the binary and the type: Nx . from_binary ( &lt;&lt; 0 , 1 , 2 &gt;&gt; , :u8 ) This function comes in handy when working with published datasets because they must often be processed. Elixir binaries make quick work of dealing with numerical data structured for platforms other than Elixir. We can get any cell of the tensor: tensor [ 0 ] [ 1 ] Now, try getting the first row of the tensor: # ...your code here... We can also get a whole dimension: tensor [ x : 1 ] or a range: tensor [ y : 0 .. 1 ] Now, create your own {3, 3} tensor with named dimensions return a {2, 2} tensor containing the first two columns of the first two rows We can get information about this most recent term with the IEx helper i , like this: i tensor The tensor is a struct that supports the usual Inspect protocol. The struct has keys, but we typically treat the Nx.Tensor as an opaque data type (meaning we typically access the contents and shape of a tensor using the tensor's API instead of the struct). Primarily, a tensor is a struct, and the functions to access it go through a specific backend. We'll get to the backend details in a moment. For now, use the IEx h helper to get more documentation about tensors. We could also open a Code cell, type Nx.tensor, and hover the cursor over the word tensor to see the help about that function. We can get the shape of the tensor with Nx.shape/1 : Nx . shape ( tensor ) We can also create a new tensor with a new shape using Nx.reshape/2 : Nx . reshape ( tensor , { 1 , 4 } , names : [ :batches , :values ] ) This operation reuses all of the tensor data and simply changes the metadata, so it has no notable cost. The new tensor has the same type, but a new shape. Now, reshape the tensor to contain three dimensions with one batch, one row, and four columns. # ...your code here... We can create a tensor with named dimensions, a type, a shape, and our target data. A dimension is called an axis , and axes can have names. We can specify the tensor type and dimension names with options, like this: Nx . tensor ( [ [ 1 , 2 , 3 ] ] , names : [ :rows , :cols ] , type : :u8 ) We created a tensor of the shape {1, 3} , with the type u8 , the values [1, 2, 3] , and two axes named rows and cols . Now we know how to create tensors, so it's time to do something with them.","ref":"intro-to-nx.html#nx-and-tensors","title":"Introduction to Nx - Nx and tensors","type":"extras"},{"doc":"In the last section, we created a s64[2][2] tensor. In this section, we'll use Nx functions to work with it. Here's the value of tensor : tensor We can use IEx.Helpers.exports/1 or code completion to find some functions in the Nx module that operate on tensors: exports Nx You might recognize that many of those functions have names that suggest that they would work on primitive values, called scalars. Indeed, a tensor can be a scalar: pi = Nx . tensor ( 3.1415 , type : :f32 ) Take the cosine: Nx . cos ( pi ) That function took the cosine of pi . We can also call them on a whole tensor, like this: Nx . cos ( tensor ) We can also call a function that aggregates the contents of a tensor. For example, to get a sum of the numbers in tensor , we can do this: Nx . sum ( tensor ) That's 1 + 2 + 3 + 4 , and Nx went to multiple dimensions to get that sum. To get the sum of values along the x axis instead, we'd do this: Nx . sum ( tensor , axes : [ :x ] ) Nx sums the values across the x dimension: 1 + 2 in the first row and 3 + 4 in the second row. Now, create a {2, 2, 2} tensor with the values 1..8 with dimension names [:z, :y, :x] calculate the sums along the y axis # ...your code here... Sometimes, we need to combine two tensors together with an operator. Let's say we wanted to subtract one tensor from another. Mathematically, the expression looks like this: $$ \\begin{bmatrix} 5 &amp; 6 \\\\ 7 &amp; 8 \\end{bmatrix} - \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} = \\begin{bmatrix} 4 &amp; 4 \\\\ 4 &amp; 4 \\end{bmatrix} $$ To solve this problem, subtract each right-hand integer from the corresponding left-hand integer. Unfortunately, we cannot use Elixir's built-in subtraction operator as it is not tensor-aware. Luckily, we can use the Nx.subtract/2 function to solve the problem: tensor2 = Nx . tensor ( [ [ 5 , 6 ] , [ 7 , 8 ] ] ) Nx . subtract ( tensor2 , tensor ) We get a {2, 2} shaped tensor full of fours, exactly as we expected. When calling Nx.subtract/2 , both operands had the same shape. Sometimes, you might want to process functions where the dimensions don't match. To solve this problem, Nx takes advantage of a concept called broadcasting .","ref":"intro-to-nx.html#tensor-aware-functions","title":"Introduction to Nx - Tensor aware functions","type":"extras"},{"doc":"Often, the dimensions of tensors in an operator don't match. For example, you might want to subtract a 1 from every element of a {2, 2} tensor, like this: $$ \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} - 1 = \\begin{bmatrix} 0 &amp; 1 \\\\ 2 &amp; 3 \\end{bmatrix} $$ Mathematically, it's the same as this: $$ \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} - \\begin{bmatrix} 1 &amp; 1 \\\\ 1 &amp; 1 \\end{bmatrix} = \\begin{bmatrix} 0 &amp; 1 \\\\ 2 &amp; 3 \\end{bmatrix} $$ That means we need a way to convert 1 to a {2, 2} tensor. Nx.broadcast/2 solves that problem. This function takes a tensor or a scalar and a shape. Nx . broadcast ( 1 , { 2 , 2 } ) This broadcast takes the scalar 1 and translates it to a compatible shape by copying it. Sometimes, it's easier to provide a tensor as the second argument, and let broadcast/2 extract its shape: Nx . broadcast ( 1 , tensor ) The code broadcasts 1 to the shape of tensor . In many operators and functions, the broadcast happens automatically: Nx . subtract ( tensor , 1 ) This result is possible because Nx broadcasts both tensors in subtract/2 to compatible shapes. That means you can provide scalar values as either argument: Nx . subtract ( 10 , tensor ) Or subtract a row or column. Mathematically, it would look like this: $$ \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} - \\begin{bmatrix} 1 &amp; 2 \\end{bmatrix} = \\begin{bmatrix} 0 &amp; 0 \\\\ 2 &amp; 2 \\end{bmatrix} $$ which is the same as this: $$ \\begin{bmatrix} 1 &amp; 2 \\\\ 3 &amp; 4 \\end{bmatrix} - \\begin{bmatrix} 1 &amp; 2 \\\\ 1 &amp; 2 \\end{bmatrix} = \\begin{bmatrix} 0 &amp; 0 \\\\ 2 &amp; 2 \\end{bmatrix} $$ This rewrite happens in Nx too, also through a broadcast. We want to broadcast the tensor [1, 2] to match the {2, 2} shape, like this: Nx . broadcast ( Nx . tensor ( [ 1 , 2 ] ) , { 2 , 2 } ) The subtract function in Nx takes care of that broadcast implicitly, as before: Nx . subtract ( tensor , Nx . tensor ( [ 1 , 2 ] ) ) The broadcast worked as advertised, copying the [1, 2] row enough times to fill a {2, 2} tensor. A tensor with a dimension of 1 will broadcast to fill the tensor: [ [ 1 ] , [ 2 ] ] |&gt; Nx . tensor ( ) |&gt; Nx . broadcast ( { 1 , 2 , 2 } ) [ [ [ 1 , 2 , 3 ] ] ] |&gt; Nx . tensor ( ) |&gt; Nx . broadcast ( { 4 , 2 , 3 } ) Both of these examples copy parts of the tensor enough times to fill out the broadcast shape. You can check out the Nx broadcasting documentation for more details: h Nx . broadcast Much of the time, you won't have to broadcast yourself. Many of the functions and operators Nx supports will do so automatically. We can use tensor-aware operators via various Nx functions and many of them implicitly broadcast tensors. Throughout this section, we have been invoking Nx.subtract/2 and our code would be more expressive if we could use its equivalent mathematical operator. Fortunately, Nx provides a way. Next, we'll dive into numerical definitions using defn .","ref":"intro-to-nx.html#broadcasts","title":"Introduction to Nx - Broadcasts","type":"extras"},{"doc":"The defn macro simplifies the expression of mathematical formulas containing tensors. Numerical definitions have two primary benefits over classic Elixir functions. They are tensor-aware . Nx replaces operators like Kernel.-/2 with the Defn counterparts &amp;mdash; which in turn use Nx functions optimized for tensors &amp;mdash; so the formulas we express can use tensors out of the box. defn definitions allow for building computation graph of all the individual operations and using a just-in-time (JIT) compiler to emit highly specialized native code for the desired computation unit. We don't have to do anything special to get access to get tensor awareness beyond importing Nx.Defn and writing our code within a defn block. To use Nx in a Mix project or a notebook, we need to include the :nx dependency and import the Nx.Defn module. The dependency is already included, so import it in a Code cell, like this: import Nx.Defn Just as the Elixir language supports def , defmacro , and defp , Nx supports defn . There are a few restrictions. It allows only numerical arguments in the form of primitives or tensors as arguments or return values, and supports only a subset of the language. The subset of Elixir allowed within defn is quite broad, though. We can use macros, pipes, and even conditionals, so we're not giving up much when you're declaring mathematical functions. Additionally, despite these small concessions, defn provides huge benefits. Code in a defn block uses tensor aware operators and types, so the math beneath your functions has a better chance to shine through. Numerical definitions can also run on accelerated numerical processors like GPUs and TPUs. Here's an example numerical definition: defmodule TensorMath do import Nx.Defn defn subtract ( a , b ) do a - b end end This module has a numerical definition that will be compiled. If we wanted to specify a compiler for this module, we could add a module attribute before the defn clause. One of such compilers is the EXLA compiler . You'd add the mix dependency for EXLA and do this: @defn_compiler EXLA defn subtract ( a , b ) do a - b end Now, it's your turn. Add a defn to TensorMath that accepts two tensors representing the lengths of sides of a right triangle and uses the pythagorean theorem to return the length of the hypotenuse . Add your function directly to the previous Code cell. The last major feature we'll cover is called auto-differentiation, or autograd.","ref":"intro-to-nx.html#numerical-definitions-defn","title":"Introduction to Nx - Numerical definitions (defn)","type":"extras"},{"doc":"An important mathematical property for a function is the rate of change, or the gradient. These gradients are critical for solving systems of equations and building probabilistic models. In advanced math, derivatives, or differential equations, are used to take gradients. Nx can compute these derivatives automatically through a feature called automatic differentiation, or autograd. Here's how it works. h Nx.Defn . grad We'll build a module with a few functions, and then create another function to create the gradients of those functions. The function grad/1 takes a function, and returns a function returning the gradient. We have two functions: poly/1 is a simple numerical definition, and poly_slope_at/1 returns its gradient: $$ poly: f(x) = 3x^2 + 2x + 1 \\\\ $$ $$ polySlopeAt: g(x) = 6x + 2 $$ Here's the Elixir equivalent of those functions: defmodule Funs do defn poly ( x ) do 3 * Nx . power ( x , 2 ) + 2 * x + 1 end defn poly_slope_at ( x ) do grad ( &amp; poly / 1 ) . ( x ) end end Notice the second defn . It uses grad/1 to take its derivative using autograd. It uses the intermediate defn AST and mathematical composition to compute the derivative. You can see it at work here: Funs . poly_slope_at ( 2 ) Nice. If you plug the number 2 into the function $$ 6x + 2 $$ you get 14! Said another way, if you look at the graph at exactly 2, the rate of increase is 14 units of poly(x) for every unit of x , precisely at x . Nx also has helpers to get gradients corresponding to a number of inputs. These come into play when solving systems of equations. Now, you try. Find a function computing the gradient of a sin wave. # your code here","ref":"intro-to-nx.html#automatic-differentiation-autograd","title":"Introduction to Nx - Automatic differentiation (autograd)","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Enhancements [Nx] Add Nx.Batch and Nx.Serving [Nx] Implement Nx.Container for numbers, complex, and tensors for completeness [Nx] Support batches in Nx.eye/2 Bug fixes [Nx] Keep input tensor names on associative scan [Nx.BinaryBackend] Differentiate between complex and real output in as_type [Nx.BinaryBackend] Fix loss of precision in Nx.complex/2 [Nx.BinaryBackend] Preserve NaNs in window and reduce operations [Nx.Random] Do not return infinity on normal/2 for f16","ref":"changelog.html#v0-4-1-2022-12-07","title":"Changelog - v0.4.1 (2022-12-07)","type":"extras"},{"doc":"Enhancements [Nx] Add Nx.rename/2 , Nx.median/2 , Nx.weighted_mean/3 , and Nx.mode/2 [Nx] Implement cumulative operations using associative scan for improved performance [Nx.Constants] Add min and max [Nx.Defn] Allow lists and functions anywhere as arguments in defn , jit and compile [Nx.Defn] Add Nx.LazyContainer that allows a data-structure to lazily define tensors [Nx.Defn] Allow tensors and ranges as generators inside while [Nx.Defn] Add debug_expr/2 and debug_expr_apply/3 [Nx.Defn.Evaluator] Calculate cache lifetime to reduce memory usage on large numerical programs [Nx.LinAlg] Handle Hermitian matrices in eigh [Nx.LinAlg] Support batched operations in adjoint , cholesky , determinant , eigh , invert , lu , matrix_power , solve , svd , and triangular_solve [Nx.Random] Support pseudo random number generators algorithms Bug fixes [Nx] Perform window_reduce / reduce operations from infinity and negative infinity [Nx.Defn] Ensure defnp emits warnings when unused [Nx.Defn] Warn on unused variables in while Deprecations [Nx] Deprecate tensor as shape in Nx.eye/2 and Nx.iota/2 [Nx] Deprecate Nx.random_uniform/2 and Nx.random_normal/2","ref":"changelog.html#v0-4-0-2022-10-25","title":"Changelog - v0.4.0 (2022-10-25)","type":"extras"},{"doc":"Enhancements [Nx] Improve support for non-finite values in Nx.broadcast/2 , Nx.all_close/2 , and more [Nx] Add Nx.is_nan/1 and Nx.is_infinite/1 [Nx] Support booleans in Nx.tensor/2 [Nx] Add Nx.fft/2 and Nx.ifft/2 [Nx] Rename Nx.logistic/1 to Nx.sigmoid/1 [Nx] Add Nx.put_diagonal/3 and Nx.indexed_put/3 [Nx] Add :reverse to cummulative functions [Nx] Add Nx.to_batched/3 which returns a stream [Nx] Support batched tensors in Nx.LinAlg.qr/1 [Nx.Defn] Add Nx.Defn.compile/3 for precompiling expressions [Nx.Defn] Add deftransform/2 and deftransformp/2 for easier to define transforms [Nx.Defn] Add div/2 [Nx.Defn] Support case/2 , raise/1 , and raise/2 [Nx.Defn] Support booleans in if , cond , and boolean operators [Nx.Defn] Perform branch elimitation in if and cond and execute branches lazily [Nx.Defn.Evaluator] Garbage collect after evaluation (it can be disabled by setting the :garbage_collect compiler option to false) Deprecations [Nx] Nx.to_batched_list/3 is deprecated in favor of Nx.to_batched/3 [Nx.Defn] transform/2 is deprecated in favor of deftransform/2 and deftransformp/2 [Nx.Defn] assert_shape/2 and assert_shape_pattern/2 are deprecated in favor of case/2 + raise/2 [Nx.Defn] inspect_expr/1 and inspect_value/1 are deprecated in favor of print_expr/1 and print_value/1 respectively","ref":"changelog.html#v0-3-0-2022-08-13","title":"Changelog - v0.3.0 (2022-08-13)","type":"extras"},{"doc":"Enhancements [Nx] Improve support for non-finite values in Nx.tensor/1 [Nx] Use iovec on serialization to avoid copying binaries [Nx.BinaryBackend] Improve for complex numbers in Nx.tensor/1 [Nx.Defn] Improve for complex numbers inside defn Bug fixes [Nx] Properly normalize type in Nx.from_binary/3 [Nx.Defn] Raise on Nx.Defn.Expr as JIT argument [Nx.Defn.Evaluator] Handle concatenate arguments on evaluator","ref":"changelog.html#v0-2-1-2022-06-04","title":"Changelog - v0.2.1 (2022-06-04)","type":"extras"},{"doc":"This version requires Elixir v1.13+. Enhancements [Nx] Support atom notation as the type option throughout the API (for example, :u8 , :f64 , etc) [Nx] Add support for complex numbers (c64, c128) [Nx] Add Nx.cumulative_sum/2 , Nx.cumulative_product/2 , Nx.cumulative_min/2 , Nx.cumulative_max/2 [Nx] Add Nx.conjugate/1 , Nx.phase/1 , Nx.real/1 , and Nx.imag/1 [Nx] Initial support for NaN and Infinity [Nx] Add :axis option to Nx.shuffle/2 [Nx] Add Nx.axis_index/2 [Nx] Add Nx.variance/2 to Nx.standard_deviation/2 [Nx] Rename Nx.slice_axis/3 to Nx.slice_along_axis/4 [Nx.Backend] Add support for optional backends [Nx.Constants] Provide a convenient module to host constants [Nx.Defn] Improve error messages throughout the compiler","ref":"changelog.html#v0-2-0-2022-04-28","title":"Changelog - v0.2.0 (2022-04-28)","type":"extras"},{"doc":"First release.","ref":"changelog.html#v0-1-0-2022-01-06","title":"Changelog - v0.1.0 (2022-01-06)","type":"extras"}]